﻿using System;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Globalization;
using System.Reflection;
using System.Resources;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Sbo.Logex.Butternut.BusinessObjects.Definitions.Tables;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_ProductionIssueBNScript
{
	/// <summary>
	/// 
	/// </summary>
	public class WorkflowScript_ProductionIssueBNScript : WorkflowInstanceScriptBase
	{
		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private static class Defs
		{
			/// <summary>
			/// The time to sleep in milliseconds
			/// </summary>
			public const int SleepTime = 2000;
		}

		// Input parameters
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		ISboProviderService m_sboProviderService;

		// Output parameters
		// <none>

		// Sub-flows
		private WorkflowInstanceBase m_beforeProductionOrderClosedHookFlow;
		private WorkflowInstanceBase m_moveToProductionLineFlow;
		private WorkflowInstanceBase m_afterQuantityProducedHookFlow;

		private string m_moveToProductionLineFlow_StrProductionOrderNumber;
		private string m_moveToProductionLineFlow_InputLocationCode;
		private string m_moveToProductionLineFlow_ProductionLineCode;
		private int? m_moveToProductionLineFlow_ProductionOrderDocEntry;
		private int? m_moveToProductionLineFlow_CurrentProdBatch;


		private WorkflowInstanceBase m_printItemLabelFlow;

		private bool m_printItemLabelFlow_BackRequested = false;
		private string m_printItemLabelFlow_LocationCode;
		private string m_printItemLabelFlow_NeededItemCode;
		private string m_printItemLabelFlow_NeededBatchNumber;
		private string m_printItemLabelFlow_NeededBatchNumber2;
		private List<string> m_printItemLabelFlow_NeededSerialNumbers;
		private DateTime? m_printItemLabelFlow_NeededBestBeforeDate;
		private int? m_printItemLabelFlow_NeededNrOfLabels;
		private bool m_printItemLabelFlow_AskBatchInfo;
		private double? m_printItemLabelFlow_EnteredQuantity;


		private WorkflowInstanceBase m_enterProduceQuantityHookFlow;

		private bool m_enterProduceQuantityHookFlow_BackRequested;
		private double m_enterProduceQuantityHookFlow_QuantityEntered;
		private double m_enterProduceQuantityHookFlow_MaximumQuantity;
		private int? m_enterProduceQuantityHookFlow_ByProductLineNum;
		private ProductionInfo m_enterProduceQuantityHookFlow_ProductionInfo;
		private IPreAndPostActionInTransaction m_enterProduceQuantityHookFlow_PreAndPostActionObject;


		WorkflowInstanceBase m_enterQuantitySubFlow;

		protected bool m_subFlowBackRequested;
		protected Collection<string> m_subFlowListOfSerialNumbers;
		protected double m_subFlowQuantity;

		protected double? m_subFlowQuantityUom2;
		protected int? m_subFlowProductionOrderDocEntry;

		WorkflowInstanceBase m_timeRegistrationSubFlow;

		/// <summary>
		/// The m_enter batch attributes flow
		/// </summary>
		private WorkflowInstanceBase m_enterBatchAttributesFlow;
		/// <summary>
		/// The m_enter batch attributes flow_ list of batch attributes
		/// </summary>
		protected Collection<ItemTransactionalBatchAttributeInfo> m_enterBatchAttributesFlow_ListOfBatchAttributes;


		/// <summary>
		/// The m_enter quality values flow
		/// </summary>
		private WorkflowInstanceBase m_enterQualityValuesFlow;

		/// <summary>
		/// The m_enter quality values flow_ list of quality values
		/// </summary>
		protected Collection<QualityValueInfo> m_enterQualityValuesFlow_ListOfQualityValues;

		/// <summary>
		/// Initializes a new instance of the <see cref="WorkflowScript_ProductionScript"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="factory">The factory.</param>
		public WorkflowScript_ProductionIssueBNScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{
			m_moveToProductionLineFlow = this.Create("MoveToProductionLineScript");
			m_printItemLabelFlow = Create("PrintItemLabelScript");
			m_enterProduceQuantityHookFlow = Create("EnterProduceQuantityHookFlow");
			m_enterQuantitySubFlow = this.Create("EnterQuantityScript");
			m_timeRegistrationSubFlow = Create("TimeRegistrationScript");
			m_enterBatchAttributesFlow = this.Create("EnterBatchAttributesScript");
			m_enterQualityValuesFlow = this.Create("EnterQualityValuesScript");
			m_beforeProductionOrderClosedHookFlow = this.Create("BeforeProductionOrderClosedHookScript");
			m_afterQuantityProducedHookFlow = this.Create("AfterQuantityProducedHookScript");
		}

		#region WorkflowInstanceScriptBase Members

		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;


			//reset in case of error

			string deviceID = session.DeviceInstanceId;
			string query = null;

			Produmex.Foundation.Messages.Message msg = null;

			string productionLineCode = null;
			string productionLineCodeForDevice = null;
			PmxOseWarehouseInfo whsInfo = null;
			PmxOseWarehouseInfo whsInfoDevice = null;
			DataSet dsProductionLines = null;
			string inputLocationCode = null;
			string restLocationCode = null;
			string outputLocationCode = null;
			string initialErrorKey = null;
			DataSet dsStartedProductionOrder = null;
			GeneratedBatchAndBestBeforeDateInfo batchAndBestBeforeDateInfo = null;
			string batchNumber = null;
			string batchNumber2 = null;
			DateTime? bestBeforeDate = null;
			//bool isUserAllowedToChangeBatchNumber = false;
			//bool isUserAllowedToChangeBestBeforeDate = false;
			int? productionOrderDocEntry = null;
			DataSet dsProductionOrder = null;
			string customer = null;
			DateTime dueDate = DateTime.Now;
			string itemToProduce = null;
			string itemCodeToProduce = null;
			int productionOrderNumber = 0;
			string productionComments = null;
			double stillToProduceQuantity = 0;
			int? stepListKey = null;
			string productionLineCodeFromProductionOrder = null;
			ProductionInfo productionInformation = null;
			bool startedPRDFound = false;
			List<string> allowedButtonKeysForProductionInfoScreen = null;
			DataSet dsHasToBeLinedUpLocations = null;
			Collection<KeyValuePair<string, Tuple<int, bool>>> listLinedUpLocations = null;
			bool isLinedUpLocationWithAnIngredientMustBeProduced = false;
			DataSet dsCanBeLinedUpLocations = null;
			PmxItemInfo itemInfo = null;
			PmxItemInfo itemInfoByProduct = null;
			PmxItemInfo itemInfoToProduce = null;
			DataSet dsIngredients = null;
			string buttonKeySelectProductionTaskScreen = null;
			double? maximumQuantity = null;
			string existingSSCC = null;
			int? existingLuid = null;
			string logisticCarrierCode = null;
			DataSet dsLogisticCarriers = null;
			DataSet dsByProducts = null;
			DataSet dsProductionLinesScanSSCCToConsume = null;
			string buttonKeyOtherTasksLogisticCarriers = null;
			Collection<PackagingTypeInfo> listOfPackagingTypeInfosForScreen = null;
			Collection<ItemTransactionalPackagingTypeInfo> currentListOfTransactionalPackagingTypeInfos;
			PmxDictionary<PmxMomentOfCapture, Collection<QualityValueInfo>> currentListOfQualityValueInfos;
			double quantityToProduce = 0;
			double? quantityUom2ToProduce = null;
			//int? createdLUID = null;
			bool askReprint = false;
			int numberOfReports = 0;
			bool stopProduction = false;
			string buttonKeyOtherTasks = null;
			double quantityForRun = 0;
			DataSet dsProductOnLine = null;
			Collection<KeyValuePair<string, ItemSetpointInfo>> listOfItemCodesAndQuantity = null;
			Collection<ConsumeItemInfo> listConsumeItemsToSend = null;
			bool justStoppedPRD = false;
			ProductionOrderStepListInputAndOutput stepListInputAndOutputForOptional = null;
			ProductionOrderStepListInputAndOutput stepListInputAndOutputForConsumption = null;
			//ProductionOrderStepListInput currentOptionalInput = null;
			PmxDictionary<string, double> listIntermediateItemsToProduce = new PmxDictionary<string, double>();
			PmxDictionary<string, double> listOptionalItemsToConsume = new PmxDictionary<string, double>();
			DataSet ds = null;
			PmxUserGroupAuthorizationCodes authorizationStopButton = PmxUserGroupAuthorizationCodes.Enabled;//Default enabled
			PmxUserGroupAuthorizationCodes authorizationOnHoldButton = PmxUserGroupAuthorizationCodes.Enabled;//Default enabled
			string userCode = null;
			Collection<string> listOfSerialNumbers = null;
			List<string> allowedButtonsForStopProductionTask = null;
			Collection<ItemTransactionalBatchAttributeInfo> currentListOfTransactionalBatchAttributeInfos;
			List<int> listOfCreatedReceipts = null;
			Collection<int> listOfCreatedIssues = null;

			string itemCodeByProduct = null;
			int? lineNumByProduct = null;

			DataSet dsSSCC = null;
			DataSet dsSSCCToBeIssued = null;
	

			PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo> batchInfo = new PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo>();
			PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo> tempBatchInfo = new PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo>();
			List<string> listOfItemCodes = new List<string>();

			//   m_sboProviderService.GetCurrentUser().;

			//Get default settings
			bool useWaste = false;
			bool confirmProducedQtyAfterProduction = false;
			bool goBack = false;
			bool autoMoveAllItemsLinkedToBOM = false;
			bool allowToMoveFromRest = false;
			bool autoFillConsumedQtyFromPreparedQty = false;
			bool splitQtyInLogUnitsOfItemDefQtyOnLogUnit = false;
			bool skipConsumptionScreenForLinkedComponents = false;
			bool lineUpLocationFromProductionTask = false;
			bool setFocusToQuantityControl = false;
			bool showStepsForStepListProduction = true;
			bool automaticallyCloseOnCompletion = false;

			m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
			  delegate (PmxDbConnection conn, object[] parameters)
			  {

				  PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
				  PmxDictionary<string, PmxParameterValue> plcParams = prdProv.GetParametersProductionController();


				  useWaste = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_USE_WASTE, false);
				  confirmProducedQtyAfterProduction = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_CONFIRM_QUANTITY, false);
				  autoMoveAllItemsLinkedToBOM = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTO_MOVE_ITEMS_LINKED_TO_BOM, false);
				  allowToMoveFromRest = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_MOVE_TO_PROD_LINE_FROM_REST, false);
				  autoFillConsumedQtyFromPreparedQty = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTO_FILL_CONSUMED_QTY_FROM_PREPARED_QTY, false);
				  splitQtyInLogUnitsOfItemDefQtyOnLogUnit = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_SPLIT_QTY_IN_LOG_UNITS_OF_ITEM_DEF_QTY_ON_LOG_UNIT, false);
				  skipConsumptionScreenForLinkedComponents = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_SKIP_CONSUMPTION_SCREEN_ON_FLOW_FOR_LINKED_COMPONENTS, false);
				  automaticallyCloseOnCompletion = PmxParameterValueProvider.GetParameterValue<bool>(plcParams, PmxExtensionParameterCodes.PRODUCTION_CONTROLLER_AUTOMATICALLY_CLOSE_PRODUCTION_ORDERS_ON_COMPLETION, false);

				  return null;
			  });


			productionLineCodeForDevice = m_sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null,
			delegate (PmxDbConnection conn, object[] parameters)
			{
				PmxOseProductionLineProvider plProv = new PmxOseProductionLineProvider(conn);
				return plProv.GetParentProductionLineForLocation(deviceID);
			});
			m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
			   delegate (PmxDbConnection conn, object[] parameters)
			   {
				   PmxUserProvider usProv = new PmxUserProvider();
				   userCode = SboHelper.SboUsername;
				   PmxDictionary<string, PmxUserGroupAuthorizationCodes> authorizations = usProv.GetAllPmxGroupAuthorizationsOfUser(userCode, null, conn);//.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowStopButton.CODE, out authorizationStopButton);
				   authorizations.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowStopButton.CODE, out authorizationStopButton);
				   authorizations.TryGetValue(PmxUserAuthorizationColumnValues.ProductionFlowOnHoldButton.CODE, out authorizationOnHoldButton);

				   return null;
			   });


			Collection<string> qualityStatuses = m_sboProviderService.InvokeMethodWithDbConnection<Collection<string>>(false, false, null, null, new DelegateWithPmxDbConnection<Collection<string>>(ExecuteGetAllowedQualityStatusesForProduction));

			whsInfoDevice = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);

			whsInfo = whsInfoDevice;

			IBarcodeController barcodeController = null;
			m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
				delegate (PmxDbConnection conn, object[] parameters)
				{
					PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
					barcodeController = bcProv.GetBarcodeController();

					return null;
				});


			//bind parameters to sub flow
			m_moveToProductionLineFlow.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadWriteBinder<int?>("ProductionOrderDocEntry", this, "m_moveToProductionLineFlow_ProductionOrderDocEntry"),
				new ReadWriteBinder<string>("ProductionLineLocationCode", this, "m_moveToProductionLineFlow_ProductionLineCode"),
				new ReadWriteBinder<string>("InputLocationCode", this, "m_moveToProductionLineFlow_InputLocationCode"),
				 new ReadOnlyBinder<string>("ProductionOrderDocNum", this, "m_moveToProductionLineFlow_StrProductionOrderNumber"),
				 new ReadOnlyBinder<int?>("CurrentProdBatch", this, "m_moveToProductionLineFlow_CurrentProdBatch"));

			//initialize sub flows
			m_printItemLabelFlow.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadOnlyBinder<string>("LocationCode", this, "m_printItemLabelFlow_LocationCode"),
				new ReadOnlyBinder<string>("NeededItemCode", this, "m_printItemLabelFlow_NeededItemCode"),
				new ReadOnlyBinder<string>("NeededBatchNumber", this, "m_printItemLabelFlow_NeededBatchNumber"),
				new ReadOnlyBinder<string>("NeededBatchNumber2", this, "m_printItemLabelFlow_NeededBatchNumber2"),
				new ReadOnlyBinder<List<string>>("NeededSerialNumbers", this, "m_printItemLabelFlow_NeededSerialNumbers"),
				new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", this, "m_printItemLabelFlow_NeededBestBeforeDate"),
				new ReadOnlyBinder<int?>("NeededNrOfLabels", this, "m_printItemLabelFlow_NeededNrOfLabels"),
				new ReadOnlyBinder<bool>("AskBatchInfo", this, "m_printItemLabelFlow_AskBatchInfo"),
				new ReadWriteBinder<bool>("BackRequested", this, "m_printItemLabelFlow_BackRequested"),
				new ReadWriteBinder<double?>("EnteredQuantity", this, "m_printItemLabelFlow_EnteredQuantity")
				);

			m_enterProduceQuantityHookFlow.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadWriteBinder<bool>("BackRequested", this, "m_enterProduceQuantityHookFlow_BackRequested"),
				new ReadWriteBinder<double>("QuantityEntered", this, "m_enterProduceQuantityHookFlow_QuantityEntered"),
				new ReadOnlyBinder<double>("MaximumQuantity", this, "m_enterProduceQuantityHookFlow_MaximumQuantity"),
				new ReadOnlyBinder<ProductionInfo>("ProductionInfo", this, "m_enterProduceQuantityHookFlow_ProductionInfo"),
				new ReadOnlyBinder<int?>("ByProductLineNum", this, "m_enterProduceQuantityHookFlow_ByProductLineNum"),
				new ReadWriteBinder<IPreAndPostActionInTransaction>("PreAndPostActionObject", this, "m_enterProduceQuantityHookFlow_PreAndPostActionObject")

				);


			Step_SelectProductionLine:

			#region SELECT PRODUCTION LINE

			query = BuildQuery.GetProductionLines(whsInfoDevice.PmxWarehouseCode, deviceID);
			dsProductionLines = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsProductionLines) > 0 &&
				!string.IsNullOrEmpty(productionLineCodeForDevice) &&
				!goBack)
			{
				productionLineCode = productionLineCodeForDevice;
				goto Step_GetProductionLineInfo;
			}

			if (this.GetNumberOfRows(dsProductionLines) == 1 && !goBack)
			{
				//auto select location
				productionLineCode = Convert.ToString(dsProductionLines.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
				goto Step_GetProductionLineInfo;
			}

			goBack = false;

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
			   DefaultCultureInfo.Get(), BuildParamCollection(
			   "InitialErrorKey", initialErrorKey,
			   "TitleKey", "MSG_TITLE_SELECT_PRODUCTION_LINE",
			   "LocationDS", dsProductionLines
			   ));
			msg = WaitForMessage();
			initialErrorKey = null;
			if (msg.Name.EndsWith(".LocationSelected"))
			{
				productionLineCode = ExtractParameter<string>(msg.Parameters, "locationCode");
				goto Step_GetProductionLineInfo;
			}

			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_SelectProductionLine;
			}

			//back requested
			return;

			#endregion

			Step_GetProductionLineInfo:

			#region GET PRODUCTION LINE INFO


			if (this.GetNumberOfRows(dsProductionLines) > 0)
			{
				DataRow[] rows = dsProductionLines.Tables[0].Select(LocationDataSet.Columns.CODE.Name + " = '" + productionLineCode + "'");
				if (rows.Length > 0)
				{
					DataRow row = rows[0];
					inputLocationCode = row[PmxOseProductionLineTable.Columns.InputLocationCode.NAME] as string;
					outputLocationCode = row[PmxOseProductionLineTable.Columns.OutputLocationCode.NAME] as string;
					restLocationCode = row[PmxOseProductionLineTable.Columns.RestLocationCode.NAME] as string;
					string warehouseCode = row[LocationDataSet.Columns.WAREHOUSECODE.Name] as string;
					if (warehouseCode != whsInfo.PmxWarehouseCode)
					{
						whsInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), warehouseCode);
					}

				}
				else
				{
					//production line not found!
					//This should not happen
					goBack = true;
					goto Step_SelectProductionLine;
				}
			}
			else
			{
				//production line not found!
				//This should not happen
				goBack = true;
				goto Step_SelectProductionLine;
			}

			goto Step_GetStartedProductionOrder;

			#endregion

			Step_GetStartedProductionOrder:

			#region GET STARTED PRODUCTION ORDER

			batchAndBestBeforeDateInfo = null;
			batchNumber = null;
			batchNumber2 = null;
			bestBeforeDate = null;
			//isUserAllowedToChangeBatchNumber = false;
			//isUserAllowedToChangeBestBeforeDate = false;
			productionOrderDocEntry = null;
			startedPRDFound = false;
			justStoppedPRD = false;

			query = BuildQuery.GetStartedProductionOrders(productionLineCode, m_sboProviderService.GetDbTool());
			dsStartedProductionOrder = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsStartedProductionOrder) == 0)
			{
				goto Step_SelectProductionOrder;
			}

			//Select first started prd
			startedPRDFound = true;
			productionOrderDocEntry = Convert.ToInt32(dsStartedProductionOrder.Tables[0].Rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name]);

			goto Step_SelectProductionOrder;

			#endregion

			Step_SelectProductionOrder:

			#region SELECT PRODUCTION ORDER

			query = BuildQuery.GetProductionOrders(productionLineCode, productionOrderDocEntry, whsInfo.SboWarehouseCode, m_sboProviderService.GetDbTool());
			dsProductionOrder = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsProductionOrder) == 1 &&
				productionOrderDocEntry.HasValue &&
				!goBack)
			{
				goto Step_GetProductionInfo;
			}

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_PACKAGING_ORDER",
					"ProductionOrderDS", dsProductionOrder));
			msg = WaitForMessage();

			initialErrorKey = null;
			goBack = false;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_SelectProductionLine;
			}
			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_SelectProductionOrder;
			}
			if (!msg.Name.EndsWith(".ProductionOrderSelected"))
			{
				initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
				goto Step_SelectProductionOrder;
			}

			//User selected a production order
			productionOrderDocEntry = ExtractParameter<int>(msg.Parameters, "productionOrderDocEntry");

			goto Step_GetProductionInfo;


			#endregion

			Step_GetProductionInfo:

			#region GET PRODUCTION INFO

			lineUpLocationFromProductionTask = false;

			if (this.GetNumberOfRows(dsProductionOrder) > 0)
			{
				DataRow[] rows = dsProductionOrder.Tables[0].Select(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name + "=" + productionOrderDocEntry.Value.ToString());
				if (rows.Length > 0)
				{
					customer = rows[0][ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name] as string;
					dueDate = Convert.ToDateTime(rows[0][ProductionOrderDataSet.Columns.DUE_DATE.Name]);
					itemToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name]);
					itemCodeToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMCODE.Name]);
					productionOrderNumber = Convert.ToInt32(rows[0][ProductionOrderDataSet.Columns.DOC_NUM.Name]);
					productionComments = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.COMMENTS.Name]);
					stillToProduceQuantity = Convert.ToDouble(rows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);
					stepListKey = Convertor.ConvertToInt32Nullable(rows[0][ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name]);
					productionLineCodeFromProductionOrder = rows[0][ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name] as string;

					productionInformation = new ProductionInfo();
					productionInformation.Customer = customer;
					productionInformation.DueDate = dueDate;
					productionInformation.ItemNameToProduce = itemToProduce;
					productionInformation.ItemCodeToProduce = itemCodeToProduce;
					productionInformation.ProductionOrderNumber = productionOrderNumber;
					productionInformation.ProductionComments = productionComments;
					productionInformation.StillToProduce = stillToProduceQuantity;
					productionInformation.NrOfBatches = Convertor.ConvertToInt32Nullable(rows[0][PmxProductionOrderTable.Columns.NrOfProdBatches.NAME]);
					productionInformation.ProdBatchQuantity = Convertor.ConvertToDoubleNullable(rows[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);
					productionInformation.PlannedQuantity = Convert.ToDouble(rows[0][PmxProductionOrderTable.Columns.PlannedQuantity.NAME]);
					productionInformation.CurrentProdBatch = Convert.ToInt32(rows[0][PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME]);

					productionInformation.SerialManagingByPool = Convert.ToString(rows[0][PmxProductionOrderTable.Columns.SerialNumberManagementMethod.NAME]) == PmxSerialNumberManagementMethodColumnValues.Pool.VALUE;
					productionInformation.SerialNumberPoolCode = Convert.ToString(rows[0][PmxProductionOrderTable.Columns.SerialNumberPoolCode.NAME]);


					productionInformation.IsDisassembly = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) == SboProductionOrderTypeValues.Disassembly.VALUE;

					if (productionInformation.NrOfBatches.HasValue && productionInformation.ProdBatchQuantity.HasValue)
					{
						productionInformation.StillToProduceCurrentProdBatch = productionInformation.ProdBatchQuantity.Value;
					}

					if (startedPRDFound)
					{
						goto Step_GetBatchBBD;
					}
					else
					{
						goto Step_ShowProductionInfo;
					}
				}
			}
			else
			{
				goto Step_SelectProductionLine;
			}

			#endregion

			Step_ShowProductionInfo:

			#region SHOW PRODUCTION INFO


			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IShowProductionInfoScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_PRODUCTION_INFO",
					"AllowedButtons", new string[1] { "MSG_BUTTON_START_PRODUCTION" },
					"ProductionInfo", productionInformation
					));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_GetStartedProductionOrder;
			}
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				goto Step_StartProductionOrder;
			}



			#endregion

			Step_StartProductionOrder:

			#region START PRODUCTION ORDER

			#region ASK START QUALITY VALUES

			m_enterQualityValuesFlow.BindParameters(
			 this.DefaultCultureInfo,
			 this.PmxOseCompany,
			 new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.Start),
			 new ReadOnlyBinder<string>("BaseType", SboObjectTypes.ProductionOrder),
			 new ReadOnlyBinder<int?>("BaseEntry", productionOrderDocEntry),
			 new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
			 new ReadOnlyBinder<bool>("SaveData", true),

			 new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
			 new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

			 );

			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

			if (m_subFlowBackRequested)
			{
				goBack = true;
				goto Step_SelectProductionLine;
			}

			#endregion

			try
			{

				m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
				 delegate (PmxDbConnection conn, object[] parameters)
				 {

					 PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
					 prdProv.StartProductionOrder(productionOrderDocEntry.Value);

					 return null;
				 });

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();

					if (((ProdumexException)ex).Error.ErrorCode == Produmex.Sbo.Logex.Data.ProdumexErrorCodes.ProductionOrderCannotBeStartedBeforeWeighingComponents)
					{
						goto Step_SelectProductionLine;
					}
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_StartProductionOrder;
				}

				goBack = true;
				goto Step_SelectProductionLine;

			}

			goto Step_MoveStockToRest;

			#endregion

			Step_MoveStockToRest:

			#region MOVE STOCK TO REST

			try
			{
				m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
				 delegate (PmxDbConnection conn, object[] parameters)
				 {
					 PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
					 prov.MoveStockToRestLocation(productionLineCode, productionOrderDocEntry, productionInformation.CurrentProdBatch);

					 return null;
				 });

				if (autoMoveAllItemsLinkedToBOM && !justStoppedPRD &&
					productionOrderDocEntry.HasValue)
				{

					#region MOVE ALL ITEMS LINKED TO BOM

					//get all itemcodes for production order
					Collection<string> listOfItemCodesToMove = new Collection<string>();

					if (!productionInformation.IsDisassembly)
					{

						//get list of itemcodes
						query = BuildQuery.GetProductsFromStockLinkedToProductionOrder(productionOrderDocEntry.Value, inputLocationCode, allowToMoveFromRest ? restLocationCode : null, m_sboProviderService.GetDbTool());
						DataSet dsItems = m_sboProviderService.RunView(true, null, null, query);

						if (GetNumberOfRows(dsItems) > 0)
						{
							foreach (DataRow row in dsItems.Tables[0].Rows)
							{
								string itemCodeToMove = Convert.ToString(row[ProductDataSet.Columns.PRODUCT_CODE.Name]);

								if (!listOfItemCodesToMove.Contains(itemCodeToMove))
								{
									listOfItemCodesToMove.Add(itemCodeToMove);
								}

							}
						}
					}
					else
					{
						//add main item
						listOfItemCodesToMove.Add(productionInformation.ItemCodeToProduce);
					}

					//move all data linked to production order
					m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
						 delegate (PmxDbConnection conn, object[] parameters)
						 {
							 PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
							 prov.MoveItemsFromInputToProductionLine(productionLineCode, listOfItemCodesToMove, allowToMoveFromRest, productionOrderDocEntry);
							 return null;
						 });


					#endregion



				}


			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_MoveStockToRest;
				}

				goBack = true;
				goto Step_SelectProductionLine;
			}

			if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
			{
				goto Step_SetProductionLineOnOrder;
			}
			else
			{
				if (justStoppedPRD)
				{
					justStoppedPRD = false;
					goto Step_GetStartedProductionOrder;
				}
				else
				{
					goto Step_SetProductionLineOnOrder;
				}
			}

			#endregion

			Step_SetProductionLineOnOrder:

			#region SET PRODUCTION LINE ON ORDER

			if (string.IsNullOrEmpty(productionLineCodeFromProductionOrder))
			{

				try
				{
					m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
					 delegate (PmxDbConnection conn, object[] parameters)
					 {
						 PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
						 prov.SetProductionLine(productionOrderDocEntry.Value, productionLineCode);

						 return null;
					 });

				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					if (ex is ProdumexException)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"PmxError", ((ProdumexException)ex).Error,
								"ShowButton", true));
						WaitForMessage();
					}
					else
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
								"NoTranslationOfMessageKey", true,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();
					}

					//ask user if he wants to try again after error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
							"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_SetProductionLineOnOrder;
					}

					goBack = true;
					goto Step_SelectProductionLine;

				}
			}

			goto Step_GetBatchBBD;


			#endregion

			Step_GetBatchBBD:

			#region GET BATCH BBD

			batchInfo.Clear();
			if (!productionInformation.IsDisassembly)
			{
				batchAndBestBeforeDateInfo = m_sboProviderService.InvokeMethodWithDbConnection<GeneratedBatchAndBestBeforeDateInfo>(true, false, null, null,
					 delegate (PmxDbConnection conn, object[] parameters)
					 {
						 PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
						 return prov.GetBatchAndBestBeforeDate(productionOrderDocEntry.Value);
					 });
				batchInfo.Add(itemCodeToProduce, batchAndBestBeforeDateInfo);
			}
			else
			{
				batchInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxDictionary<string, GeneratedBatchAndBestBeforeDateInfo>>(true, false, null, null,
					 delegate (PmxDbConnection conn, object[] parameters)
					 {
						 PmxGeneralProvider prov = new PmxGeneralProvider(conn);
						 return prov.GetBatchAndBestBeforeDateForDisassembly(productionOrderDocEntry.Value);
					 });
			}

			goto Step_EnterBatch;


			#endregion

			Step_EnterBatch:

			#region ENTER BATCH

			if (batchAndBestBeforeDateInfo != null &&
			   batchAndBestBeforeDateInfo.HasBatchNumber)
			{
				batchNumber = batchAndBestBeforeDateInfo.BatchNumber;

				if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber && !productionInformation.IsDisassembly)
				{
					//user can change the batch, so ask it
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
					 DefaultCultureInfo.Get(), BuildParamCollection(
						 "InitialErrorKey", initialErrorKey,
						 "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER_FOR_PRODUCTION",
						 "ForceDataEntry", true,
						 "ForceUppercase", true,
						 "AllowToGoBack", true,
						 "DefaultValue", batchNumber,
						 "Information", itemToProduce + Environment.NewLine + "Production order: " +  productionOrderNumber,
						 "MessageAddendum", base.ResourceStore.GetValue<string>("MSG_DEFAULT_BATCH_NUMBER") + Environment.NewLine + batchAndBestBeforeDateInfo.BatchNumber
						 ));
					msg = WaitForMessage();

					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".StringEntered"))
					{
						batchNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						goBack = true;
						goto Step_SelectProductionLine;
					}
				}

			}

			goto Step_EnterBatch2;

			#endregion

			Step_EnterBatch2:

			#region ENTER BATCH2

			if (batchAndBestBeforeDateInfo != null && batchAndBestBeforeDateInfo.HasBatchNumber2)
			{
				batchNumber2 = batchAndBestBeforeDateInfo.BatchNumber2;

				if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber2 && !productionInformation.IsDisassembly)
				{
					//user can change the batch, so ask it
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
					 DefaultCultureInfo.Get(), BuildParamCollection(
						 "InitialErrorKey", initialErrorKey,
						 "TitleKey", "MSG_TITLE_ENTER_BATCH_NUMBER2_FOR_PRODUCTION",
						 "ForceDataEntry", true,
						 "ForceUppercase", true,
						 "AllowToGoBack", true,
						 "DefaultValue", batchNumber2,
						 "Information", "MSG_DEFAULT_BATCH_NUMBER",
						 "MessageAddendum", batchAndBestBeforeDateInfo.BatchNumber2
						 ));
					msg = WaitForMessage();

					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".StringEntered"))
					{
						batchNumber2 = ExtractParameter<string>(msg.Parameters, "stringValue");
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber)
						{
							goto Step_EnterBatch;
						}

						goBack = true;
						goto Step_SelectProductionLine;
					}
				}

			}

			goto Step_EnterBBD;

			#endregion

			Step_EnterBBD:

			#region ENTER BBD

			if (batchAndBestBeforeDateInfo != null && batchAndBestBeforeDateInfo.HasBestBeforeDate)
			{
				bestBeforeDate = batchAndBestBeforeDateInfo.BestBeforeDate;

				if (!batchAndBestBeforeDateInfo.GenerateBBDOnEveryReceipt)
				{
					if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBestBeforeDate && !productionInformation.IsDisassembly)
					{
						//user can change the batch, so ask it
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterDateScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE_FOR_PRODUCTION",
							 "ForceDataEntry", true,
							 "ForceUppercase", true,
							 "AllowToGoBack", true,
							 "DefaultDate", bestBeforeDate,
							 "Information", "MSG_DEFAULT_BEST_BEFORE_DATE"
							 ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".DateEntered"))
						{
							bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "date");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							if (batchAndBestBeforeDateInfo.HasBatchNumber2)
							{
								if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber2)
								{
									goto Step_EnterBatch2;
								}
							}
							if (batchAndBestBeforeDateInfo.HasBatchNumber)
							{
								if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBatchNumber)
								{
									goto Step_EnterBatch;
								}
							}

							goBack = true;
							goto Step_SelectProductionLine;
						}
					}
				}
			}

			goto Step_SetLinedUpLocations;

			#endregion

			Step_SetLinedUpLocations:

			#region SET LINED UP LOCATIONS

			isLinedUpLocationWithAnIngredientMustBeProduced = false;

			m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
			 delegate (PmxDbConnection conn, object[] parameters)
			 {

				 PmxLinedUpLocationProvider prov = new PmxLinedUpLocationProvider(conn);
				 prov.DeleteAllLinedUpLocationsForProductionLine(productionLineCode, productionOrderDocEntry.Value);

				 return null;
			 });


			query = BuildQuery.GetHasToBeLinedUpProductionLines(productionOrderDocEntry.Value, m_sboProviderService.GetDbTool());
			dsHasToBeLinedUpLocations = m_sboProviderService.RunView(false, null, null, query);

			listLinedUpLocations = new Collection<KeyValuePair<string, Tuple<int, bool>>>();


			if (this.GetNumberOfRows(dsHasToBeLinedUpLocations) > 0)
			{
				foreach (DataRow rowToConsume in dsHasToBeLinedUpLocations.Tables[0].Rows)
				{
					string lineUpItemCode = rowToConsume[HasToBeLinedUpProductionLinesDataSet.Columns.ITEMCODE.Name] as string;
					string lineUpItemDescription = rowToConsume[HasToBeLinedUpProductionLinesDataSet.Columns.ITEMDESCRIPTION.Name] as string;
					string lineUpItemBarcode = rowToConsume[HasToBeLinedUpProductionLinesDataSet.Columns.ITEMBARCODE.Name] as string;
					int lineUpLineNum = Convert.ToInt32(rowToConsume[HasToBeLinedUpProductionLinesDataSet.Columns.LINENUM.Name]);
					bool allowToPickForLineUp = (rowToConsume[HasToBeLinedUpProductionLinesDataSet.Columns.HASTOBELINEDUP.Name] as string) == SboBooleanColumnValues.Y.VALUE;


					string linedUpLocation = null;
					bool ingredientMustBeProduced = false;

					query = BuildQuery.GetCanBeLinedUpLocations(productionLineCode, allowToPickForLineUp ? null : lineUpItemCode);
					dsCanBeLinedUpLocations = m_sboProviderService.RunView(false, null, null, query);

					if (this.GetNumberOfRows(dsCanBeLinedUpLocations) == 0)
					{
						if (productionInformation.IsDisassembly)
						{
							//no location found, for item
							//For disassembly, user can select any lined up location
							query = BuildQuery.GetCanBeLinedUpLocations(productionLineCode, null);
							dsCanBeLinedUpLocations = m_sboProviderService.RunView(false, null, null, query);
						}
					}

					if (this.GetNumberOfRows(dsCanBeLinedUpLocations) == 0)
					{
						initialErrorKey = "MSG_ERROR_NO_POSSIBLE_LINE_UP_LOCATION";
						listLinedUpLocations = null;
						goBack = true;
						goto Step_SelectProductionLine;
					}

					if (this.GetNumberOfRows(dsCanBeLinedUpLocations) == 1)
					{
						//auto select first one
						linedUpLocation = dsCanBeLinedUpLocations.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name] as string;

						if (!productionInformation.IsDisassembly)
						{
							ingredientMustBeProduced = dsCanBeLinedUpLocations.Tables[0].Rows[0][PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME] as string == "Y";
						}
					}
					else
					{

						Step_ReEnterLinedUpLocation:

						//Let user select it
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectLineUpLocationScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"TitleKey", "MSG_TITLE_SELECT_LINE_UP_LOCATION",
								"ProductionInfo", productionInformation,
								"IngredientCode", lineUpItemCode,
								"IngredientDescription", lineUpItemDescription,
								"IngredientBarcode", lineUpItemBarcode,
								"DSLocations", dsCanBeLinedUpLocations
								));
						msg = WaitForMessage();

						initialErrorKey = null;

						if (msg.Name.EndsWith(".BackRequested"))
						{
							if (lineUpLocationFromProductionTask)
							{
								//back is not allowed
								goto Step_ReEnterLinedUpLocation;
							}
							else
							{
								listLinedUpLocations = null;
								goBack = true;
								goto Step_SelectProductionOrder;
							}
						}
						if (msg.Name.EndsWith(".LocationSelected"))
						{
							linedUpLocation = ExtractParameter<string>(msg.Parameters, "locationCode");

							if (!productionInformation.IsDisassembly)
							{
								foreach (DataRow rowLinedUpLocation in dsCanBeLinedUpLocations.Tables[0].Rows)
								{
									if (linedUpLocation == rowLinedUpLocation[LocationDataSet.Columns.CODE.Name] as string)
									{
										ingredientMustBeProduced = rowLinedUpLocation[PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME] as string == "Y";
										break;
									}
								}
							}
						}
					}

					if (!string.IsNullOrEmpty(linedUpLocation))
					{
						listLinedUpLocations.Add(new KeyValuePair<string, Tuple<int, bool>>(linedUpLocation, new Tuple<int, bool>(lineUpLineNum, ingredientMustBeProduced)));

						if (ingredientMustBeProduced)
						{
							isLinedUpLocationWithAnIngredientMustBeProduced = true;
						}
					}
				}
			}

			goto Step_SetLinedUpLocationsToDB;

			#endregion

			Step_SetLinedUpLocationsToDB:

			#region SET LINED UP LOCATIONS TO DB


			try
			{

				if (listLinedUpLocations != null &&
					listLinedUpLocations.Count > 0)
				{
					m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
					   delegate (PmxDbConnection conn, object[] parameters)
					   {
						   PmxLinedUpLocationProvider prov = new PmxLinedUpLocationProvider(conn);
						   prov.AddLinedUpLocations(productionLineCode, productionOrderDocEntry.Value, listLinedUpLocations);

						   return null;
					   });

				}

				goto Step_ShowProductionTasks;

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_SetLinedUpLocationsToDB;
				}

				goBack = true;
				goto Step_SelectProductionLine;

			}




			#endregion

			Step_ShowProductionTasks:

			#region SHOW PRODUCTION TASKS

			lineUpLocationFromProductionTask = false;
			itemCodeByProduct = null;
			lineNumByProduct = null;
			dsByProducts = null;
			itemInfoToProduce = null;
			dsProductionLinesScanSSCCToConsume = null;

			bool blnScanSSCCToConsume = false;
			query = BuildQuery.GetProductionLineScanSSCCToConsume(productionLineCode, m_sboProviderService.GetDbTool());
			dsProductionLinesScanSSCCToConsume = m_sboProviderService.RunView(false, null, null, query);
			if (this.GetNumberOfRows(dsProductionLinesScanSSCCToConsume) > 0)
			{
				blnScanSSCCToConsume = true;
			}

			query = BuildQuery.GetProductionOrders(productionLineCode, productionOrderDocEntry, whsInfo.SboWarehouseCode, m_sboProviderService.GetDbTool());
			dsProductionOrder = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsProductionOrder) > 0)
			{
				DataRow[] rows = dsProductionOrder.Tables[0].Select(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name + "=" + productionOrderDocEntry.Value.ToString());
				if (rows.Length > 0)
				{
					customer = rows[0][ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name] as string;
					dueDate = Convert.ToDateTime(rows[0][ProductionOrderDataSet.Columns.DUE_DATE.Name]);
					itemToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name]);
					itemCodeToProduce = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.ITEMCODE.Name]);
					productionOrderNumber = Convert.ToInt32(rows[0][ProductionOrderDataSet.Columns.DOC_NUM.Name]);
					productionComments = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.COMMENTS.Name]);
					stillToProduceQuantity = Convert.ToDouble(rows[0][ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name]);
					stepListKey = Convertor.ConvertToInt32Nullable(rows[0][ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name]);
					productionLineCodeFromProductionOrder = rows[0][ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name] as string;

					productionInformation = new ProductionInfo();
					productionInformation.Customer = customer;
					productionInformation.DueDate = dueDate;
					productionInformation.ItemNameToProduce = itemToProduce;
					productionInformation.ItemCodeToProduce = itemCodeToProduce;
					productionInformation.ProductionOrderNumber = productionOrderNumber;
					productionInformation.ProductionComments = productionComments;
					productionInformation.StillToProduce = stillToProduceQuantity;
					productionInformation.ProductionOrderDocEntry = productionOrderDocEntry.Value;

					productionInformation.NrOfBatches = Convertor.ConvertToInt32Nullable(rows[0][PmxProductionOrderTable.Columns.NrOfProdBatches.NAME]);
					productionInformation.ProdBatchQuantity = Convertor.ConvertToDoubleNullable(rows[0][PmxProductionOrderTable.Columns.ProdBatchSize.NAME]);
					productionInformation.PlannedQuantity = Convert.ToDouble(rows[0][PmxProductionOrderTable.Columns.PlannedQuantity.NAME]);
					productionInformation.CurrentProdBatch = Convert.ToInt32(rows[0][PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME]);

					productionInformation.IsDisassembly = Convert.ToString(rows[0][ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name]) == SboProductionOrderTypeValues.Disassembly.VALUE;

					productionInformation.SerialManagingByPool = Convert.ToString(rows[0][PmxProductionOrderTable.Columns.SerialNumberManagementMethod.NAME]) == PmxSerialNumberManagementMethodColumnValues.Pool.VALUE;
					productionInformation.SerialNumberPoolCode = Convert.ToString(rows[0][PmxProductionOrderTable.Columns.SerialNumberPoolCode.NAME]);

					if (productionInformation.NrOfBatches.HasValue && productionInformation.ProdBatchQuantity.HasValue && productionInformation.CurrentProdBatch.HasValue)
					{
						query = BuildQuery.GetQuantityAlreadyIssued(productionOrderDocEntry.Value, productionInformation.CurrentProdBatch.Value, m_sboProviderService.GetDbTool());
						DataSet tempDS = m_sboProviderService.RunView(false, null, null, query);
						double alreadyProducedQty = Convert.ToDouble(tempDS.Tables[0].Rows[0][PmxProductionReceiptLineTable.Columns.Quantity.NAME]);
						productionInformation.StillToProduceCurrentProdBatch = productionInformation.ProdBatchQuantity.Value - alreadyProducedQty;
					}

					itemInfo = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCodeToProduce);

					if (productionInformation.IsDisassembly)
					{
						query = BuildQuery.GetIngredientsForProductionDisassembly(productionOrderDocEntry.Value, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, productionInformation.NrOfBatches.HasValue ? productionInformation.CurrentProdBatch : null, m_sboProviderService.GetDbTool());
					}
					else
					{
						bool showItemsWithLowTolReached = false;
						query = BuildQuery.GetIngredientsForProduction(productionOrderDocEntry.Value, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, showItemsWithLowTolReached, productionInformation.NrOfBatches.HasValue ? productionInformation.CurrentProdBatch : null, m_sboProviderService.GetDbTool());
					}
					dsIngredients = m_sboProviderService.RunView(false, null, null, query);

					//Get by products
					query = BuildQuery.GetByProducts(productionOrderDocEntry.Value, false, m_sboProviderService.GetDbTool());
					dsByProducts = m_sboProviderService.RunView(false, null, null, query);


					if (this.GetNumberOfRows(dsByProducts) > 0)
					{
						productionInformation.ByProducts = new PmxDictionary<string, int>();

						foreach (DataRow row in dsByProducts.Tables[0].Rows)
						{
							productionInformation.ByProducts.Add(row[ProductionOrderLineDataSet.Columns.ITEMCODE.Name] as string, Convert.ToInt32(row[ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name]));
						}

					}


				}
			}
			else
			{
				goto Step_SelectProductionLine;
			}



			List<string> buttons = new List<string>();
			buttons.Add("MSG_BUTTON_SCAN_SSCC_TO_CONSUME");
			buttons.Add(KnownMessageKeys.MSG_BUTTON_ISSUED_COMPONENTS);

			ToolBarButtons toolBarButtons = ToolBarButtons.Back;

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", base.ResourceStore.GetValue<string>("MSG_TITLE_SELECT_TASK")  + " - PO(" + productionOrderNumber +")",
					"AllowedButtons", buttons.ToArray(),
					"EnabledToolBarButtons", toolBarButtons
					));
			msg = WaitForMessage();

			initialErrorKey = null;
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				if (ExtractParameter<string>(msg.Parameters, "buttonKey") == "MSG_BUTTON_SCAN_SSCC_TO_CONSUME")
				{
					goto Step_ScanSSCCToConsume;
				}
				if (ExtractParameter<string>(msg.Parameters, "buttonKey") == KnownMessageKeys.MSG_BUTTON_ISSUED_COMPONENTS)
				{
					goto Step_ShowIssuedComponents;
				}			
			}
			if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_SelectProductionOrder;
			}
			allowedButtonKeysForProductionInfoScreen = new List<string>();

			allowedButtonKeysForProductionInfoScreen.Add(productionInformation.IsDisassembly ? "MSG_BUTTON_DISASSEMBLE" : "MSG_BUTTON_PRODUCE");
		

			//Do we have by products?
			if (productionInformation.ByProducts != null &&
			productionInformation.ByProducts.Count > 0 &&
			!productionInformation.IsDisassembly)
			{
				allowedButtonKeysForProductionInfoScreen.Add("MSG_BUTTON_PRODUCE_BY_PRODUCTS");
			}

			allowedButtonKeysForProductionInfoScreen.AddRange(new List<string>{
						 "MSG_BUTTON_CHANGE_LINED_UP_LOCATIONS",
						(blnScanSSCCToConsume ? "MSG_BUTTON_SCAN_SSCC_TO_CONSUME" : "MSG_BUTTON_ADD_ITEMS_TO_USE"),
						"MSG_BUTTON_PRINT_PRODUCT_LABEL",
						"MSG_BUTTON_END_PRODUCTION_ORDER" });


			buttonKeySelectProductionTaskScreen = null;

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionTaskScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_TASK",
					"AllowedButtons", allowedButtonKeysForProductionInfoScreen.ToArray(),
					"DSPickItemsForProduction", dsIngredients,
					"ProductionInfo", productionInformation
					));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_SelectProductionOrder;
			}
			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_ShowProductionTasks;
			}
			if (msg.Name.EndsWith(".TimeRequested"))
			{
				goto Step_TimeRegistration;
			}
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				buttonKeySelectProductionTaskScreen = ExtractParameter<string>(msg.Parameters, "buttonKey");

				switch (buttonKeySelectProductionTaskScreen)
				{
					case "MSG_BUTTON_PRODUCE":
					case "MSG_BUTTON_DISASSEMBLE":
						goto Step_Produce;
					case "MSG_BUTTON_PRODUCE_BY_PRODUCTS":
						goto Step_SelectByProduct;
					case "MSG_BUTTON_CHANGE_LINED_UP_LOCATIONS":
						lineUpLocationFromProductionTask = true;
						goto Step_SetLinedUpLocations;
					case "MSG_BUTTON_ADD_ITEMS_TO_USE":

						m_moveToProductionLineFlow_StrProductionOrderNumber = productionOrderNumber.ToString();
						m_moveToProductionLineFlow_InputLocationCode = inputLocationCode;
						m_moveToProductionLineFlow_ProductionLineCode = productionLineCode;
						m_moveToProductionLineFlow_ProductionOrderDocEntry = productionOrderDocEntry;
						m_moveToProductionLineFlow_CurrentProdBatch = productionInformation.NrOfBatches.HasValue ? productionInformation.CurrentProdBatch : null;

						//call flow
						base.StartSubFlowAndWait(m_moveToProductionLineFlow, true);


						goto Step_ShowProductionTasks;

					case "MSG_BUTTON_SCAN_SSCC_TO_CONSUME":


						//query = BuildQuery.GetSSCCOnInputLocation(productionLineCode, "-1");
						//dsSSCCToBeIssued = m_sboProviderService.RunView(false, null, null, query);

						goto Step_ScanSSCCToConsume;

					case "MSG_BUTTON_PRINT_PRODUCT_LABEL":

						this.m_printItemLabelFlow_LocationCode = productionLineCode;
						this.m_printItemLabelFlow_NeededBatchNumber = batchNumber;
						this.m_printItemLabelFlow_NeededBatchNumber2 = batchNumber2;
						this.m_printItemLabelFlow_NeededBestBeforeDate = bestBeforeDate;
						this.m_printItemLabelFlow_NeededItemCode = itemInfo.ItemCode;
						this.m_printItemLabelFlow_AskBatchInfo = false;

						this.m_printItemLabelFlow_EnteredQuantity = null;
						if (quantityToProduce != 0)
						{
							this.m_printItemLabelFlow_EnteredQuantity = quantityToProduce;
						}

						if (productionInformation.SerialManagingByPool)
						{
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"TitleKey", "MSG_TITLE_SELECT_TASK",
								"AllowedButtons", new List<string>() { "MSG_BUTTON_PRINT_ITEM_LABEL", "MSG_PRINT_SERIAL_NUMBERS" }.ToArray()));
							msg = WaitForMessage();

							initialErrorKey = null;

							if (msg.Name.EndsWith(".BackRequested"))
							{
								goto Step_ShowProductionTasks;
							}
							if (msg.Name.EndsWith(".ButtonClicked"))
							{
								string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

								if (buttonKey == "MSG_PRINT_SERIAL_NUMBERS")
								{

									List<string> serialNumbers = new List<string>();
									try
									{
										serialNumbers = m_sboProviderService.InvokeMethodWithDbConnection<List<string>>(true, false, null, null,
										 delegate (PmxDbConnection conn, object[] parameters)
										 {
											 PmxSerialNumberProvider prov = new PmxSerialNumberProvider(conn);
											 List<string> result = prov.ReserveSerialNumbersForDocument((int)productionInformation.StillToProduce, productionInformation.SerialNumberPoolCode, itemInfo.ItemCode, SboObjectTypes.ProductionOrder, productionOrderDocEntry.Value).Keys.ToList();
											 if (result.Count < this.m_printItemLabelFlow_EnteredQuantity)
											 {
												 initialErrorKey = "MSG_ERROR_NOT_ENOUGH_SERIAL_NUMBERS_TO_RESERVE";
												 throw new Exception();
											 }
											 return result;
										 });
									}
									catch
									{
										if (initialErrorKey == "MSG_ERROR_NOT_ENOUGH_SERIAL_NUMBERS_TO_RESERVE")
										{
											initialErrorKey = Produmex.Sbo.Logex.Data.ProdumexErrors.NotEnoughSerialNumbersInThePool(s_log, itemInfo.ItemCode, productionInformation.SerialNumberPoolCode, (int)productionInformation.StillToProduce).ToString();

											goto Step_ShowProductionTasks;
										}
										else
										{
											throw;
										}
									}

									if (serialNumbers.Count == 0)
									{
										initialErrorKey = "MSG_ERROR_NOTHING_TO_PRINT";
										goto Step_ShowProductionTasks;
									}

									this.m_printItemLabelFlow_NeededSerialNumbers = serialNumbers;
									this.m_printItemLabelFlow_EnteredQuantity = productionInformation.StillToProduce;
									this.m_printItemLabelFlow_NeededNrOfLabels = 1;
								}
							}
						}

						base.StartSubFlowAndWait(m_printItemLabelFlow, true);

						//call flow
						goto Step_ShowProductionTasks;

					case "MSG_BUTTON_END_PRODUCTION_ORDER":
						goto Step_StopProductionTask;
					default:
						initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
						goto Step_ShowProductionTasks;

				}

			}



			#endregion

			Step_ScanSSCCToConsume:

			#region SCAN SSCC TO CONSUME 

			query = BuildQuery.GetSSCCOnInputLocation(productionLineCode, null);
			dsSSCC = m_sboProviderService.RunView(false, null, null, query);

			//	dsSSCC = GetNewSSCCDataSet();
			//List<DataRow> rows_to_remove = new List<DataRow>();
			//foreach (DataRow row1 in dsSSCC.Tables[0].Rows)
			//{
			//	foreach (DataRow row2 in dsSSCCToBeIssued.Tables[0].Rows)
			//	{
			//		if (row1[SSCCForProductionDataSet.Columns.SSCC.Name].ToString() == row2[SSCCForProductionDataSet.Columns.SSCC.Name].ToString())
			//		{
			//			rows_to_remove.Add(row1);
			//		}
			//	}
			//}

			//foreach (DataRow row in rows_to_remove)
			//{
			//	dsSSCC.Tables[0].Rows.Remove(row);
			//	dsSSCC.Tables[0].AcceptChanges();
			//}

			

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectSSCCScreen),
				   DefaultCultureInfo.Get(), BuildParamCollection(
				   "InitialErrorKey", initialErrorKey,
				   "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
				   "BarcodeController", barcodeController,
				   "ShowButtonNoSSCC", false,
				   "HasQuantity", true,
				   "SSCCDS", dsSSCC));
			msg = WaitForMessage();
			

			initialErrorKey = null;

			if (msg.Name.EndsWith(".SSCCSelected"))
			{
				DataRow ssccRow = dsSSCC.Tables[0].Select(SSCCForProductionDataSet.Columns.SSCC.Name + "=" + ExtractParameter<string>(msg.Parameters, "sscc")).First();

				//dsSSCCToBeIssued.Tables[0].ImportRow(ssccRow);
				goto Step_ConsumeComponents;
			}

				if (msg.Name.EndsWith(".BackRequested"))
			{
				dsSSCCToBeIssued = null;

				goto Step_ShowProductionTasks;
			}
			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_ScanSSCCToConsume;
			}
		
			

			goto Step_ScanSSCCToConsume;

			#endregion
			Step_ConsumeComponents:
			#region CONSUME COMPONENTS



			// Build the list of items to consume, given the SSCC
			DataRow[] itemRows = dsSSCC.Tables[0].Select(SSCCForProductionDataSet.Columns.SSCC.Name + "='" + ExtractParameter<string>(msg.Parameters, "sscc") + "'");

			listConsumeItemsToSend = new Collection<ConsumeItemInfo>();

			foreach (DataRow itemRow in itemRows)
			{
				ConsumeItemInfo ConsitemInfo = new ConsumeItemInfo();
				if (!itemRow.IsNull(SSCCForProductionDataSet.Columns.BATCH_NUMBER_1.Name))
					ConsitemInfo.BatchNumber1 = (string)itemRow[SSCCForProductionDataSet.Columns.BATCH_NUMBER_1.Name];
				if (!itemRow.IsNull(SSCCForProductionDataSet.Columns.BATCH_NUMBER_2.Name))
					ConsitemInfo.BatchNumber2 = (string)itemRow[SSCCForProductionDataSet.Columns.BATCH_NUMBER_2.Name];
				if (!itemRow.IsNull(SSCCForProductionDataSet.Columns.BEST_BEFORE_DATE.Name))
					ConsitemInfo.BestBeforeDate = (DateTime)itemRow[SSCCForProductionDataSet.Columns.BEST_BEFORE_DATE.Name];
				ConsitemInfo.ItemCode = (string)itemRow[SSCCForProductionDataSet.Columns.ITEM_CODE.Name];
				ConsitemInfo.LocationCode = (string)itemRow[SSCCForProductionDataSet.Columns.LOCATION_CODE.Name];
				ConsitemInfo.LogisticUnitIdentKey = (int)itemRow[SSCCForProductionDataSet.Columns.LUID.Name];
				ConsitemInfo.QualityStatusCode = (string)itemRow[SSCCForProductionDataSet.Columns.QUALITY_STATUS_CODE.Name];
				ConsitemInfo.QuantityUsed = Convert.ToDouble(itemRow[SSCCForProductionDataSet.Columns.QUANTITY.Name]);


				if (!qualityStatuses.Contains(ConsitemInfo.QualityStatusCode))
				{
					initialErrorKey = KnownMessageKeys.MSG_ERROR_INVALID_QUALITY_STATUS;
					goto Step_ScanSSCCToConsume;
				}

				if (ConsitemInfo.BestBeforeDate.HasValue && ConsitemInfo.BestBeforeDate.Value < DateTime.Now)
				{
					initialErrorKey = "Expired product";
					goto Step_ScanSSCCToConsume;
				}



				listConsumeItemsToSend.Add(ConsitemInfo);
			}



			try
			{
				m_sboProviderService.InvokeMethodWithDbConnection<bool>(false, null, null,
					new DelegateWithSboConnection<bool>(ExecuteConsume),
					productionOrderDocEntry, listConsumeItemsToSend, false);
			}
			catch (Produmex.Foundation.ProdumexException ex)
			{
				s_log.Error("Exception during ExecuteConsume(...)", ex);

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", ex.Error,
						"ShowButton", true));

				msg = WaitForMessage();
			}
			catch (Exception ex)
			{
				s_log.Error("Exception during ExecuteConsume(...)", ex);

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
						"ShowButton", true));

				msg = WaitForMessage();
			}
			goto Step_ScanSSCCToConsume;

			#endregion

			Step_ShowIssuedComponents:
			#region SHOW ISSUED COMPONENTS

			// Get the list of issued components
			query = BuildQuery.GetIssuedComponents(Convert.ToInt32(productionOrderDocEntry));
			dsSSCC = m_sboProviderService.RunView(false, null, null, query);

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectSSCCScreen),
			DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", KnownMessageKeys.MSG_TITLE_ISSUED_COMPONENTS,
					   "BarcodeController", barcodeController,
					"SSCCDS", dsSSCC,
					 "ShowButtonNoSSCC", false));
			msg = WaitForMessage();
		

			goto Step_ShowProductionTasks;
			#endregion


			Step_SelectByProduct:

			#region SELECT BY PRODUCT



			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.ISelectProductionOrderLineScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_PRODUCT",
					"ProductionOrderLineDS", dsByProducts
					));
			msg = WaitForMessage();

			//clear data
			initialErrorKey = null;

			if (msg.Name.EndsWith(".ProductionOrderLineSelected"))
			{
				lineNumByProduct = ExtractParameter<int?>(msg.Parameters, "productionOrderLineNum");

				if (lineNumByProduct.HasValue)
				{
					DataRow[] rows = dsByProducts.Tables[0].Select(ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name + " = " + lineNumByProduct.Value.ToString());

					if (rows.Length > 0)
					{

						itemCodeByProduct = rows[0][ProductionOrderLineDataSet.Columns.ITEMCODE.Name] as string;
						itemInfoByProduct = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCodeByProduct);

						goto Step_Produce;
					}
				}

				//Go back
				goto Step_ShowProductionTasks;
			}
			else
			{
				//Go back
				goto Step_ShowProductionTasks;
			}

			#endregion

			Step_Produce:

			#region PRODUCE


			if (lineNumByProduct.HasValue)
			{
				//this is for by products
				itemInfoToProduce = itemInfoByProduct;
			}
			else
			{
				itemInfoToProduce = itemInfo;
			}

			if (!stepListKey.HasValue)
			{
				if (string.IsNullOrEmpty(itemCodeByProduct))
				{
					//No steplist AND not a by product.
					maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
					 delegate (PmxDbConnection conn, object[] parameters)
					 {
						 PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
						 return prov.GetMaximumQuantityToProduce(productionOrderDocEntry.Value, null, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, whsInfo.StorageLocationCodeLogisticCarriers, itemInfoToProduce);
					 });


					if (!maximumQuantity.HasValue ||
						(maximumQuantity.Value == 0 && !blnScanSSCCToConsume))
					{
						initialErrorKey = productionInformation.IsDisassembly ? "MSG_ERROR_NOT_ENOUGH_STOCK_TO_DISASSEMBLE" : "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE";
						goto Step_ShowProductionTasks;
					}
				}
				else
				{
					maximumQuantity = -1;
				}

				if (blnScanSSCCToConsume)
				{
					maximumQuantity = -1;
				}

				//run hook flow
				m_enterProduceQuantityHookFlow_BackRequested = false;
				m_enterProduceQuantityHookFlow_QuantityEntered = 0;
				m_enterProduceQuantityHookFlow_MaximumQuantity = maximumQuantity.Value;
				m_enterProduceQuantityHookFlow_ProductionInfo = productionInformation;
				m_enterProduceQuantityHookFlow_ByProductLineNum = lineNumByProduct;

				//call flow
				base.StartSubFlowAndWait(m_enterProduceQuantityHookFlow, true);

				if (m_enterProduceQuantityHookFlow_BackRequested)
				{
					goto Step_ShowProductionTasks;
				}
			}
			else
			{
				maximumQuantity = -1;
			}


			//clear some data
			existingSSCC = null;
			existingLuid = null;
			logisticCarrierCode = null;
			bool noSSCC = false;

			if (itemInfoToProduce.IsLogisticUnit)
			{
				goto Step_EnterPackagingQuantity;
			}
			else
			{
				goto Step_SelectLogisticCarrier;
			}



			#endregion

			Step_SelectLogisticCarrier:

			#region SELECT LOGISTIC CARRIERS

			bool foundLogisticCarriers = true;


			logisticCarrierCode = null;
			existingSSCC = null;
			existingLuid = null;

			if (!string.IsNullOrEmpty(itemInfoToProduce.DefaultLogisticCarrierProduction) &&
				!productionInformation.IsDisassembly)
			{
				logisticCarrierCode = itemInfoToProduce.DefaultLogisticCarrierProduction;
				goto Step_OtherTasksLogisticCarrier;
			}

			query = BuildQuery.GetLogisticCarriers(true, true, whsInfo.StorageLocationCodeLogisticCarriers, m_sboProviderService.GetDbTool());
			dsLogisticCarriers = m_sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
			{
				//We found logistic carriers, so we ask to select

				// Show screen with list
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
						"ProductDS", dsLogisticCarriers,
						"ButtonKey", "MSG_BUTTON_OTHER_TASK"));
				msg = WaitForMessage();

				initialErrorKey = null;

				if (msg.Name.EndsWith(".BackRequested"))
				{
					goto Step_ShowProductionTasks;
				}
				if (msg.Name.EndsWith(".RefreshRequested"))
				{
					goto Step_SelectLogisticCarrier;
				}
				if (msg.Name.EndsWith(".ButtonClicked"))
				{
					goto Step_OtherTasksLogisticCarrier;
				}
				if (!msg.Name.EndsWith(".ProductSelected"))
				{
					initialErrorKey = KnownMessageKeys.MSG_ERROR_OPERATION_NOT_SUPPORTED;
					goto Step_SelectLogisticCarrier;
				}

				logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

				goto Step_EnterPackagingQuantity;


			}
			else
			{
				foundLogisticCarriers = false;

				goto Step_OtherTasksLogisticCarrier;
			}

			#endregion

			Step_OtherTasksLogisticCarrier:

			#region OTHER TASKS LOGISTIC CARRIERS

			buttonKeyOtherTasksLogisticCarriers = null;


			List<string> listOfButtons = new List<string>();

			listOfButtons.Add("MSG_BUTTON_NO_SSCC");

			if (!foundLogisticCarriers)
			{
				listOfButtons.Add("MSG_BUTTON_PRODUCE_NEW_SSCC");
			}
			else
			{
				listOfButtons.Add(string.IsNullOrEmpty(logisticCarrierCode) ? "MSG_BUTTON_NO_LOGISTIC_CARRIER" : "MSG_BUTTON_PROCEED");
			}

			listOfButtons.Add(productionInformation.IsDisassembly ? "MSG_BUTTON_DISASSEMBLE_ON_EXISTING_SSCC" : "MSG_BUTTON_PRODUCE_ON_EXISTING_SSCC");

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_TASK",
					"AllowedButtons", listOfButtons.ToArray()));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				if (this.GetNumberOfRows(dsLogisticCarriers) > 0)
				{
					goto Step_SelectLogisticCarrier;
				}
				else
				{
					goto Step_ShowProductionTasks;
				}
			}
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				buttonKeyOtherTasksLogisticCarriers = ExtractParameter<string>(msg.Parameters, "buttonKey");

				if (buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_NO_SSCC")
				{
					noSSCC = true;
				}

				if (buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_NO_LOGISTIC_CARRIER" ||
					buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_PROCEED" ||
					buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_PRODUCE_NEW_SSCC" ||
					buttonKeyOtherTasksLogisticCarriers == "MSG_BUTTON_NO_SSCC")
				{
					goto Step_EnterPackagingQuantity;
				}
				else
				{
					goto Step_SelectExistingSSCC;
				}

			}

			#endregion

			Step_SelectExistingSSCC:

			#region SELECT EXISTING SSCC

			existingSSCC = null;
			existingLuid = null;

			query = BuildQuery.GetExistingSSSCCsLinkedToProductionOrder(productionOrderDocEntry.Value, m_sboProviderService.GetDbTool());

			dsSSCC = m_sboProviderService.RunView(false, null, null, query);

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IIdentifyMultipleSSCCScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
					"BarcodeController", barcodeController,
					"SSCCDS", dsSSCC,
					"ForceDataEntry", true
				   ));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goto Step_OtherTasksLogisticCarrier;
			}

			if (msg.Name.EndsWith(".SSCCScanned") || msg.Name.EndsWith(".ForwardRequested"))
			{
				existingSSCC = ExtractParameter<string>(msg.Parameters, "sscc");

				query = BuildQuery.GetProductOnSSCC(existingSSCC, null, m_sboProviderService.GetDbTool());
				ds = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{
					//clear logistic carrier if it was set by default logistic carrier
					logisticCarrierCode = null;
					existingLuid = Convert.ToInt32((ds.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]));
					goto Step_EnterPackagingQuantity;
				}
				else
				{
					try
					{
						//new valid sscc
						m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
						 delegate (PmxDbConnection conn, object[] parameters)
						 {
							 PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider((SboConnection)conn);
							 PmxSSCCProvider ssccProv = new PmxSSCCProvider((SboConnection)conn);
							 return existingLuid = luidProv.NewLogisticUnitID(existingSSCC, "");
						 });
						goto Step_EnterPackagingQuantity;
					}
					catch
					{
						initialErrorKey = "MSG_ERROR_INCORRECT_SSCC";
						goto Step_SelectExistingSSCC;
					}
				}
			}

			#endregion

			Step_EnterPackagingQuantity:

			#region ENTER PACKAGING QUANTITY

			if (!stepListKey.HasValue &&
				(!maximumQuantity.HasValue ||
				maximumQuantity.Value == 0))
			{
				//no quantity allowed
				initialErrorKey = productionInformation.IsDisassembly ? "MSG_ERROR_NOT_ENOUGH_STOCK_TO_DISASSEMBLE" : "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE";
				goto Step_ShowProductionTasks;
			}

			if ((itemInfoToProduce.HasBatchnumber || itemInfoToProduce.HasSecondBatchNumber || itemInfoToProduce.HasBestBeforeDate) && !productionInformation.IsDisassembly)
			{

				//batch not found, so we need to enter the quantity for the packaging types
				currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfoToProduce);
				bool foundAtLeastOnePackagingTypeToShowOnScreen = false;
				bool isFirst = true;
				listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();
				foreach (PmxItemPackagingTypeInfo packType in itemInfoToProduce.ListOfPackagingTypes)
				{

					PackagingTypeInfo info = new PackagingTypeInfo();

					if (isFirst)
					{
						info.NumberOfDecimals = itemInfoToProduce.UomDecimals;
					}

					info.Quantity = packType.Quantity;
					info.PackagingTypeCode = packType.PackagingTypeCode;
					info.PackagingTypeName = packType.PackagingTypeName;

					info.ShowOnScreen = packType.AskDuringReception;

					listOfPackagingTypeInfosForScreen.Add(info);

					if (!foundAtLeastOnePackagingTypeToShowOnScreen &&
						info.ShowOnScreen)
					{
						foundAtLeastOnePackagingTypeToShowOnScreen = true;
					}

					isFirst = false;
				}

				if (foundAtLeastOnePackagingTypeToShowOnScreen)
				{
					initialErrorKey = "MSG_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES";
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES",
						"ItemDescription", itemInfoToProduce.Description,
						"ItemCode", itemInfoToProduce.ItemCode,
						"ItemBarcode", itemInfoToProduce.CodeBars, "UOM", itemInfoToProduce.InventoryUom,
						"ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
						"IsForEnteringDefaultPackagingQuantity", true,
						"BatchNumber", batchNumber,
						"BestBeforeDate", bestBeforeDate,
						"SSCC", existingSSCC,
						"DocNum", productionOrderNumber.ToString()
						));

					msg = WaitForMessage();

					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".QuantitiesEntered"))
					{
						PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

						//build the list of packaging quantities
						foreach (ItemTransactionalPackagingTypeInfo info in currentListOfTransactionalPackagingTypeInfos)
						{
							//try to get new qty from user
							if (screenListOfPackagingTypeInfos.PackagingTypeInfoList.ContainsKey(info.PackagingTypeCode))
							{
								info.Quantity = screenListOfPackagingTypeInfos.PackagingTypeInfoList[info.PackagingTypeCode].Quantity;
							}
						}

						goto Step_EnterBatchAttributes;
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						if (itemInfoToProduce.IsLogisticUnit)
						{
							goto Step_ShowProductionTasks;
						}
						goto Step_SelectLogisticCarrier;
					}

				}

				goto Step_EnterBatchAttributes;
			}
			else if (!productionInformation.IsDisassembly)
			{
				//no batch info
				//get the packaging types qty from item
				currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfoToProduce);

				goto Step_EnterBatchAttributes;
			}
			else
			{
				currentListOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

				goto Step_EnterBatchAttributes;
			}

			#endregion

			Step_EnterBatchAttributes:

			#region ENTER BATCH ATTRIBUTES

			currentListOfTransactionalBatchAttributeInfos = null;

			if ((itemInfoToProduce.HasBatchnumber || itemInfoToProduce.HasSecondBatchNumber || itemInfoToProduce.HasBestBeforeDate))
			{

				m_enterBatchAttributesFlow.BindParameters(
						  this.DefaultCultureInfo,
						  this.PmxOseCompany,
						  new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToProduce),
						  new ReadOnlyBinder<string>("BatchNumber", batchNumber),
						  new ReadOnlyBinder<string>("BatchNumber2", batchNumber2),
						  new ReadOnlyBinder<DateTime?>("BBD", bestBeforeDate),
						  new ReadOnlyBinder<bool>("SaveData", false),
						  new ReadOnlyBinder<BarcodeContainer>("Barcode", null),

						  new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
						  new ReadWriteBinder<Collection<ItemTransactionalBatchAttributeInfo>>("ListOfBatchAttributes", this, "m_enterBatchAttributesFlow_ListOfBatchAttributes")

						  );
				m_subFlowBackRequested = false;
				this.StartSubFlowAndWait(m_enterBatchAttributesFlow, true);

				if (m_subFlowBackRequested)
				{
					if (itemInfoToProduce.IsLogisticUnit)
					{
						goto Step_ShowProductionTasks;
					}
					goto Step_SelectLogisticCarrier;
				}

				currentListOfTransactionalBatchAttributeInfos = m_enterBatchAttributesFlow_ListOfBatchAttributes;




				goto Step_EnterQuantity;
			}
			else
			{
				goto Step_EnterQuantity;
			}

			#endregion

			Step_EnterQuantity:

			#region ENTER QUANTITY

			goBack = false;

			quantityToProduce = 0;
			quantityUom2ToProduce = null;


			listOfSerialNumbers = null;
			m_subFlowListOfSerialNumbers = null;

			bool getNormalQuantity = true;

			if ((itemInfoToProduce.HasSerialNumber || itemInfoToProduce.HasSAPSerialNumber) &&
				!productionInformation.IsDisassembly && !productionInformation.SerialManagingByPool)
			{
				#region SERIAL NUMBER


				getNormalQuantity = false;

				//show screen to ask user what to do.
				List<string> buttonKeys = new List<string>(new Collection<string>() { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" });

				if (!(//itemInfoToProduce.TrackLocationSerialNumber ||
					(itemInfoToProduce.HasSAPSerialNumber && itemInfoToProduce.ManageMethod == SboItemManageMethod.OnEveryTransaction)))
				{
					buttonKeys.Add("MSG_BUTTON_NO_SERIAL_NUMBER");
				}


				toolBarButtons = ToolBarButtons.Back;

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
						"AllowedButtons", buttonKeys.ToArray(),
						"EnabledToolBarButtons", toolBarButtons
						));
				msg = WaitForMessage();
				initialErrorKey = null;
				if (msg.Name.EndsWith(".ButtonClicked"))
				{
					string buttonKeySerial = ExtractParameter<string>(msg.Parameters, "buttonKey");

					if (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER" ||
						buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
					{
						#region ENTER SERIAL NUMBERS


						//get ITRI
						int? itriKey = null;
						if (itemInfoToProduce.HasBestBeforeDate ||
							itemInfoToProduce.HasSecondBatchNumber ||
							itemInfoToProduce.HasBatchnumber)
						{
							itriKey = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecuteGetCreateItemTransactionalInfoKey), itemInfoToProduce.ItemCode, batchNumber, batchNumber2, bestBeforeDate);
						}


						bool isSerialNumberRange = (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

						m_enterQuantitySubFlow.BindParameters(
							this.DefaultCultureInfo,
							this.PmxOseCompany,
							new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
							new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToProduce),
							new ReadOnlyBinder<string>("BatchNumber", batchNumber),
							new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
							new ReadOnlyBinder<int?>("ItriKey", itriKey),
							new ReadOnlyBinder<string>("LocationCode", productionLineCode),
							new ReadOnlyBinder<string>("SSCC", null),

							new ReadOnlyBinder<string>("DocNum", productionOrderNumber.ToString()),
							new ReadOnlyBinder<bool>("UsePickingInfo", false),

							new ReadOnlyBinder<bool>("AllowZero", false),
							new ReadOnlyBinder<double>("InitialQuantity", 1),
							new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"),
							new ReadOnlyBinder<double>("StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0),
							new ReadOnlyBinder<double>("MaximumQuantity", maximumQuantity.Value),
							new ReadOnlyBinder<double>("MinimumQuantity", 0),
							new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
							new ReadOnlyBinder<string>("UOM", itemInfoToProduce.InventoryUom),
							new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToProduce != null) ? itemInfoToProduce.UomDecimals : 0),

							new ReadOnlyBinder<double>("InnerQuantity", 1),
							new ReadOnlyBinder<string>("InnerUOM", ""),
							new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
							new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.ProductionOrder).ToString()),
							new ReadOnlyBinder<int?>("DocEntry", productionOrderNumber),
							new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
							new ReadOnlyBinder<string>("LocationCode", productionLineCode),

							new ReadOnlyBinder<int?>("LUID", null),


							new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
							new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
							new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

							);

						m_subFlowBackRequested = false;
						this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

						if (m_subFlowBackRequested)
						{
							if (itemInfoToProduce.IsLogisticUnit)
							{
								goto Step_ShowProductionTasks;
							}
							goto Step_SelectLogisticCarrier;
						}

						listOfSerialNumbers = m_subFlowListOfSerialNumbers;

						quantityToProduce = m_subFlowQuantity;

						goto Step_ConfirmQuantity;

						#endregion

					}

					getNormalQuantity = true;
				}

				if (msg.Name.EndsWith(".BackRequested"))
				{
					if (itemInfoToProduce.IsLogisticUnit)
					{
						goto Step_ShowProductionTasks;
					}
					goto Step_SelectLogisticCarrier;
				}



				#endregion
			}

			if (getNormalQuantity)
			{

				if (itemInfoToProduce.ListOfPackagingTypes != null &&
					currentListOfTransactionalPackagingTypeInfos.Count > 0 &&
					(!itemInfoToProduce.IsCatchWeightItem || productionInformation.IsDisassembly))
				{
					#region PACKAGING TYPES

					//build list of packaging types
					listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();

					//add first one for the uom

					PackagingTypeInfo info = new PackagingTypeInfo();

					info.Quantity = productionInformation.IsDisassembly ? 1 : GetDefaultQuantityForProduction(itemInfoToProduce);
					info.PackagingTypeCode = null;
					info.PackagingTypeName = itemInfoToProduce.InventoryUom == null ? itemInfoToProduce.ItemCode : itemInfoToProduce.InventoryUom;
					info.QuantityPerPack = 1;
					if (!string.IsNullOrEmpty(itemInfoToProduce.CodeBars))
					{
						info.Barcode.Add(itemInfoToProduce.CodeBars);
					}
					info.NumberOfDecimals = itemInfoToProduce.UomDecimals;
					info.InitialQuantity = 1;
					listOfPackagingTypeInfosForScreen.Add(info);


					foreach (ItemTransactionalPackagingTypeInfo packType in currentListOfTransactionalPackagingTypeInfos)
					{
						bool hideDuringEnteringQuantity = GetHideDuringEnteringQuantity(packType.PackagingTypeCode, itemInfoToProduce.ListOfPackagingTypes);

						if (!hideDuringEnteringQuantity)
						{
							info = new PackagingTypeInfo();

							info.Quantity = packType.Quantity;
							info.PackagingTypeCode = packType.PackagingTypeCode;
							info.PackagingTypeName = GetPackTypeName(packType.PackagingTypeCode, itemInfoToProduce.ListOfPackagingTypes);
							info.Barcode = GetPackBarcode(packType.PackagingTypeCode, itemInfoToProduce.ListOfPackagingTypes);

							info.HideDuringEnteringQuantity = hideDuringEnteringQuantity;

							info.InitialQuantity = 0;

							info.QuantityPerPack = packType.Quantity;

							listOfPackagingTypeInfosForScreen.Add(info);
						}
					}


					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
										"InitialErrorKey", initialErrorKey,
										"TitleKey", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
										"ItemDescription", itemInfoToProduce.Description,
										"ItemCode", itemInfoToProduce.ItemCode,
										"ItemBarcode", itemInfoToProduce.CodeBars,
										"UOM", itemInfoToProduce.InventoryUom,
										"ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
										"IsForEnteringDefaultPackagingQuantity", false,
										"BatchNumber", batchNumber,
										"BestBeforeDate", bestBeforeDate,
										"SSCC", existingSSCC,
										"DocNum", productionOrderNumber.ToString(),
										"StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0,
										"MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY",
										"AllowZero", false,
										"MaximumQuantity", maximumQuantity.Value
										));

					msg = WaitForMessage();

					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".QuantitiesEntered"))
					{
						PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

						//build the list of packaging quantities
						foreach (PackagingTypeInfo infoForQty in screenListOfPackagingTypeInfos.PackagingTypeInfoList.Values)
						{
							quantityToProduce += (infoForQty.Quantity * infoForQty.QuantityPerPack);
						}

						goto Step_ConfirmQuantity;
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						if (itemInfoToProduce.IsLogisticUnit)
						{
							goto Step_ShowProductionTasks;
						}
						goto Step_SelectLogisticCarrier;
					}
					#endregion
				}
				else
				{
					if (!(itemInfoToProduce.IsCatchWeightItem && productionInformation.IsDisassembly))
					{

						//Call enterquantity script
						m_enterQuantitySubFlow.BindParameters(
							this.DefaultCultureInfo,
							this.PmxOseCompany,
							new ReadOnlyBinder<string>("TitleKey", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED"),
							new ReadOnlyBinder<string>("TitleKeyWeight", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_WEIGHT_DISASSEMBLED" : "MSG_TITLE_ENTER_WEIGHT_PRODUCED"),
							new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToProduce),
							new ReadOnlyBinder<string>("BatchNumber", batchNumber),
							new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
							new ReadOnlyBinder<int?>("ItriKey", null),
							new ReadOnlyBinder<string>("LocationCode", null),
							new ReadOnlyBinder<string>("SSCC", null),

							new ReadOnlyBinder<string>("DocNum", productionInformation.ProductionOrderNumber.ToString()),
							new ReadOnlyBinder<bool>("UsePickingInfo", false),
							new ReadOnlyBinder<bool>("IsAlternatePicking", false),

							new ReadOnlyBinder<bool>("AllowZero", false),
							new ReadOnlyBinder<double>("InitialQuantity", productionInformation.IsDisassembly ? 0 : GetDefaultQuantityForProduction(itemInfoToProduce)),
							new ReadOnlyBinder<double>("StillToQuantity", stillToProduceQuantity),
							new ReadOnlyBinder<string>("MessageKeyStillQuantity", null), //"MSG_LBL_MAXIMUM_QUANTITY"),
							new ReadOnlyBinder<double>("MaximumQuantity", maximumQuantity.Value),
							new ReadOnlyBinder<double>("MinimumQuantity", 0),
							new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
							new ReadOnlyBinder<string>("UOM", itemInfoToProduce.InventoryUom),
							new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToProduce != null) ? itemInfoToProduce.UomDecimals : 0),

							new ReadOnlyBinder<double>("InnerQuantity", 1),
							new ReadOnlyBinder<string>("InnerUOM", itemInfoToProduce.InventoryUom),
							new ReadOnlyBinder<bool>("ForceInputSerialNumbers", false),
							new ReadOnlyBinder<string>("ObjectType", Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.SboObjectTypes.ProductionOrder),
							new ReadOnlyBinder<int?>("DocEntry", productionInformation.ProductionOrderDocEntry),
							new ReadOnlyBinder<bool>("AutoFillQuantityForPackagingTypes", false),
							new ReadOnlyBinder<bool>("OverrideAskingWeight", false),
							new ReadWriteBinder<double?>("QuantityUom2Entered", this, "m_subFlowQuantityUom2"),
							new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
							new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
							new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

							);

						m_subFlowBackRequested = false;
						this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

						if (m_subFlowBackRequested)
						{
							if (itemInfoToProduce.IsLogisticUnit)
							{
								goto Step_ShowProductionTasks;
							}
							goto Step_SelectLogisticCarrier;
						}

						listOfSerialNumbers = m_subFlowListOfSerialNumbers;
						quantityToProduce = m_subFlowQuantity;
						quantityUom2ToProduce = m_subFlowQuantityUom2;

						goto Step_ConfirmQuantity;
					}
					else
					{
						//This is a disassembly, but header item is a catch weight item.
						//In this case we only need to ask for the pieces, because they are taken in account for the receipt of the ingredients.
						//The weight of the header item has no use ...
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", productionInformation.IsDisassembly ? "MSG_TITLE_ENTER_QUANTITY_DISASSEMBLED" : "MSG_TITLE_ENTER_QUANTITY_PRODUCED",
							"InitialQuantity", 1,
							"NumberOfDigitsAfterDecimal", itemInfoToProduce.UomDecimals,
							"MaximumQuantity", maximumQuantity.Value,
							"UOM", itemInfoToProduce.InventoryUom,
							"DocNum", productionOrderNumber.ToString(),
							"ItemDescription", itemInfoToProduce.Description,
							"ItemCode", itemInfoToProduce.ItemCode,
							"ItemBarcode", itemInfoToProduce.CodeBars,
							"BatchNumber", batchNumber,
							"BestBeforeDate", bestBeforeDate,
							"StillToQuantity", maximumQuantity.Value > 0 ? maximumQuantity.Value : 0,
							"MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY",
							"SetFocusToQuantityControl", setFocusToQuantityControl
							));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantityEntered"))
						{
							quantityToProduce = ExtractParameter<double>(msg.Parameters, "quantity");
							goto Step_ConfirmQuantity;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							if (itemInfoToProduce.IsLogisticUnit)
							{
								goto Step_ShowProductionTasks;
							}
							goto Step_SelectLogisticCarrier;

						}
					}

				}
			}



			#endregion

			Step_ConfirmQuantity:

			#region CONFIRM QUANTITY

			if (confirmProducedQtyAfterProduction && !productionInformation.IsDisassembly)
			{
				//check if qty deviates from default qty
				if (
					itemInfoToProduce.DefaultProductionQuantity > 0 &&
					(quantityToProduce > itemInfoToProduce.DefaultProductionQuantity + (itemInfoToProduce.DefaultProductionQuantity * itemInfoToProduce.AllowedProductionDeviationPercentage / 100) ||
					quantityToProduce < itemInfoToProduce.DefaultProductionQuantity - (itemInfoToProduce.DefaultProductionQuantity * itemInfoToProduce.AllowedProductionDeviationPercentage / 100))
					)
				{

					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_CONFIRM_PROD_QTY_AFTER_PROD",
							"MessageKey", "MSG_QUESTION_CONFIRM_PROD_QTY_AFTER_PROD",
							"Parameters", new object[] { quantityToProduce, itemInfoToProduce.Description }));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".No"))
					{
						//Go back
						goto Step_EnterQuantity;
					}

				}
			}
			else if (goBack)
			{
				goto Step_EnterQuantity;
			}

			goto Step_GenerateBBD;


			#endregion

			Step_GenerateBBD:
			#region GENERATE BBD

			listOfItemCodes = new List<string>();
			// This foreach provides asking BBD for every component during disassembly, but not supported yet
			foreach (string item in batchInfo.Keys)
			{
				listOfItemCodes.Add(item);
			}
			tempBatchInfo = batchInfo;

			//Step_StartGenerateBBD:

			batchAndBestBeforeDateInfo = tempBatchInfo[listOfItemCodes[0]];

			if (batchAndBestBeforeDateInfo.GenerateBBDOnEveryReceipt)
			{
				goBack = false;

				if (batchAndBestBeforeDateInfo.HasBestBeforeDate)
				{
					bestBeforeDate = batchAndBestBeforeDateInfo.BestBeforeDate;

					if (batchAndBestBeforeDateInfo.IsUserAllowedToChangeBestBeforeDate)
					{
						//user can change the best before date, so ask it
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterDateScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE_FOR_PRODUCTION",
							 "ForceDataEntry", true,
							 "ForceUppercase", true,
							 "AllowToGoBack", true,
							 "DefaultDate", bestBeforeDate,
							 "Information", "MSG_DEFAULT_BEST_BEFORE_DATE"
							 ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".DateEntered"))
						{
							bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "date");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
							goto Step_ConfirmQuantity;
						}
					}

				}
			}

			goto Step_ProduceStock;
			#endregion

			Step_ProduceStock:

			#region PRODUCE STOCK

			//createdLUID = null;

			//get maximum qty again, but now with the entered qty.
			//This is needed for the steplist.
			//It depends on the number of items to use, to know if we have enough stock of the ingredients
			if (!lineNumByProduct.HasValue)
			{
				maximumQuantity = m_sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
				delegate (PmxDbConnection conn, object[] parameters)
				{
					PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
					return prov.GetMaximumQuantityToProduce(productionOrderDocEntry.Value, quantityToProduce, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation, whsInfo.StorageLocationCodeLogisticCarriers, itemInfoToProduce);
				});
				if (blnScanSSCCToConsume)
				{
					maximumQuantity = -1;
				}
				//if (!maximumQuantity.HasValue ||
				//    maximumQuantity.Value <= 0)
				if (maximumQuantity.HasValue &&
					maximumQuantity.Value != -1 &&
					maximumQuantity.Value < quantityToProduce)
				{
					initialErrorKey = productionInformation.IsDisassembly ? "MSG_ERROR_NOT_ENOUGH_STOCK_TO_DISASSEMBLE" : "MSG_ERROR_NOT_ENOUGH_STOCK_TO_PRODUCE";
					goto Step_EnterPackagingQuantity;
				}
			}

			Step_Produce_Again:


			// serial numbers will be emptied after produce
			Collection<string> listOfSerialNumbersForHookFlow = new Collection<string>();
			if (listOfSerialNumbers != null)
			{
				foreach (string serialNumber in listOfSerialNumbers)
				{
					listOfSerialNumbersForHookFlow.Add(serialNumber);
				}
			}

			//produce stock
			listOfCreatedReceipts = new List<int>();
			List<int> createdLUIDs = new List<int>();

			try
			{
				double splitQty = quantityToProduce;
				double? splitQtyUom2 = quantityUom2ToProduce;

				int splitInNrOfLogUnits = 1;
				if (splitQtyInLogUnitsOfItemDefQtyOnLogUnit && GetDefaultQuantityForProduction(itemInfoToProduce) > 0
					&& (!existingLuid.HasValue /*|| !existingLUIDIsInStock*/ ) /*&& advanceNoticeQty == 0*/ )
				{
					splitInNrOfLogUnits = (int)Produmex.Foundation.Utilities.FormattingHelper.CeilingForDecimals(splitQty / GetDefaultQuantityForProduction(itemInfoToProduce), 0);
				}


				m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);


						List<InventoryItemInfo> listOfConsumedItems = new List<InventoryItemInfo>();



						Collection<string> fullListOfSerialNumbers = listOfSerialNumbers;

						double producedQuantity = 0;

						for (int splitLogUnitIdx = 0; splitLogUnitIdx < splitInNrOfLogUnits; splitLogUnitIdx++)
						{
							double logUnitQty = splitQty;
							double? logUnitQtyUom2 = splitQtyUom2;

							if (splitInNrOfLogUnits > 1)
							{
								logUnitQty = Math.Min(splitQty, GetDefaultQuantityForProduction(itemInfoToProduce));
								splitQty = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(splitQty - logUnitQty, itemInfoToProduce.UomDecimals);

								if (itemInfoToProduce.IsCatchWeightItem)
								{
									if (splitLogUnitIdx + 1 == splitInNrOfLogUnits)
									{
										//Keep remainder
										logUnitQtyUom2 = splitQtyUom2;
									}
									else
									{
										logUnitQtyUom2 = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((quantityUom2ToProduce.Value / splitInNrOfLogUnits) * logUnitQty, itemInfoToProduce.Uom2Decimals);
									}
									splitQtyUom2 = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(splitQtyUom2.Value - logUnitQtyUom2.Value, itemInfoToProduce.Uom2Decimals);
								}
							}

							Collection<string> tempListOfSerialNumbers = null;

							if (fullListOfSerialNumbers != null && fullListOfSerialNumbers.Count > 0)
							{
								tempListOfSerialNumbers = new Collection<string>();

								for (int iLoop = 0; iLoop < logUnitQty; iLoop++)
								{
									tempListOfSerialNumbers.Add(fullListOfSerialNumbers[iLoop]);
								}

							}


							batchAndBestBeforeDateInfo.BatchNumber = batchNumber;
							batchAndBestBeforeDateInfo.BatchNumber2 = batchNumber2;
							batchAndBestBeforeDateInfo.BestBeforeDate = bestBeforeDate;

							if (lineNumByProduct.HasValue)
							{
								if (itemInfoByProduct.HasBatchnumber)
								{
									if (batchInfo.ContainsKey(itemCodeByProduct))
									{
										batchInfo[itemCodeByProduct] = batchAndBestBeforeDateInfo;
									}
									else
									{
										batchInfo.Add(itemCodeByProduct, batchAndBestBeforeDateInfo);
									}
								}
							}
							else
							{

								if (batchInfo.ContainsKey(itemCodeToProduce))
								{
									batchInfo[itemCodeToProduce] = batchAndBestBeforeDateInfo;
								}
								else
								{
									batchInfo.Add(itemCodeToProduce, batchAndBestBeforeDateInfo);
								}
							}

							List<InventoryItemInfo> listOfProducedItems = new List<InventoryItemInfo>();

							//produce         
							int? createdLUID = null;

							if (isLinedUpLocationWithAnIngredientMustBeProduced)
							{
								List<InventoryItemInfo> producedItems = prdProv.ProduceAndLock(productionOrderDocEntry.Value, logUnitQty, 0,
									PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, logisticCarrierCode, existingLuid, !noSSCC,
									itemCodeToProduce, outputLocationCode, currentListOfTransactionalPackagingTypeInfos, batchInfo, fullListOfSerialNumbers, currentListOfTransactionalBatchAttributeInfos,
									logUnitQtyUom2, lineNumByProduct, splitInNrOfLogUnits, ref listOfConsumedItems, ref listOfProducedItems, out listOfCreatedReceipts);

								if (producedItems != null)
								{
									foreach (var invItemInfo in producedItems)
									{
										if (invItemInfo.ItemCode == itemCodeToProduce)
										{
											createdLUID = invItemInfo.LogisticUnitIdentKey;
											break;
										}
									}
								}
							}
							else
							{
								createdLUID = prdProv.ProduceExtended(productionOrderDocEntry.Value, logUnitQty, outputLocationCode, whsInfo.PmxWarehouseCode, logisticCarrierCode, existingLuid, !noSSCC,
								true, currentListOfTransactionalPackagingTypeInfos, productionInformation.CurrentProdBatch, batchInfo, tempListOfSerialNumbers, currentListOfTransactionalBatchAttributeInfos,
								logUnitQtyUom2, lineNumByProduct, ref listOfConsumedItems, ref listOfProducedItems, out listOfCreatedReceipts);
							}

							producedQuantity += logUnitQty;

							if (tempListOfSerialNumbers != null)
							{
								//remove serial numbers from full list, so next run will get new serial numbers
								foreach (string serNum in tempListOfSerialNumbers)
								{
									fullListOfSerialNumbers.Remove(serNum);
								}
							}


							if (createdLUID.HasValue)
							{
								createdLUIDs.Add(createdLUID.Value);
							}
						}


						return null;

					});
				/*tempBatchInfo.Remove(listOfItemCodes[0]);
				listOfItemCodes.Remove(listOfItemCodes[0]);

				if (tempBatchInfo.Count > 0)
				{
					goto Step_StartGenerateBBD;
				}*/
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_Produce_Again;
				}
				goto Step_ShowProductionTasks;

			}

			goto Step_PrintProductionLabel;


			#endregion

			Step_PrintProductionLabel:

			#region PRINT PRODUCTION LABEL

			numberOfReports = 0;

			if (createdLUIDs.Count > 0)
			{
				try
				{
					//trigger event
					foreach (int createdLUID in createdLUIDs)
					{
						numberOfReports += m_sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters2)
						{
							PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
							return printProv.PrintReportsForEvent(PmxPrintReportEventType.ProductionLogisticUnitProduced,
								productionLineCode, deviceID, createdLUID, out askReprint);
						});
					}
				}
				catch (Produmex.Foundation.ProdumexException ex)
				{
					s_log.Error(ex.ToString());
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
							"MessageAddendum", ex.Message,
							"PmxError", ex.Error,
							"ShowButton", true));

					msg = WaitForMessage();
				}
				catch (Exception ex)
				{
					s_log.Error(ex.ToString());
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
							"MessageAddendum", ex.Message,
							"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
							"ShowButton", true));

					msg = WaitForMessage();
				}


				if (numberOfReports > 0 && askReprint)
				{
					goto Step_DocumentsPrinted;
				}

				//Produced items
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
				   DefaultCultureInfo.Get(), BuildParamCollection(
					   "InitialErrorKey", initialErrorKey,
					   "MessageKey", productionInformation.IsDisassembly ? "MSG_ITEM_IS_DISASSEMBLED" : "MSG_ITEM_IS_PRODUCED",
					   "ShowButton", true));

				this.WaitForMessage();
			}

			goto Step_AfterProduced;

			#endregion

			Step_DocumentsPrinted:

			#region DOCUMENTS PRINTED

			if (numberOfReports > 0 &&
				askReprint)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"MessageKey", "MSG_QUESTION_REPRINT_LABEL",
						"TitleKey", "MSG_TITLE_REPRINT_LABEL"));

				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_PrintProductionLabel;
				}

			}

			#endregion

			Step_AfterProduced:

			#region AFTER PRODUCED

			//HOOK
			m_afterQuantityProducedHookFlow.BindParameters(
				  this.DefaultCultureInfo,
				  this.PmxOseCompany,
				  new ReadOnlyBinder<Collection<string>>("ListOfSerialNumbers", listOfSerialNumbersForHookFlow),
				  new ReadOnlyBinder<List<int>>("CreatedLUIDs", createdLUIDs),
				  new ReadOnlyBinder<int?>("ProdOrderDocEntry", productionOrderDocEntry)
			  );
			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_afterQuantityProducedHookFlow, true);

			createdLUIDs.Clear();

			//Put away for production
			try
			{
				foreach (int receiptDocEntry in listOfCreatedReceipts)
				{
					m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(ExecuteGeneratePutAwayProduction), receiptDocEntry, outputLocationCode);
				}
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (!(ex is ProdumexException))
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();

				}

				//retry?
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_RETRY_UPDATE",
						"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
				msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto Step_AfterProduced;
				}
			}



			if (productionInformation.StillToProduce <= quantityToProduce &&
				!lineNumByProduct.HasValue)
			{
				goto Step_StopProductionTask;
			}

			goto Step_ShowProductionTasks;

			#endregion


			Step_StopProductionTask:

			#region STOP PRODUCTION TASK

			stopProduction = false;
			buttonKeyOtherTasks = null;

			allowedButtonsForStopProductionTask = new List<string>();

			if (authorizationStopButton == PmxUserGroupAuthorizationCodes.Enabled)
			{
				allowedButtonsForStopProductionTask.Add("MSG_BUTTON_STOP_PRODUCTION");
			}
			if (authorizationOnHoldButton == PmxUserGroupAuthorizationCodes.Enabled)
			{
				allowedButtonsForStopProductionTask.Add("MSG_BUTTON_ON_HOLD_PRODUCTION");
			}
			allowedButtonsForStopProductionTask.Add("MSG_BUTTON_BACK_TO_PRODUCTION_LINE");
			if (productionInformation.NrOfBatches.HasValue && productionInformation.NrOfBatches.Value != productionInformation.CurrentProdBatch)
			{
				allowedButtonsForStopProductionTask.Add("MSG_BUTTON_NEXT_PRODUCTION_BATCH");
			}

			// Show screen with list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_TASK",
					"AllowedButtons", allowedButtonsForStopProductionTask.ToArray()
					));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goto Step_ShowProductionTasks;
			}
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				buttonKeyOtherTasks = ExtractParameter<string>(msg.Parameters, "buttonKey");

				if (buttonKeyOtherTasks == "MSG_BUTTON_BACK_TO_PRODUCTION_LINE")
				{
					goBack = true;
					goto Step_SelectProductionLine;
				}
				else if (buttonKeyOtherTasks == "MSG_BUTTON_STOP_PRODUCTION")
				{
					stopProduction = true;
					if (blnScanSSCCToConsume)
					{
						try
						{
							m_sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null,
								new DelegateWithSboConnection<object>(ExecuteCloseProductionOrder),
								productionOrderDocEntry);
						}
						catch (Produmex.Foundation.ProdumexException ex)
						{
							s_log.Error("Exception during ExecuteCloseProductionOrder(...)", ex);

							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
									"MessageAddendum", ex.Message,
									"PmxError", ex.Error,
									"ShowButton", true));

							msg = WaitForMessage();

							goto Step_ShowProductionTasks;
						}
						catch (Exception ex)
						{
							s_log.Error("Exception during ExecuteCloseProductionOrder(...)", ex);

							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
									"MessageAddendum", ex.Message,
									"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
									"ShowButton", true));

							msg = WaitForMessage();

							goto Step_ShowProductionTasks;
						}
						goto Step_SelectProductionLine;
					}
					goto Step_GetQuantityForRun;

				}
				else if (buttonKeyOtherTasks == "MSG_BUTTON_ON_HOLD_PRODUCTION")
				{
					stopProduction = false;

					//on hold
					if (blnScanSSCCToConsume)
					{

						m_sboProviderService.InvokeMethodWithDbConnection<int?>(true, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters)
						{

							PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
							//on hold
							prdProv.PutOnHold(productionOrderDocEntry.Value);

							return null;
						});

						goto Step_ShowProductionTasks;
					}
					else
					{
						goto Step_GetQuantityForRun;
					}

				}
				else if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
				{

					//ask user if he wants start the next batch
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_NEXT_PRODUCTION_BATCH",
							"MessageKey", "MSG_QUESTION_NEXT_PRODUCTION_BATCH"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_GetQuantityForRun;
					}

					goto Step_ShowProductionTasks;

				}
				initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
				goto Step_StopProductionTask;
			}

			#endregion

			Step_GetQuantityForRun:

			#region GET QUANTITY FOR RUN

			quantityForRun = m_sboProviderService.InvokeMethodWithDbConnection<double>(true, false, null, null,
			  delegate (PmxDbConnection conn, object[] parameters)
			  {
				  PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
				  return prdProv.GetQuantityProducedForCurrentRun(productionOrderDocEntry.Value, productionInformation.IsDisassembly);
			  });

			if (stepListKey.HasValue)
			{
				//PRD is for step list
				goto Step_StartConsumeForStepList;
			}
			else
			{
				//No step list
				goto Step_StartConsumeNoStepList;
			}

			#endregion

			Step_StartConsumeNoStepList:

			#region START CONSUME NO STEP LIST

			if (productionInformation.IsDisassembly)
			{
				query = BuildQuery.GetProductionOrderLinesDisassembly(productionOrderDocEntry.Value, null, m_sboProviderService.GetDbTool());
			}
			else
			{
				query = BuildQuery.GetProductionOrderLines(productionOrderDocEntry.Value, null, m_sboProviderService.GetDbTool());
			}
			listConsumeItemsToSend = new Collection<ConsumeItemInfo>();


			DataSet dsProductionOrderLines = m_sboProviderService.RunView(false, null, null, query);


			if (productionInformation.IsDisassembly)
			{
				query = BuildQuery.GetProductsOnLineDisassembly(productionOrderDocEntry.Value, productionLineCode, null, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, m_sboProviderService.GetDbTool());
			}
			else
			{
				query = BuildQuery.GetProductsOnLine(productionOrderDocEntry.Value, productionLineCode, null, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, m_sboProviderService.GetDbTool());
			}

			dsProductOnLine = m_sboProviderService.RunView(false, null, null, query);


			PmxItemInfo itemInfoToConsume = null;


			listOfItemCodesAndQuantity = new Collection<KeyValuePair<string, ItemSetpointInfo>>();

			PmxDictionary<string, ItemSetpointInfo> listOfQuantityByItemCode = new PmxDictionary<string, ItemSetpointInfo>();
			PmxDictionary<int, ItemSetpointInfo> listOfLineNumQuantity = new PmxDictionary<int, ItemSetpointInfo>();
			//Build list of quantity by item code
			if (this.GetNumberOfRows(dsProductionOrderLines) > 0)
			{
				foreach (DataRow row in dsProductionOrderLines.Tables[0].Rows)
				{
					string tempItemCode = row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string;

					double baseQuantity = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name]);
					double qtyTolPercentage = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name]);
					double qtySetpoint = baseQuantity * quantityForRun;

					double qtyLowerLimit = qtySetpoint * (100.0 - qtyTolPercentage) / 100.0;
					double qtyUpperLimit = qtySetpoint * (100.0 + qtyTolPercentage) / 100.0;
					if (qtyUpperLimit <= 0 && qtyTolPercentage >= 100)
						qtyUpperLimit = double.MaxValue;

					if (!listOfQuantityByItemCode.ContainsKey(tempItemCode))
					{
						listOfQuantityByItemCode.Add(tempItemCode, new ItemSetpointInfo(0, 0, 0));
					}

					if (!productionInformation.IsDisassembly)
					{
						int lineNum = Convert.ToInt32(row[PmxProductionOrderLineTable.Columns.LineNum.NAME]);
						if (!listOfLineNumQuantity.ContainsKey(lineNum))
						{
							listOfLineNumQuantity.Add(lineNum, new ItemSetpointInfo(qtyLowerLimit, qtySetpoint, qtyUpperLimit));
						}
					}



					listOfQuantityByItemCode[tempItemCode].Setpoint += qtySetpoint;
					listOfQuantityByItemCode[tempItemCode].LowerLimit += qtyLowerLimit;
					listOfQuantityByItemCode[tempItemCode].UpperLimit = Math.Min(double.MaxValue, listOfQuantityByItemCode[tempItemCode].UpperLimit + qtyUpperLimit);
				}
			}


			Collection<string> listOfPreparedItemCodes = new Collection<string>();
			listConsumeItemsToSend = new Collection<ConsumeItemInfo>();
			foreach (string tempItemCode in listOfQuantityByItemCode.Keys)
			{
				itemInfoToConsume = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), tempItemCode);

				DataRow[] rows = dsProductOnLine.Tables[0].Select(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name + " = '" + tempItemCode + "'");
				int? currentProductionOrderLineNum = null;

				double quantityToUseRemaining = listOfQuantityByItemCode[tempItemCode].Setpoint;

				//copy structure
				DataSet dsConsumeItemsByItemCode = dsProductOnLine.Clone();





				foreach (DataRow row in rows)
				{
					int productionOrderLineNum = -1;
					if (!productionInformation.IsDisassembly)
					{
						productionOrderLineNum = Convert.ToInt32(row[PmxProductionOrderLineTable.Columns.LineNum.NAME]);
						if (!currentProductionOrderLineNum.HasValue)
						{
							currentProductionOrderLineNum = productionOrderLineNum;
						}
					}

					//When multiple production order lines exist for the same item code, the inventory lines are duplicated.
					//So we only take the stock lines for the first production order line
					if (productionInformation.IsDisassembly ||
						productionOrderLineNum == currentProductionOrderLineNum)
					{
						double qtyUsed = 0;
						double? qtyUom2Used = 0;

						bool isPrepared = !productionInformation.IsDisassembly && Convertor.ConvertToInt32(row[ConsumeItemsForProductionDataSet.Columns.IS_PREPARED.Name]) != 0;
						if (isPrepared && !listOfPreparedItemCodes.Contains(row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string))
						{
							listOfPreparedItemCodes.Add(row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string);
						}

						double quantityOnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]);
						double quantityUom2OnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name]);

						if (autoFillConsumedQtyFromPreparedQty && isPrepared)
						{
							qtyUsed = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]);
							if (itemInfoToConsume.IsCatchWeightItem)
							{
								qtyUom2Used = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name]);
							}
						}
						else
						{

							qtyUsed = Math.Min(quantityToUseRemaining, quantityOnLine);
							quantityToUseRemaining -= qtyUsed;

							if (itemInfoToConsume.IsCatchWeightItem)
							{
								if (quantityOnLine == qtyUsed)
								{
									qtyUom2Used = quantityUom2OnLine;
								}
								else
								{
									//Use default ratio
									qtyUom2Used = PmxItemAllConnectionsProvider.CalculateWeightFromPieces(qtyUsed, itemInfoToConsume);
								}
							}


						}





						row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name] = qtyUsed;
						if (itemInfoToConsume.IsCatchWeightItem)
						{
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name] = qtyUom2Used;
						}

						//set the rest
						row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name] = quantityOnLine - qtyUsed;
						if (itemInfoToConsume.IsCatchWeightItem)
						{
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name] = quantityUom2OnLine - qtyUom2Used;
						}


						dsConsumeItemsByItemCode.Tables[0].Rows.Add(row.ItemArray);
					}


				}


				DataSet dsConsumedItems = null;

				ItemSetpointInfo totalToConsumeForItem = listOfQuantityByItemCode[tempItemCode];

				if ((skipConsumptionScreenForLinkedComponents && listOfPreparedItemCodes.Contains(tempItemCode)) ||
					(totalToConsumeForItem.Setpoint == 0 && totalToConsumeForItem.UpperLimit == 0))
				{
					dsConsumedItems = dsConsumeItemsByItemCode;
				}
				else
				{
					//show screen
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IEnterConsumeItemsScreen),
					   DefaultCultureInfo.Get(), BuildParamCollection(
					   "InitialErrorKey", initialErrorKey,
					   "TitleKey", "MSG_TITLE_ENTER_CONSUMED_QUANTITY",
					   "DSConsumeItems", dsConsumeItemsByItemCode,
					   "TotalQuantityToConsumeLowerLimit", totalToConsumeForItem.LowerLimit,
					   "TotalQuantityToConsume", totalToConsumeForItem.Setpoint,
					   "TotalQuantityToConsumeUpperLimit", totalToConsumeForItem.UpperLimit,
					   "NumbersOfDigitsAfterDecimal", itemInfoToConsume.UomDecimals,
					   "UseWaste", useWaste,
					   "IsCatchWeight", itemInfoToConsume.IsCatchWeightItem,
					   "NumbersOfDigitsAfterDecimalUom2", itemInfoToConsume.Uom2Decimals,
					   "TotalQuantityUom2ToConsume", PmxItemAllConnectionsProvider.CalculateWeightFromPieces(totalToConsumeForItem.Setpoint, itemInfoToConsume),
					   "WeightByPiece", itemInfoToConsume.DefaultWeightForPiece

					   ));
					msg = WaitForMessage();
					initialErrorKey = null;
					if (msg.Name.EndsWith(".DataEntered"))
					{
						dsConsumedItems = ExtractParameter<DataSet>(msg.Parameters, "dsConsumedItems");
					}
					else
					{
						//go back
						goto Step_StopProductionTask;
					}

				}


				PmxDictionary<int, PmxDictionary<string, int>> serialNumbersByLuid = new PmxDictionary<int, PmxDictionary<string, int>>();

				bool isSerialNumber = itemInfoToConsume.HasSerialNumber || itemInfoToConsume.HasSAPSerialNumber;


				foreach (DataRow row in dsConsumedItems.Tables[0].Rows)
				{
					ConsumeItemInfo consume = new ConsumeItemInfo();

					consume.BatchNumber1 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name] as string;
					consume.BatchNumber2 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name] as string;
					if (row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name] is DateTime)
					{
						consume.BestBeforeDate = Convert.ToDateTime(row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name]);
					}
					consume.ItemCode = row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string;
					consume.ItemDescription = row[ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name] as string;
					consume.LocationCode = row[ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name] as string;
					consume.LogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[ConsumeItemsForProductionDataSet.Columns.LUID.Name]);
					consume.QualityStatusCode = row[ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name] as string;
					consume.QuantityUsed = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name]);
					consume.QuantityWaste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name]);
					consume.SSCC = row[ConsumeItemsForProductionDataSet.Columns.SSCC.Name] as string;

					if (itemInfoToConsume.IsCatchWeightItem)
					{
						consume.QuantityUom2Used = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name]);
						consume.QuantityUom2Waste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name]);
					}

					if (isSerialNumber)
					{

						if (consume.QuantityUsed > 0)
						{
							PmxDictionary<string, int> serialNumbers = new PmxDictionary<string, int>();
							// ask to fill in serial numbers unless they're full pallets with items with location tracking
							//show screen to ask user what to do.
							string[] buttonKeys = new string[2] { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" };

							toolBarButtons = ToolBarButtons.Back;

							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
									"AllowedButtons", buttonKeys,
									"EnabledToolBarButtons", toolBarButtons
									));
							msg = WaitForMessage();
							initialErrorKey = null;
							if (msg.Name.EndsWith(".ButtonClicked"))
							{
								string filterButtonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

								if (filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER" ||
									filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
								{
									#region ENTER SERIAL NUMBERS

									Collection<SerialNumberInfo> listOfSerialNumbersToUse = new Collection<SerialNumberInfo>();

									bool isSerialNumberRange = (filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

									m_enterQuantitySubFlow.BindParameters(
										this.DefaultCultureInfo,
										this.PmxOseCompany,
										new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
										new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToConsume),
										new ReadOnlyBinder<string>("BatchNumber", consume.BatchNumber1),
										new ReadOnlyBinder<DateTime?>("BestBeforeDate", consume.BestBeforeDate),
										new ReadOnlyBinder<int?>("ItriKey", Convertor.ConvertToInt32Nullable(row[ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name])),
										new ReadOnlyBinder<string>("LocationCode", productionLineCode),
										new ReadOnlyBinder<string>("SSCC", consume.SSCC),

										new ReadOnlyBinder<string>("DocNum", productionOrderNumber.ToString()),
										new ReadOnlyBinder<bool>("UsePickingInfo", false),

										new ReadOnlyBinder<bool>("AllowZero", false),
										new ReadOnlyBinder<double>("InitialQuantity", 1),
										new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"),
										new ReadOnlyBinder<double>("StillToQuantity", consume.QuantityUsed),
										new ReadOnlyBinder<double>("MaximumQuantity", consume.QuantityUsed),
										new ReadOnlyBinder<double>("MinimumQuantity", consume.QuantityUsed),
										new ReadOnlyBinder<double>("TotalQuantityToPick", consume.QuantityUsed),
										new ReadOnlyBinder<string>("UOM", itemInfoToConsume.InventoryUom),
										new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToConsume != null) ? itemInfoToConsume.UomDecimals : 0),

										new ReadOnlyBinder<double>("InnerQuantity", 1),
										new ReadOnlyBinder<string>("InnerUOM", ""),
										new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
										new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.NoLink).ToString()),
										new ReadOnlyBinder<int?>("DocEntry", productionOrderDocEntry),
										new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
										new ReadOnlyBinder<string>("LocationCode", productionLineCode),

										new ReadOnlyBinder<int?>("LUID", consume.LogisticUnitIdentKey),


										new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
										new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
										new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

										);

									//call flow
									base.StartSubFlowAndWait(m_enterQuantitySubFlow, true);


									if (m_subFlowBackRequested)
									{
										goto Step_StopProductionTask;
									}

									query = BuildQuery.GetSerialNumbers(itemInfoToConsume.ItemCode, m_subFlowListOfSerialNumbers, m_sboProviderService.GetDbTool());
									DataSet dsSerialNumberCount = m_sboProviderService.RunView(true, null, null, query);

									foreach (DataRow rowSer in dsSerialNumberCount.Tables[0].Rows)
									{
										int internalKey = Convert.ToInt32(rowSer[PmxSerialNumberTable.Columns.InternalKey.NAME]);
										int luid = Convert.ToInt32(rowSer[PmxSerialNumberTable.Columns.Luid.NAME]);
										string serialNumber = Convert.ToString(rowSer[PmxSerialNumberTable.Columns.SerialNumber.NAME]);

										serialNumbers.Add(serialNumber, internalKey);
									}


									#endregion

								}

							}
							else
							{
								//go back
								goto Step_StopProductionTask;
							}
							consume.ListOfSerialNumbersUsed = serialNumbers;
						}

						if (consume.QuantityWaste > 0)
						{
							PmxDictionary<string, int> serialNumbers = new PmxDictionary<string, int>();
							// ask to fill in serial numbers unless they're full pallets with items with location tracking
							//show screen to ask user what to do.
							string[] buttonKeys = new string[2] { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" };

							toolBarButtons = ToolBarButtons.Back;

							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
									"AllowedButtons", buttonKeys,
									"EnabledToolBarButtons", toolBarButtons
									));
							msg = WaitForMessage();
							initialErrorKey = null;
							if (msg.Name.EndsWith(".ButtonClicked"))
							{
								string filterButtonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

								if (filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER" ||
									filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
								{
									#region ENTER SERIAL NUMBERS

									Collection<SerialNumberInfo> listOfSerialNumbersToUse = new Collection<SerialNumberInfo>();

									bool isSerialNumberRange = (filterButtonKey == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

									m_enterQuantitySubFlow.BindParameters(
										this.DefaultCultureInfo,
										this.PmxOseCompany,
										new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
										new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfoToConsume),
										new ReadOnlyBinder<string>("BatchNumber", consume.BatchNumber1),
										new ReadOnlyBinder<DateTime?>("BestBeforeDate", consume.BestBeforeDate),
										new ReadOnlyBinder<int?>("ItriKey", Convertor.ConvertToInt32Nullable(row[ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name])),
										new ReadOnlyBinder<string>("LocationCode", productionLineCode),
										new ReadOnlyBinder<string>("SSCC", consume.SSCC),

										new ReadOnlyBinder<string>("DocNum", productionOrderNumber.ToString()),
										new ReadOnlyBinder<bool>("UsePickingInfo", false),

										new ReadOnlyBinder<bool>("AllowZero", false),
										new ReadOnlyBinder<double>("InitialQuantity", 1),
										new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_MAXIMUM_QUANTITY"),
										new ReadOnlyBinder<double>("StillToQuantity", consume.QuantityWaste),
										new ReadOnlyBinder<double>("MaximumQuantity", consume.QuantityWaste),
										new ReadOnlyBinder<double>("MinimumQuantity", consume.QuantityWaste),
										new ReadOnlyBinder<double>("TotalQuantityToPick", consume.QuantityWaste),
										new ReadOnlyBinder<string>("UOM", itemInfoToConsume.InventoryUom),
										new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfoToConsume != null) ? itemInfoToConsume.UomDecimals : 0),

										new ReadOnlyBinder<double>("InnerQuantity", 1),
										new ReadOnlyBinder<string>("InnerUOM", ""),
										new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
										new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.NoLink).ToString()),
										new ReadOnlyBinder<int?>("DocEntry", productionOrderDocEntry),
										new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
										new ReadOnlyBinder<string>("LocationCode", productionLineCode),

										new ReadOnlyBinder<int?>("LUID", consume.LogisticUnitIdentKey),

										new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
										new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
										new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

										);

									//call flow
									base.StartSubFlowAndWait(m_enterQuantitySubFlow, true);


									if (m_subFlowBackRequested)
									{
										goto Step_StopProductionTask;
									}

									query = BuildQuery.GetSerialNumbers(itemInfoToConsume.ItemCode, m_subFlowListOfSerialNumbers, m_sboProviderService.GetDbTool());
									DataSet dsSerialNumberCount = m_sboProviderService.RunView(true, null, null, query);

									foreach (DataRow rowSer in dsSerialNumberCount.Tables[0].Rows)
									{
										int internalKey = Convert.ToInt32(rowSer[PmxSerialNumberTable.Columns.InternalKey.NAME]);
										int luid = Convert.ToInt32(rowSer[PmxSerialNumberTable.Columns.Luid.NAME]);
										string serialNumber = Convert.ToString(rowSer[PmxSerialNumberTable.Columns.SerialNumber.NAME]);

										serialNumbers.Add(serialNumber, internalKey);
									}


									#endregion

								}

							}
							else
							{
								//go back
								goto Step_StopProductionTask;
							}
							consume.ListOfSerialNumbersWaste = serialNumbers;
						}


					}

					//It is possible that we have multiple production order lines for the same item code.
					//if that is the case, we need to create multiple consume items, linked to correct line numbers

					DataRow[] prdLineRows = dsProductOnLine.Tables[0].Select(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name + " = '" + tempItemCode + "'");



					if (prdLineRows.Length > 1)
					{
						//double qtyRemaining = consume.QuantityUsed;
						//double? qtyRemainingUom2 = consume.QuantityUom2Used;
						bool isFirst = true;
						for (int iLoop = 0; iLoop < prdLineRows.Length; iLoop++)
						{
							DataRow prdLineRow = prdLineRows[iLoop];
							int productionOrderLineNum = Convert.ToInt32(prdLineRow[PmxProductionOrderLineTable.Columns.LineNum.NAME]);

							ItemSetpointInfo setPoint = listOfLineNumQuantity[productionOrderLineNum];

							ConsumeItemInfo consumeItem = RuntimeHelper.DeepCopy(consume);


							if (!isFirst)
							{
								//Only add waste to first line
								consumeItem.QuantityUom2Waste = null;
								consumeItem.QuantityWaste = 0;
							}

							if (iLoop + 1 < prdLineRows.Length)
							{
								//This is the NOT last line, so take partial quantity
								consumeItem.QuantityUsed = Math.Min(consume.QuantityUsed, setPoint.Setpoint);

								if (consumeItem.QuantityUsed > 0)
								{
									setPoint.Setpoint -= consumeItem.QuantityUsed;
									consume.QuantityUsed -= consumeItem.QuantityUsed;
									if (itemInfoToConsume.IsCatchWeightItem)
									{
										consumeItem.QuantityUom2Used = Math.Min(consume.QuantityUom2Used.Value, PmxItemAllConnectionsProvider.CalculateWeightFromPieces(consumeItem.QuantityUsed, itemInfoToConsume).Value);
										consume.QuantityUom2Used -= consumeItem.QuantityUom2Used;
									}

									if (isSerialNumber)
									{
										//We need to remove items from the serial number list
										for (int iSerial = 0; iSerial < consumeItem.ListOfSerialNumbersUsed.Count; iSerial++)
										{
											string serial = consumeItem.ListOfSerialNumbersUsed.Keys.ToList()[iSerial];
											if (iSerial < consumeItem.QuantityUsed)
											{
												//Remove the serial number from main consume object, because we are using it in the new consume item
												consume.ListOfSerialNumbersUsed.Remove(serial);
											}
											else
											{
												//Remove the serial number from new consume item object, because we only have a partial number of serial numbers
												consumeItem.ListOfSerialNumbersUsed.Remove(serial);
											}
										}
									}
								}
							}
							else
							{
								//This is the last line, so we don't change quantity, because we want to take the remainder ...

							}

							if (consumeItem.QuantityUsed > 0)
							{
								consumeItem.LineNum = productionOrderLineNum;
								listConsumeItemsToSend.Add(consumeItem);

								isFirst = false;
							}
						}
					}
					else
					{
						if (!productionInformation.IsDisassembly)
						{
							consume.LineNum = Convert.ToInt32(prdLineRows[0][PmxProductionOrderLineTable.Columns.LineNum.NAME]);
						}
						listConsumeItemsToSend.Add(consume);
					}


				}



			}

			goto Step_CheckForMissingTimeRegistrations;

			#endregion

			Step_CheckForMissingTimeRegistrations:

			#region CHECK FOR MISSING TIME REGISTRATION

			if (stopProduction)
			{

				query = BuildQuery.GetMissingTimeRegistrations(productionOrderDocEntry.Value, m_sboProviderService.GetDbTool());
				ds = m_sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{
					foreach (DataRow row in ds.Tables[0].Rows)
					{
						m_timeRegistrationSubFlow.BindParameters(
						   this.DefaultCultureInfo,
						   this.PmxOseCompany,
						   new ReadOnlyBinder<string>("ObjectType", SboObjectTypes.ProductionOrder),
						   new ReadOnlyBinder<int>("DocEntry", productionOrderDocEntry.Value),
						   new ReadOnlyBinder<int?>("LineNum", null),
						   new ReadOnlyBinder<string>("ItemCode", row[PmxProductionOrderLineTable.Columns.ItemCode.NAME] as string),
						   new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")
						   );

						this.StartSubFlowAndWait(m_timeRegistrationSubFlow, true);

						if (m_subFlowBackRequested)
						{
							goto Step_StopProductionTask;
						}


					}
				}
			}

			goto Step_AskEndQualityValues;



			#endregion

			Step_AskEndQualityValues:

			#region ASK END QUALITY VALUES

			m_enterQualityValuesFlow_ListOfQualityValues = null;

			m_enterQualityValuesFlow.BindParameters(
			 this.DefaultCultureInfo,
			 this.PmxOseCompany,
			 new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.End),
			 new ReadOnlyBinder<string>("BaseType", SboObjectTypes.ProductionOrder),
			 new ReadOnlyBinder<int?>("BaseEntry", productionOrderDocEntry),
			 new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
			 new ReadOnlyBinder<bool>("SaveData", false),

			 new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
			 new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

			 );

			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

			if (m_subFlowBackRequested)
			{
				goto Step_StopProductionTask;
			}

			if (stepListKey.HasValue)
			{
				//PRD is for step list
				goto Step_ConsumeStepList;
			}
			else
			{
				//No step list
				goto Step_ConsumeNoStepList;
			}

			#endregion

			Step_ConsumeNoStepList:

			#region CONSUME NO STEP LIST

			listOfCreatedIssues = new Collection<int>();

			if (listConsumeItemsToSend != null &&
				listConsumeItemsToSend.Count > 0)
			{


				//consume stock
				try
				{

					m_sboProviderService.InvokeMethodWithDbConnection<int?>(true, false, null, null,
					delegate (PmxDbConnection conn, object[] parameters)
					{

						PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
						PmxProductionOrderProvider.CreatedIssues issues = new PmxProductionOrderProvider.CreatedIssues();

						issues = prdProv.Consume(productionOrderDocEntry.Value, listConsumeItemsToSend, false);
						if (issues.IssueUsedMaterialDocEntry.HasValue)
						{
							listOfCreatedIssues.Add(Convert.ToInt32(issues.IssueUsedMaterialDocEntry));
						}
						if (issues.IssueWastedMaterialDocEntry.HasValue)
						{
							listOfCreatedIssues.Add(Convert.ToInt32(issues.IssueWastedMaterialDocEntry));
						}
						if (issues.IssueTimeRegistrationDocEntry.HasValue)
						{
							listOfCreatedIssues.Add(Convert.ToInt32(issues.IssueTimeRegistrationDocEntry));
						}

						PmxQualityValueProvider qvProv = new PmxQualityValueProvider(conn);

						if (m_enterQualityValuesFlow_ListOfQualityValues != null)
						{
							foreach (QualityValueInfo qvInfo in m_enterQualityValuesFlow_ListOfQualityValues)
							{
								using (PmxQualityValue qv = qvProv.GetNewBO())
								{
									qv.BaseEntry = productionOrderDocEntry.Value;
									qv.BaseType = SboObjectTypes.ProductionOrder;
									qv.MomentOfCapture = qvInfo.MomentOfCapture;
									qv.QualityTypeCode = qvInfo.QualityTypeCode;
									qv.QualityValue = qvInfo.QualityValue;
									qv.ValueConvertor = qvInfo.ValueConvertor;

									qvProv.AddBO(qv);
								}
							}
						}

						return null;
					});

				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					if (ex is ProdumexException)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"PmxError", ((ProdumexException)ex).Error,
								"ShowButton", true));
						WaitForMessage();
					}
					else
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
								"NoTranslationOfMessageKey", true,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();
					}

					//ask user if he wants to try again after error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
							"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_ConsumeNoStepList;
					}
					goto Step_ShowProductionTasks;

				}
			}


			goto Step_UpdateProductionStatus;


			#endregion

			Step_UpdateProductionStatus:

			#region UPDATE PRODUCTION STATUS

			try
			{

				if (buttonKeyOtherTasks == "MSG_BUTTON_NEXT_PRODUCTION_BATCH")
				{

					m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters)
						{
							PmxProductionOrderProvider prov = new PmxProductionOrderProvider((SboConnection)conn);
							prov.IncreaseCurrentProductionBatch(productionOrderDocEntry);
							return null;
						});

					goto Step_MoveStockToRest;
				}
				else
				{
					if (stopProduction)
					{
						// HOOK
						m_subFlowBackRequested = false;
						m_subFlowProductionOrderDocEntry = productionOrderDocEntry;

						m_beforeProductionOrderClosedHookFlow.BindParameters(
							this.DefaultCultureInfo,
							this.PmxOseCompany,
							new ReadOnlyBinder<int?>("ProductionOrderDocEntry", this, "m_subFlowProductionOrderDocEntry"),
							new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")
						);
						try
						{
							//call flow
							this.StartSubFlowAndWait(m_beforeProductionOrderClosedHookFlow, true);

							if (m_subFlowBackRequested)
							{
								goto Step_ShowProductionTasks;
							}
						}
						catch (Exception ex)
						{
							s_log.Error("", ex);
							if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
							{
								// Inform the user with the error
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
										"NoTranslationOfMessageKey", true,
										"IsError", true,
										"ShowButton", true));
								WaitForMessage();

							}
							else
							{
								// Inform the user with the error
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"PmxError", ((ProdumexException)ex).Error,
										"ShowButton", true));
								WaitForMessage();
							}
						}
					}


					m_sboProviderService.InvokeMethodWithDbConnection<int?>(true, false, null, null,
					delegate (PmxDbConnection conn, object[] parameters)
					{

						PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);

						if (stopProduction)
						{
							prdProv.CloseOrderAndConsumeTimeRegistration(productionOrderDocEntry.Value);
						}
						else
						{
							//on hold
							prdProv.PutOnHold(productionOrderDocEntry.Value);
						}

						return null;
					});
				}

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_UpdateProductionStatus;
				}
				goto Step_ShowProductionTasks;

			}

			//release lined up locations
			try
			{

				m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
				   delegate (PmxDbConnection conn, object[] parameters)
				   {

					   PmxLinedUpLocationProvider prov = new PmxLinedUpLocationProvider(conn);
					   prov.DeleteAllLinedUpLocationsForProductionLine(productionLineCode, productionOrderDocEntry.Value);

					   return null;
				   });

			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

			}


			justStoppedPRD = true;
			goto Step_MoveStockToRest;


			#endregion

			Step_StartConsumeForStepList:

			#region START CONSUME FOR STEP LIST

			//Get steplist items
			stepListInputAndOutputForOptional = m_sboProviderService.InvokeMethodWithDbConnection<ProductionOrderStepListInputAndOutput>(true, false, null, null,
			 delegate (PmxDbConnection conn, object[] parameters)
			 {
				 PmxProductionOrderStepListProvider prov = new PmxProductionOrderStepListProvider(conn);
				 return prov.GetProductionInputAndOutputForStepList(stepListKey.Value, 1);
			 });

			listIntermediateItemsToProduce = new PmxDictionary<string, double>();
			listOptionalItemsToConsume = new PmxDictionary<string, double>();

			if (showStepsForStepListProduction)
			{
				foreach (ProductionOrderStepListInput currentOptionalInput in stepListInputAndOutputForOptional.InputList)
				{
					if (currentOptionalInput.Optional)
					{
						itemInfoToConsume = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), currentOptionalInput.ItemCode);


						Step_EnterIntermediateItem_Hook:


						double intermediateQuantity = 0;


						//Enter the quantity of remaining optional items

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
						  DefaultCultureInfo.Get(), BuildParamCollection(
							  "InitialErrorKey", initialErrorKey,
							  "TitleKey", "MSG_TITLE_ENTER_QUANTITY_INTERMEDIATE_ITEM",
							  "InitialQuantity", 0,
							  "AllowZero", true,
							  "NumberOfDigitsAfterDecimal", itemInfoToConsume.UomDecimals,
							  "MaximumQuantity", -1,
							  "UOM", itemInfoToConsume.InventoryUom,
							  "DocNum", productionOrderNumber.ToString(),
							  "ItemDescription", itemInfoToConsume.Description,
							  "ItemCode", itemInfoToConsume.ItemCode,
							  "ItemBarcode", itemInfoToConsume.CodeBars,
							  "SetFocusToQuantityControl", setFocusToQuantityControl
							  ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantityEntered"))
						{
							intermediateQuantity = ExtractParameter<double>(msg.Parameters, "quantity");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goto Step_ShowProductionTasks;
						}


						if (intermediateQuantity > 0)
						{
							//add item to produce
							listIntermediateItemsToProduce.Add(currentOptionalInput.ItemCode, intermediateQuantity);
						}



						//get the products on the line
						query = BuildQuery.GetProductsOnLine(productionOrderDocEntry.Value, productionLineCode, currentOptionalInput.ItemCode, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, m_sboProviderService.GetDbTool());
						DataSet dsOptionalProductOnLine = m_sboProviderService.RunView(false, null, null, query);

						double quantityOnLineForOptionalItem = 0;
						if (this.GetNumberOfRows(dsOptionalProductOnLine) > 0)
						{
							foreach (DataRow row in dsOptionalProductOnLine.Tables[0].Rows)
							{
								quantityOnLineForOptionalItem += Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]);
							}
						}


						double optionalQuantityToConsume = 0;

						if (quantityOnLineForOptionalItem > 0)
						{
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
							  DefaultCultureInfo.Get(), BuildParamCollection(
								  "InitialErrorKey", initialErrorKey,
								  "TitleKey", "MSG_TITLE_ENTER_OPTIONAL_CONSUME_QUANTITY",
								  "InitialQuantity", 0,
								  "AllowZero", true,
								  "NumberOfDigitsAfterDecimal", itemInfoToConsume.UomDecimals,
								  "MaximumQuantity", quantityOnLineForOptionalItem,
								  "UOM", itemInfoToConsume.InventoryUom,
								  "DocNum", productionOrderNumber.ToString(),
								  "ItemDescription", itemInfoToConsume.Description,
								  "ItemCode", itemInfoToConsume.ItemCode,
								  "ItemBarcode", itemInfoToConsume.CodeBars,
								  "SetFocusToQuantityControl", setFocusToQuantityControl
								  ));
							msg = WaitForMessage();

							// reset error
							initialErrorKey = null;

							if (msg.Name.EndsWith(".QuantityEntered"))
							{
								optionalQuantityToConsume = ExtractParameter<double>(msg.Parameters, "quantity");
							}
							else if (msg.Name.EndsWith(".BackRequested"))
							{
								goto Step_EnterIntermediateItem_Hook;
							}

							if (optionalQuantityToConsume > 0)
							{
								//Add to list
								listOptionalItemsToConsume.Add(currentOptionalInput.ItemCode, optionalQuantityToConsume);
							}



						}








					}
				}//foreach
			}
			goto Step_InputConsumeForStepList;


			#endregion

			Step_InputConsumeForStepList:

			#region INPUT CONSUME FOR STEP LIST


			//get list for consumption
			stepListInputAndOutputForConsumption = m_sboProviderService.InvokeMethodWithDbConnection<ProductionOrderStepListInputAndOutput>(true, false, null, null,
			 delegate (PmxDbConnection conn, object[] parameters)
			 {
				 PmxProductionOrderStepListProvider prov = new PmxProductionOrderStepListProvider(conn);
				 return prov.GetProductionInputAndOutputForConsumation(stepListKey.Value, quantityForRun, listIntermediateItemsToProduce, listOptionalItemsToConsume);
			 });


			listConsumeItemsToSend = new Collection<ConsumeItemInfo>();

			foreach (ProductionOrderStepListInput input in stepListInputAndOutputForConsumption.InputList)
			{

				KeyValuePair<string, ItemSetpointInfo> currentToConsumeItemCodeQuantity = new KeyValuePair<string, ItemSetpointInfo>(input.ItemCode, new ItemSetpointInfo(input.Quantity));

				//get the products on the line
				query = BuildQuery.GetProductsOnLine(productionOrderDocEntry.Value, productionLineCode, input.ItemCode, PmxOseCompany.Get().LogisticCarriersStoredOnOneStorageLocation ? whsInfo.StorageLocationCodeLogisticCarriers : null, m_sboProviderService.GetDbTool());
				DataSet dsActualProductOnLine = m_sboProviderService.RunView(false, null, null, query);

				bool skipConsumeScreen = true;
				//set the total quantity
				ItemSetpointInfo totalToConsumeForItem = currentToConsumeItemCodeQuantity.Value;

				if (totalToConsumeForItem.Setpoint > 0)
				{
					skipConsumeScreen = false;
				}

				//calculate the used, and rest quantity
				if (this.GetNumberOfRows(dsActualProductOnLine) > 0)
				{
					double quantityToBeUsed = currentToConsumeItemCodeQuantity.Value.Setpoint;

					foreach (DataRow row in dsActualProductOnLine.Tables[0].Rows)
					{
						double quantityOnLine = 0;
						quantityOnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name]);

						double quantityUom2OnLine = Convertor.ConvertToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name]);


						if (quantityOnLine > 0)
						{
							skipConsumeScreen = false;
						}

						double currentQuantity = Math.Min(quantityOnLine, quantityToBeUsed);
						row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name] = currentQuantity;

						quantityToBeUsed -= currentQuantity;

						//set the waste
						row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name] = quantityOnLine - currentQuantity;


						if (input.ItemInfo.IsCatchWeightItem)
						{
							double? quantityUom2ToUse = 0;
							if (quantityOnLine == currentQuantity)
							{
								quantityUom2ToUse = quantityUom2OnLine;
							}
							else
							{
								//Use default ratio
								quantityUom2ToUse = PmxItemAllConnectionsProvider.CalculateWeightFromPieces(currentQuantity, input.ItemInfo);
							}

							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name] = quantityUom2ToUse;
							row[ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name] = quantityUom2OnLine - quantityUom2ToUse;


						}


						if (quantityToBeUsed <= 0)
						{
							break;
						}
					}

				}

				if (!skipConsumeScreen)
				{
					//show screen
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Production.IEnterConsumeItemsScreen),
					   DefaultCultureInfo.Get(), BuildParamCollection(
					   "InitialErrorKey", initialErrorKey,
					   "TitleKey", "MSG_TITLE_ENTER_CONSUMED_QUANTITY",
					   "DSConsumeItems", dsActualProductOnLine,
					   "TotalQuantityToConsumeLowerLimit", totalToConsumeForItem.LowerLimit,
					   "TotalQuantityToConsume", totalToConsumeForItem.Setpoint,
					   "TotalQuantityToConsumeUpperLimit", totalToConsumeForItem.UpperLimit,
					   "NumbersOfDigitsAfterDecimal", input.ItemInfo.UomDecimals,
					   "UseWaste", useWaste,
					   "IsCatchWeight", input.ItemInfo.IsCatchWeightItem,
					   "NumbersOfDigitsAfterDecimalUom2", input.ItemInfo.Uom2Decimals,
					   "TotalQuantityUom2ToConsume", PmxItemAllConnectionsProvider.CalculateWeightFromPieces(totalToConsumeForItem.Setpoint, input.ItemInfo),
					   "WeightByPiece", input.ItemInfo.DefaultWeightForPiece

					   ));
					msg = WaitForMessage();
					initialErrorKey = null;
					if (msg.Name.EndsWith(".DataEntered"))
					{
						DataSet dsConsumedItems = ExtractParameter<DataSet>(msg.Parameters, "dsConsumedItems");

						foreach (DataRow row in dsConsumedItems.Tables[0].Rows)
						{
							ConsumeItemInfo consume = new ConsumeItemInfo();

							consume.BatchNumber1 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name] as string;
							consume.BatchNumber2 = row[ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name] as string;
							if (row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name] is DateTime)
							{
								consume.BestBeforeDate = Convert.ToDateTime(row[ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name]);
							}
							consume.ItemCode = row[ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name] as string;
							consume.ItemDescription = row[ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name] as string;
							consume.LocationCode = row[ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name] as string;
							consume.LogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[ConsumeItemsForProductionDataSet.Columns.LUID.Name]);
							consume.QualityStatusCode = row[ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name] as string;
							consume.QuantityUsed = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name]);
							consume.QuantityWaste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name]);
							consume.SSCC = row[ConsumeItemsForProductionDataSet.Columns.SSCC.Name] as string;

							if (input.ItemInfo.IsCatchWeightItem)
							{
								consume.QuantityUom2Used = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name]);
								consume.QuantityUom2Waste = Convert.ToDouble(row[ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name]);
							}

							listConsumeItemsToSend.Add(consume);
						}

					}
					else
					{
						//go back
						goto Step_ShowProductionTasks;
					}
				}


			}//foreach

			goto Step_OutputConsumeForStepList;

			#endregion

			Step_OutputConsumeForStepList:

			#region OUTPUT CONSUME FOR STEP LIST

			if (showStepsForStepListProduction)
			{
				foreach (ProductionOrderStepListOutput output in stepListInputAndOutputForConsumption.OutputList)
				{

					PmxItemInfo itemInfoOutput = m_sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), output.ItemCode);


					double outputQuantity = 0;

					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
							   DefaultCultureInfo.Get(), BuildParamCollection(
								   "InitialErrorKey", initialErrorKey,
								   "TitleKey", "MSG_TITLE_ENTER_QUANTITY_OUTPUT",
								   "InitialQuantity", 0, //output.Quantity,
								   "AllowZero", true,
								   "NumberOfDigitsAfterDecimal", itemInfoOutput.UomDecimals,
								   "MaximumQuantity", output.Quantity,
								   "UOM", itemInfoOutput.InventoryUom,
								   "DocNum", productionOrderNumber.ToString(),
								   "ItemDescription", itemInfoOutput.Description,
								   "ItemCode", itemInfoOutput.ItemCode,
								   "ItemBarcode", itemInfoOutput.CodeBars,
								   "SetFocusToQuantityControl", setFocusToQuantityControl
								   ));
					msg = WaitForMessage();

					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".QuantityEntered"))
					{
						outputQuantity = ExtractParameter<double>(msg.Parameters, "quantity");
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						goto Step_ShowProductionTasks;
					}

					output.Quantity = outputQuantity;
				}//foreach
			}
			//remove empty ones
			for (int counterOutput = 0; counterOutput > stepListInputAndOutputForConsumption.OutputList.Count; counterOutput++)
			{
				ProductionOrderStepListOutput currentOutput = stepListInputAndOutputForConsumption.OutputList[counterOutput];
				if (currentOutput.Quantity <= 0)
				{
					stepListInputAndOutputForConsumption.OutputList.Remove(currentOutput);
					counterOutput--;
				}
			}

			goto Step_CheckForMissingTimeRegistrations;

			//goto Step_ConsumeStepList;

			#endregion

			Step_ConsumeStepList:

			#region CONSUME STEP LIST

			listOfCreatedIssues = new Collection<int>();
			if (listConsumeItemsToSend != null &&
				listConsumeItemsToSend.Count > 0)
			{
				//consume stock
				try
				{
					m_sboProviderService.InvokeMethodWithDbConnection<Collection<int>>(true, false, null, null,
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxQualityValueProvider qvProv = new PmxQualityValueProvider(conn);

						if (m_enterQualityValuesFlow_ListOfQualityValues != null)
						{
							foreach (QualityValueInfo qvInfo in m_enterQualityValuesFlow_ListOfQualityValues)
							{
								using (PmxQualityValue qv = qvProv.GetNewBO())
								{
									qv.BaseEntry = productionOrderDocEntry.Value;
									qv.BaseType = SboObjectTypes.ProductionOrder;
									qv.MomentOfCapture = qvInfo.MomentOfCapture;
									qv.QualityTypeCode = qvInfo.QualityTypeCode;
									qv.QualityValue = qvInfo.QualityValue;
									qv.ValueConvertor = qvInfo.ValueConvertor;

									qvProv.AddBO(qv);
								}
							}
						}

						PmxProductionOrderProvider prdProv = new PmxProductionOrderProvider((SboConnection)conn);
						listOfCreatedIssues = prdProv.ConsumeAndProduceStepList(productionOrderDocEntry.Value, listConsumeItemsToSend, listIntermediateItemsToProduce, stepListInputAndOutputForConsumption.OutputList, outputLocationCode, whsInfo.PmxWarehouseCode);
						return listOfCreatedIssues;
					});

				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					if (ex is ProdumexException)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"PmxError", ((ProdumexException)ex).Error,
								"ShowButton", true));
						WaitForMessage();
					}
					else
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
								"NoTranslationOfMessageKey", true,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();
					}

					//ask user if he wants to try again after error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
							"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_ConsumeNoStepList;
					}
					goto Step_ShowProductionTasks;

				}
			}

			goto Step_UpdateProductionStatus;

			#endregion

			Step_TimeRegistration:

			#region TimeRegistration


			m_timeRegistrationSubFlow.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadOnlyBinder<string>("ObjectType", SboObjectTypes.ProductionOrder),
				new ReadOnlyBinder<int>("DocEntry", productionOrderDocEntry.Value),
				new ReadOnlyBinder<int?>("LineNum", null),
				new ReadOnlyBinder<string>("ItemCode", null),
				new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")
				);

			this.StartSubFlowAndWait(m_timeRegistrationSubFlow, true);

			goto Step_ShowProductionTasks;


			#endregion

		}

		#endregion

		#region CLASSES




		/// <summary>
		/// 
		/// </summary>
		public class ItemSetpointInfo
		{
			private double m_lowerLimit;
			private double m_upperLimit;
			private double m_setPoint;

			/// <summary>
			/// Gets or sets the lower limit.
			/// </summary>
			/// <value>The lower limit.</value>
			public double LowerLimit {
				get {
					return m_lowerLimit;
				}
				set {
					m_lowerLimit = value;
				}
			}

			/// <summary>
			/// Gets or sets the setpoint.
			/// </summary>
			/// <value>The setpoint.</value>
			public double Setpoint {
				get {
					return m_setPoint;
				}
				set {
					m_setPoint = value;
				}
			}

			/// <summary>
			/// Gets or sets the upper limit.
			/// </summary>
			/// <value>The upper limit.</value>
			public double UpperLimit {
				get {
					return m_upperLimit;
				}
				set {
					m_upperLimit = value;
				}
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="ItemSetpointInfo"/> class.
			/// </summary>
			/// <param name="lowerLimit">The lower limit.</param>
			/// <param name="setpoint">The setpoint.</param>
			/// <param name="upperLimit">The upper limit.</param>
			public ItemSetpointInfo(double lowerLimit, double setpoint, double upperLimit)
			{
				LowerLimit = lowerLimit;
				Setpoint = setpoint;
				UpperLimit = upperLimit;
			}

			/// <summary>
			/// Initializes a new instance of the <see cref="ItemSetpointInfo"/> class.
			/// </summary>
			/// <param name="setpoint">The setpoint.</param>
			public ItemSetpointInfo(double setpoint)
			{
				LowerLimit = setpoint;
				Setpoint = setpoint;
				UpperLimit = setpoint;
			}
		}


		#endregion

		#region HELPERS



		private double GetDefaultQuantityForProduction(PmxItemInfo itemInfo)
		{
			double defaultQuantity = itemInfo.DefaultProductionQuantity;
			if (defaultQuantity == 0)
			{
				defaultQuantity = itemInfo.DefaultQuantityOnLogisticUnit;
			}
			return defaultQuantity;
		}


		private static bool GetHideDuringEnteringQuantity(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						return packType.HideDuringEnteringQuantity;
					}
				}
			}

			return true;
		}

		private static int? ExecuteGetCreateItemTransactionalInfoKey(PmxDbConnection conn, object[] parameters)
		{
			PmxItemTransactionalInfoProvider prov = new PmxItemTransactionalInfoProvider(conn);
			string itemCode = (string)parameters[0];
			string batchNumber = (string)parameters[1];
			string batchNumber2 = (string)parameters[2];
			DateTime? bbd = (DateTime?)parameters[3];

			return prov.GetCreateItemTransactionalInfoKey(itemCode, batchNumber, batchNumber2, bbd);
		}

		/// <summary>
		/// Executes the get PMX warehouse code.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null && ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}
			return 0;
		}

		private static string GetPackTypeName(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						return packType.PackagingTypeName;
					}
				}
			}

			return null;
		}

		private static Collection<string> GetPackBarcode(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						return packType.Barcodes;
					}
				}
			}

			return null;
		}

		/// <summary>
		/// Sets the item transactional packaging type info list.
		/// </summary>
		/// <param name="itemInfo">The item info.</param>
		/// <returns></returns>
		private Collection<ItemTransactionalPackagingTypeInfo> SetItemTransactionalPackagingTypeInfoList(PmxItemInfo itemInfo)
		{
			//get the packaging types qty from item
			Collection<ItemTransactionalPackagingTypeInfo> listOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

			foreach (PmxItemPackagingTypeInfo packInfo in itemInfo.ListOfPackagingTypes)
			{
				ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

				newPackInfo.PackagingTypeCode = packInfo.PackagingTypeCode;
				newPackInfo.Quantity = packInfo.Quantity;
				newPackInfo.ItemCode = itemInfo.ItemCode;

				listOfTransactionalPackagingTypeInfos.Add(newPackInfo);

			}

			return listOfTransactionalPackagingTypeInfos;
		}

		/// <summary>
		/// Executes the generate put away production.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static object ExecuteGeneratePutAwayProduction(PmxDbConnection conn, object[] parameters)
		{
			PmxMoveOrderProvider prov = new PmxMoveOrderProvider(conn);
			PmxPriorityProvider prioProv = new PmxPriorityProvider(conn);

			prov.GeneratePutAwayProduction((int)parameters[0], prioProv.GetDefaultPriority().Sequence, null, true, (string)parameters[1]);


			return null;
		}
		/// <summary>
		/// Executes the get allowed quality statuses for production.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static Collection<string> ExecuteGetAllowedQualityStatusesForProduction(PmxDbConnection conn, object[] parameters)
		{
			PmxQualityStatusProvider qsProv = new PmxQualityStatusProvider(conn);
			return qsProv.GetQualityStatusesCanBeUsedForProductionAsCollection();
		}

		/// <summary>
		/// Consume the given items.
		/// </summary>
		/// <param name="conn">The SboConnection to use.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteConsume(SboConnection conn, object[] parameters)
		{
			int productionOrderDocEntry = (int)parameters[0];
			Collection<ConsumeItemInfo> consumeItemInfos = (Collection<ConsumeItemInfo>)parameters[1];
			bool optionalComponent = (bool)parameters[2];

			Produmex.Sbo.Logex.Data.Providers.PmxProductionOrderProvider prov = new Produmex.Sbo.Logex.Data.Providers.PmxProductionOrderProvider(conn);
			prov.Consume(productionOrderDocEntry, consumeItemInfos, false);

			//if (!optionalComponent)
			//{
			//	// If this component is not optional then remove the link of prepared component
			//	foreach (ConsumeItemInfo itemInfo in consumeItemInfos)
			//	{
			//		string query = BuildQuery.DeletePreparedComponent(productionOrderDocEntry, (int)itemInfo.LogisticUnitIdentKey);
			//		SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);
			//	}
			//}

			return true;
		}
		/// <summary>
		/// Executes the close production order.
		/// </summary>
		/// <param name="conn">The SboConnection to use.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static object ExecuteCloseProductionOrder(SboConnection conn, object[] parameters)
		{
			int productionOrderDocEntry = (int)parameters[0];

			PmxProductionOrderProvider prov = new PmxProductionOrderProvider(conn);
			prov.CloseOrder(productionOrderDocEntry);
			return null;
		}



		/// <summary>
		/// Gets the new SSCC data set.
		/// </summary>
		/// <returns></returns>
		private DataSet GetNewSSCCDataSet()
		{
			DataSet ds = new DataSet();
			ds.Tables.Add(SSCCDataSet.NAME);
			ds.Tables[0].Columns.Add(SSCCDataSet.Columns.LUID.Name, SSCCDataSet.Columns.LUID.Types[1]);
			ds.Tables[0].Columns.Add(SSCCDataSet.Columns.SSCC.Name, SSCCDataSet.Columns.SSCC.Types[0]);
			ds.Tables[0].Columns.Add(SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Types[0]);
			ds.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Types[0]);
			ds.Tables[0].Columns.Add(SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Types[0]);

			return ds;

		}


		#endregion

		#region QUERY

		/// <summary>
		/// Static class grouping our native queries
		/// </summary>
		private class BuildQuery
		{
			/// <summary>
			/// Gets the serial numbers
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="listOfSerialNumbers">The list of serial numbers.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetSerialNumbers(string itemCode, Collection<string> listOfSerialNumbers, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxSerialNumberTable.NAME, PmxSerialNumberTable.Columns.InternalKey.NAME);
				qryBuilder.Append(", ");

				qryBuilder.Flags(QBF.Push | QBF.Translate);

				qryBuilder.AppendFormat("IFNULL( {0}, -1) ",
					SqlCommandHelper.EscapeTableColumnName(PmxSerialNumberTable.NAME, PmxSerialNumberTable.Columns.Luid.NAME));
				qryBuilder.AppendFormat(" AS {0} ",
					SqlCommandHelper.EscapeColumnName(PmxSerialNumberTable.Columns.Luid.NAME));

				qryBuilder.Flags(QBF.Pop);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxSerialNumberTable.NAME, PmxSerialNumberTable.Columns.SerialNumber.NAME);


				qryBuilder.Append("FROM ");
				SqlCommandHelper.AddTableName(qryBuilder, PmxSerialNumberTable.NAME);


				qryBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxSerialNumberTable.NAME, PmxSerialNumberTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxSerialNumberTable.NAME, PmxSerialNumberTable.Columns.SerialNumber.NAME);
				qryBuilder.Append(" IN ( ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(listOfSerialNumbers, CultureInfo.InvariantCulture));
				qryBuilder.Append(" ) ");

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the products from stock linked to production order.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="locationCode">The location code.</param>
			/// <param name="restLocationCode">The rest location code.</param>
			/// <returns></returns>
			public static string GetProductsFromStockLinkedToProductionOrder(int productionOrderDocEntry, string locationCode, string restLocationCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);


				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				//inner join Product - inventory
				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(" ON ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));

				//inner join Product - production order
				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
				qryBuilder.Append(" ON ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.Columns.ItemCode.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));


				qryBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				string operand = " AND ";
				if (!string.IsNullOrEmpty(locationCode))
				{
					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
					if (string.IsNullOrEmpty(restLocationCode))
					{
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
					}
					else
					{
						//input and rest location are allowed
						qryBuilder.Append(" IN ( ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(restLocationCode, CultureInfo.InvariantCulture));
						qryBuilder.Append(" ) ");
					}
					operand = " AND ";
				}


				qryBuilder.Append(operand);
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" ORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the production order lines.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetProductionOrderLines(int? productionOrderDocEntry, string itemCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

				qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
				   SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
				   SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);

				qryBuilder.AppendFormat(", ( {0} / {1} ) AS {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.BASEQUANTITY.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name);


				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);



				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);


				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" <> -1 ");

				qryBuilder.Append("\r\n WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.AppendFormat(" AND {0}({1}, {2}) = {2} ",
					qryBuilder.IFNULL,
					SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemType.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionItemTypeColumnValues.Item.VALUE, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(itemCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				}
				else
				{
					qryBuilder.AppendFormat(" AND {0} >= 0",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME));
				}

				qryBuilder.AppendFormat("\r\n ORDER BY {0} DESC",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name));

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the products on the line.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="locationCode">The location code.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="logisticCarrierLocationCode">The logistic carrier location code.</param>
			/// <returns></returns>
			public static string GetProductsOnLine(int? productionOrderDocEntry, string locationCode, string itemCode, string logisticCarrierLocationCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

				qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(", 0) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name));


				qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
				qryBuilder.Append(", 0) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));

				qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
				   SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
				   SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

				qryBuilder.AppendFormat(", ( {0} / {1} ) AS {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.BASEQUANTITY.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);

				qryBuilder.AppendFormat(", CASE WHEN {0} IS NULL THEN 0 ELSE 1 END AS {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.InternalKey.NAME),
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.IS_PREPARED.Name));

				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);



				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);


				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" <> -1 ");

				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

				qryBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxProductionOrderPreparedComponentTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

				qryBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME));
				qryBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
					SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
					SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.Code.NAME));
				qryBuilder.AppendFormat(" AND {0}.{1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.CanBeUsedForProduction.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				qryBuilder.Append("\r\n WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
				qryBuilder.Append(" > 0 ");

				if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				}

				if (!string.IsNullOrEmpty(itemCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				}


				if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
				{
					#region UNION

					qryBuilder.Append("\r\n UNION ");

					qryBuilder.Append(" SELECT ");

					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name);

					qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
					qryBuilder.Append(", 0) AS ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));

					qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
							  SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
							  SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
							  SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);


					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

					qryBuilder.AppendFormat(", ( {0} / {1} ) AS {2} ",
							   SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
							   SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
							   SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);

					qryBuilder.AppendFormat(", CASE WHEN {0} IS NULL THEN 0 ELSE 1 END AS {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.InternalKey.NAME),
						SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.IS_PREPARED.Name));

					qryBuilder.Append("\r\n FROM ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

					qryBuilder.Append("\r\n INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(qryBuilder.WITH_NOLOCK);
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);


					qryBuilder.Append("\r\n INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);


					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" <> -1 ");


					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(logisticCarrierLocationCode, CultureInfo.InvariantCulture));

					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

					qryBuilder.AppendFormat("\r\n LEFT JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxProductionOrderPreparedComponentTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.ProdOrderDocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderPreparedComponentTable.NAME, PmxProductionOrderPreparedComponentTable.Columns.LogUnitIdentKey.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));


					qryBuilder.Append("\r\n WHERE ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
					qryBuilder.Append(" > 0 ");


					if (!string.IsNullOrEmpty(itemCode))
					{
						qryBuilder.Append(" AND ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

					}

					#endregion
				}


				qryBuilder.AppendFormat("\r\n ORDER BY {0} DESC",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name));
				qryBuilder.AppendFormat(", {0} DESC",
					SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.IS_PREPARED.Name));

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the missing time registrations.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <returns></returns>
			public static string GetMissingTimeRegistrations(int? productionOrderDocEntry, DbTool dbTool)
			{
				return Produmex.Sbo.Logex.Data.SboCommands.GetMissingTimeRegistrationsForProduction(productionOrderDocEntry, dbTool);
			}

			/// <summary>
			/// Gets the time registrations.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <returns></returns>
			public static string GetTimeRegistrations(int productionOrderDocEntry, DbTool dbTool)
			{

				/*
select itemCode, SUM((PMX_TRHE.hours - ISNULL(PMX_TRDT.hours ,0)) * numberOfoperators)  AS hours from pmx_trhe

left join
(
SELECT SUM(Hours) AS Hours, docentry FROM PMX_TRDT group by docentry
) AS PMX_TRDT
ON PMX_TRDT.DocEntry  = pmx_trhe.docEntry

where basetype = 202
AND Baseentry = 93
group by itemcode*/


				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.ItemCode.NAME, PmxTimeRegistrationTable.Columns.ItemCode.NAME);

				qryBuilder.AppendFormat(", SUM(({0} - " + qryBuilder.IFNULL + "({1},0)) * {2}) AS {3} ",
					SqlCommandHelper.EscapeTableColumnName(PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.Hours.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxTimeRegistrationDownTimeTable.NAME, PmxTimeRegistrationDownTimeTable.Columns.Hours.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.NumberOfOperators.NAME),
					SqlCommandHelper.EscapeColumnName(PmxTimeRegistrationTable.Columns.Hours.NAME)
					);

				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxTimeRegistrationTable.NAME));

				qryBuilder.Append("\r\n LEFT JOIN (");
				{
					qryBuilder.Append("SELECT ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxTimeRegistrationDownTimeTable.NAME, PmxTimeRegistrationDownTimeTable.Columns.DocEntry.NAME, PmxTimeRegistrationDownTimeTable.Columns.DocEntry.NAME);

					qryBuilder.AppendFormat(", SUM({0}) AS {1} ",
						SqlCommandHelper.EscapeTableColumnName(PmxTimeRegistrationDownTimeTable.NAME, PmxTimeRegistrationDownTimeTable.Columns.Hours.NAME),
						SqlCommandHelper.EscapeColumnName(PmxTimeRegistrationDownTimeTable.Columns.Hours.NAME)
						);

					qryBuilder.Append("\r\n FROM ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxTimeRegistrationDownTimeTable.NAME));

					qryBuilder.Append(" GROUP BY ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationDownTimeTable.NAME, PmxTimeRegistrationDownTimeTable.Columns.DocEntry.NAME);

				}
				qryBuilder.AppendFormat(") AS {0} ",
					SqlCommandHelper.EscapeTableName(PmxTimeRegistrationDownTimeTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationDownTimeTable.NAME, PmxTimeRegistrationDownTimeTable.Columns.DocEntry.NAME);


				qryBuilder.Append("\r\n WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.ObjType.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.ProductionOrder, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n GROUP BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxTimeRegistrationTable.NAME, PmxTimeRegistrationTable.Columns.ItemCode.NAME);


				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the production order lines disassembly.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetProductionOrderLinesDisassembly(int? productionOrderDocEntry, string itemCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);


				qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
				   SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
				   SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);


				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddStringValueColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name, SboBooleanColumnValues.N.VALUE);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name, 0);
				qryBuilder.Append("\r\nFROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

				qryBuilder.Append("\r\nINNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);


				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" <> -1 ");

				qryBuilder.Append("\r\nWHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));


				if (!string.IsNullOrEmpty(itemCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				}

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the products on the line for disassembly.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="locationCode">The location code.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="logisticCarrierLocationCode">The logistic carrier location code.</param>
			/// <returns></returns>
			public static string GetProductsOnLineDisassembly(int? productionOrderDocEntry, string locationCode, string itemCode, string logisticCarrierLocationCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

				qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(", 0) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name));


				qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
				qryBuilder.Append(", 0) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));

				qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
				   SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
				   SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);



				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddStringValueColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name, SboBooleanColumnValues.N.VALUE);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name, 0);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);

				qryBuilder.Append("\r\nFROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

				qryBuilder.Append("\r\nLEFT OUTER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

				qryBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME));
				qryBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
					SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME),
					SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.Code.NAME));
				qryBuilder.AppendFormat(" AND {0}.{1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.CanBeUsedForProduction.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\nINNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);


				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
				qryBuilder.Append(" <> -1 ");

				qryBuilder.Append("\r\nLEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


				qryBuilder.Append("\r\nWHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				}

				if (!string.IsNullOrEmpty(itemCode))
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				}


				if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
				{
					#region UNION
					qryBuilder.Append("\r\nUNION ");

					qryBuilder.Append(" SELECT ");

					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ConsumeItemsForProductionDataSet.Columns.BATCHNUMBER2.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, ConsumeItemsForProductionDataSet.Columns.BESTBEFOREDATE.Name));

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ConsumeItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, ConsumeItemsForProductionDataSet.Columns.ISLOGISTICCARRIER.Name);

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, ConsumeItemsForProductionDataSet.Columns.LUID.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, ConsumeItemsForProductionDataSet.Columns.QUALITYSTATUSCODE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, ConsumeItemsForProductionDataSet.Columns.QUANTITY.Name);

					qryBuilder.Append(", ", qryBuilder.IFNULL, "( ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
					qryBuilder.Append(", 0) AS ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.QUANTITY_UOM2.Name));

					qryBuilder.AppendFormat(", " + qryBuilder.IFNULL + "({0}, {1}) AS {2} ",
							  SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
							  SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME),
							  SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.UOM.Name));



					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM2.NAME, ConsumeItemsForProductionDataSet.Columns.UOM2.Name);



					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, ConsumeItemsForProductionDataSet.Columns.LOCATIONCODE.Name);

					qryBuilder.Append(", ");
					qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, ConsumeItemsForProductionDataSet.Columns.SSCC.Name));

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddStringValueColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name, SboBooleanColumnValues.N.VALUE);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleQuantityColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QTYTOLPERCENTAGE.Name, 0);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.InternalKey.NAME, ConsumeItemsForProductionDataSet.Columns.INTERNALKEY.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED.Name);


					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYREST_UOM2.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYWASTE_UOM2.Name);

					qryBuilder.Append(", ");
					SqlCommandHelper.AddDoubleZeroColumnWithName(qryBuilder, ConsumeItemsForProductionDataSet.Columns.QUANTITYUSED_UOM2.Name);

					qryBuilder.Append("\r\nFROM ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

					qryBuilder.Append("\r\nLEFT OUTER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(logisticCarrierLocationCode, CultureInfo.InvariantCulture));

					qryBuilder.Append("\r\nINNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(qryBuilder.WITH_NOLOCK);
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);


					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" <> -1 ");



					qryBuilder.Append("\r\nLEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);



					qryBuilder.Append("\r\nWHERE ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

					if (!string.IsNullOrEmpty(itemCode))
					{
						qryBuilder.Append(" AND ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

					}

					#endregion
				}


				qryBuilder.Append("\r\nORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.OPTIONAL.Name));
				qryBuilder.Append(" DESC, ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ConsumeItemsForProductionDataSet.Columns.ITEMCODE.Name));

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Command for getting all products order by name
			/// </summary>
			/// <param name="sscc">The SSCC.</param>
			/// <param name="locationCode">The location code.</param>
			/// <returns>
			/// Command for getting all products order by name
			/// </returns>
			public static string GetProductOnSSCC(string sscc, string locationCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT ");

				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));


				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME);

				qryBuilder.Append(", SUM( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(" ) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME));


				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

				qryBuilder.Append(" LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

				qryBuilder.Append(" LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);


				qryBuilder.Append(" WHERE ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(locationCode))
				{
					qryBuilder.Append(" AND ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
				}


				qryBuilder.Append(" GROUP BY ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemName.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);


				qryBuilder.Append(" ORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
				return qryBuilder.ToString();
			}

			/// <summary>
			/// Command for getting all logistic carriers order by name
			/// </summary>
			/// <param name="onlyInStock">if set to <c>true</c> the system will get the logistic carriers that are in stock and have no products on it (LUID IS NULL)</param>
			/// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
			/// <returns>
			/// Command for getting all logistic carriers order by name
			/// </returns>
			public static string GetLogisticCarriers(bool onlyInStock, bool filterOutFrozenItems, string storLocCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));

				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				if (onlyInStock)
				{
					qryBuilder.Append(" INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(" ON ", qryBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" = ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" AND ", qryBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
					qryBuilder.Append(" IS NULL ");
				}

				qryBuilder.Append(" WHERE ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(storLocCode))
				{
					qryBuilder.Append(" AND ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(storLocCode, CultureInfo.InvariantCulture));
				}

				if (filterOutFrozenItems)
				{
					Produmex.Sbo.Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(qryBuilder, " AND ");
				}


				qryBuilder.Append(" ORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
				return qryBuilder.ToString();

			}



			/// <summary>
			/// Gets the items to receive.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order document entry.</param>
			/// <param name="onlyHasToBeLinedUp">if set to <c>true</c> [only has to be lined up].</param>
			/// <param name="useItemNameAsDescription">if set to <c>true</c> [use item name as description].</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetByProducts(int productionOrderDocEntry, bool useItemNameAsDescription, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT DISTINCT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductionOrderLineDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, useItemNameAsDescription ? SboItemTable.Columns.ItemName.NAME : PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderLineDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME, ProductionOrderLineDataSet.Columns.IS_OPTIONAL.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_LINE_NUM.Name);


				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME, ProductionOrderLineDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

				//Only needed for screen
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME, ProductionOrderLineDataSet.Columns.QUANTITY.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME, ProductionOrderLineDataSet.Columns.HAS_TO_BE_LINED_UP.Name);



				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);



				qryBuilder.Append("\r\n WHERE ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
				qryBuilder.Append(" < ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(0, CultureInfo.InvariantCulture));


				qryBuilder.Append("\r\n AND ", qryBuilder.IFNULL, "(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME);
				qryBuilder.AppendFormat(", {0} ) = {0} ",
				SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));




				qryBuilder.Append("\r\n ORDER BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);


				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the production orders ingredients.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="areLogisticCarriersStoredOnOneLocation">if set to <c>true</c> [are logistic carriers stored on one location].</param>
			/// <param name="showItemsWithLowTolReached">if set to <c>true</c> [show items with low tol reached].</param>
			/// <param name="currentProdBatchNumber">The current prod batch number.</param>
			/// <returns></returns>
			public static string GetIngredientsForProduction(int productionOrderDocEntry, bool areLogisticCarriersStoredOnOneLocation, bool showItemsWithLowTolReached, int? currentProdBatchNumber, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, PickItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME, PickItemsForProductionDataSet.Columns.ITEMBARCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, PickItemsForProductionDataSet.Columns.ITEMCODE.Name);



				qryBuilder.AppendFormat(", (SUM({0}) / SUM({1}) ) AS {2} ",
					 SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
					 SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					 SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.BASEQUANTITY.Name));



				if (currentProdBatchNumber.HasValue)
				{
					qryBuilder.Append(", (( ");
					//SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
					qryBuilder.AppendFormat(" (SUM({0}) / " + qryBuilder.IFNULL + "( {1}, 1) ) * {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
					 SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.Value, CultureInfo.InvariantCulture));

				}
				else
				{
					qryBuilder.Append(", ((SUM(");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
					qryBuilder.Append(" )");

				}

				qryBuilder.Append(") - SUM(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.IssuedQuantity.NAME);

				qryBuilder.Append(") - ", qryBuilder.IFNULL, " (  ( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(" ),0) ");


				qryBuilder.Append(" ) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.QUANTITY.Name));

				qryBuilder.Append(", \r\n");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, PickItemsForProductionDataSet.Columns.UOM.Name);



				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

				qryBuilder.Append("\r\n INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);


				qryBuilder.Append("\r\n LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderDocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderLineNum.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME);


				/* 
 LEFT OUTER JOIN "PMX_INVT" ON "PMX_INVT"."StorLocCode" =  CASE WHEN ISNULL("WOR1"."U_PMX_HBLU" , N'N' ) = N'Y' AND WOR1.U_PMX_APLU = 'Y' THEN PMX_LIUP.LinedUpLocationCode ELSE "OWOR"."U_PMX_PLCD" END AND "WOR1"."ItemCode" = "PMX_INVT"."ItemCode"
*/


				qryBuilder.Append("\r\n LEFT OUTER JOIN (");
				{

					qryBuilder.Append("SELECT ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					qryBuilder.Append(", SUM ( ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
					qryBuilder.AppendFormat(") AS {0} ",
					SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.Quantity.NAME));


					qryBuilder.Append("\r\n FROM ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));


					qryBuilder.Append("\r\n WHERE ");

					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
					qryBuilder.Append(" IN ( ");
					{

						qryBuilder.Append("SELECT ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

						qryBuilder.Append("\r\n FROM ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

						qryBuilder.Append("\r\n WHERE ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

					}
					qryBuilder.Append(" ) ");


					qryBuilder.Append(" GROUP BY ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);



				}
				qryBuilder.Append("\r\n) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				qryBuilder.Append(" = CASE WHEN ", qryBuilder.IFNULL, "(", SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME));
				qryBuilder.AppendFormat(" , {0} ) = {1} ",
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				qryBuilder.Append(" AND ", SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.AllowToPickForLineUp.NAME),
					" = ", SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" THEN ", SqlCommandHelper.EscapeTableColumnName(PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME));
				qryBuilder.Append(" ELSE ", SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME));
				qryBuilder.Append(" END ");

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);


				qryBuilder.Append("\r\n WHERE ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME);
				qryBuilder.Append(" > 0 ");

				{
					qryBuilder.Append("\r\n AND (", qryBuilder.IFNULL, "(");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
					qryBuilder.AppendFormat(" , {0} ) = {0} ",
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
					qryBuilder.Append(" OR ", SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.AllowToPickForLineUp.NAME),
						" = ", SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture), " ) ");
				}

				qryBuilder.Append("\r\n AND ", qryBuilder.IFNULL, "(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.Optional.NAME);
				qryBuilder.AppendFormat(" , {0} ) = {0} ",
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				if (areLogisticCarriersStoredOnOneLocation)
				{
					qryBuilder.Append("\r\n AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
				}

				qryBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append("\r\n AND ", qryBuilder.IFNULL, "(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UseForTimeRegistration.NAME);
				qryBuilder.AppendFormat(", {0} ) = {0} ",
				SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.AppendFormat("\r\n AND {0} >= 0",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME));

				qryBuilder.Append("\r\n GROUP BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);


				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);



				if (showItemsWithLowTolReached)//2013-11-04: is never used?
				{
					qryBuilder.AppendFormat("\r\n HAVING SUM({0}) + " + qryBuilder.IFNULL + "( ( {1} ), 0 ) < (SUM({2}/" + qryBuilder.IFNULL + "({3},1)) * {4})",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.IssuedQuantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
						currentProdBatchNumber.HasValue ? SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME) : SqlCommandHelper.FormatAndEscapeColumnValue(1, CultureInfo.InvariantCulture),
						SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.HasValue ? currentProdBatchNumber.Value : 1, CultureInfo.InvariantCulture));
				}
				else
				{
					qryBuilder.AppendFormat("\r\n HAVING SUM({0}) + " + qryBuilder.IFNULL + "( ( {1} ), 0 ) < (SUM({2}/{4}) * {5}) * ( 100.0 - {3} ) / 100.0",
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.IssuedQuantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.PlannedQuantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.QuantityTolerancePercentage.NAME),
						currentProdBatchNumber.HasValue ? SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME) : SqlCommandHelper.FormatAndEscapeColumnValue(1, CultureInfo.InvariantCulture),
						SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.HasValue ? currentProdBatchNumber.Value : 1, CultureInfo.InvariantCulture));

				}

				qryBuilder.Append("\r\n ORDER BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the ingredients for production for disassembly.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="areLogisticCarriersStoredOnOneLocation">if set to <c>true</c> [are logistic carriers stored on one location].</param>
			/// <param name="currentProdBatchNumber">The current product batch number.</param>
			/// <returns></returns>
			public static string GetIngredientsForProductionDisassembly(int productionOrderDocEntry, bool areLogisticCarriersStoredOnOneLocation, int? currentProdBatchNumber, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, PickItemsForProductionDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME, PickItemsForProductionDataSet.Columns.ITEMBARCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, PickItemsForProductionDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddIntQuantityColumnWithName(qryBuilder, PickItemsForProductionDataSet.Columns.BASEQUANTITY.Name, 1);
				//no value but used as base qty? no WOR1.Baseqty / OWOR.BaseQty applied


				if (currentProdBatchNumber.HasValue)
				{
					qryBuilder.Append(", (( ");
					qryBuilder.AppendFormat(" ({0} / " + qryBuilder.IFNULL + "( {1}, 1) ) * {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
					 SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.Value, CultureInfo.InvariantCulture));

				}
				else
				{
					qryBuilder.Append(", ((");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);

				}

				qryBuilder.Append(") - ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

				qryBuilder.Append(" - ", qryBuilder.IFNULL, " ( SUM ( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(" ),0) ");


				qryBuilder.Append(" ) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.QUANTITY.Name));

				//strBuilder.Append(", (");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
				//strBuilder.Append(" - ");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

				//strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				//strBuilder.Append(" ),0) ");


				//strBuilder.Append(" ) AS ");
				//strBuilder.Append(SqlCommandHelper.EscapeColumnName(PickItemsForProductionDataSet.Columns.QUANTITY.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, PickItemsForProductionDataSet.Columns.UOM.Name);



				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" LEFT OUTER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);


				qryBuilder.Append(" WHERE ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));

				if (areLogisticCarriersStoredOnOneLocation)
				{
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
				}

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" GROUP BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME);

				qryBuilder.Append("HAVING (( ");
				if (currentProdBatchNumber.HasValue)
				{
					qryBuilder.AppendFormat(" ({0} / " + qryBuilder.IFNULL + "( {1}, 1) ) * {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),
					 SqlCommandHelper.FormatAndEscapeColumnValue(currentProdBatchNumber.Value, CultureInfo.InvariantCulture));

				}
				else
				{
					SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
				}

				qryBuilder.Append(") - ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);

				qryBuilder.Append(" - ", qryBuilder.IFNULL, " ( SUM ( ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				qryBuilder.Append(" ),0) ) > 0");

				//strBuilder.Append(" HAVING (");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
				//strBuilder.Append(" - ");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
				//strBuilder.Append(" - ", strBuilder.IFNULL, " ( SUM ( ");
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
				//strBuilder.Append(" ),0) ");
				//strBuilder.Append(" ) ");
				//strBuilder.Append(" > 0 ");

				qryBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);


				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the can be lined up locations.
			/// </summary>
			/// <param name="productionLineCode">The production line code.</param>
			/// <param name="itemCode">The item code.</param>
			/// <returns></returns>
			public static string GetCanBeLinedUpLocations(string productionLineCode, string itemCode)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.IngredientMustBeProduced.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


				if (!string.IsNullOrEmpty(itemCode))
				{
					strBuilder.Append(" INNER JOIN ");
					strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					strBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);

				}
				strBuilder.Append(" WHERE ");

				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(itemCode))
				{
					strBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
				}

				strBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);


				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the has to be lined up production lines.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="dbTool">The dbTool</param>
			/// <returns></returns>
			public static string GetHasToBeLinedUpProductionLines(int productionOrderDocEntry, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, HasToBeLinedUpProductionLinesDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CodeBars.NAME, HasToBeLinedUpProductionLinesDataSet.Columns.ITEMBARCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME, HasToBeLinedUpProductionLinesDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME, HasToBeLinedUpProductionLinesDataSet.Columns.LINENUM.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.AllowToPickForLineUp.NAME, HasToBeLinedUpProductionLinesDataSet.Columns.HASTOBELINEDUP.Name);


				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" WHERE ");

				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture));


				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));




				qryBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.ItemCode.NAME);


				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the production orders.
			/// </summary>
			/// <param name="productionLineCode">The production line code.</param>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <returns></returns>
			public static string GetProductionOrders(string productionLineCode, int? productionOrderDocEntry, string sboWhsCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.Append(
					"SELECT ", qryBuilder.AsIdentifierWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name),

					", CAST(", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
					" AS NVARCHAR) AS ", qryBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name),

					", CAST(", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME),
					" AS NVARCHAR) AS ", qryBuilder.AsIdentifier(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name),

					", ( ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),
					 " - ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME),
					 " - ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME),
					" ) AS ", qryBuilder.AsIdentifier(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.InputLocationCode.NAME, ProductionOrderDataSet.Columns.INPUT_LOCATION_CODE.Name),

					", ", SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.BatchNumber.NAME, ProductionOrderDataSet.Columns.PMX_BATCH_NUM.Name),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME, PmxProductionOrderTable.Columns.NrOfProdBatches.NAME),

					", ", qryBuilder.AsIdentifierWithAlias(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME, PmxProductionOrderTable.Columns.ProdBatchSize.NAME),

					", ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME),

					", ", qryBuilder.IFNULL, "(", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME), ", 1)",
						" AS ", qryBuilder.AsIdentifier(PmxProductionOrderTable.Columns.CurrentProductionBatch.NAME),

					", ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.SerialNumberManagementMethod.NAME),
					", ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.SerialNumberPoolCode.NAME),



					"\r\n FROM ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME),

					"\r\n INNER JOIN ", qryBuilder.AsIdentifier(PmxWarehouseTable.NAME), qryBuilder.WITH_NOLOCK,
								" ON ", qryBuilder.AsIdentifier(PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME),
								 " = ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Warehouse.NAME),
							   " AND ", qryBuilder.AsIdentifier(PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME),
								 " = ", qryBuilder.AsString(SboBooleanColumnValues.Y.VALUE),

					"\r\n INNER JOIN ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME), qryBuilder.WITH_NOLOCK,
								" ON ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME),
								 " = ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME),

					"\r\n LEFT JOIN ", qryBuilder.AsIdentifier(SboBusinessPartnerTable.NAME),
							   " ON ", qryBuilder.AsIdentifier(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME),
								" = ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME),

					"\r\n LEFT JOIN ", qryBuilder.AsIdentifier(PmxOseProductionLineTable.NAME),
							   " ON ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
								" = ", qryBuilder.AsIdentifier(PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME),


				"\r\n WHERE ( ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
							 " IS NULL ");
				if (!string.IsNullOrEmpty(productionLineCode))
				{
					qryBuilder.Append(
						" OR ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
						 " = ", qryBuilder.AsString(productionLineCode));
				}
				qryBuilder.Append(
					" )",
					"\r\n AND ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME),
						  " = ", qryBuilder.AsString(SboProductionOrderStatusColumnValues.Released.VALUE));

				if (productionOrderDocEntry.HasValue)
				{
					qryBuilder.Append(
						"\r\n AND ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
							  " = ", productionOrderDocEntry.Value);
				}

				qryBuilder.Append(
						"\r\n AND ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Warehouse.NAME),
							  " = ", qryBuilder.AsString(sboWhsCode));


				qryBuilder.Append(
					"\r\n ORDER BY ", qryBuilder.AsIdentifier(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME));


				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the started production order for the Production line.
			/// </summary>
			/// <param name="productionLineCode">The production line code.</param>
			/// <param name="dbTool">The dbTool</param>
			/// <returns></returns>
			public static string GetStartedProductionOrders(string productionLineCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductionOrderDataSet.Columns.ITEMDESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME, ProductionOrderDataSet.Columns.ITEMCODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY.Name);

				qryBuilder.Append(", CAST(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" AS NVARCHAR) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_ENTRY_AS_STRING.Name));

				qryBuilder.Append(", CAST(");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME);
				qryBuilder.Append(" AS NVARCHAR) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.PRODUCTION_ORDER_DOC_NUM_AS_STRING.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Comments.NAME, ProductionOrderDataSet.Columns.COMMENTS.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocNum.NAME, ProductionOrderDataSet.Columns.DOC_NUM.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME, ProductionOrderDataSet.Columns.PRODUCTION_LINE_CODE.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME, ProductionOrderDataSet.Columns.DUE_DATE.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME, ProductionOrderDataSet.Columns.CUSTOMER_CODE.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardName.NAME, ProductionOrderDataSet.Columns.CUSTOMER_DESCRIPTION.Name));

				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionOrderStepListKey.NAME, ProductionOrderDataSet.Columns.STEP_LIST_KEY.Name));

				qryBuilder.Append(", (");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.PlannedQuantity.NAME);
				qryBuilder.Append(" - ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CompletedQuantity.NAME);
				qryBuilder.Append(" - ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.RejectedQuantity.NAME);

				qryBuilder.Append(" ) AS ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductionOrderDataSet.Columns.OPEN_QUANTITY.Name));


				qryBuilder.Append(", ");
				qryBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, ProductionOrderDataSet.Columns.UOM.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Type.NAME, ProductionOrderDataSet.Columns.PRODUCTION_ORDER_TYPE.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME, ProductionOrderDataSet.Columns.PMX_PRODUCTION_STATUS.Name);


				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" LEFT JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.CardCode.NAME);


				qryBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(productionLineCode, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.Status.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboProductionOrderStatusColumnValues.Released.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionStatus.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxProductionStatusColumnValues.Started.VALUE, CultureInfo.InvariantCulture));



				qryBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DueDate.NAME);


				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets all the production lines.
			/// </summary>
			/// <param name="warehouseCode">The warehouse code.</param>
			/// <returns>The sql statement to get all the production lines</returns>
			public static string GetProductionLines(string warehouseCode, string deviceID)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.InputLocationCode.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.OutputLocationCode.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.RestLocationCode.NAME));

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);


	

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);


				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxProductionLineTableBN.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxProductionLineTableBN.NAME, PmxProductionLineTableBN.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



				if (!String.IsNullOrEmpty(warehouseCode) || !String.IsNullOrEmpty(deviceID))
				{
					strBuilder.Append(" AND ( ");
					if (!String.IsNullOrEmpty(warehouseCode))
					{
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));
					}
					if (!String.IsNullOrEmpty(deviceID))
					{
						if (!String.IsNullOrEmpty(warehouseCode)) strBuilder.Append(" OR ");

						strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
							SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
							SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
					}
					strBuilder.Append(" )");
				}


				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


				return strBuilder.ToString();
			}




			/// <summary>
			/// Gets the quantity already issued.
			/// </summary>
			/// <param name="productionOrderDocEntry">The production order doc entry.</param>
			/// <param name="currentProductionBatch">The current production batch.</param>
			/// <param name="dbTool">The db tool.</param>
			/// <returns></returns>
			internal static string GetQuantityAlreadyIssued(int productionOrderDocEntry, int currentProductionBatch, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.AppendFormat("SELECT " + qryBuilder.IFNULL + "(SUM({0}),0) AS {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.Quantity.NAME),
					 SqlCommandHelper.EscapeColumnName(PmxProductionReceiptLineTable.Columns.Quantity.NAME)
					);

				qryBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxProductionReceiptLineTable.NAME)
				 );

				qryBuilder.AppendFormat("\r\n WHERE {0} = {1} AND {2} = {3}",
					SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(productionOrderDocEntry, CultureInfo.InvariantCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxProductionReceiptLineTable.NAME, PmxProductionReceiptLineTable.Columns.ProductionBatch.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(currentProductionBatch, CultureInfo.InvariantCulture)
				);

				return qryBuilder.ToString();

			}

			internal static string GetExistingSSSCCsLinkedToProductionOrder(int productionOrderDocEntry, DbTool dbTool)
			{
				/*
				  select distinct PMX_LUID.SSCC
					from PMX_INVD
					inner join PMX_LUID on PMX_INVD.LogUnitIdentKey = PMX_LUID.InternalKey
					inner join PMX_INVT on PMX_INVT.LogUnitIdentKey = PMX_LUID.InternalKey
					where PMX_INVD.BaseType  = N'202' 
					and TransType = N'59'
					and PMX_INVD.BaseEntry = docEntry
				 */

				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.AppendFormat("SELECT DISTINCT {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name)
					);

				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name)
					);


				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name)
					);

				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));


				qryBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME)
				 );

				qryBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));

				qryBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));


				qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
				SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

				qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));


				qryBuilder.AppendFormat("\r\n WHERE {0} = {1} AND {2} = {3} AND {4} = {5}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypeValues.ProductionOrder, CultureInfo.InvariantCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.TransType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(((int)SboObjectType.ProductionReceipt).ToString(), CultureInfo.InvariantCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.EscapeColumnValue(productionOrderDocEntry));

				return qryBuilder.ToString();

			}
			/// <summary>
			/// Gets the production orders.
			/// </summary>
			/// <param name="productionLineCode">The production line code.</param>
			/// <returns></returns>
			public static string GetProductionLineScanSSCCToConsume(string productionLineCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();

				qryBuilder.
					Select.Distinct.
						EscTableColumnNameWithAlias(PmxProductionLineTableBN.NAME, PmxProductionLineTableBN.Columns.Code.NAME, PmxProductionLineTableBN.Columns.Code.NAME).
						CRLF.
					From.
						EscTableName(PmxProductionLineTableBN.NAME).WithNoLock.
						CRLF.
					Where.
						EscTableColumnName(PmxProductionLineTableBN.NAME, PmxProductionLineTableBN.Columns.Code.NAME).Eq.Append(qryBuilder.AsOwnType(productionLineCode));

				return qryBuilder.ToString();
			}

			internal static string GetIssuedComponents(int productionOrderDocEntry)
			{
				StringBuilder strBuilder = new StringBuilder();
			
				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.InternalKey.NAME, SSCCForProductionDataSet.Columns.LUID.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.SSCC.NAME, SSCCForProductionDataSet.Columns.SSCC.Name);
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemName.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemCode.NAME, SSCCForProductionDataSet.Columns.ITEM_CODE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemName.NAME, SSCCForProductionDataSet.Columns.ITEM_NAME.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.StorLocCode.NAME, SSCCForProductionDataSet.Columns.LOCATION_CODE.Name);
			
				strBuilder.AppendFormat(", ABS({0}) as {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.Quantity.NAME),
					SqlCommandHelper.EscapeTableName(SSCCForProductionDataSet.Columns.QUANTITY.Name)
					);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UOM.NAME, SSCCForProductionDataSet.Columns.UOM.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.UomDecimals.NAME, SSCCForProductionDataSet.Columns.UOM_DECIMALS.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, SSCCForProductionDataSet.Columns.BATCH_NUMBER_1.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, SSCCForProductionDataSet.Columns.BATCH_NUMBER_2.Name);
				strBuilder.Append(',');
				strBuilder.AppendFormat("CAST( CAST( {0}.{1} AS NVARCHAR ) AS DATETIME ) AS {2}",
					SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
					SSCCForProductionDataSet.Columns.BEST_BEFORE_DATE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.QualityStatusCode.NAME, SSCCForProductionDataSet.Columns.QUALITY_STATUS_CODE.Name);

				strBuilder.AppendFormat(" FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME));

				strBuilder.AppendFormat(" INNER JOIN {0}", SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				strBuilder.AppendFormat(" ON {0}.{1} = {2}.{3}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.ItemCode.NAME),
					SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME),
					SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));

				strBuilder.AppendFormat(" LEFT JOIN {0}", SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.AppendFormat(" ON CAST( CAST( {0}.{1} AS NVARCHAR ) AS INT ) = {2}.{3}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.ItemTransactionalInfoKey.NAME),
					SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));

				strBuilder.AppendFormat(" WHERE {0}.{1} = '{2}'",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.BaseType.NAME),
					(int)SboLinkDocumentType.ProductionOrder);
				strBuilder.AppendFormat(" AND {0}.{1} = {2}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.BaseEntry.NAME),
					productionOrderDocEntry);
				strBuilder.AppendFormat(" AND {0}.{1} = '{2}'",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.TransType.NAME),
					(int)SboObjectType.ProductionIssue);

				strBuilder.AppendFormat(" ORDER BY {0}.{1}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeColumnName(PmxInventoryDetailTable.Columns.ItemCode.NAME));

				return strBuilder.ToString();
			}

			internal static string GetSSCCOnInputLocation(string inputLocation, string sscc)
			{
				StringBuilder strBuilder = new StringBuilder();

				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCForProductionDataSet.Columns.LUID.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCForProductionDataSet.Columns.SSCC.Name);				
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTable.Columns.ItemName.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME, SSCCForProductionDataSet.Columns.ITEM_CODE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemName.NAME, SSCCForProductionDataSet.Columns.ITEM_NAME.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, SSCCForProductionDataSet.Columns.BATCH_NUMBER_1.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, SSCCForProductionDataSet.Columns.BATCH_NUMBER_2.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, SSCCForProductionDataSet.Columns.BEST_BEFORE_DATE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCForProductionDataSet.Columns.LOCATION_CODE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCForProductionDataSet.Columns.LUID.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME, SSCCForProductionDataSet.Columns.QUALITY_STATUS_CODE.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME, SSCCForProductionDataSet.Columns.QUANTITY.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME, SSCCForProductionDataSet.Columns.UOM.Name);
				strBuilder.Append(',');
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME, SSCCForProductionDataSet.Columns.UOM_DECIMALS.Name);


				strBuilder.AppendFormat("FROM {0} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

				strBuilder.AppendFormat("INNER JOIN {0} ", SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
				strBuilder.AppendFormat("ON {0}.{1} = {2}.{3} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME), SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME), SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.Columns.InternalKey.NAME));


				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
				SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));


				strBuilder.AppendFormat("INNER JOIN {0} ", SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
				strBuilder.AppendFormat("ON {0}.{1} = {2}.{3} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME), SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME), SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.InputLocationCode.NAME));


				strBuilder.AppendFormat("INNER JOIN {0} ", SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
				strBuilder.AppendFormat("ON {0}.{1} = {2}.{3} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME), SqlCommandHelper.EscapeTableName(PmxItemTable.NAME), SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.ItemCode.NAME));

				strBuilder.AppendFormat("LEFT JOIN {0} ", SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.AppendFormat("ON {0}.{1} = {2}.{3} ", SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME), SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME), SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME), SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));


				strBuilder.AppendFormat("WHERE {0}.{1} = '{2}' ", SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME), SqlCommandHelper.EscapeColumnName(PmxOseProductionLineTable.Columns.Code.NAME), inputLocation);
				if (!String.IsNullOrEmpty(sscc))
				{
					strBuilder.AppendFormat("AND {0}.{1} = '{2}' ", SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME), SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.Columns.SSCC.NAME), sscc);

				}

				strBuilder.AppendFormat("ORDER BY {0}.{1}", SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME), SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.Columns.SSCC.NAME));

				return strBuilder.ToString();
			}

		}

		#endregion
	}
}
