using System;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Utilities;
using Produmex.Sbo.Logex.Data.Generators.LocationController;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.Data;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	/// <summary>
	/// Reception
	/// </summary>
	public class WorkflowScript_ReceptionBNScript : WorkflowInstanceScriptBase
	{
		/// <summary>
		/// Definitions
		/// </summary>
		private static class Definitions
		{
			/// <summary>
			/// The separator for separating strings values
			/// </summary>
			public const string SEPARATOR = ", ";

			/// <summary>
			/// Minimum number of characters when entering reason
			/// </summary>
			public const int MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON = 10;

		}


		// Input parameters
		/// <summary>
		/// The default culture information
		/// </summary>
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
		/// <summary>
		/// The PMX ose company
		/// </summary>
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
		/// <summary>
		/// The cross docking
		/// </summary>
		public ReadOnlyBinder<bool> CrossDocking;

		/// <summary>
		/// Gets the current_ default culture information.
		/// </summary>
		/// <value>
		/// The current_ default culture information.
		/// </value>
		private CultureInfo current_DefaultCultureInfo {
			get { return DefaultCultureInfo.Get(); }
		}

		/// <summary>
		/// Gets the current PMX ose company.
		/// </summary>
		/// <value>
		/// The current PMX ose company.
		/// </value>
		private PmxOseCompany currentPmxOseCompany {
			get { return PmxOseCompany.Get(); }
		}


		//Subflows
		/// <summary>
		/// The m_enter quantity sub flow
		/// </summary>
		WorkflowInstanceBase m_enterQuantitySubFlow;

		/// <summary>
		/// The m_sub flow back requested
		/// </summary>
		protected bool m_subFlowBackRequested;
		/// <summary>
		/// The m_sub flow list of serial numbers
		/// </summary>
		protected Collection<string> m_subFlowListOfSerialNumbers;
		/// <summary>
		/// The m_sub flow quantity
		/// </summary>
		protected double m_subFlowQuantity;


		/// <summary>
		/// The m_print item label flow
		/// </summary>
		private WorkflowInstanceBase m_printItemLabelFlow;
		/// <summary>
		/// The m_print item label flow_ back requested
		/// </summary>
		private bool m_printItemLabelFlow_BackRequested = false;

		/// <summary>
		/// The m_print item label flow_ location code
		/// </summary>
		private string m_printItemLabelFlow_LocationCode;
		/// <summary>
		/// The m_print item label flow_ needed item code
		/// </summary>
		private string m_printItemLabelFlow_NeededItemCode;
		/// <summary>
		/// The m_print item label flow_ needed packaging type
		/// </summary>
		private string m_printItemLabelFlow_NeededPackagingType;

		private int? m_printItemLabelFlow_NeededQuantityByPackagingType;


		/// <summary>
		/// The m_print item label flow_ needed batch number
		/// </summary>
		private string m_printItemLabelFlow_NeededBatchNumber;
		/// <summary>
		/// The m_print item label flow_ needed batch number2
		/// </summary>
		private string m_printItemLabelFlow_NeededBatchNumber2;
		/// <summary>
		/// The m_print item label flow_ needed best before date
		/// </summary>
		private DateTime? m_printItemLabelFlow_NeededBestBeforeDate;
		/// <summary>
		/// The m_print item label flow_ needed nr of labels
		/// </summary>
		private int? m_printItemLabelFlow_NeededNrOfLabels;
		/// <summary>
		/// The m_print item label flow_ entered quantity
		/// </summary>
		private double? m_printItemLabelFlow_EnteredQuantity;

		private double? m_printItemLabelFlow_EnteredQuantityUom2;
		private bool m_printItemLabelFlow_ShowPrintMessage;

		/// <summary>
		/// The m_enter batch attributes flow
		/// </summary>
		private WorkflowInstanceBase m_enterBatchAttributesFlow;
		/// <summary>
		/// The m_enter batch attributes flow_ list of batch attributes
		/// </summary>
		protected Collection<ItemTransactionalBatchAttributeInfo> m_enterBatchAttributesFlow_ListOfBatchAttributes;

		/// <summary>
		/// The m_enter quality values flow
		/// </summary>
		private WorkflowInstanceBase m_enterQualityValuesFlow;

		/// <summary>
		/// The m_enter quality values flow_ list of quality values
		/// </summary>
		protected Collection<QualityValueInfo> m_enterQualityValuesFlow_ListOfQualityValues;

		/// <summary>
		/// The s_log
		/// </summary>
		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


		private WorkflowInstanceBase m_afterLuidReceivedHookFlow;
		private List<LogisticUnitGoodsReceipt> m_subFlowLUIDs;


		private WorkflowInstanceBase m_afterLogisticUnitItemEnteredHookScript;
		protected LogisticUnitItemGoodReceipt m_afterLogisticUnitItemEnteredHookScript_LogUnitItem;

		private WorkflowInstanceBase m_beforeReceptionQuantityHookScript;
		protected string m_beforeReceptionQuantityHookScript_ScaleCode;

		PmxDictionary<int, double> m_uomCoefficients = new PmxDictionary<int, double>();

		private string initialErrorKey = null;
		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="WorkflowScript_ReceptionScript"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="factory">The factory.</param>
		public WorkflowScript_ReceptionBNScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{
			m_enterQuantitySubFlow = this.Create("EnterQuantityScript");
			m_enterBatchAttributesFlow = this.Create("EnterBatchAttributesScript");
			m_enterQualityValuesFlow = this.Create("EnterQualityValuesScript");
			m_afterLuidReceivedHookFlow = this.Create("AfterLogisticUnitReceivedHookScript");
			m_afterLogisticUnitItemEnteredHookScript = this.Create("AfterLogisticUnitItemEnteredHookScript");
			m_beforeReceptionQuantityHookScript = this.Create("BeforeReceptionQuantityHookScript");
		}

		#endregion

		#region WorkflowInstanceScriptBase Members

		/// <summary>
		/// Executes this instance.
		/// </summary>
		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			PmxBusinessPartnerInfo businessPartnerInfo = null;

			string deviceID = session.DeviceInstanceId;
			SboUserInfo userInfo = sboProviderService.GetCurrentUser();
			Message msg = null;
			string query = null;
			PmxOseWarehouseInfo whsInfo = null;
			DataSet dsDocks = null;
			string currentDockCode = null;
			DataSet dsPurchaseOrders = null;
			DataSet dsContainers = null;
			DocumentRef purchaseDocRef = null;
			int? containerDocEntry = null;
			int? purchaseOrderDocNum = null;
			string filterGTIN = null;
			string cardCode = null;
			string filterButtonKey = null;
			DataSet dsSuppliers = null;
			BarcodeContainer barcode = null;
			string supplierPalletNumber = null;
			int batchReceivingNumber = 1;
			List<string> buttonKeys = null;
			string titleKey = null;
			List<string> allowedButtonsForLabelScreen = null;
			string SSCC = null;
			int? luid = null;
			string scannedSSCC = null;
			//BarcodeContainer productBarCode = null;
			string itemCode = null;
			string itemDescription = null;
			PmxItemInfo itemInfo = null;
			string gtin = null;
			DataSet dsItems = null;
			Collection<LogisticUnitGoodsReceipt> purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();
			string logisticCarrierCode = null;
			DataSet dsLogisticCarriers = null;
			double stillToReceiveInventory = 0;
			LogisticUnitGoodsReceipt currentLogUnit = null;
			string batchNumber = null;
			string secondBatchNumber = null;
			DateTime? bestBeforeDate = null;
			double quantity = 0;
			double? quantityUom2 = null;
			GeneratedBatchNumberReception generatedBatch1Info = null;
			GeneratedBatchNumberReception generatedBatch2Info = null;
			string tempBatchNumber = null;
			LogisticUnitItemGoodReceipt currentLogisticCarrier = null;
			DataSet dsExistingBatch = null;
			Collection<ItemTransactionalPackagingTypeInfo> currentListOfTransactionalPackagingTypeInfos;
			Collection<ItemTransactionalBatchAttributeInfo> currentListOfTransactionalBatchAttributeInfos;
			PmxDictionary<PmxMomentOfCapture, Collection<QualityValueInfo>> currentListOfQualityValueInfos;
			Collection<PackagingTypeInfo> listOfPackagingTypeInfosForScreen = null;
			int? numberOfItemLabels = 0;
			double initialQuantity = 0;
			double? initialWeight = null;
			double maxQuantity = 0;
			string supplierRef = null;
			KeyValuePair<Collection<DocumentRef>, Collection<int>> listOfGeneratedDocEntry = new KeyValuePair<Collection<DocumentRef>, Collection<int>>();
			int numberOfReports = 0;
			bool receiveAllItemsOnOnePurchaseDelivery = false;
			double? numberOfItemsInPurchaseUnit = null;
			string purchaseUom = null;
			List<string> allowedButtonsIdentifyProductScreen = null;
			bool goBack = false;
			DataSet dsCheckItems = null;
			DataSet dsItemsNonDistinct = null;
			DataSet dsSSCC = null;
			bool gotoReceiveItem = false;
			ToolBarButtons toolBarButtons = ToolBarButtons.Back;
			Collection<string> listOfSerialNumbers = new Collection<string>();
			bool autoSelectPO = false;
			bool askReprint = false;
			bool isContainer = false;
			string tempGTIN = null;
			bool errorInGS1IdentifyProductScreen = false;
			GeneratedBestBeforeDateReception generatedBBDInfo = null;

			bool needToEnterSupplierPalletNumber = false;
			bool allowLabelsToBeScannedOnVendor = false;
			bool allowLogisticCarriersOnVendor = false;
			bool allowToChangeGeneratedBBDOnReception = false;
			bool setFocusToQuantityControl = false;

			int? lineNr = null;
			double? lineQty = null;

			DataSet dsReasons = null;

			string zoneTypeCode = null;
			string reasonCode = null;
			string reasonName = null;
			string reasonFreeText = null;
			PmxDictionary<string, bool> reasonsCodesRequiresFreeText = null;

			string reasonCodeForZoneTypeScreen = null;
			string reasonNameForZoneTypeScreen = null;
			string reasonFreeTextForZoneTypeScreen = null;
			List<string> batchNumber1ToCheck = new List<string>();
			List<string> batchNumber2ToCheck = new List<string>();
			List<DateTime?> bestBeforeDatesToCheck = new List<DateTime?>();
			List<string> serialNumbersToCheck = new List<string>();

			PmxDictionary<string, bool> reasonsCodesRequiresFreeTextForZoneTypeScreen = null;

			DataSet dsZoneTypes = null;
			DataSet dsZoneTypesAndReasons = null;
			DataSet dsPOLines = null;
			PmxDictionary<string, ReasonInfo> listOfReasonsCodesAndZoneTypes = null;

			whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);

			receiveAllItemsOnOnePurchaseDelivery = sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteGetReceiveAllItemsOnOnePurchaseDelivery));

			bool allowToSelectBatch = sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteGetAllowToSelectBatches));
			bool crossDocking = this.CrossDocking != null && this.CrossDocking.Get();


			bool autoSelectInternalBatch = false;
			bool allowToEnterReason = false;
			bool splitQtyInLogUnitsOfItemDefQtyOnLogUnit = false;
			bool checkDataInUDFPOLine = false;
			bool isPOManuallySelected = false;
			bool isGS1DataEnteredOnScanLogDataScreen = false;
			bool createSsccOnReception = true;
			bool reserveInvoicesAreAllowed = false;
			bool askUserToPrintLabels = false;
			string remarksViewName = string.Empty;
			string purchaseLineRemarksViewName = string.Empty;

			DataSet remarks = null;
			List<int> asnKeys = null;

			WeighData weighData = new WorkflowScripts.WorkflowScript_ReceptionBNScript.WeighData();
			DataSet dsScales = null;

			Collection<string> listOfLocationsForWA = new Collection<string>();

			sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, new DelegateWithPmxDbConnection<object>(
				delegate (PmxDbConnection conn, object[] parameters)
				{
					PmxContainerProvider cntprov = new PmxContainerProvider((SboConnection)conn);
					PmxDictionary<string, PmxParameterValue> ctnparams = cntprov.GetParametersContainerManagementController();

					reserveInvoicesAreAllowed = PmxParameterValueProvider.GetParameterValue<bool>(ctnparams, PmxExtensionParameterCodes.CONTAINER_MANAGEMENT_CONTROLLER_PURCHASE_RESERVE_INVOICES_ALLOWED, false);

					return null;
				}));

			IBarcodeController barcodeController = null;


			sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
			 delegate (PmxDbConnection conn, object[] parameters)
			 {
				 PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
				 barcodeController = bcProv.GetBarcodeController();

				 PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider();
				 PmxDictionary<string, PmxParameterValue> pdParams = prov.GetParametersPurchaseOrderGenerator(conn);

				 autoSelectInternalBatch = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_AUTO_SELECT_INTERNAL_BATCH, false);
				 allowToEnterReason = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ENTER_REASON_BY_ITEM, false);
				 splitQtyInLogUnitsOfItemDefQtyOnLogUnit = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_SPLIT_QTY_IN_LOG_UNITS_OF_ITEM_DEF_QTY_ON_LOG_UNIT, false);
				 checkDataInUDFPOLine = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_CHECK_DATA_FROM_PO, false);
				 askUserToPrintLabels = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ASK_TO_PRINT_LABEL, false);
				 remarksViewName = PmxParameterValueProvider.GetParameterValue<string>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_REMARKS_VIEW_NAME, string.Empty);
				 purchaseLineRemarksViewName = PmxParameterValueProvider.GetParameterValue<string>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_LINE_REMARKS_VIEW_NAME, string.Empty);

				 PmxOseWarehouseAutomationProvider waProv = new PmxOseWarehouseAutomationProvider(conn);
				 listOfLocationsForWA = waProv.GetAllInputLocationsForWA(true);

				 return null;
			 });





			//initialize sub flows
			m_printItemLabelFlow = Create("PrintItemLabelScript");
			m_printItemLabelFlow.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadOnlyBinder<string>("LocationCode", this, "m_printItemLabelFlow_LocationCode"),
				new ReadOnlyBinder<string>("NeededItemCode", this, "m_printItemLabelFlow_NeededItemCode"),
				new ReadOnlyBinder<string>("NeededPackagingType", this, "m_printItemLabelFlow_NeededPackagingType"),
				new ReadOnlyBinder<int?>("NeededQuantityByPackagingType", this, "m_printItemLabelFlow_NeededQuantityByPackagingType"),
				new ReadOnlyBinder<string>("NeededBatchNumber", this, "m_printItemLabelFlow_NeededBatchNumber"),
				new ReadOnlyBinder<string>("NeededBatchNumber2", this, "m_printItemLabelFlow_NeededBatchNumber2"),
				new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", this, "m_printItemLabelFlow_NeededBestBeforeDate"),
				new ReadOnlyBinder<int?>("NeededNrOfLabels", this, "m_printItemLabelFlow_NeededNrOfLabels"),
				new ReadWriteBinder<bool>("BackRequested", this, "m_printItemLabelFlow_BackRequested"),
				new ReadWriteBinder<double?>("EnteredQuantity", this, "m_printItemLabelFlow_EnteredQuantity"),
				new ReadWriteBinder<double?>("EnteredQuantityUom2", this, "m_printItemLabelFlow_EnteredQuantityUom2"),
				new ReadWriteBinder<bool>("ShowPrintMessage", this, "m_printItemLabelFlow_ShowPrintMessage"));


			bool receiveGoodsOnSelectedLocation = false;
			bool showReserveInvoices = false;
			bool allowToExceedOrderedQuantity = true;
			PmxParameterValue crossDockingPickListType = null;

			sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, new DelegateWithPmxDbConnection<object>(
				delegate (PmxDbConnection conn, object[] parameters)
				{
					PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);
					PmxDictionary<string, PmxParameterValue> parms = prov.GetParametersPurchaseOrderGenerator(conn);

					receiveGoodsOnSelectedLocation = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_RECEIVE_ON_LOCATION, false);
					showReserveInvoices = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_SHOW_RESERVE_INVOICES, false);
					allowToExceedOrderedQuantity = PmxParameterValueProvider.GetParameterValue<bool>(parms, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ALLOW_TO_EXCEED_ORDERED_QTY, true);
					if (crossDocking)
					{
						parms.TryGetValue(PmxExtensionParameterCodes.PURCHASE_DELIVERY_CROSS_DOCKING_PICKLISTTYPE, out crossDockingPickListType);
					}

					return null;
				}));

			PmxDictionary<string, ReasonInfo> listOfReasonCodeByItem = null;
			bool enterReason = false;

			bool askForPieces = false;
			bool askForWeight = false;
			string scannedItemCode = null;
			bool isNoPo = false;

			List<string> buttons = null;

			#region LOCATION SUGGESTIONS

			bool useLocationSuggestions = false;
			Collection<LocationInfo> listOfLocationInfos = null;
			PmxDictionary<string, LocationInfo> listOfLocationInfosAsDictionary = null;
			bool needsToEnterReasonForNotAllowedLocation = false;
			string reasonCodeLocations = null;
			string reasonNameLocations = null;
			string reasonFreeTextLocations = null;
			string reasonLocationCode = null;

			Collection<string> listOfUsedLocations = null;

			#endregion



			Step_SelectDock:

			#region SELECT DOCK

			// Get locations
			query = BuildQuery.GetCodeNameDockOrderByName(crossDocking ? true : (bool?)null, true, true, whsInfo.PmxWarehouseCode, deviceID);
			dsDocks = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsDocks) == 1 && !goBack)
			{
				currentDockCode = Convert.ToString(dsDocks.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
				//reset warehouse info
				whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);
				goto Step_SelectPOFilter;
			}
			goBack = false;

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", "MSG_TITLE_SELECT_LOCATION",
				"LocationDS", dsDocks
				));

			msg = WaitForMessage();
			goBack = false;
			initialErrorKey = null;
			if (msg.Name.EndsWith(".LocationSelected"))
			{
				currentDockCode = ExtractParameter<string>(msg.Parameters, "locationCode");
				//reset warehouse info
				whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);

				goto Step_SelectPOFilter;
			}

			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_SelectDock;
			}

			//back requested
			return;

			#endregion

			Step_SelectPOFilter:

			#region SELECT PO FILTER

			allowLabelsToBeScannedOnVendor = false;
			allowLogisticCarriersOnVendor = false;

			if (crossDocking)
			{
				if (goBack /*|| !ExecuteCrossDocking( purchaseOrderDocEntry, whsInfo, currentDockCode, session, sboProviderService )*/ )
				{
					//goBack = true;
					goto Step_SelectDock;
				}

				// Always select Purchase Order in case of cross-docking
				filterButtonKey = "MSG_BUTTON_PO_FILTER_ORDER";
				goto Step_SelectOrder;
			}

			filterGTIN = null;
			filterButtonKey = null;
			cardCode = null;
			purchaseDocRef = null;
			containerDocEntry = null;
			autoSelectPO = false;
			isContainer = false;
			isPOManuallySelected = false;
			asnKeys = null;
			isNoPo = false;
			buttonKeys = new List<string>(new Collection<string>() { "MSG_BUTTON_PO_FILTER_EAN", "MSG_BUTTON_PO_FILTER_ORDER", "MSG_BUTTON_PO_FILTER_NO_PO", "MSG_BUTTON_CONTAINER" });

			toolBarButtons = ToolBarButtons.Stop | ToolBarButtons.Back;

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_PO_FILTER",
					"AllowedButtons", buttonKeys.ToArray(),
					"EnabledToolBarButtons", toolBarButtons
					));
			msg = WaitForMessage();
			goBack = false;
			initialErrorKey = null;
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				filterButtonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

				if (filterButtonKey == "MSG_BUTTON_PO_FILTER_EAN")
				{
					goto Step_FilterOnEAN;
				}

				if (filterButtonKey == "MSG_BUTTON_PO_FILTER_ORDER")
				{
					goto Step_SelectOrder;
				}

				if (filterButtonKey == "MSG_BUTTON_CONTAINER")
				{
					isContainer = true;
					goto Step_SelectContainer;
				}

				isNoPo = true;
				goto Step_SelectSupplier;
			}

			if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_SelectDock;
			}

			if (msg.Name.EndsWith(".StopRequested"))
			{
				return;
			}

			#endregion

			Step_SelectSupplier:

			#region SELECT SUPPLIER

			allowLabelsToBeScannedOnVendor = false;
			allowLogisticCarriersOnVendor = false;

			cardCode = null;

			query = BuildQuery.GetSuppliersOrderByName();
			dsSuppliers = sboProviderService.RunView(false, null, null, query);

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectSupplierScreen),
				 DefaultCultureInfo.Get(), BuildParamCollection(
				 "InitialErrorKey", initialErrorKey,
				 "TitleKey", "MSG_TITLE_SELECT_SUPPLIER",
				 "SupplierDS", dsSuppliers,
				 "BarcodeController", barcodeController
				 ));



			msg = WaitForMessage();
			goBack = false;
			initialErrorKey = null;
			if (msg.Name.EndsWith(".SupplierSelected"))
			{
				cardCode = ExtractParameter<string>(msg.Parameters, "supplierCode");

				businessPartnerInfo = sboProviderService.InvokeMethodWithDbConnection<PmxBusinessPartnerInfo>(true, false, null, null, new DelegateWithPmxDbConnection<PmxBusinessPartnerInfo>(ExecuteGetBusinessPartnerInfo), cardCode);

				allowLabelsToBeScannedOnVendor = !businessPartnerInfo.HasNoLabelsAtReception;
				allowLogisticCarriersOnVendor = !businessPartnerInfo.HasNoLogisticCarriersAtReception;

				goto Step_StartReceiving;
			}

			goto Step_SelectPOFilter;

			#endregion

			Step_FilterOnEAN:

			#region FILTER ON EAN

			allowLabelsToBeScannedOnVendor = false;
			allowLogisticCarriersOnVendor = false;

			filterGTIN = null;


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SCAN_GS1_LABEL",
					"BarcodeController", barcodeController,
					"ImageType", GS1ImageType.BoxImage));

			msg = WaitForMessage();
			goBack = false;
			// Reset latest error
			initialErrorKey = null;
			barcode = null;
			filterGTIN = null;

			if (msg.Name.EndsWith(".DataEntered"))
			{
				barcode = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

				//check if there is a product present
				if (barcode != null &&
				!string.IsNullOrEmpty(BarcodeHelper.GetProductIdentification(barcode)))
				{
					filterGTIN = BarcodeHelper.GetProductIdentification(barcode);
					goto Step_SelectOrder;

				}
				else
				{
					initialErrorKey = "MSG_ERROR_NO_PRODUCT_ON_BARCODE";
					goto Step_FilterOnEAN;
				}
			}
			else if (msg.Name.EndsWith(".BackRequested"))
			{
				goto Step_SelectPOFilter;
			}

			initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
			goto Step_FilterOnEAN;


			#endregion

			Step_SelectOrder:

			#region SELECT ORDER

			allowLabelsToBeScannedOnVendor = false;
			allowLogisticCarriersOnVendor = false;

			if (crossDocking)
			{
				if (purchaseDocRef != null && !ExecuteCrossDocking(purchaseDocRef.DocEntry, whsInfo, currentDockCode, crossDockingPickListType, session, sboProviderService))
				{
					goBack = true;
					goto Step_SelectDock;
				}
			}

			purchaseDocRef = null;
			containerDocEntry = null;
			purchaseOrderDocNum = null;
			query = BuildQuery.GetPOsOrderByPONumber(/*cardCode*/ null, filterGTIN, crossDocking, showReserveInvoices && !crossDocking, whsInfo.SboWarehouseCode, sboProviderService.GetDbTool());  // Don't show reserve invoices when cross-docking
			dsPurchaseOrders = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsPurchaseOrders) == 0)
			{
				initialErrorKey = "MSG_ERROR_NO_PO_FOUND";

				goBack = true;
				goto Step_SelectPOFilter;
			}

			if (this.GetNumberOfRows(dsPurchaseOrders) == 1 &&
				filterButtonKey != "MSG_BUTTON_PO_FILTER_ORDER" &&
				!goBack)
			{
				DataRow row = dsPurchaseOrders.Tables[0].Rows[0];

				//we found just one row, and we do not want to select it manually, so auto-select
				purchaseDocRef = new DocumentRef(Convert.ToString(row[PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name]), Convert.ToInt32(row[PurchaseOrderDataSet.Columns.PO_ID.Name]));
				cardCode = Convert.ToString(row[PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name]);
				purchaseOrderDocNum = Convertor.ConvertToInt32Nullable(row[PurchaseOrderDataSet.Columns.PO_NUMBER.Name]);
				goto Step_ShowRemarks;//				Step_StartReceiving;
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectPOScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", "MSG_TITLE_SELECT_PO",
				"ButtonKey", crossDocking ? "" : "MSG_BUTTON_PO_FILTER_AUTO_SELECT_ORDER",
				"PODS", dsPurchaseOrders,
				 "BarcodeController", barcodeController
				));

			msg = WaitForMessage();
			goBack = false;
			initialErrorKey = null;
			if (msg.Name.EndsWith(".POSelected"))
			{
				purchaseDocRef = new DocumentRef(ExtractParameter<string>(msg.Parameters, "docObjType"), ExtractParameter<int>(msg.Parameters, "docEntry"));

				DataRow[] rows = dsPurchaseOrders.Tables[0].Select(PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name + " = " + purchaseDocRef.DocType
					+ " AND " + PurchaseOrderDataSet.Columns.PO_ID.Name + " = " + purchaseDocRef.DocEntry.ToString());

				if (rows.Length > 0)
				{
					cardCode = Convert.ToString(rows[0][PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name]);
					purchaseOrderDocNum = Convertor.ConvertToInt32Nullable(rows[0][PurchaseOrderDataSet.Columns.PO_NUMBER.Name]);
				}

				//The UDF-check can ONLY be performed when PO is MANUALLY selected.
				isPOManuallySelected = true;

				goto Step_ShowRemarks;// Step_StartReceiving;
			}

			if (msg.Name.EndsWith("ISelectPOScreenReply.RefreshRequested"))
			{
				goto Step_SelectOrder;
			}

			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				autoSelectPO = true;
				goto Step_SelectSupplier;
			}

			goBack = true;
			goto Step_SelectPOFilter;

			#endregion

			Step_SelectContainer:

			#region SELECT CONTAINER

			allowLabelsToBeScannedOnVendor = false;
			allowLogisticCarriersOnVendor = false;

			purchaseDocRef = null;
			containerDocEntry = null;
			purchaseOrderDocNum = null;
			query = BuildQuery.GetContainers(whsInfo.SboWarehouseCode, sboProviderService.GetDbTool());
			dsContainers = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsContainers) == 0)
			{
				initialErrorKey = "MSG_ERROR_NO_CONTAINER_FOUND";

				goBack = true;
				goto Step_SelectPOFilter;
			}


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectContainerScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", "MSG_TITLE_SELECT_CONTAINER",
				"ContainerDS", dsContainers
				));

			msg = WaitForMessage();
			goBack = false;
			initialErrorKey = null;
			if (msg.Name.EndsWith(".ContainerSelected"))
			{
				containerDocEntry = ExtractParameter<int>(msg.Parameters, "docEntry");

				DataRow[] rows = dsContainers.Tables[0].Select(ContainerDataSet.Columns.DOC_ENTRY.Name + " = " + containerDocEntry.Value.ToString());

				if (rows.Length > 0)
				{
					cardCode = Convert.ToString(rows[0][PmxContainerTable.Columns.CardCode.NAME]);

					if (!string.IsNullOrEmpty(cardCode))
					{
						businessPartnerInfo = sboProviderService.InvokeMethodWithDbConnection<PmxBusinessPartnerInfo>(true, false, null, null, new DelegateWithPmxDbConnection<PmxBusinessPartnerInfo>(ExecuteGetBusinessPartnerInfo), cardCode);
						allowLabelsToBeScannedOnVendor = !businessPartnerInfo.HasNoLabelsAtReception;
						allowLogisticCarriersOnVendor = !businessPartnerInfo.HasNoLogisticCarriersAtReception;
					}
				}

				goto Step_ShowRemarks;// Step_StartReceiving;
			}

			if (msg.Name.EndsWith(".RefreshRequested"))
			{
				goto Step_SelectContainer;
			}


			goBack = true;
			goto Step_SelectPOFilter;

			#endregion

			Step_ShowRemarks:

			#region SHOW REMARKS

			if (!string.IsNullOrEmpty(remarksViewName) && purchaseDocRef != null)
			{
				query = BuildQuery.GetRemarks(purchaseDocRef.DocEntry, purchaseDocRef.DocType, remarksViewName, sboProviderService.GetDbTool());
				DataSet ds = sboProviderService.RunView(true, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{

					ds.Tables[0].Columns.Remove(PmxDocumentLineLockTable.Columns.ObjType.NAME);
					ds.Tables[0].Columns.Remove(PmxDocumentLineLockTable.Columns.DocEntry.NAME);

					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowRemarksScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"RemarksTable", ds.Tables[0],
							"EnabledToolBarButtons", ToolBarButtons.Forward));
					WaitForMessage();
				}
			}

			goto Step_StartReceiving;

			#endregion

			Step_StartReceiving:

			#region START RECEIVING

			string customerInfo = null;

			if (purchaseDocRef != null)
			{
				query = BuildQuery.GetCustomerInfo(purchaseDocRef.DocEntry, purchaseDocRef.DocType, sboProviderService.GetDbTool());
				DataSet dsCustomerInfo = sboProviderService.RunView(false, null, null, query);

				customerInfo = GetCustomerInfo(dsCustomerInfo);
			}

			//This is the start of a reception.
			//Cleanup lists
			purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();
			listOfReasonCodeByItem = new PmxDictionary<string, ReasonInfo>();

			listOfUsedLocations = new Collection<string>();

			currentListOfQualityValueInfos = new PmxDictionary<PmxMomentOfCapture, Collection<QualityValueInfo>>();
			asnKeys = null;


			goto Step_AskStartQualityValues;

			#endregion

			Step_AskStartQualityValues:

			#region ASK START QUALITY VALUES

			m_enterQualityValuesFlow.BindParameters(
			 this.DefaultCultureInfo,
			 this.PmxOseCompany,
			 new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.Start),
			 new ReadOnlyBinder<string>("BaseType", SboObjectTypes.PurchaseDelivery),
			 new ReadOnlyBinder<int?>("BaseEntry", null),
			 new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
			 new ReadOnlyBinder<bool>("SaveData", false),

			 new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
			 new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

			 );
			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

			if (m_subFlowBackRequested)
			{
				goBack = true;
				if (filterButtonKey == "MSG_BUTTON_PO_FILTER_NO_PO")
				{
					goto Step_SelectPOFilter;
				}
				else if (filterButtonKey == "MSG_BUTTON_PO_FILTER_EAN")
				{
					goto Step_FilterOnEAN;
				}
				else if (isContainer)
				{
					goto Step_SelectContainer;
				}
				else
				{
					goto Step_SelectOrder;
				}
			}

			if (!currentListOfQualityValueInfos.ContainsKey(PmxMomentOfCapture.Start))
			{
				currentListOfQualityValueInfos[PmxMomentOfCapture.Start] = m_enterQualityValuesFlow_ListOfQualityValues;
			}

			if (autoSelectPO)
			{
				//Check if there are ASN rows found.
				//If they are found, ask user if he wants to continue 

				if (sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(
							delegate (PmxDbConnection conn, object[] parameters)
							{
								PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

								List<PmxAdvanceShippingNotice> asnBOs = asnProv.GetOpenAsnListForAutoSelectPO(cardCode, conn.InternalKeySboUser, "", 1);
								return asnBOs.Count > 0;

							})))
				{
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_AUTO_SELECT_PO_ASN_FOUND",
							"MessageKey", "MSG_QUESTION_AUTO_SELECT_PO_ASN_FOUND",
							"AllowToGoBack", true,
							"ButtonKeyYes", "MSG_BUTTON_AUTO_SELECT_PO_CONTINUE",
							"ButtonKeyNo", "MSG_BUTTON_AUTO_SELECT_PO_REMOVE_ASN"
							));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_EnterSupplierReference;
					}
					else if (msg.Name.EndsWith(".No"))
					{
						//Cancel, so delete the entries
						sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
						delegate (PmxDbConnection conn, object[] parameters)
						{
							PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

							asnProv.RemoveASNForAutoSelectPO(cardCode, conn.InternalKeySboUser);

							return null;

						}));

						goto Step_EnterSupplierReference;
					}
					else
					{
						//Go back
						goto Step_SelectPOFilter;
					}
				}

			}

			goto Step_EnterSupplierReference;

			#endregion

			Step_EnterSupplierReference:

			#region ENTER SUPPLIER REFERENCE

			supplierRef = null;

			if (!isContainer)
			{

				if (PmxOseCompany.Get().GoodsReceiptRequiresSupplierRef)
				{
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
					 DefaultCultureInfo.Get(), BuildParamCollection(
						 "InitialErrorKey", initialErrorKey,
						 "TitleKey", "MSG_TITLE_ENTER_SUPPLIER_REF",
						 "ForceDataEntry", true,
						 "ForceUppercase", true,
						 "AllowToGoBack", true,
						 "Information", "MSG_ENTER_SUPPLIER_REF_FOR_PO_DELIVERY"
						 ));
					msg = WaitForMessage();
					goBack = false;
					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".StringEntered"))
					{
						supplierRef = ExtractParameter<string>(msg.Parameters, "stringValue");
						//goto Step_SelectLogisticCarrier;
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						goBack = true;
						if (filterButtonKey == "MSG_BUTTON_PO_FILTER_NO_PO")
						{
							goto Step_SelectPOFilter;
						}
						else if (filterButtonKey == "MSG_BUTTON_PO_FILTER_EAN")
						{
							goto Step_FilterOnEAN;
						}
						else if (isContainer)
						{
							goto Step_SelectContainer;
						}
						else
						{
							goto Step_SelectOrder;
						}
					}
				}

				needToEnterSupplierPalletNumber = sboProviderService.InvokeMethodWithDbConnection<bool>(true, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteNeedToEnterSupplierPalletNumber), cardCode);

				if (!string.IsNullOrEmpty(cardCode))
				{
					businessPartnerInfo = sboProviderService.InvokeMethodWithDbConnection<PmxBusinessPartnerInfo>(true, false, null, null, new DelegateWithPmxDbConnection<PmxBusinessPartnerInfo>(ExecuteGetBusinessPartnerInfo), cardCode);
					allowLabelsToBeScannedOnVendor = !businessPartnerInfo.HasNoLabelsAtReception;
					allowLogisticCarriersOnVendor = !businessPartnerInfo.HasNoLogisticCarriersAtReception;
				}

			}
			else
			{
				//supplier not known for container
				needToEnterSupplierPalletNumber = false;
			}




			//Check if we have ASN and all items are on ASN
			if (containerDocEntry.HasValue || purchaseDocRef != null)
			{
				query = BuildQuery.GetCodeNameItemsOrderByName(null, tempGTIN, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, scannedItemCode, sboProviderService.GetDbTool());
				dsItemsNonDistinct = sboProviderService.RunView(true, null, null, query);
				RemoveReceivedItems(purchaseDeliveryNoteList, dsItemsNonDistinct, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

				if (this.GetNumberOfRows(dsItemsNonDistinct) == 0 && !errorInGS1IdentifyProductScreen)
				{
					//No rows found, but ASN rows found, so all stock is available on ASN
					if (sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(
							delegate (PmxDbConnection conn, object[] parameters)
							{
								PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

								List<PmxAdvanceShippingNotice> asnBOs = null;
								if (autoSelectPO)
								{
									asnBOs = asnProv.GetOpenAsnListForAutoSelectPO(cardCode, conn.InternalKeySboUser, "", 1);
								}
								else
								{
									asnBOs = asnProv.GetOpenAsnList(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : purchaseDocRef.DocType,
										containerDocEntry.HasValue ? containerDocEntry.Value : purchaseDocRef.DocEntry, "", 1, conn.InternalKeySboUser);
								}
								return asnBOs.Count > 0;

							})))
					{
						//we want to go back, but we have some items in the list
						//ask user what to do
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"TitleKey", "MSG_TITLE_PRODUCTS_ON_LIST_TO_RECEIVE",
								"MessageKey", "MSG_QUESTION_RECEPTION_ALL_GOODS_ON_ASN",
								"AllowToGoBack", true,
								"ButtonKeyYes", "MSG_BUTTON_BOOK_RECEIPT",
								"ButtonKeyNo", "MSG_BUTTON_CANCEL_RECEIPT"
								));
						msg = WaitForMessage();

						if (msg.Name.EndsWith(".Yes"))
						{
							//there are pallets received, but user wants to go back, so we will receive them right now
							goBack = true;

							// Inform the user items are receiving
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"MessageKey", "MSG_RECEIVING_ALL_ITEMS",
									"NoTranslationOfMessageKey", false,
									"ShowButton", false));

							goto Step_AskEndQualityValues;
						}
						else if (msg.Name.EndsWith(".No"))
						{
							//Cancel, so delete the entries
							sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
							delegate (PmxDbConnection conn, object[] parameters)
							{
								PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

								if (autoSelectPO)
								{
									asnProv.RemoveASNForAutoSelectPO(cardCode, conn.InternalKeySboUser);
								}
								else
								{
									asnProv.RemoveASNForUser(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : purchaseDocRef.DocType,
										containerDocEntry.HasValue ? containerDocEntry.Value : purchaseDocRef.DocEntry, null, conn.InternalKeySboUser);
								}
								return null;

							}));

							goto Step_AskStartQualityValues;
						}
						else
						{
							//Go back
							goto Step_SelectPOFilter;
						}
					}

				}

			}




			goto Step_AskBatchReceiving;


			#endregion

			Step_ShowMessageNextLogisticUnit:

			#region SHOW MESSAGE NEXT LOGISTIC UNIT

			// Inform the user the items are received
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"MessageKey", "MSG_ENTER_NEXT_LOGISTIC_UNIT",
					"ShowButton", true));
			WaitForMessage();

			goto Step_AskBatchReceiving;

			#endregion

			Step_AskBatchReceiving:

			#region ASK BATCH RECEIVING

			enterReason = false;

			batchReceivingNumber = 1;
			barcode = null;
			buttonKeys = new List<string>(new Collection<string> { "MSG_BUTTON_BATCH_RECEIVING", "MSG_BUTTON_NO_BATCH_RECEIVING" });

			if (businessPartnerInfo != null)
			{
				if (needToEnterSupplierPalletNumber ||
					autoSelectPO || businessPartnerInfo.HasNoIdenticalLogUnitsOnReception)
				{
					//we need to enter supplier pallet number, so do not ask for batch receiving, because it is not allowed!
					goto Step_ScanLabelStart;
				}
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_TASK",
					"AllowedButtons", buttonKeys.ToArray(),
					"ShowStopButton", false
					));
			msg = WaitForMessage();
			initialErrorKey = null;
			goBack = false;
			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				if (ExtractParameter<string>(msg.Parameters, "buttonKey") == "MSG_BUTTON_BATCH_RECEIVING")
				{
					goto Step_EnterBatchReceiving;
				}
				else
				{
					goto Step_ScanLabelStart;
				}
			}
			goBack = true;
			//go back
			goto Step_GoBack;


			#endregion

			Step_EnterBatchReceiving:

			#region ENTER BATCH RECEIVING

			batchReceivingNumber = 1;

			//ask for quantity
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_BATCH_RECEIVING",
					"BarcodeController", barcodeController,
					"InitialQuantity", 1,
					"NumberOfDigitsAfterDecimal", 0,
					"MaximumQuantity", -1,
					"UOM", "",
					"SetFocusToQuantityControl", true
				   ));
			msg = WaitForMessage();

			//clear data
			initialErrorKey = null;

			if (msg.Name.EndsWith(".QuantityEntered"))
			{
				batchReceivingNumber = Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
				goto Step_ScanLabelStart;
			}

			goto Step_AskBatchReceiving;

			#endregion

			Step_ScanLabelStart:

			#region SCAN LABEL START


			titleKey = "MSG_TITLE_SCAN_GS1_LABEL";
			allowedButtonsForLabelScreen = new List<string>();
			allowedButtonsForLabelScreen.Add("MSG_BUTTON_NO_LABEL");
			if (!enterReason && allowToEnterReason)
			{
				allowedButtonsForLabelScreen.Add("MSG_BUTTON_ENTER_REASON_RECEPTION");
			}


			barcode = null;
			currentLogUnit = new LogisticUnitGoodsReceipt();

			goto Step_ScanLabel;

			#endregion

			Step_ScanLabel:

			#region SCAN LABEL

			scannedItemCode = null;
			lineNr = null;

			if (!allowLabelsToBeScannedOnVendor && !containerDocEntry.HasValue)
			{
				goto Step_CheckSSCC;
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
			DefaultCultureInfo.Get(), BuildParamCollection(
				"InitialErrorKey", initialErrorKey,
				"TitleKey", titleKey,
				"BarcodeController", barcodeController,
				"ImageType", GS1ImageType.PalletImage,
				"AllowedButtons", allowedButtonsForLabelScreen.ToArray(),
				"GS1DataObject", barcode == null ? null : barcode.GS1DataObject
				));
			goBack = false;
			msg = WaitForMessage();
			// Reset latest error
			initialErrorKey = null;

			if (msg.Name.EndsWith(".DataEntered"))
			{
				isGS1DataEnteredOnScanLogDataScreen = true;
				barcode = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

				scannedItemCode = barcode == null ? null : barcode.ItemCode;


				if (this.IsProductOnBarcode(barcode))
				{
					gtin = BarcodeHelper.GetProductIdentification(barcode);

					query = BuildQuery.GetCodeNameItemsOrderByName(null, gtin, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, true, crossDocking, containerDocEntry, false, isNoPo, autoSelectPO, scannedItemCode, sboProviderService.GetDbTool());
					DataSet dsTempItemsForBarcode = sboProviderService.RunView(true, null, null, query);
					if (GetNumberOfRows(dsTempItemsForBarcode) == 0)
					{
						initialErrorKey = "MSG_ERROR_INCORRECT_PRODUCT";
						goto Step_ScanLabelStart;
					}

					itemCode = dsTempItemsForBarcode.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
					itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode);

					if (autoSelectPO)
					{
						query = BuildQuery.GetOpenPOLinesForCardCodeAndItemCode(itemCode, cardCode, sboProviderService.GetDbTool());
						DataSet tempDS = sboProviderService.RunView(true, null, null, query);
						if (GetNumberOfRows(tempDS) == 0)
						{
							initialErrorKey = "MSG_ERROR_CANNOT_FIND_OPEN_PO_LINES_FOR_CARDCODE_AND_ITEMCODE";
							goto Step_ScanLabelStart;
						}
					}
				}

				if (
					this.IsSSCCOnBarcode(barcode.GS1DataObject) && itemInfo != null && (
					 ((itemInfo.HasBatchnumber && this.IsBatchNumberOnBarcode(barcode.GS1DataObject)) || (!itemInfo.HasBatchnumber && !this.IsBatchNumberOnBarcode(barcode.GS1DataObject)))
					&& ((itemInfo.HasBestBeforeDate & this.IsBestBeforeDateOnBarcode(barcode.GS1DataObject)) || (!itemInfo.HasBestBeforeDate & !this.IsBestBeforeDateOnBarcode(barcode.GS1DataObject)))
					)
					)
				{
					goto Step_CheckSSCC;
				}
				//else if (this.IsProductOnBarcode(barcode))
				//{
				//    itemCode = BarcodeHelper.GetProductIdentification(barcode);
				//    itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode);

				//    if (itemInfo.HasBatchnumber && this.IsBatchNumberOnBarcode(barcode.GS1DataObject))
				//    {
				//        batchNumber = barcode.GS1DataObject.LotNumber;
				//    }

				//    if (itemInfo.HasBestBeforeDate && this.IsBestBeforeDateOnBarcode(barcode.GS1DataObject))
				//    {
				//        bestBeforeDate = barcode.GS1DataObject.BestBeforeDate;
				//    }
				//}
				else
				{
					titleKey = "MSG_TITLE_SCAN_NEXT_GS1_LABEL";

					allowedButtonsForLabelScreen = new List<string>();
					allowedButtonsForLabelScreen.Add("MSG_BUTTON_DONE");

					if (!enterReason && allowToEnterReason)
					{
						allowedButtonsForLabelScreen.Add("MSG_BUTTON_ENTER_REASON_RECEPTION");
					}

					goto Step_ScanLabel;
				}

			}
			else if (msg.Name.EndsWith(".BackRequested"))
			{
				goBack = true;
				goto Step_GoBack;
			}
			else if (msg.Name.EndsWith(".ButtonClicked"))
			{
				//get buttonkey
				string btnKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

				if (btnKey == "MSG_BUTTON_ENTER_REASON_RECEPTION")
				{
					enterReason = true;
					goto Step_ScanLabelStart;
				}

				goto Step_CheckSSCC;
			}

			initialErrorKey = "MSG_ERROR_OPERATION_NOT_SUPPORTED";
			goto Step_ScanLabel;


			#endregion

			Step_CheckSSCC:

			#region CHECK SSCC

			SSCC = null;
			scannedSSCC = null;
			luid = null;

			if (barcode != null &&
				barcode.GS1DataObject != null &&
				!string.IsNullOrEmpty(barcode.GS1DataObject.Sscc))
			{
				scannedSSCC = barcode.GS1DataObject.Sscc;

				//check if the scanned SSCC is valid
				if (!barcode.GS1DataObject.ValidateSSCC || PmxSSCCProvider.IsValidChecksumSSCC(scannedSSCC))
				{
					SSCC = scannedSSCC;
					int tempLUID = sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, new DelegateWithPmxDbConnection<int>(ExecuteGetLUIDBySSCC), SSCC);
					if (tempLUID > 0)
					{
						luid = tempLUID;
					}
					else
					{
						//no luid, so check if scanned sscc already exists in the 'in memory' list
						foreach (LogisticUnitGoodsReceipt foundLUID in purchaseDeliveryNoteList)
						{
							if (foundLUID.SSCC == SSCC)
							{
								luid = foundLUID.LogisticUnitId;
								break;
							}
						}

						if (!luid.HasValue && purchaseDocRef != null)
						{
							//Still no LUID, so get it from ASN table
							luid = GetLuidFromASN(sboProviderService, cardCode, containerDocEntry, autoSelectPO, purchaseDocRef, SSCC);
						}
					}



					goto Step_SelectLogisticCarrier;
				}
				else
				{
					//invalid
					initialErrorKey = "MSG_ERROR_INVALID_SSCC";
					goto Step_ScanLabelStart;
				}
			}

			goto Step_SelectLogisticCarrier;


			#endregion

			Step_SelectLogisticCarrier:

			#region SELECT LOGISTIC CARRIER


			logisticCarrierCode = null;
			currentLogisticCarrier = null;

			//set luid
			currentLogUnit.LogisticUnitId = luid;
			currentLogUnit.SSCC = SSCC;

			PmxItemInfo tempItemInfo = null;

			if (this.IsLUOnBarcode(barcode))
			{
				tempGTIN = BarcodeHelper.GetProductIdentification(barcode);
				scannedItemCode = barcode == null ? null : barcode.ItemCode;

				query = BuildQuery.GetCodeNameItemsOrderByName(null, tempGTIN, purchaseDocRef, whsInfo, null, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, true, false, null, false, isNoPo, autoSelectPO, scannedItemCode, sboProviderService.GetDbTool());
				DataSet ds = sboProviderService.RunView(true, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{
					//get itemcode of first row
					string tempItemCode = ds.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
					tempItemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), tempItemCode);

				}


			}

			if (luid > 0 ||
				(this.IsLUOnBarcode(barcode) && tempItemInfo != null && tempItemInfo.IsLogisticUnit) ||
				(currentLogUnit != null && currentLogUnit.ItemsOnLogisticUnit.Count > 0))
			{
				//we do not need to add logistic carrier
				goto Step_EnterSupplierPalletNumber;
			}


			//ask for logistic carrier
			query = BuildQuery.GetLogisticCarriersOrderByName(false, true, sboProviderService.GetDbTool());
			dsLogisticCarriers = sboProviderService.RunView(false, null, null, query);

			if (this.GetNumberOfRows(dsLogisticCarriers) == 0 ||
				!allowLogisticCarriersOnVendor)
			{
				//There are no logistic carriers, so we do not show screen
				logisticCarrierCode = null;
				goto Step_EnterSupplierPalletNumber;
			}


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
					"ProductDS", dsLogisticCarriers,
					"ButtonKey", "MSG_BUTTON_NO_LOGISTIC_CARRIER"
					));
			msg = WaitForMessage();
			goBack = false;
			//clear data
			initialErrorKey = null;

			if (msg.Name.EndsWith(".ProductSelected"))
			{
				logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

				//add to list

				currentLogisticCarrier = new LogisticUnitItemGoodReceipt();
				currentLogisticCarrier.BatchNumber1 = null;
				currentLogisticCarrier.BatchNumber2 = null;
				currentLogisticCarrier.BestBeforeDate = null;
				currentLogisticCarrier.ItemCode = logisticCarrierCode;
				currentLogisticCarrier.Quantity = 1;
				currentLogisticCarrier.IsLogisticCarrierForLogisticUnit = true;
				currentLogisticCarrier.IsLogisticCarrier = true;

				goto Step_EnterSupplierPalletNumber;
			}

			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				logisticCarrierCode = null;
				goto Step_EnterSupplierPalletNumber;
			}

			goBack = true;
			//go back
			goto Step_GoBack;



			#endregion

			Step_EnterSupplierPalletNumber:

			#region ENTER SUPPLIER PALLET NUMBER

			supplierPalletNumber = null;

			if (needToEnterSupplierPalletNumber)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
				 DefaultCultureInfo.Get(), BuildParamCollection(
					 "InitialErrorKey", initialErrorKey,
					 "TitleKey", "MSG_TITLE_ENTER_SUPPLIER_PALLET_NUMBER",
					 "ForceDataEntry", true,
					 "ForceUppercase", true,
					 "AllowToGoBack", true,
					 "Information", "MSG_ENTER_SUPPLIER_PALLET_NUMBER"
					 ));
				msg = WaitForMessage();
				goBack = false;
				// reset error
				initialErrorKey = null;

				if (msg.Name.EndsWith(".StringEntered"))
				{
					supplierPalletNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
					currentLogUnit.SupplierPalletNumber = supplierPalletNumber;
					goto Step_IdentifyProduct;
				}
				else if (msg.Name.EndsWith(".BackRequested"))
				{
					goBack = true;
					goto Step_GoBack;
				}
			}

			if (goBack
				//&& (!allowLogisticCarriersOnVendor || string.IsNullOrEmpty(whsInfo.StorageLocationCodeLogisticCarriers) &&
				//        !allowLabelsToBeScannedOnVendor && businessPartnerInfo.HasNoIdenticalLogUnitsOnReception)
				)
			{
				goto Step_GoBack;
			}

			goto Step_IdentifyProduct;


			#endregion

			Step_GoBack:

			#region GO BACK

			//go back
			if ((currentLogUnit != null &&
				currentLogUnit.ItemsOnLogisticUnit.Count > 0))
			{
				//we want to go back, but we have some items in the list
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_PRODUCTS_ON_LOGISTIC_UNIT_TO_RECEIVE",
						"MessageKey", "MSG_QUESTION_PRODUCTS_ON_LIST_TO_RECEIVE",
						"AllowToGoBack", true,
						"ButtonKeyYes", "MSG_BUTTON_BOOK_RECEIPT",
						"ButtonKeyNo", "MSG_BUTTON_CANCEL_RECEIPT"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					//receive the items
					goto Step_AskEndQualityValues;
				}
				else if (msg.Name.EndsWith(".No"))
				{
					//Do nothing
				}
				else
				{
					//Go back
					//Set goBack to false, because we reached this point because of a go back
					//If we don't set it to false, we will end up here again ...
					goBack = false;
					goto Step_ScanLabelStart;
				}
			}

			bool asnFound = false;

			//Check if we have ASN
			if (containerDocEntry.HasValue || purchaseDocRef != null)
			{
				//Only if there are base documents we can remove the ASN 

				//get asn
				asnFound = sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

						List<PmxAdvanceShippingNotice> asnBOs = asnProv.GetOpenAsnList(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : purchaseDocRef.DocType,
							containerDocEntry.HasValue ? containerDocEntry.Value : purchaseDocRef.DocEntry, "", 1, conn.InternalKeySboUser);
						return asnBOs.Count > 0;

					}));
			}
			else if (autoSelectPO)
			{
				//get asn
				asnFound = sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

						List<PmxAdvanceShippingNotice> asnBOs = asnProv.GetOpenAsnListForAutoSelectPO(cardCode, conn.InternalKeySboUser, "", 1);
						return asnBOs.Count > 0;

					}));
			}

			if (asnFound)
			{
				//we want to go back, but we have some items in the list
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_PRODUCTS_ON_LIST_TO_RECEIVE",
						"MessageKey", "MSG_QUESTION_PRODUCTS_ON_LIST_TO_RECEIVE",
						"AllowToGoBack", true,
						"ButtonKeyYes", "MSG_BUTTON_BOOK_RECEIPT",
						"ButtonKeyNo", "MSG_BUTTON_CANCEL_RECEIPT"
						));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					//there are pallets received, but user wants to go back, so we will receive them right now
					goBack = true;

					// Inform the user items are receiving
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", "MSG_RECEIVING_ALL_ITEMS",
							"NoTranslationOfMessageKey", false,
							"ShowButton", false));

					goto Step_AskEndQualityValues;
				}
				else if (msg.Name.EndsWith(".No"))
				{
					//Cancel, so delete the entries
					sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

						if (autoSelectPO)
						{
							asnProv.RemoveASNForAutoSelectPO(cardCode, conn.InternalKeySboUser);
						}
						else
						{
							asnProv.RemoveASNForUser(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : purchaseDocRef.DocType,
								containerDocEntry.HasValue ? containerDocEntry.Value : purchaseDocRef.DocEntry, null, conn.InternalKeySboUser);
						}
						return null;

					}));
				}
				else
				{
					//Go back
					//Set goBack to false, because we reached this point because of a go back
					//If we don't set it to false, we will end up here again ...
					goBack = false;
					goto Step_SelectPOFilter;
				}
			}

			//clear data
			cardCode = null;

			if (filterButtonKey == "MSG_BUTTON_PO_FILTER_NO_PO")
			{
				goto Step_SelectPOFilter;
			}
			else if (filterButtonKey == "MSG_BUTTON_PO_FILTER_EAN")
			{
				goto Step_FilterOnEAN;
			}
			else if (isContainer)
			{
				goBack = true;
				goto Step_SelectContainer;
			}
			else
			{
				goBack = true;
				goto Step_SelectOrder;
			}

			#endregion

			Step_ClearDataBeforeNextItem:

			#region CLEAR DATA BEFORE NEXT ITEM


			barcode = null;

			goto Step_IdentifyProduct;

			#endregion

			Step_IdentifyProduct:

			#region IDENTIFY PRODUCT

			itemCode = null;
			itemDescription = null;
			itemInfo = null;
			numberOfItemsInPurchaseUnit = null;
			purchaseUom = null;

			if (goBack)
			{
				barcode = null;
			}

			tempGTIN = BarcodeHelper.GetProductIdentification(barcode);
			scannedItemCode = barcode == null ? null : barcode.ItemCode;

			query = BuildQuery.GetCodeNameItemsOrderByName(null, tempGTIN, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, true, crossDocking, containerDocEntry, false, isNoPo, autoSelectPO, scannedItemCode, sboProviderService.GetDbTool());
			dsItems = sboProviderService.RunView(true, null, null, query);
			query = BuildQuery.GetCodeNameItemsOrderByName(null, tempGTIN, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, scannedItemCode, sboProviderService.GetDbTool());
			dsItemsNonDistinct = sboProviderService.RunView(true, null, null, query);

			RemoveReceivedItems(purchaseDeliveryNoteList, dsItemsNonDistinct, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

			// If items removed in dsItemsNonDistinct => check dsItems if they also need to be removed
			for (int i = 0; i < dsItems.Tables[0].Rows.Count; i++)
			{
				string productCode = Convert.ToString(dsItems.Tables[0].Rows[i][ProductDataSet.Columns.PRODUCT_CODE.Name]);
				// If there are items in the non distincted dataset, keep the item in the distincted dataset.
				if (dsItemsNonDistinct.Tables[0].Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + productCode + "'").Length == 0)
				{
					dsItems.Tables[0].Rows[i].Delete();
				}
			}

			dsItems.AcceptChanges();

			//if (this.GetNumberOfRows(dsItems) == 1 &&
			//    !goBack)
			//{
			//    itemCode = dsItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
			//    itemDescription = dsItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name] as string;

			//    gtin = tempGTIN;

			//    if (dsItems.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME))
			//    {
			//        numberOfItemsInPurchaseUnit = Convert.ToDouble(dsItems.Tables[0].Rows[0][SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME]);
			//    }

			//    goto Step_GetItemInfo;
			//}

			if (this.GetNumberOfRows(dsItems) == 0 && !errorInGS1IdentifyProductScreen)
			{
				//product not found
				initialErrorKey = "MSG_ERROR_INCORRECT_PRODUCT";
				goto Step_ScanLabelStart;
			}

			if (!string.IsNullOrEmpty(tempGTIN) &&
				this.GetNumberOfRows(dsItems) == 1)
			{
				if (goBack)
				{
					goto Step_ScanLabelStart;
				}
				itemCode = dsItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
				itemDescription = dsItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name] as string;

				gtin = tempGTIN;

				if (dsItemsNonDistinct.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME))
				{
					numberOfItemsInPurchaseUnit = Convert.ToDouble(dsItemsNonDistinct.Tables[0]
						.Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
						[0][SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME]);
					purchaseUom = Convert.ToString(dsItemsNonDistinct.Tables[0]
						.Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
						[0][SboPurchaseOrderLineTable.Columns.UomCode.NAME]);
				}
				if (checkDataInUDFPOLine && isPOManuallySelected)
				{
					if (GetNumberOfRows(dsItemsNonDistinct) > 1)
					{
						dsPOLines = dsItemsNonDistinct;
						goto Step_SelectProduct;
					}
					else
					{
						lineNr = Convert.ToInt32(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.LINE_NUMBER.Name].ToString());
						batchNumber1ToCheck = ExtractStringValuesFromPipes(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BATCH1.Name].ToString());
						batchNumber2ToCheck = ExtractStringValuesFromPipes(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BATCH2.Name].ToString());
						bestBeforeDatesToCheck = ExtractDateTimeValuesFromPipes(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString());
						serialNumbersToCheck = ExtractStringValuesFromPipes(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.SERIAL_NUMBER_INFO.Name].ToString());
					}
				}
				goto Step_GetItemInfo;
			}

			allowedButtonsIdentifyProductScreen = new List<string>();

			if (!PmxOseCompany.Get().DisableItemSelectionInFlows)
			{
				allowedButtonsIdentifyProductScreen.Add("MSG_BUTTON_SELECT_PRODUCT");
			}

			if ((currentLogUnit != null &&
				currentLogUnit.ItemsOnLogisticUnit.Count > 0))
			{
				//we have data in lists
				allowedButtonsIdentifyProductScreen.Add("MSG_BUTTON_RECEIVE_ITEMS");
			}


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_IDENTIFY_PRODUCT",
					"BarcodeController", barcodeController,
					"AllowedButtons", allowedButtonsIdentifyProductScreen.Count > 0 ? allowedButtonsIdentifyProductScreen.ToArray() : null,
					"ImageType", GS1ImageType.BoxImage
					));
			msg = WaitForMessage();
			goBack = false;
			//clear data
			initialErrorKey = null;
			//We need to keep old barcode scanned on logistic unit screen
			//barcode = null;
			gtin = null;
			itemInfo = null;
			errorInGS1IdentifyProductScreen = false;
			if (msg.Name.EndsWith(".DataEntered"))
			{
				isGS1DataEnteredOnScanLogDataScreen = false;
				BarcodeContainer tempBarcode = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

				//check if there is a product present
				if (tempBarcode != null &&
				(!string.IsNullOrEmpty(BarcodeHelper.GetProductIdentification(tempBarcode)) || !string.IsNullOrEmpty(tempBarcode.ItemCode)))
				{
					if (string.IsNullOrEmpty(tempBarcode.FreeBarcode) ||
						!string.IsNullOrEmpty(tempBarcode.ItemCode))
					{
						//EAN128 barcode scanned, so copy it to global one
						barcode = tempBarcode;
					}

					gtin = BarcodeHelper.GetProductIdentification(tempBarcode);

					query = BuildQuery.GetCodeNameItemsOrderByName(null, gtin, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, true, crossDocking, containerDocEntry, false, isNoPo, autoSelectPO, tempBarcode.ItemCode, sboProviderService.GetDbTool());
					DataSet dsTempItems = sboProviderService.RunView(true, null, null, query);

					query = BuildQuery.GetCodeNameItemsOrderByName(null, gtin, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, tempBarcode.ItemCode, sboProviderService.GetDbTool());
					DataSet dsTempItemsNonDistinct = sboProviderService.RunView(true, null, null, query);


					RemoveReceivedItems(purchaseDeliveryNoteList, dsTempItemsNonDistinct, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

					if (GetNumberOfRows(dsTempItems) > 0 &&
						GetNumberOfRows(dsTempItemsNonDistinct) > 0)
					{
						//copy the data
						itemCode = dsTempItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
						itemDescription = dsTempItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name] as string;

						if (dsTempItemsNonDistinct.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME))
						{
							numberOfItemsInPurchaseUnit = Convert.ToDouble(dsTempItemsNonDistinct.Tables[0]
							  .Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
							  [0][SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME]);
							purchaseUom = Convert.ToString(dsTempItemsNonDistinct.Tables[0]
								.Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
								[0][SboPurchaseOrderLineTable.Columns.UomCode.NAME]);
						}

						if (checkDataInUDFPOLine && isPOManuallySelected)
						{
							if (GetNumberOfRows(dsTempItemsNonDistinct) > 1)
							{
								dsPOLines = dsTempItemsNonDistinct;
								goto Step_SelectProduct;
							}
							else
							{
								lineNr = Convert.ToInt32(dsItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.LINE_NUMBER.Name].ToString());
								batchNumber1ToCheck = ExtractStringValuesFromPipes(dsTempItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BATCH1.Name].ToString());
								batchNumber2ToCheck = ExtractStringValuesFromPipes(dsTempItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BATCH2.Name].ToString());
								bestBeforeDatesToCheck = ExtractDateTimeValuesFromPipes(dsTempItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString());
								serialNumbersToCheck = ExtractStringValuesFromPipes(dsTempItemsNonDistinct.Tables[0].Rows[0][ProductDataSet.Columns.SERIAL_NUMBER_INFO.Name].ToString());
								lineQty = Convert.ToDouble(dsTempItemsNonDistinct.Tables[0].Rows[0][SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME]);// *numberOfItemsInPurchaseUnit;
							}
						}
						goto Step_GetItemInfo;

					}
					else
					{
						//reset data
						BarcodeHelper.ResetProductIdentification(barcode);
						initialErrorKey = "MSG_ERROR_PRODUCT_NOT_FOUND";
						errorInGS1IdentifyProductScreen = true;
						goto Step_IdentifyProduct;
					}

				}
				else
				{
					//reset data
					BarcodeHelper.ResetProductIdentification(barcode);
					initialErrorKey = "MSG_ERROR_NO_PRODUCT_ON_BARCODE";
					goto Step_IdentifyProduct;
				}

			}

			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				string tempButtonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

				if (tempButtonKey == "MSG_BUTTON_SELECT_PRODUCT")
				{
					//select the product
					dsPOLines = dsItemsNonDistinct;
					goto Step_SelectProduct;
				}
				else
				{
					//receive items
					// Inform the user items are receiving
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", "MSG_RECEIVING_ALL_ITEMS",
							"NoTranslationOfMessageKey", false,
							"ShowButton", false));

					goto Step_AskEndQualityValues;
				}
			}

			//go back
			goBack = true;
			if (currentLogUnit != null &&
				currentLogUnit.ItemsOnLogisticUnit.Count > 0)
			{
				//we want to go back, but we have some items in the list
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
						"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_ScanLabelStart;
				}

				goto Step_IdentifyProduct;

			}
			else
			{
				if (!allowLabelsToBeScannedOnVendor)
				{
					goto Step_AskBatchReceiving;
				}
				goto Step_ScanLabelStart;
			}

			#endregion

			Step_SelectProduct:

			#region SELECT PRODUCT


			if (isPOManuallySelected && checkDataInUDFPOLine)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectPOLineScreen),
			   DefaultCultureInfo.Get(), BuildParamCollection(
				   "InitialErrorKey", initialErrorKey,
				   "TitleKey", "MSG_TITLE_SELECT_PRODUCT",
				   "POLineDS", dsPOLines
				   ));
				msg = WaitForMessage();
				//clear data
				initialErrorKey = null;
				goBack = false;

				if (msg.Name.EndsWith(".POLineSelected"))
				{
					lineNr = ExtractParameter<int?>(msg.Parameters, "LineNr");

					DataRow[] rows = dsItemsNonDistinct.Tables[0].Select(string.Format("{0} = '{1}'", ProductDataSet.Columns.LINE_NUMBER.Name, lineNr));
					if (rows.Length > 0)
					{
						itemCode = rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name] as string;
						itemDescription = rows[0][ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name] as string;
						batchNumber1ToCheck = ExtractStringValuesFromPipes(rows[0][ProductDataSet.Columns.BATCH1.Name].ToString());
						batchNumber2ToCheck = ExtractStringValuesFromPipes(rows[0][ProductDataSet.Columns.BATCH2.Name].ToString());
						bestBeforeDatesToCheck = ExtractDateTimeValuesFromPipes(rows[0][ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString());
						serialNumbersToCheck = ExtractStringValuesFromPipes(rows[0][ProductDataSet.Columns.SERIAL_NUMBER_INFO.Name].ToString());
						numberOfItemsInPurchaseUnit = Convertor.ConvertToDoubleNullable(rows[0][SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME]);
						purchaseUom = rows[0][SboPurchaseOrderLineTable.Columns.UomCode.NAME] as string;
						lineQty = Convert.ToDouble(rows[0][SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME]);// *numberOfItemsInPurchaseUnit;
					}

					goto Step_GetItemInfo;
				}

				goBack = true;
				goto Step_IdentifyProduct;

			}


			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_PRODUCT",
					"ProductDS", dsItems
					));
			msg = WaitForMessage();

			//clear data
			initialErrorKey = null;

			if (msg.Name.EndsWith(".ProductSelected"))
			{
				itemCode = ExtractParameter<string>(msg.Parameters, "itemcode");
				//get itemdescription
				DataRow[] rows = dsItems.Tables[0].Select(string.Format("{0} = '{1}'", ProductDataSet.Columns.PRODUCT_CODE.Name, itemCode));
				if (rows.Length > 0)
				{
					//copy the data
					itemDescription = rows[0][ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name] as string;
				}

				if (dsItemsNonDistinct.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME))
				{
					numberOfItemsInPurchaseUnit = Convert.ToDouble(dsItemsNonDistinct.Tables[0]
						.Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
						[0][SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME]);
					purchaseUom = Convert.ToString(dsItemsNonDistinct.Tables[0]
						.Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "'")
						[0][SboPurchaseOrderLineTable.Columns.UomCode.NAME]);
				}

				goto Step_GetItemInfo;
			}
			goBack = true;
			goto Step_IdentifyProduct;

			#endregion

			Step_GetItemInfo:

			#region GET ITEM INFO

			if (autoSelectPO)
			{
				query = BuildQuery.GetOpenPOLinesForCardCodeAndItemCode(itemCode, cardCode, sboProviderService.GetDbTool());
				DataSet tempDS = sboProviderService.RunView(true, null, null, query);
				if (GetNumberOfRows(tempDS) == 0)
				{
					initialErrorKey = "MSG_ERROR_CANNOT_FIND_OPEN_PO_LINES_FOR_CARDCODE_AND_ITEMCODE";
					goto Step_ClearDataBeforeNextItem;
				}
			}



			if (itemInfo == null)
			{
				itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode);
			}

			//if (batchReceivingNumber > 1 &&
			//    (itemInfo.TrackLocationSerialNumber ||
			//    ((itemInfo.HasSAPSerialNumber || itemInfo.HasSerialNumber) && itemInfo.ManageMethod == SboItemManageMethod.OnEveryTransaction)))
			//{
			//    initialErrorKey = "MSG_ERROR_BATCH_RECEIVING_NOT_ALLOWED_FOR_SERIAL_NUMBERS";
			//    goto Step_IdentifyProduct;
			//}

			if (checkDataInUDFPOLine && lineQty.HasValue)
			{
				stillToReceiveInventory = lineQty.Value;
			}
			else
			{
				stillToReceiveInventory = GetStillToReceive(dsItemsNonDistinct, itemCode, numberOfItemsInPurchaseUnit);
			}

			if (stillToReceiveInventory == 0 &&
				((purchaseDocRef != null && purchaseDocRef.DocType == SboObjectTypes.PurchaseInvoice) ||
			(purchaseDocRef != null && !allowToExceedOrderedQuantity) ||
			(containerDocEntry.HasValue && !allowToExceedOrderedQuantity)))
			{
				//No more items to receive, but user selected/scanned this item
				initialErrorKey = "MSG_RECEPTION_MAXIMUM_EXCEEDED";
				if (barcode != null)
				{
					barcode.ClearProductInfo();
				}
				goto Step_IdentifyProduct;
			}

			if (!numberOfItemsInPurchaseUnit.HasValue)
			{
				//take default 
				numberOfItemsInPurchaseUnit = itemInfo.NumberOfItemsPerPurchaseUnit;
				purchaseUom = itemInfo.PurchaseUom;
			}

			//        if (crossDocking && (itemInfo.HasSAPSerialNumber || itemInfo.HasSerialNumber))
			//        {
			//            initialErrorKey = "MSG_ERROR_CROSS_DOCKING_NOT_ALLOWED_FOR_SERIAL_NUMBERS";
			//if (barcode != null)
			//{
			//	barcode.ClearProductInfo();
			//}
			//            goto Step_IdentifyProduct;
			//        }

			goto Step_ShowPurchaseLineRemarks;

			#endregion

			Step_ShowPurchaseLineRemarks:

			#region SHOW PURCHASE LINE REMARKS

			if (!string.IsNullOrEmpty(purchaseLineRemarksViewName) && !autoSelectPO && (purchaseDocRef != null || containerDocEntry != null))
			{
				if (purchaseDocRef != null)
					query = BuildQuery.GetPurchaseLineRemarks(purchaseDocRef.DocEntry, purchaseDocRef.DocType, lineNr, itemCode, purchaseLineRemarksViewName, sboProviderService.GetDbTool());
				else
					query = BuildQuery.GetPurchaseLineRemarks((int)containerDocEntry, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer, lineNr, itemCode, purchaseLineRemarksViewName, sboProviderService.GetDbTool());

				DataSet ds = sboProviderService.RunView(true, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{

					ds.Tables[0].Columns.Remove(PmxReceptionLineRemarksViewDefinition.Columns.DocEntry.NAME);
					ds.Tables[0].Columns.Remove(PmxReceptionLineRemarksViewDefinition.Columns.ObjType.NAME);
					ds.Tables[0].Columns.Remove(PmxReceptionLineRemarksViewDefinition.Columns.LineNum.NAME);
					ds.Tables[0].Columns.Remove(PmxReceptionLineRemarksViewDefinition.Columns.ItemCode.NAME);

					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowRemarksScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"RemarksTable", ds.Tables[0],
							"EnabledToolBarButtons", ToolBarButtons.Forward));
					WaitForMessage();
				}
			}

			goto Step_AskForReason;

			#endregion

			Step_AskForReason:

			#region ASK FOR REASON

			//this step will only be shown if the user has set that the GS1Label screen for log units is skipped but does want scanner-user to use reasons later on
			if (!allowLabelsToBeScannedOnVendor && allowToEnterReason && !listOfReasonCodeByItem.ContainsKey(itemCode))
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					this.DefaultCultureInfo, BuildParamCollection(
				  "InitialErrorKey", initialErrorKey,
				  "TitleKey", "MSG_TITLE_ASK_TO_ENTER_REASON_LATER",
				  "MessageKey", "MSG_INFO_ASK_TO_ENTER_REASON_LATER",
				  "Parameters", new object[] { StringHelper.Truncate(string.Format("{0} - {1}", itemCode, itemInfo.Description), 50) }
				  ));
				msg = WaitForMessage();

				enterReason = msg.Name.EndsWith(".Yes") ? true : false;
			}

			if (!listOfReasonCodeByItem.ContainsKey(itemCode) &&
				enterReason &&
				allowToEnterReason)
			{
				//ask for reason
				query = BuildQuery.GetReasons();
				dsReasons = sboProviderService.RunView(false, null, null, query);

				reasonsCodesRequiresFreeText = new PmxDictionary<string, bool>();
				foreach (DataRow row in dsReasons.Tables[0].Rows)
				{
					reasonsCodesRequiresFreeText.Add(
						row[ReasonDataSet.Columns.REASON_CODE.Name].ToString(),
						(SboBooleanConvertor.ObjectToBooleanOrNull(row[ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name]).HasValue ? (SboBooleanConvertor.ObjectToBooleanOrNull(row[PmxReasonTable.Columns.RequiresFreeText.NAME])).Value : false)
						);
				}

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreen),
				  this.DefaultCultureInfo, BuildParamCollection(
				  "InitialErrorKey", initialErrorKey,
				  "TitleKey", "MSG_TITLE_SELECT_REASON",
				  "MessageKey", "MSG_SELECT_PURCHASE_RECEIPT_REASON",
				  "MessageParams", new object[] { StringHelper.Truncate(string.Format("{0} - {1}", itemCode, itemInfo.Description), 50) },
				  "ReasonDS", dsReasons));
				msg = WaitForMessage();
				initialErrorKey = null;

				if (msg.Name != "Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreenReply.ReasonSelected")
				{
					//go back
					if (this.GetNumberOfRows(dsItems) == 1)
					{
						//go back
						if (currentLogUnit != null &&
							currentLogUnit.ItemsOnLogisticUnit.Count > 0)
						{
							//we want to go back, but we have some items in the list
							//ask user what to do
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
									"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
							msg = WaitForMessage();

							if (msg.Name.EndsWith(".Yes"))
							{
								goto Step_ScanLabelStart;
							}

							goto Step_SelectBatchNumber;
						}
						goBack = true;
						goto Step_IdentifyProduct;
					}

					goBack = true;
					//there where more items in the list
					goto Step_IdentifyProduct;
				}

				reasonCode = ExtractParameter<string>(msg.Parameters, "reasonCode");

				DataRow[] rows = dsReasons.Tables[0].Select(String.Format("{0} = '{1}'", ReasonDataSet.Columns.REASON_CODE.Name, reasonCode));
				if (rows.Length > 0)
				{
					reasonName = rows[0][ReasonDataSet.Columns.REASON_NAME.Name] as string;
				}

				#region ENTER REASON FREE TEXT

				if (reasonsCodesRequiresFreeText.ContainsKey(reasonCode) &&
					reasonsCodesRequiresFreeText[reasonCode])
				{


					// Show screen with list
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_ENTER_REASON",
							"Information", "MSG_ENTER_FREE_REASON_DEFAULT",
							"Parameters", new object[] { reasonName },
							"AllowToGoBack", true,
							"ForceDataEntry", true,
							"AllowMultiLine", true,
							"MinimumNumberOfCharacters", Definitions.MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON
							));

					msg = WaitForMessage();

					initialErrorKey = null;

					if (msg.Name.EndsWith(".StringEntered"))
					{
						reasonFreeText = ExtractParameter<string>(msg.Parameters, "stringValue");

					}
					else
					{
						//go back
						goto Step_AskForReason;
					}
				}


				#endregion

				listOfReasonCodeByItem.Add(itemCode, new ReasonInfo(reasonCode, reasonFreeText));

			}


			goto Step_SelectZoneTypeStart;

			#endregion

			Step_SelectZoneTypeStart:

			#region SELECT ZONE TYPE START

			listOfReasonsCodesAndZoneTypes = new PmxDictionary<string, ReasonInfo>();

			//query = BuildQuery.GetZoneTypesOnItem(itemInfo.ItemCode, current_DefaultCultureInfo);
			query = BuildQuery.GetZoneTypesOnItemCode(itemInfo.ItemCode, current_DefaultCultureInfo, sboProviderService.GetDbTool());
			dsZoneTypesAndReasons = sboProviderService.RunView(false, null, null, query);

			dsZoneTypesAndReasons.Tables[0].Columns.Add(ZoneTypeDataSet.Columns.REASON_CODE.Name, ZoneTypeDataSet.Columns.REASON_CODE.Types[0]);
			dsZoneTypesAndReasons.Tables[0].Columns.Add(ZoneTypeDataSet.Columns.REASON_NAME.Name, ZoneTypeDataSet.Columns.REASON_NAME.Types[0]);

			#endregion

			Step_SelectZoneType:

			#region SELECT ZONE TYPE

			if (GetNumberOfRows(dsZoneTypesAndReasons) == 0)
			{
				goto Step_CanItemBeStoredOnLocation;
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectZoneTypeScreen),
			   DefaultCultureInfo.Get(), BuildParamCollection(
				   "InitialErrorKey", initialErrorKey,
				   "TitleKey", "MSG_TITLE_SELECT_ZONE_TYPE",
				  "ZoneTypeDS", dsZoneTypesAndReasons,
				   "ButtonKey", "MSG_SELECT_PURCHASE_RECEIPT_REASON_SELECT_ZONETYPE",
				   "ShowStopButton", false
				   ));

			msg = WaitForMessage();
			initialErrorKey = null;
			goBack = false;

			if (msg.Name.EndsWith(".ButtonClicked"))
			{
				zoneTypeCode = ExtractParameter<string>(msg.Parameters, "ZoneTypeCode");

				goto Step_AskForReasonForZoneType;
			}
			else if (msg.Name.EndsWith(".ForwardRequested"))
			{
				goto Step_CanItemBeStoredOnLocation;
			}
			else if (msg.Name.EndsWith("BackRequested"))
			{
				goBack = true;
				goto Step_GoBack;
			}
			return;

			#endregion

			Step_AskForReasonForZoneType:

			#region ASK FOR REASON FOR ZONE TYPE

			//ask for reason
			query = BuildQuery.GetReasonsForZoneTypes();
			dsReasons = sboProviderService.RunView(false, null, null, query);

			reasonsCodesRequiresFreeTextForZoneTypeScreen = new PmxDictionary<string, bool>();

			foreach (DataRow row in dsReasons.Tables[0].Rows)
			{
				reasonsCodesRequiresFreeTextForZoneTypeScreen.Add(
					row[ReasonDataSet.Columns.REASON_CODE.Name].ToString(),
					(SboBooleanConvertor.ObjectToBooleanOrNull(row[ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name]).HasValue ? (SboBooleanConvertor.ObjectToBooleanOrNull(row[PmxReasonTable.Columns.RequiresFreeText.NAME])).Value : false)
					);
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreen),
			  this.DefaultCultureInfo, BuildParamCollection(
			  "InitialErrorKey", initialErrorKey,
			  "TitleKey", "MSG_TITLE_SELECT_REASON",
			  "MessageKey", "MSG_SELECT_PURCHASE_RECEIPT_REASON_FOR_ZONE_TYPE",
			  "MessageParams", new object[] { zoneTypeCode, StringHelper.Truncate(string.Format("{0} - {1}", itemCode, itemDescription), 50) },
			  "ReasonDS", dsReasons));

			msg = WaitForMessage();
			initialErrorKey = null;
			goBack = false;
			if (msg.Name != "Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreenReply.ReasonSelected")
			{
				//go back   
				goBack = true;
				goto Step_SelectZoneType;
			}

			reasonCodeForZoneTypeScreen = ExtractParameter<string>(msg.Parameters, "reasonCode");

			DataRow[] temp_rows = dsReasons.Tables[0].Select(String.Format("{0} = '{1}'", ReasonDataSet.Columns.REASON_CODE.Name, reasonCodeForZoneTypeScreen));
			if (temp_rows.Length > 0)
			{
				reasonNameForZoneTypeScreen = temp_rows[0][ReasonDataSet.Columns.REASON_NAME.Name] as string;
			}

			#region ENTER REASON FREE TEXT

			if (reasonsCodesRequiresFreeTextForZoneTypeScreen.ContainsKey(reasonCodeForZoneTypeScreen) &&
				reasonsCodesRequiresFreeTextForZoneTypeScreen[reasonCodeForZoneTypeScreen])
			{
				// Show screen with list
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_ENTER_REASON",
						"Information", "MSG_ENTER_FREE_REASON_DEFAULT",
						"Parameters", new object[] { reasonNameForZoneTypeScreen },
						"AllowToGoBack", true,
						"ForceDataEntry", true,
						"AllowMultiLine", true,
						"MinimumNumberOfCharacters", Definitions.MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON
						));

				msg = WaitForMessage();
				goBack = false;
				initialErrorKey = null;

				if (msg.Name.EndsWith(".StringEntered"))
				{
					reasonFreeTextForZoneTypeScreen = ExtractParameter<string>(msg.Parameters, "stringValue");
				}
				else
				{
					//goback
					goBack = true;
					goto Step_AskForReasonForZoneType;
				}
			}
			#endregion

			if (listOfReasonsCodesAndZoneTypes.ContainsKey(zoneTypeCode))
			{
				listOfReasonsCodesAndZoneTypes[zoneTypeCode] = new ReasonInfo(reasonCodeForZoneTypeScreen, reasonFreeTextForZoneTypeScreen);
			}
			else
			{
				listOfReasonsCodesAndZoneTypes.Add(zoneTypeCode, new ReasonInfo(reasonCodeForZoneTypeScreen, reasonFreeTextForZoneTypeScreen));
			}

			foreach (string zoneTypeCodeItem in listOfReasonsCodesAndZoneTypes.Keys)
			{
				foreach (DataRow row in dsZoneTypesAndReasons.Tables[0].Select(ZoneTypeDataSet.Columns.ZONETYPE_CODE.Name as string + "= '" + zoneTypeCodeItem + "'"))
				{
					row[ZoneTypeDataSet.Columns.REASON_CODE.Name] = listOfReasonsCodesAndZoneTypes[zoneTypeCodeItem].ReasonCode;
					row[ZoneTypeDataSet.Columns.REASON_NAME.Name] = reasonNameForZoneTypeScreen;//dsReasons.Tables[0].Select(ReasonDataSet.Columns.REASON_CODE.Name + "= '" + listOfReasonsCodesAndZoneTypes[zoneTypeCodeItem]. + "'")[0][ReasonDataSet.Columns.REASON_NAME.Name];
				}
			}

			goto Step_SelectZoneType;

			#endregion

			Step_CanItemBeStoredOnLocation:

			#region CAN ITEM BE STORED ON LOCATION

			if (!sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteCanItemBeStoredOnStorageLocation), currentDockCode, itemCode))
			{
				initialErrorKey = "MSG_ERROR_STORAGE_LOCATION_NOT_ALLOWED_FOR_ITEM";
				goto Step_IdentifyProduct;
			}


			batchNumber = null;
			secondBatchNumber = null;
			bestBeforeDate = null;
			quantity = 0;
			quantityUom2 = null;



			#endregion

			Step_SelectBatchNumber:

			#region SELECT BATCH NUMBER

			batchNumber = null;

			if (itemInfo.HasBatchnumber)
			{

				generatedBatch1Info = sboProviderService.InvokeMethodWithDbConnection<GeneratedBatchNumberReception>(false, false, null, null, new DelegateWithPmxDbConnection<GeneratedBatchNumberReception>(ExecuteGenerateBatchNumber1ForReception), purchaseDocRef, null, itemCode, cardCode, true);

				tempBatchNumber = null;

				if (barcode != null &&
					barcode.GS1DataObject != null &&
					!string.IsNullOrEmpty(barcode.GS1DataObject.LotNumber))
				{
					tempBatchNumber = barcode.GS1DataObject.LotNumber.ToUpper();
				}

				//If the batchnumber can be overwritten by GS1 batchnumber and it is filled in then
				//we use this batchnumber
				if (generatedBatch1Info.CanBeOverwrittenByGS1BatchNumber && (tempBatchNumber != null) && (tempBatchNumber.Length > 0))
				{
					if (goBack)
					{
						goto Step_IdentifyProduct;
					}
					//user cannot change batchnumber
					batchNumber = tempBatchNumber;
					goto Step_SecondBatchNumber;
				}
				else
				{
					tempBatchNumber = generatedBatch1Info.BatchNumber;

					if (generatedBatch1Info.IsUserAllowedToChange || string.IsNullOrEmpty(generatedBatch1Info.BatchNumber))
					{
						DataSet dsBatchToSelect = null;

						if (allowToSelectBatch)
						{
							query = BuildQuery.GetAllBatchesInStock(itemInfo.ItemCode);
							dsBatchToSelect = sboProviderService.RunView(true, null, null, query);
						}


						//no batchnumber provided, so ask for batchnumber
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IIdentifyBatchNumberScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_IDENTIFY_BATCH_NUMBER",
							 "PONumber", purchaseOrderDocNum.HasValue ? purchaseOrderDocNum.Value : (int?)null,
							 "ItemDescription", itemDescription,
							 "ItemCode", itemInfo.ItemCode,
							 "ItemBarcode", itemInfo.CodeBars,
							 "StillToReceive", (itemInfo.UomDecimals == 0 ? Math.Floor(stillToReceiveInventory / batchReceivingNumber) : Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals)),
							 "CurrentSSCC", SSCC,
							 "InitialBatchNumber", tempBatchNumber,
							 "AllowToSelect", (this.GetNumberOfRows(dsBatchToSelect) > 0 && allowToSelectBatch),
							 "BatchDS", dsBatchToSelect,
							 //"Uom", string.IsNullOrEmpty(itemInfo.PurchaseUom) ? itemInfo.InventoryUom : itemInfo.PurchaseUom
							 "Uom", itemInfo.InventoryUom
							));

						msg = WaitForMessage();
						goBack = false;
						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".BatchNumberEntered"))
						{
							batchNumber = ExtractParameter<string>(msg.Parameters, "batchNumber");

							goto Step_SecondBatchNumber;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							if (this.GetNumberOfRows(dsItems) == 1)
							{
								//go back
								if (currentLogUnit != null &&
									currentLogUnit.ItemsOnLogisticUnit.Count > 0)
								{
									//we want to go back, but we have some items in the list
									//ask user what to do
									session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
											"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
											"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
									msg = WaitForMessage();

									if (msg.Name.EndsWith(".Yes"))
									{
										goto Step_ScanLabelStart;
									}

									goto Step_SelectBatchNumber;
								}
								goBack = true;
								goto Step_IdentifyProduct;
							}

							goBack = true;
							//there where more items in the list
							goto Step_IdentifyProduct;
						}

					}
					else
					{
						if (goBack)
						{
							goto Step_IdentifyProduct;
						}
						batchNumber = tempBatchNumber;
					}

					goBack = false;

					goto Step_SecondBatchNumber;

				}
			}
			else
			{
				//no batchnumber, so we try to get second batch number
				goto Step_SecondBatchNumber;
			}

			#endregion

			Step_SecondBatchNumber:

			#region ENTER SECOND BATCH NUMBER

			//if allowed, check the given batch
			if (checkDataInUDFPOLine && isPOManuallySelected && batchNumber1ToCheck.Count > 0 &&
		  (itemInfo.HasBatchnumber && !batchNumber1ToCheck.Contains(batchNumber)))
			{
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_ENTERED_DATA_FAILED_CHECK",
						"MessageKey", "MSG_QUESTION_ENTERED_DATA_FAILED_CHECK_BATCH1"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					if (!(barcode != null &&
						barcode.GS1DataObject != null &&
						!string.IsNullOrEmpty(barcode.GS1DataObject.LotNumber)))
					{
						goto Step_SelectBatchNumber;
					}
					else if (isGS1DataEnteredOnScanLogDataScreen)
					{
						goBack = true;
						goto Step_ScanLabelStart;
					}
				}
				goBack = true;
				goto Step_IdentifyProduct;
			}

			secondBatchNumber = null;

			if (itemInfo.HasSecondBatchNumber)
			{
				generatedBatch2Info = sboProviderService.InvokeMethodWithDbConnection<GeneratedBatchNumberReception>(false, false, null, null, new DelegateWithPmxDbConnection<GeneratedBatchNumberReception>(ExecuteGenerateBatchNumber1ForReception), purchaseDocRef, null, itemCode, cardCode, false);

				tempBatchNumber = null;

				if (barcode != null &&
					barcode.GS1DataObject != null &&
					!string.IsNullOrEmpty(barcode.GS1DataObject.LotNumber))
				{
					tempBatchNumber = barcode.GS1DataObject.LotNumber.ToUpper();
				}

				//If the batchnumber can be overwritten by GS1 batchnumber and it is filled in then
				//we use this batchnumber
				if (generatedBatch2Info.CanBeOverwrittenByGS1BatchNumber && (tempBatchNumber != null) && (tempBatchNumber.Length > 0))
				{
					if (goBack)
					{
						goto Step_SelectBatchNumber;
					}
					//user cannot change batchnumber
					secondBatchNumber = tempBatchNumber;
					goto Step_BestBeforeDate;
				}
				else
				{
					tempBatchNumber = generatedBatch2Info.BatchNumber;

					DataSet dsInternalBatchToSelect = null;

					if (allowToSelectBatch)
					{
						query = BuildQuery.GetAllInternalBatchesInStock(itemInfo.ItemCode, batchNumber);
						dsInternalBatchToSelect = sboProviderService.RunView(true, null, null, query);

						if (this.GetNumberOfRows(dsInternalBatchToSelect) == 1)
						{
							tempBatchNumber = dsInternalBatchToSelect.Tables[0].Rows[0][TextDataSet.Columns.TEXT.Name] as string;
						}

					}


					if (generatedBatch2Info.IsUserAllowedToChange || string.IsNullOrEmpty(generatedBatch2Info.BatchNumber))
					{
						//no batchnumber provided, so ask for batchnumber
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IIdentifyBatchNumberScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_IDENTIFY_BATCH_NUMBER2",
							 "PONumber", purchaseOrderDocNum.HasValue ? purchaseOrderDocNum.Value : (int?)null,
							 "ItemDescription", itemDescription,
							 "ItemCode", itemInfo.ItemCode,
							 "ItemBarcode", itemInfo.CodeBars,
							 "StillToReceive", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals),
							 "CurrentSSCC", SSCC,
							 //"Uom", string.IsNullOrEmpty(itemInfo.PurchaseUom) ? itemInfo.InventoryUom : itemInfo.PurchaseUom,
							 "Uom", itemInfo.InventoryUom,
							 "AllowToSelect", (this.GetNumberOfRows(dsInternalBatchToSelect) > 0 && autoSelectInternalBatch),
							 "BatchDS", dsInternalBatchToSelect,
							 "InitialBatchNumber", tempBatchNumber
							));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;
						goBack = false;

						if (msg.Name.EndsWith(".BatchNumberEntered"))
						{
							secondBatchNumber = ExtractParameter<string>(msg.Parameters, "batchNumber");
							goto Step_BestBeforeDate;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
							if (itemInfo.HasBatchnumber)
							{
								goto Step_SelectBatchNumber;
							}
							else
							{
								// if there's no batch number, use the same code as when back is pressed on the batch number
								if (this.GetNumberOfRows(dsItems) == 1)
								{
									//go back
									if (currentLogUnit != null &&
										currentLogUnit.ItemsOnLogisticUnit.Count > 0)
									{
										//we want to go back, but we have some items in the list
										//ask user what to do
										session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
											DefaultCultureInfo.Get(), BuildParamCollection(
												"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
												"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
										msg = WaitForMessage();

										if (msg.Name.EndsWith(".Yes"))
										{
											goto Step_ScanLabelStart;
										}

										goto Step_SelectBatchNumber;
									}
									goBack = true;
									goto Step_IdentifyProduct;
								}

								goBack = true;
								//there where more items in the list
								goto Step_IdentifyProduct;
							}
						}

					}
					else
					{
						if (goBack)
						{
							goto Step_SelectBatchNumber;
						}
						secondBatchNumber = tempBatchNumber;
					}

					goBack = false;

					goto Step_BestBeforeDate;

				}
			}
			else
			{
				//no second batchnumber, so we try to get best before date
				goto Step_BestBeforeDate;
			}

			#endregion

			Step_BestBeforeDate:

			#region PROVIDE BEST BEFORE DATE

			//if allowed, check the given batch2
			if (checkDataInUDFPOLine && isPOManuallySelected && batchNumber2ToCheck.Count > 0 &&
				(itemInfo.HasSecondBatchNumber && !batchNumber2ToCheck.Contains(secondBatchNumber)))
			{
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ENTERED_DATA_FAILED_CHECK",
						"MessageKey", "MSG_QUESTION_ENTERED_DATA_FAILED_CHECK_BATCH2"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					if (!(barcode != null &&
						barcode.GS1DataObject != null &&
						!string.IsNullOrEmpty(barcode.GS1DataObject.LotNumber)))
					{
						goto Step_SecondBatchNumber;
					}
					else if (isGS1DataEnteredOnScanLogDataScreen)
					{
						goBack = true;
						goto Step_ScanLabelStart;
					}
				}
				goBack = true;
				goto Step_IdentifyProduct;
			}

			if (itemInfo.HasBestBeforeDate)
			{

				if (goBack)
				{
					if (itemInfo.HasSecondBatchNumber)
					{
						goto Step_SecondBatchNumber;
					}
					else if (itemInfo.HasBatchnumber)
					{
						goto Step_SelectBatchNumber;
					}


					if (this.GetNumberOfRows(dsItems) == 1)
					{
						//go back
						if (currentLogUnit != null &&
							currentLogUnit.ItemsOnLogisticUnit.Count > 0)
						{
							//we want to go back, but we have some items in the list
							//ask user what to do
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
									"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
							msg = WaitForMessage();

							if (msg.Name.EndsWith(".Yes"))
							{
								goto Step_ScanLabelStart;
							}

							goto Step_BestBeforeDate;
						}
					}
					goto Step_IdentifyProduct;

				}

				if (barcode != null &&
					barcode.GS1DataObject != null &&
					barcode.GS1DataObject.BestBeforeDate.HasValue)
				{

					bestBeforeDate = barcode.GS1DataObject.BestBeforeDate.Value;

					//check if BBD is valid
					if (SboHelper.ConvertDateToSboDate(bestBeforeDate.Value) < SboHelper.ConvertDateToSboDate(DateTime.Now.AddDays(itemInfo.ShelfLifeInDaysReception)))
					{
						// Inform the user the BBD is incorrect
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", "MSG_ERROR_RECEPTION_BBD_DOES_NOT_MEET_SHELF_LIFE",
								"MessageAddendum", bestBeforeDate.Value.ToShortDateString(),
								"ShowButton", true));
						WaitForMessage();
					}

					//now check if there is an item in stock with later BBD
					query = BuildQuery.GetStockWithLaterBBD(itemCode, bestBeforeDate.Value);
					DataSet ds = sboProviderService.RunView(false, null, null, query);

					if (this.GetNumberOfRows(ds) > 0)
					{
						//we found stock with later BBD
						// Inform the user the BBD is incorrect
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", "MSG_ERROR_RECEPTION_LATER_BBD_FOUND",
								"MessageAddendum", ((DateTime)ds.Tables[0].Rows[0][PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME]).ToShortDateString(),
								"ShowButton", true));
						WaitForMessage();
					}

					goto Step_EnterPackagingQuantity;
				}
				else
				{
					generatedBBDInfo = sboProviderService.InvokeMethodWithDbConnection<GeneratedBestBeforeDateReception>(false, false, null, null, new DelegateWithPmxDbConnection<GeneratedBestBeforeDateReception>(ExecuteGenerateBBDForReception), itemCode);

					bestBeforeDate = generatedBBDInfo.GeneratedDateTime.Value;
					allowToChangeGeneratedBBDOnReception = generatedBBDInfo.CanUserChangeGeneratedBBD;

					if (allowToChangeGeneratedBBDOnReception)
					{
						bool allowToCalculateBasedOnManufacturingDate = sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteCheckIfAllowToCalculateBasedOnManufacturingDate));

						goBack = false;

						//ask for best before date
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IEnterBestBeforeDateScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
							 "InitialErrorKey", initialErrorKey,
							 "TitleKey", "MSG_TITLE_ENTER_BEST_BEFORE_DATE",
							 "DefaultDateTime", bestBeforeDate,
							 "PONumber", purchaseOrderDocNum.HasValue ? purchaseOrderDocNum.Value : (int?)null,
							 "ItemDescription", itemDescription,
							 "ItemCode", itemInfo.ItemCode,
							 "ItemBarcode", itemInfo.CodeBars,
							 "StillToReceive", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals),
							 "CurrentSSCC", SSCC,
							 //"Uom", string.IsNullOrEmpty(itemInfo.PurchaseUom) ? itemInfo.InventoryUom : itemInfo.PurchaseUom,
							 "Uom", itemInfo.InventoryUom,
							 "BatchNumber", batchNumber,
							 "ButtonKey", allowToCalculateBasedOnManufacturingDate ? "MSG_TITLE_ENTER_MANUFACTURING_DATE" : string.Empty
							 ));
						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".ButtonClicked"))
						{
							goto Step_ManufacturingDate;
						}
						else if (msg.Name.EndsWith(".BestBeforDateEntered"))
						{
							bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "bestBeforeDate");
							goto Step_CheckStock;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goto Step_BBDBack;
						}
					}

					goto Step_EnterPackagingQuantity;
				}
			}
			else
			{
				//no best before date
				goto Step_EnterPackagingQuantity;
			}

			#endregion

			Step_ManufacturingDate:

			#region MANUFACTURING DATE

			bestBeforeDate = DateTime.Now;

			//ask for manufacturing before date
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IEnterBestBeforeDateScreen),
			 DefaultCultureInfo.Get(), BuildParamCollection(
				 "InitialErrorKey", initialErrorKey,
				 "TitleKey", "MSG_TITLE_ENTER_MANUFACTURING_DATE",
				 "DefaultDateTime", bestBeforeDate,
				 "PONumber", purchaseOrderDocNum.HasValue ? purchaseOrderDocNum.Value : (int?)null,
				 "ItemDescription", itemDescription,
				 "ItemCode", itemInfo.ItemCode,
				 "ItemBarcode", itemInfo.CodeBars,
				 "StillToReceive", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals),
				 "CurrentSSCC", SSCC,
				 "Uom", itemInfo.InventoryUom,
				 "BatchNumber", batchNumber
				 ));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".BestBeforDateEntered"))
			{
				bestBeforeDate = ExtractParameter<DateTime?>(msg.Parameters, "bestBeforeDate");

				generatedBBDInfo = sboProviderService.InvokeMethodWithDbConnection<GeneratedBestBeforeDateReception>(false, false, null, null, new DelegateWithPmxDbConnection<GeneratedBestBeforeDateReception>(ExecuteGenerateBBDForReception), itemCode, bestBeforeDate);

				bestBeforeDate = generatedBBDInfo.GeneratedDateTime.Value;

				goto Step_CheckStock;
			}
			else if (msg.Name.EndsWith(".BackRequested"))
			{
				goto Step_BBDBack;
			}


			#endregion

			Step_BBDBack:

			#region BBDBack:

			goBack = true;
			if (itemInfo.HasSecondBatchNumber)
			{
				goto Step_SecondBatchNumber;
			}
			else if (itemInfo.HasBatchnumber)
			{
				goto Step_SelectBatchNumber;
			}


			if (this.GetNumberOfRows(dsItems) == 1)
			{
				//go back
				if (currentLogUnit != null &&
					currentLogUnit.ItemsOnLogisticUnit.Count > 0)
				{
					//we want to go back, but we have some items in the list
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
							"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_ScanLabelStart;
					}

					goto Step_BestBeforeDate;
				}
			}

			//there where more items in the list
			goto Step_IdentifyProduct;

			#endregion

			Step_CheckStock:

			#region CHECK STOCK
			{
				//check if BBD is valid
				if (SboHelper.ConvertDateToSboDate(bestBeforeDate.Value) < SboHelper.ConvertDateToSboDate(DateTime.Now.AddDays(itemInfo.ShelfLifeInDaysReception)))
				{
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_RECEPTION_BBD_DOES_NOT_MEET_SHELF_LIFE",
							"MessageKey", "MSG_QUESTION_RECEPTION_BBD_DOES_NOT_MEET_SHELF_LIFE"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_BestBeforeDate;
					}
				}

				//now check if there is an item in stock with later BBD
				query = BuildQuery.GetStockWithLaterBBD(itemCode, bestBeforeDate.Value);
				DataSet ds = sboProviderService.RunView(false, null, null, query);

				if (this.GetNumberOfRows(ds) > 0)
				{
					//we found stock with later BBD
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_RECEPTION_LATER_BBD_FOUND",
							"MessageKey", "MSG_QUESTION_RECEPTION_LATER_BBD_FOUND",
							"Parameters", new object[] { ((DateTime)ds.Tables[0].Rows[0][PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME]).ToShortDateString() }));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_BestBeforeDate;
					}
				}
			}
			goto Step_EnterPackagingQuantity;
			#endregion

			Step_EnterPackagingQuantity:

			#region ENTER PACKAGING QUANTITY

			//if allowed, check the given bestbeforedate
			if (checkDataInUDFPOLine && isPOManuallySelected && bestBeforeDatesToCheck.Count > 0 &&
				(itemInfo.HasBestBeforeDate &&
				!bestBeforeDatesToCheck.Contains(bestBeforeDate)))
			{
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ENTERED_DATA_FAILED_CHECK",
						"MessageKey", "MSG_QUESTION_ENTERED_DATA_FAILED_CHECK_BBD"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					if (!(barcode != null &&
						barcode.GS1DataObject != null &&
						!barcode.GS1DataObject.BestBeforeDate.HasValue))
					{
						goto Step_BestBeforeDate;
					}
					else if (isGS1DataEnteredOnScanLogDataScreen)
					{
						goBack = true;
						goto Step_ScanLabelStart;
					}
				}
				goBack = true;
				goto Step_IdentifyProduct;
			}

			if ((itemInfo.HasBatchnumber || itemInfo.HasSecondBatchNumber || itemInfo.HasBestBeforeDate))
			{

				currentListOfTransactionalPackagingTypeInfos = GetTransactionalPackagingInfo(purchaseDeliveryNoteList, batchNumber, secondBatchNumber, bestBeforeDate, itemCode);

				if (currentListOfTransactionalPackagingTypeInfos == null)
				{
					query = BuildQuery.GetExistingITRI(itemCode, batchNumber, secondBatchNumber, bestBeforeDate);
					dsExistingBatch = sboProviderService.RunView(false, null, null, query);

					if (this.GetNumberOfRows(dsExistingBatch) == 0)
					{
						//batch not found, so we need to enter the quantity for the packaging types
						currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfo);
						bool foundAtLeastOnePackagingTypeToShowOnScreen = false;
						bool isFirst = true;
						listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();
						foreach (PmxItemPackagingTypeInfo packType in itemInfo.ListOfPackagingTypes)
						{

							PackagingTypeInfo info = new PackagingTypeInfo();

							if (isFirst)
							{
								//info.NumberOfDecimals = itemInfo.UomDecimals;
							}

							info.Quantity = packType.Quantity;
							info.PackagingTypeCode = packType.PackagingTypeCode;
							info.PackagingTypeName = packType.PackagingTypeName;

							info.ShowOnScreen = (packType.AskDuringReception &&
							(!packType.Barcodes.Contains(gtin)));

							listOfPackagingTypeInfosForScreen.Add(info);

							if (!foundAtLeastOnePackagingTypeToShowOnScreen &&
								info.ShowOnScreen)
							{
								foundAtLeastOnePackagingTypeToShowOnScreen = true;
							}

							isFirst = false;
						}

						if (foundAtLeastOnePackagingTypeToShowOnScreen)
						{
							initialErrorKey = "MSG_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES";
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"TitleKey", "MSG_TITLE_ENTER_QUANTITY_OF_ALL_PACKAGING_TYPES",
								"BarcodeController", barcodeController,
								"ItemDescription", itemDescription,
								"ItemCode", itemInfo.ItemCode,
								"ItemBarcode", itemInfo.CodeBars,
								"UOM", itemInfo.InventoryUom,
								"ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
								"IsForEnteringDefaultPackagingQuantity", true,
								"BatchNumber", batchNumber,
								"BestBeforeDate", bestBeforeDate,
								"SSCC", SSCC,
								"DocNum", Convert.ToString(purchaseOrderDocNum),
								"InitialErrorKeyIsError", false
								));

							msg = WaitForMessage();
							goBack = false;
							// reset error
							initialErrorKey = null;

							if (msg.Name.EndsWith(".QuantitiesEntered"))
							{
								PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

								//build the list of packaging quantities
								foreach (ItemTransactionalPackagingTypeInfo info in currentListOfTransactionalPackagingTypeInfos)
								{
									//try to get new qty from user
									if (screenListOfPackagingTypeInfos.PackagingTypeInfoList.ContainsKey(info.PackagingTypeCode))
									{
										info.Quantity = screenListOfPackagingTypeInfos.PackagingTypeInfoList[info.PackagingTypeCode].Quantity;
									}
								}

								goto Step_EnterBatchAttributes;
							}
							else if (msg.Name.EndsWith(".BackRequested"))
							{
								goBack = true;
								if (itemInfo.HasBestBeforeDate)
								{
									goto Step_BestBeforeDate;
								}
								else if (itemInfo.HasSecondBatchNumber)
								{
									goto Step_SecondBatchNumber;
								}
								goto Step_SelectBatchNumber;
							}

						}

					}
					else
					{
						//we found the batch, so now fill in the packaging data from the batch
						currentListOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();
						foreach (DataRow row in dsExistingBatch.Tables[0].Rows)
						{
							ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

							newPackInfo.PackagingTypeCode = Convert.ToString(row[PmxItemTransactionalPackagingTypeTable.Columns.PackagingTypeCode.NAME]);
							newPackInfo.Quantity = Convert.ToDouble(row[PmxItemTransactionalPackagingTypeTable.Columns.Quantity.NAME]);
							newPackInfo.ItriKey = Convert.ToInt32(row[PmxItemTransactionalInfoTable.Columns.InternalKey.NAME]);

							newPackInfo.ItemCode = itemInfo.ItemCode;

							currentListOfTransactionalPackagingTypeInfos.Add(newPackInfo);
						}

						goto Step_EnterBatchAttributes;
					}
				}

				goto Step_EnterBatchAttributes;
			}
			else
			{
				//no batch info
				//get the packaging types qty from item
				currentListOfTransactionalPackagingTypeInfos = SetItemTransactionalPackagingTypeInfoList(itemInfo);



				goto Step_EnterBatchAttributes;
			}

			#endregion

			Step_EnterBatchAttributes:

			#region ENTER BATCH ATTRIBUTES

			currentListOfTransactionalBatchAttributeInfos = null;

			if ((itemInfo.HasBatchnumber || itemInfo.HasSecondBatchNumber || itemInfo.HasBestBeforeDate))
			{

				m_enterBatchAttributesFlow.BindParameters(
						  this.DefaultCultureInfo,
						  this.PmxOseCompany,
						  new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
						  new ReadOnlyBinder<string>("BatchNumber", batchNumber),
						  new ReadOnlyBinder<string>("BatchNumber2", secondBatchNumber),
						  new ReadOnlyBinder<DateTime?>("BBD", bestBeforeDate),
						  new ReadOnlyBinder<bool>("SaveData", true),
						  new ReadOnlyBinder<BarcodeContainer>("Barcode", barcode),

						  new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
						  new ReadWriteBinder<Collection<ItemTransactionalBatchAttributeInfo>>("ListOfBatchAttributes", this, "m_enterBatchAttributesFlow_ListOfBatchAttributes")

						  );
				m_subFlowBackRequested = false;
				this.StartSubFlowAndWait(m_enterBatchAttributesFlow, true);

				if (m_subFlowBackRequested)
				{
					goto Step_GoBackOnEnterQuantity;
				}

				currentListOfTransactionalBatchAttributeInfos = m_enterBatchAttributesFlow_ListOfBatchAttributes;




				goto Step_EnterQuantity;
			}
			else
			{
				goto Step_EnterQuantity;
			}

			#endregion

			Step_EnterQuantity:

			#region ENTER QUANTITY

			quantity = 0;
			quantityUom2 = null;
			listOfSerialNumbers = null;
			m_subFlowListOfSerialNumbers = null;
			askForPieces = false;
			askForWeight = false;

			double defaultQty = itemInfo.DefaultQuantityOnLogisticUnit > 0 ? itemInfo.DefaultQuantityOnLogisticUnit : 1;

			if (itemInfo.IsCatchWeightItem)
			{
				//Only set qty if it needs to be set
				initialWeight = BarcodeHelper.GetQuantityForMeasureType(itemInfo.Uom2GS1MeasureType, barcode, null);

				if (IsQuantityOnBarcode(barcode))
				{
					initialQuantity = Convert.ToDouble(barcode.GS1DataObject.Count);
				}
				else
				{
					if (initialWeight.HasValue)
					{
						initialQuantity = PmxItemAllConnectionsProvider.CalculatePiecesFromWeight(initialWeight.Value, itemInfo).Value;
					}
					else
					{
						initialQuantity = defaultQty;
					}
				}
			}
			else
			{
				if (IsQuantityOnBarcode(barcode))
				{
					initialQuantity = Convert.ToDouble(barcode.GS1DataObject.Count);
				}
				else
				{
					initialQuantity = BarcodeHelper.GetQuantityForMeasureType(itemInfo.UomGS1MeasureType, barcode, defaultQty) ?? defaultQty;
				}

			}



			//if (this.IsLUOnBarcode(barcode))
			//{
			//    maxQuantity = 1;
			//}
			//else 
			if ((purchaseDocRef != null && purchaseDocRef.DocType == SboObjectTypes.PurchaseInvoice) ||
			(purchaseDocRef != null && !allowToExceedOrderedQuantity) ||
			(containerDocEntry.HasValue && !allowToExceedOrderedQuantity))
			{
				maxQuantity = stillToReceiveInventory / batchReceivingNumber;
			}
			else
			{
				maxQuantity = -1;
			}

			bool getNormalQuantity = true;

			if ((itemInfo.HasSerialNumber || itemInfo.HasSAPSerialNumber) && batchReceivingNumber == 1)
			{
				#region SERIAL NUMBER


				getNormalQuantity = false;

				//show screen to ask user what to do.
				buttonKeys = new List<string>(new Collection<string>() { "MSG_BUTTON_SERIAL_NUMBER", "MSG_BUTTON_SERIAL_NUMBER_RANGE" });

				if (!(crossDocking) &&
					!(itemInfo.HasSAPSerialNumber && itemInfo.ManageMethod == SboItemManageMethod.OnEveryTransaction))
				{
					buttonKeys.Add("MSG_BUTTON_NO_SERIAL_NUMBER");
				}


				toolBarButtons = ToolBarButtons.Back;

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_ENTER_QUANTITY_SERIAL_NUMBER_TASK",
						"AllowedButtons", buttonKeys.ToArray(),
						"EnabledToolBarButtons", toolBarButtons
						));
				msg = WaitForMessage();
				goBack = false;
				initialErrorKey = null;
				if (msg.Name.EndsWith(".ButtonClicked"))
				{
					string buttonKeySerial = ExtractParameter<string>(msg.Parameters, "buttonKey");

					if (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER" ||
						buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE")
					{
						#region ENTER SERIAL NUMBERS


						//get ITRI
						int? itriKey = null;
						if (itemInfo.HasBestBeforeDate ||
							itemInfo.HasSecondBatchNumber ||
							itemInfo.HasBatchnumber)
						{
							itriKey = sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecuteGetCreateItemTransactionalInfoKey), itemCode, batchNumber, secondBatchNumber, bestBeforeDate);
						}

						goBack = false;
						bool isSerialNumberRange = (buttonKeySerial == "MSG_BUTTON_SERIAL_NUMBER_RANGE");

						m_enterQuantitySubFlow.BindParameters(
							this.DefaultCultureInfo,
							this.PmxOseCompany,
							new ReadOnlyBinder<string>("TitleKey", "MSG_TITLE_ENTER_QUANTITY"),
							new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
							new ReadOnlyBinder<string>("BatchNumber", batchNumber),
							new ReadOnlyBinder<DateTime?>("BestBeforeDate", bestBeforeDate),
							new ReadOnlyBinder<int?>("ItriKey", itriKey),
							new ReadOnlyBinder<string>("LocationCode", currentDockCode),
							new ReadOnlyBinder<string>("SSCC", SSCC),

							new ReadOnlyBinder<string>("DocNum", Convert.ToString(purchaseOrderDocNum)),
							new ReadOnlyBinder<bool>("UsePickingInfo", false),

							new ReadOnlyBinder<bool>("AllowZero", false),
							new ReadOnlyBinder<double>("InitialQuantity", initialQuantity),
							new ReadOnlyBinder<string>("MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE"),
							new ReadOnlyBinder<double>("StillToQuantity", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals)),
							new ReadOnlyBinder<double>("MaximumQuantity", maxQuantity),
							new ReadOnlyBinder<double>("TotalQuantityToPick", 0),
							new ReadOnlyBinder<string>("UOM", itemInfo.InventoryUom),
							new ReadOnlyBinder<int>("NumberOfDigitsAfterDecimal", (itemInfo != null) ? itemInfo.UomDecimals : 0),

							new ReadOnlyBinder<double>("InnerQuantity", 1),
							new ReadOnlyBinder<string>("InnerUOM", null),
							new ReadOnlyBinder<bool>("ForceInputSerialNumbers", true),
							new ReadOnlyBinder<string>("ObjectType", ((int)SboLinkDocumentType.PurchaseOrder).ToString()),
							new ReadOnlyBinder<int?>("DocEntry", purchaseOrderDocNum),
							new ReadOnlyBinder<bool>("UseSerialNumberRange", isSerialNumberRange),
							new ReadOnlyBinder<string>("LocationCode", currentDockCode),

							new ReadOnlyBinder<int?>("LUID", null),


							new ReadWriteBinder<double>("QuantityEntered", this, "m_subFlowQuantity"),
							new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
							new ReadWriteBinder<Collection<string>>("ListOfSerialNumbers", this, "m_subFlowListOfSerialNumbers")

							);
						m_subFlowBackRequested = false;
						this.StartSubFlowAndWait(m_enterQuantitySubFlow, true);

						if (m_subFlowBackRequested)
						{
							goto Step_EnterQuantity;
						}

						listOfSerialNumbers = m_subFlowListOfSerialNumbers;

						quantity = m_subFlowQuantity / (numberOfItemsInPurchaseUnit ?? 1);

						goto Step_CheckQuantity;

						#endregion

					}

					getNormalQuantity = true;
				}

				if (msg.Name.EndsWith(".BackRequested"))
				{
					goBack = true;
					goto Step_GoBackOnEnterQuantity;
				}



				#endregion
			}

			if (itemInfo.ListOfPackagingTypes != null &&
				currentListOfTransactionalPackagingTypeInfos.Count > 0)
			{
				#region GET QUANTITY WITH PACKAGING TYPE

				//build list of packaging types

				listOfPackagingTypeInfosForScreen = new Collection<PackagingTypeInfo>();

				//add first one for the uom

				PackagingTypeInfo info = new PackagingTypeInfo();

				info.Quantity = 1;
				info.PackagingTypeCode = null;
				info.PackagingTypeName = itemInfo.InventoryUom == null ? itemInfo.ItemCode : itemInfo.InventoryUom;
				info.QuantityPerPack = 1;
				if (!string.IsNullOrEmpty(itemInfo.CodeBars))
				{
					info.Barcode.Add(itemInfo.CodeBars);
				}
				if (itemInfo.BarcodesAndUomQuantities != null)
				{
					List<string> barcodes = itemInfo.BarcodesAndUomQuantities.Where(w => w.Value == 1).Select(s => s.Key).ToList();
					if (barcodes.Count > 0)
					{
						foreach (string bc in barcodes)
						{
							info.Barcode.Add(bc);
						}
					}
				}

				info.NumberOfDecimals = itemInfo.UomDecimals;
				if (info.Barcode.Contains(gtin == null ? itemCode : gtin))
				{
					//We had a quantity scanned in barcode, so use that one 
					info.InitialQuantity = initialQuantity;
				}
				else
				{
					info.InitialQuantity = 0;
				}
				listOfPackagingTypeInfosForScreen.Add(info);


				foreach (ItemTransactionalPackagingTypeInfo packType in currentListOfTransactionalPackagingTypeInfos)
				{
					info = new PackagingTypeInfo();

					info.Quantity = packType.Quantity;
					info.PackagingTypeCode = packType.PackagingTypeCode;
					info.PackagingTypeName = GetPackTypeName(packType.PackagingTypeCode, itemInfo.ListOfPackagingTypes);
					info.Barcode = GetPackBarcode(packType.PackagingTypeCode, itemInfo.ListOfPackagingTypes);
					info.HideDuringEnteringQuantity = GetHideDuringEnteringQuantity(packType.PackagingTypeCode, itemInfo.ListOfPackagingTypes);

					if (info.Barcode.Contains(gtin))
					{
						info.InitialQuantity = 1;
						//Check if barcode is also added to inventory uom.
						//If so, we need to adjust initial qty
						if (listOfPackagingTypeInfosForScreen[0].Barcode.Contains(gtin))
						{
							listOfPackagingTypeInfosForScreen[0].InitialQuantity = 0;
						}

						if (info.HideDuringEnteringQuantity)
						{
							if (listOfPackagingTypeInfosForScreen[0].InitialQuantity.HasValue)
							{
								listOfPackagingTypeInfosForScreen[0].InitialQuantity += info.Quantity;
							}
							else
							{
								listOfPackagingTypeInfosForScreen[0].InitialQuantity = info.Quantity;
							}

							info.InitialQuantity = 0;
						}
					}
					else
					{
						info.InitialQuantity = 0;
					}

					info.QuantityPerPack = info.Quantity;

					listOfPackagingTypeInfosForScreen.Add(info);
				}

				if (listOfPackagingTypeInfosForScreen.Where(w => w.HideDuringEnteringQuantity == false).Count() == 1)
				{
					//Only 1 item in list, so this means this is only the uom
					//So now check if there was an initial quantity, and set it if needed
					if (listOfPackagingTypeInfosForScreen[0].InitialQuantity == 0)
					{
						listOfPackagingTypeInfosForScreen[0].InitialQuantity = initialQuantity;
					}
				}

				#endregion
			}

			if (itemInfo.IsCatchWeightItem)
			{
				#region CATCH WEIGHT

				getNormalQuantity = false;


				askForPieces = itemInfo.UomToUseForPurchase == PmxUomToUse.UOM1AndUOM2 ||
									itemInfo.UomToUseForPurchase == PmxUomToUse.UOM1;
				askForWeight = itemInfo.UomToUseForPurchase == PmxUomToUse.UOM1AndUOM2 ||
									itemInfo.UomToUseForPurchase == PmxUomToUse.UOM2;


				double? quantityForPieces = null;
				double? quantityForWeight = null;


				if (askForPieces)
				{

					if (itemInfo.ListOfPackagingTypes != null &&
						currentListOfTransactionalPackagingTypeInfos.Count > 0)
					{
						#region QUANTITY WITH PACKAGING TYPE

						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, null, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_ENTER_QUANTITY",
							"BarcodeController", barcodeController,
							"ItemDescription", itemDescription,
							"ItemCode", itemInfo.ItemCode,
							"ItemBarcode", itemInfo.CodeBars,
							"UOM", itemInfo.InventoryUom,
							"ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
							"IsForEnteringDefaultPackagingQuantity", false,
							"BatchNumber", batchNumber,
							"BestBeforeDate", bestBeforeDate,
							"SSCC", SSCC,
							"DocNum", Convert.ToString(purchaseOrderDocNum),
							"StillToQuantity", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((stillToReceiveInventory / batchReceivingNumber), itemInfo.UomDecimals),
							"MaximumQuantity", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(maxQuantity, itemInfo.UomDecimals),
							"MaximumUOM", itemInfo.InventoryUom,
							"MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE",
							"AllowZero", false
							));

						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantitiesEntered"))
						{
							PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

							quantityForPieces = 0;

							//build the list of packaging quantities
							foreach (PackagingTypeInfo infoForQty in screenListOfPackagingTypeInfos.PackagingTypeInfoList.Values)
							{
								quantityForPieces += (infoForQty.Quantity * infoForQty.QuantityPerPack);
							}
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}
						#endregion
					}
					else
					{
						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, null, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
						   DefaultCultureInfo.Get(), BuildParamCollection(
						   "InitialErrorKey", initialErrorKey,
						   "TitleKey", "MSG_TITLE_ENTER_QUANTITY",
						   "BarcodeController", barcodeController,
						   "ItemDescription", itemDescription,
						   "ItemCode", itemInfo.ItemCode,
						   "ItemBarcode", itemInfo.CodeBars,
						   "InitialQuantity", initialQuantity,
						   "InnerQuantity", 1,
						   "MaximumQuantity", maxQuantity,
						   "NumberOfDigitsAfterDecimal", itemInfo.UomDecimals,
						   "InnerUOM", itemInfo.InventoryUom,
						   "UOM", itemInfo.InventoryUom,
						   "BarcodesAndUomQuantities", itemInfo.PurchaseBarcodesAndUomQuantities,
						   "BatchNumber", batchNumber,
						   "BestBeforeDate", bestBeforeDate,
						   "SSCC", SSCC,
						   "DocNum", Convert.ToString(purchaseOrderDocNum),
						   "StillToQuantity", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((stillToReceiveInventory / batchReceivingNumber), itemInfo.UomDecimals),
						   "MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE",
						   "SetFocusToQuantityControl", setFocusToQuantityControl
						   ));

						msg = WaitForMessage();
						goBack = false;
						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantityEntered"))
						{
							quantityForPieces = ExtractParameter<double>(msg.Parameters, "quantity");
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}
					}
				}

				double totalInvQtyScanned = 0;

				if (askForWeight)
				{
					quantityForWeight = 0;
					double maxQuantityForWeight = -1;

					if (quantityForPieces.HasValue)
					{
						//Calculate weight maximum
						double? tempMax = PmxItemAllConnectionsProvider.GetMaximumWeightForCatchWeightItem(quantityForPieces.Value, itemInfo);

						if (itemInfo.CatchWeightWeightTolerancePercentage >= 100)
						{
							//Tolerance greater than 100%, so actually no maximum is required
							maxQuantityForWeight = -1;
						}
						else
						{
							//Calculate weight maximum
							if (tempMax.HasValue)
							{
								maxQuantityForWeight = tempMax.Value;
							}
						}
					}
					else
					{
						double? tempMax = PmxItemAllConnectionsProvider.GetMaximumWeightForCatchWeightItem(stillToReceiveInventory, itemInfo);

						maxQuantityForWeight = maxQuantity == -1 ? -1 : tempMax.HasValue ? tempMax.Value : -1;
					}

					totalInvQtyScanned = 0;

					Step_NextWeight:

					double? tempMaxToUse = null;

					double calculatedQuantityPieces = 0;

					if (itemInfo.CatchWeightScanWeightForEachCase)
					{
						//Scan each piece
						calculatedQuantityPieces = itemInfo.CatchWeightQuantityInCase;

						//It is possible that number of decimals is larger than 0, which means that it is not really pieces.
						//So we need to check if the remaining quantity is lower than 1.
						//If this is the case, we need to use this quantity!
						if (itemInfo.UomDecimals > 0 &&
							((quantityForPieces.HasValue ? quantityForPieces.Value : stillToReceiveInventory) - totalInvQtyScanned) < itemInfo.CatchWeightQuantityInCase)
						{
							calculatedQuantityPieces = (quantityForPieces.HasValue ? quantityForPieces.Value : stillToReceiveInventory) - totalInvQtyScanned;
						}

						tempMaxToUse = PmxItemAllConnectionsProvider.GetMaximumWeightForCatchWeightItem(calculatedQuantityPieces, itemInfo);

						if (totalInvQtyScanned == 0)
						{
							//This is the first time asking for weight, so allow to enter full quantity
							tempMaxToUse = PmxItemAllConnectionsProvider.GetMaximumWeightForCatchWeightItem((quantityForPieces.HasValue ? quantityForPieces.Value : stillToReceiveInventory) - totalInvQtyScanned, itemInfo);
						}

					}
					else
					{
						tempMaxToUse = PmxItemAllConnectionsProvider.GetMaximumWeightForCatchWeightItem((quantityForPieces.HasValue ? quantityForPieces.Value : stillToReceiveInventory) - totalInvQtyScanned, itemInfo);
					}

					double maxQuantityForWeightToUse = maxQuantityForWeight == -1 ? -1 : tempMaxToUse.HasValue ? tempMaxToUse.Value : -1;


					if (currentDockCode != weighData.LocationCode)
					{
						weighData.LocationCode = currentDockCode;
						weighData.ScaleData = null;
						weighData.ScaleCode = null;

						query = SboCommands.GetQueryGetAllScalesOfWeighingRooms(weighData.LocationCode, true, DefaultCultureInfo.Get());
						dsScales = sboProviderService.RunView(false, null, null, query);
						weighData.NumberOfScales = this.GetNumberOfRows(dsScales);
					}

					if (itemInfo.WeightCaptureNeeded && weighData.NumberOfScales > 0)
					{

						Step_GetCatchWeightWeight:

						weighData.ItemUom = itemInfo.Uom2;
						weighData.Quantity = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((PmxItemAllConnectionsProvider.CalculateWeightFromPieces(stillToReceiveInventory, itemInfo).Value / batchReceivingNumber), itemInfo.Uom2Decimals);

						if (weighData.ScaleData == null || weighData.ScaleCode != weighData.ScaleData.Code)
						{
							initialErrorKey = GetScaleData(sboProviderService, weighData, m_uomCoefficients);
						}

						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, weighData, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Weighing.IEnterWeightWithItemInfoScreen),
							 DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_ENTER_WEIGHT",
							"ItemDescription", itemDescription,
							"DocumentNumber", Convert.ToString(purchaseOrderDocNum),
							"BatchNumber", batchNumber,
							"BestBeforeDate", bestBeforeDate,
							"StillToReceiveQuantity", weighData.Quantity,
							"Uom", itemInfo.Uom2,
							"UomDecimals", itemInfo.Uom2Decimals,
							"ScaleUom", weighData.ScaleUom,
							"ScaleUomDecimals", weighData.ScaleData != null ? weighData.ScaleUomDecimals : itemInfo.Uom2Decimals,
							"NumberOfScales", weighData.NumberOfScales,
							"CurrentScaleCode", weighData.ScaleData != null ? weighData.ScaleData.Code : null,
							"TareCommand", weighData.TareCommand,
							"ZeroCommand", weighData.ZeroCommand,
							"WeighCommand", weighData.WeighCommand,
							"WeighResultFormat", weighData.WeighResultFormat,
							"ScaleProviderType", weighData.ScaleData != null ? (weighData.ScaleData.UseAsService ? "Service" : "GenericScale") : null,
							"ScaleSettings", weighData.ScaleData != null ? weighData.ScaleData.ScaleDefinition : null,
							"MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE"));

						msg = WaitForMessage();

						if (msg.Name.EndsWith(".SelectScale"))
						{
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectCodeNameScreen),
							   DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"TitleKey", "MSG_TITLE_SELECT_SCALE",
								"CodeNameDS", dsScales
								));

							msg = WaitForMessage();

							initialErrorKey = null;

							if (msg.Name.EndsWith(".CodeNameSelected"))
							{
								weighData.ScaleCode = ExtractParameter<string>(msg.Parameters, "code");
							}

							goto Step_GetCatchWeightWeight;
						}
					}
					else
					{

						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, null, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
					   DefaultCultureInfo.Get(), BuildParamCollection(
					   "InitialErrorKey", initialErrorKey,
					   "TitleKey", "MSG_TITLE_ENTER_WEIGHT",
					   "BarcodeController", barcodeController,
					   "ItemDescription", itemDescription,
					   "ItemCode", itemInfo.ItemCode,
					   "ItemBarcode", itemInfo.CodeBars,
					   "BarcodesAndUomQuantities", itemInfo.PurchaseBarcodesAndUomQuantities,
					   "InitialQuantity", initialWeight.HasValue ? initialWeight.Value : 0,
					   "InnerQuantity", 1,
					   "MaximumQuantity", maxQuantityForWeightToUse,
					   "NumberOfDigitsAfterDecimal", itemInfo.Uom2Decimals,
					   "InnerUOM", itemInfo.Uom2,
					   "UOM", itemInfo.Uom2,
					   "BatchNumber", batchNumber,
					   "BestBeforeDate", bestBeforeDate,
					   "SSCC", SSCC,
					   "DocNum", Convert.ToString(purchaseOrderDocNum),
					   "StillToQuantity", Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((PmxItemAllConnectionsProvider.CalculateWeightFromPieces(stillToReceiveInventory, itemInfo).Value / batchReceivingNumber), itemInfo.Uom2Decimals),
					   "MessageKeyStillQuantity", "MSG_LBL_RECEIVING",
					   "ForceGS1_128", true,
					   "GS1WeightAI", itemInfo.Uom2GS1MeasureType,
					   "UomToDisplayOnQtyProvider", itemInfo.Uom2,
					   "QtyEntered", quantityForWeight.Value,
					   "AllowZero", true,
					   "InfoMessageKey", "MSG_LBL_NUMBER_OF_PIECES",
					   "InfoValue", quantityForPieces.HasValue ? string.Format("{0}/{1}", totalInvQtyScanned, quantityForPieces.Value) : totalInvQtyScanned.ToString(),
					   "SetFocusToQuantityControl", setFocusToQuantityControl
					   ));
						msg = WaitForMessage();
					}

					goBack = false;
					// reset error
					initialErrorKey = null;

					if (msg.Name.EndsWith(".QuantityEntered") || msg.Name.EndsWith(".WeightEntered"))
					{
						double tempWeight;

						if (itemInfo.WeightCaptureNeeded)
						{
							tempWeight = ExtractParameter<double>(msg.Parameters, "weight");
							if (weighData.ScaleCoefficient.HasValue)
							{
								tempWeight = tempWeight / weighData.ScaleCoefficient.Value;
							}
						}
						else
						{
							tempWeight = ExtractParameter<double>(msg.Parameters, "quantity");
						}


						if (tempWeight != 0)
						{
							if (!quantityForWeight.HasValue)
							{
								quantityForWeight = 0;
							}

							double tempInvQty = 0;
							if (itemInfo.CatchWeightScanWeightForEachCase)
							{
								//Regard each scan as the number of pieces in a case.
								tempInvQty = calculatedQuantityPieces;

								//Check if this is within tolerance. when it is the first time
								if (totalInvQtyScanned == 0 &&
									!PmxItemAllConnectionsProvider.IsCatchWeightWithinTolerance(tempInvQty, tempWeight, itemInfo) &&
									quantityForPieces.HasValue)
								{
									//If not, check if it is within tolerance of total quantity
									if (PmxItemAllConnectionsProvider.IsCatchWeightWithinTolerance(quantityForPieces.Value, tempWeight, itemInfo))
									{
										//It is within tolerance, so regard this as a weight for the total quantity
										tempInvQty = quantityForPieces.Value;
									}
								}



							}
							else
							{
								tempInvQty = PmxItemAllConnectionsProvider.CalculatePiecesFromWeight(tempWeight, itemInfo).Value;
							}

							if (!PmxItemAllConnectionsProvider.IsCatchWeightWithinTolerance(tempInvQty, tempWeight, itemInfo))
							{
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								   this.DefaultCultureInfo, BuildParamCollection(
								   "MessageKey", "MSG_ERROR_CW_WEIGHT_NOT_WITHIN_TOLERANCE",
								   "ShowButton", true,
								   "IsError", true,
								   "Parameters", new object[] { tempWeight, tempInvQty, itemInfo.CatchWeightWeightTolerancePercentage }));
								msg = WaitForMessage();

								goto Step_NextWeight;
							}

							totalInvQtyScanned += tempInvQty;
							quantityForWeight += tempWeight;

							//Reset initial quantity
							initialWeight = null;

							#region PRINT LABELS


							//Print labels for catch weight here, because each label will have a different weight
							try
							{
								numberOfItemLabels = 0;
								if (itemInfo.PrintItemLabel)
								{
									if (currentPmxOseCompany.GoodsReceiptAutoPrintItemLabels)
									{
										if (askUserToPrintLabels)
										{
											//ask user if he wants to print item labels
											session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
												DefaultCultureInfo.Get(), BuildParamCollection(
													"TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
													"MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
											msg = WaitForMessage();

											if (msg.Name.EndsWith(".Yes"))
											{
												numberOfItemLabels = 1;
											}
										}
										else
										{
											//Print 1 label for the weight
											numberOfItemLabels = 1;
										}
									}
									else
									{
										//ask user if he wants to print item labels
										session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
											DefaultCultureInfo.Get(), BuildParamCollection(
												"TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
												"MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
										msg = WaitForMessage();

										if (msg.Name.EndsWith(".Yes"))
										{
											numberOfItemLabels = 1;
										}
									}

									if (numberOfItemLabels > 0)
									{
										// print the labels

										//Set the data into binders
										this.m_printItemLabelFlow_LocationCode = currentDockCode;
										this.m_printItemLabelFlow_NeededBatchNumber = batchNumber;
										this.m_printItemLabelFlow_NeededBatchNumber2 = secondBatchNumber;
										this.m_printItemLabelFlow_NeededBestBeforeDate = bestBeforeDate;
										this.m_printItemLabelFlow_NeededItemCode = itemInfo.ItemCode;
										this.m_printItemLabelFlow_NeededNrOfLabels = numberOfItemLabels;
										this.m_printItemLabelFlow_EnteredQuantity = quantityForPieces.HasValue ? quantityForPieces.Value : tempInvQty;
										this.m_printItemLabelFlow_EnteredQuantityUom2 = tempWeight;
										this.m_printItemLabelFlow_ShowPrintMessage = false;

										m_printItemLabelFlow_NeededPackagingType = null;
										m_printItemLabelFlow_NeededQuantityByPackagingType = null;

										if (numberOfItemLabels.HasValue)
										{
											this.m_printItemLabelFlow_NeededPackagingType = purchaseUom;

											if (numberOfItemsInPurchaseUnit.HasValue)
											{
												this.m_printItemLabelFlow_NeededQuantityByPackagingType = Convert.ToInt32(Math.Ceiling(numberOfItemsInPurchaseUnit.Value));
											}
										}



										//call flow
										base.StartSubFlowAndWait(m_printItemLabelFlow, true);
										goBack = false;
										if (m_printItemLabelFlow_BackRequested)
										{
											goBack = true;
											goto Step_GoBackOnEnterQuantity;
										}
									}


								}
							}
							catch (Exception ex)
							{
								s_log.Error("", ex);
								if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
								{
									// Inform the user with the error
									session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
											"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
											"NoTranslationOfMessageKey", true,
											"IsError", true,
											"ShowButton", true));
									WaitForMessage();

								}
								else
								{
									// Inform the user with the error
									session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
											"PmxError", ((ProdumexException)ex).Error,
											"ShowButton", true));
									WaitForMessage();
								}
							}

							#endregion

							if (!quantityForPieces.HasValue ||
								(quantityForPieces.HasValue && totalInvQtyScanned < quantityForPieces.Value))
							{
								//Pieces were asked, and the number of pieces was not reached yet, so ask next weight.
								goto Step_NextWeight;
							}
						}
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						goBack = true;
						goto Step_GoBackOnEnterQuantity;
					}
				}

				//Check if we need to calculate any of the quantities.

				if (!quantityForWeight.HasValue)
				{
					quantityForWeight = PmxItemAllConnectionsProvider.CalculateWeightFromPieces(quantityForPieces.Value, itemInfo);
				}
				else if (!quantityForPieces.HasValue)
				{
					if (totalInvQtyScanned > 0)
					{
						quantityForPieces = totalInvQtyScanned;
					}
					else
					{
						quantityForPieces = PmxItemAllConnectionsProvider.CalculatePiecesFromWeight(quantityForWeight.Value, itemInfo);
					}
				}


				//Check if quantity is within tolerance
				if (!PmxItemAllConnectionsProvider.IsCatchWeightWithinTolerance(quantityForPieces.Value, quantityForWeight.Value, itemInfo))
				{
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							this.DefaultCultureInfo, BuildParamCollection(
							"MessageKey", "MSG_ERROR_CW_WEIGHT_NOT_WITHIN_TOLERANCE",
							"ShowButton", true,
							"IsError", true,
							"Parameters", new object[] { quantityForWeight.Value, quantityForPieces.Value, itemInfo.CatchWeightWeightTolerancePercentage }));
					msg = WaitForMessage();

					goto Step_EnterQuantity;
				}


				if ((itemInfo.ListOfPackagingTypes != null &&
				  listOfPackagingTypeInfosForScreen != null &&
					listOfPackagingTypeInfosForScreen.Count > 0))
				{
					quantity = quantityForPieces.Value;

				}
				else
				{
					//The quantity from screens is in inventory uom.
					//So we need to rework to quantity in purchase uom
					quantity = quantityForPieces.Value / numberOfItemsInPurchaseUnit.Value;
				}




				quantityUom2 = quantityForWeight.Value;



				goto Step_CheckQuantity;

				#endregion
			}



			if (containerDocEntry.HasValue)
			{
				//For a container we can only set the custmomer info when we know the item to receive. 
				query = BuildQuery.GetCustomerInfoForContainer(containerDocEntry.Value, itemCode, sboProviderService.GetDbTool());
				DataSet dsCustomerInfo = sboProviderService.RunView(false, null, null, query);

				customerInfo = GetCustomerInfo(dsCustomerInfo);
			}

			if (getNormalQuantity)
			{
				if (currentDockCode != weighData.LocationCode)
				{
					weighData.LocationCode = currentDockCode;
					weighData.ScaleData = null;
					weighData.ScaleCode = null;

					query = SboCommands.GetQueryGetAllScalesOfWeighingRooms(weighData.LocationCode, true, DefaultCultureInfo.Get());
					dsScales = sboProviderService.RunView(false, null, null, query);
					weighData.NumberOfScales = this.GetNumberOfRows(dsScales);
				}

				if (itemInfo.WeightCaptureNeeded && weighData.NumberOfScales > 0)
				{
					Step_GetNormalQuantityWeight:

					weighData.ItemUom = itemInfo.InventoryUom;
					weighData.Quantity = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals);

					if (weighData.ScaleData == null || weighData.ScaleCode != weighData.ScaleData.Code)
					{
						initialErrorKey = GetScaleData(sboProviderService, weighData, m_uomCoefficients);
					}

					if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, weighData, currentDockCode, purchaseDocRef, itemInfo))
					{
						goBack = true;
						goto Step_GoBackOnEnterQuantity;
					}

					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Weighing.IEnterWeightWithItemInfoScreen),
						 DefaultCultureInfo.Get(), BuildParamCollection(
											"InitialErrorKey", initialErrorKey,
											"TitleKey", "MSG_TITLE_ENTER_WEIGHT",
											"ItemDescription", itemDescription,
											"DocumentNumber", Convert.ToString(purchaseOrderDocNum),
											"BatchNumber", batchNumber,
											"BestBeforeDate", bestBeforeDate,
											"StillToReceiveQuantity", weighData.Quantity,
											"Uom", itemInfo.InventoryUom,
											"UomDecimals", itemInfo.UomDecimals,
											"ScaleUom", weighData.ScaleUom,
											"CurrentScaleCode", weighData.ScaleData != null ? weighData.ScaleData.Code : null,
											"ScaleUomDecimals", weighData.ScaleData != null ? weighData.ScaleUomDecimals : itemInfo.UomDecimals,
											"NumberOfScales", weighData.NumberOfScales,
											"TareCommand", weighData.TareCommand,
											"ZeroCommand", weighData.ZeroCommand,
											"WeighCommand", weighData.WeighCommand,
											"WeighResultFormat", weighData.WeighResultFormat,
											"ScaleProviderType", weighData.ScaleData != null ? (weighData.ScaleData.UseAsService ? "Service" : "GenericScale") : null,
											"ScaleSettings", weighData.ScaleData != null ? weighData.ScaleData.ScaleDefinition : null,
											"MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE"));

					msg = WaitForMessage();

					goBack = false;
					initialErrorKey = null;

					if (msg.Name.EndsWith(".SelectScale"))
					{
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectCodeNameScreen),
						   DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_SELECT_SCALE",
							"CodeNameDS", dsScales
							));

						msg = WaitForMessage();

						initialErrorKey = null;

						if (msg.Name.EndsWith(".CodeNameSelected"))
						{
							weighData.ScaleCode = ExtractParameter<string>(msg.Parameters, "code");
						}

						goto Step_GetNormalQuantityWeight;
					}
					else if (msg.Name.EndsWith(".BackRequested"))
					{
						goBack = true;
					}
					else if (msg.Name.EndsWith(".WeightEntered"))
					{
						quantity = ExtractParameter<double>(msg.Parameters, "weight");
						if (weighData.ScaleCoefficient.HasValue)
						{
							quantity = quantity / weighData.ScaleCoefficient.Value;
						}
						if (numberOfItemsInPurchaseUnit != null)
						{
							quantity = quantity / numberOfItemsInPurchaseUnit.Value;
						}

						quantity = Foundation.Utilities.FormattingHelper.RoundSAPDecimal(quantity, itemInfo.UomDecimals);

						goto Step_CheckQuantity;
					}

				}
				else
				{
					if (itemInfo.ListOfPackagingTypes != null &&
						currentListOfTransactionalPackagingTypeInfos.Count > 0)
					{
						#region QUANTITY WITH PACKAGING TYPE

						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, null, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterPackagingQuantityScreen),
											DefaultCultureInfo.Get(), BuildParamCollection(
											"InitialErrorKey", initialErrorKey,
											"TitleKey", "MSG_TITLE_ENTER_QUANTITY",
											"BarcodeController", barcodeController,
											"ItemDescription", itemDescription,
											"ItemCode", itemInfo.ItemCode,
											"ItemBarcode", itemInfo.CodeBars,
											//"UOM", string.IsNullOrEmpty(itemInfo.PurchaseUom) ? itemInfo.InventoryUom : itemInfo.PurchaseUom,
											"UOM", itemInfo.InventoryUom,
											"ListOfPackagingTypeInfos", listOfPackagingTypeInfosForScreen,
											"IsForEnteringDefaultPackagingQuantity", false,
											"BatchNumber", batchNumber,
											"BestBeforeDate", bestBeforeDate,
											"SSCC", SSCC,
											"DocNum", Convert.ToString(purchaseOrderDocNum),
											"StillToQuantity", (itemInfo.UomDecimals == 0 ? Math.Floor(stillToReceiveInventory / batchReceivingNumber) : Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceiveInventory / batchReceivingNumber, itemInfo.UomDecimals)),
											"MaximumQuantity", (itemInfo.UomDecimals == 0 ? Math.Floor(maxQuantity) : Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(maxQuantity, itemInfo.UomDecimals)),
											"MaximumUOM", itemInfo.InventoryUom,
											"MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE",
											"AllowZero", false,
											"AdditionalInfo", customerInfo
											));

						msg = WaitForMessage();

						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantitiesEntered"))
						{
							PackagingTypeInfos screenListOfPackagingTypeInfos = ExtractParameter<PackagingTypeInfos>(msg.Parameters, "listOfPackagingTypeInfos");

							//build the list of packaging quantities
							foreach (PackagingTypeInfo infoForQty in screenListOfPackagingTypeInfos.PackagingTypeInfoList.Values)
							{
								quantity += (infoForQty.Quantity * infoForQty.QuantityPerPack);
							}

							goto Step_CheckQuantity;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
						}
						#endregion
					}
					else
					{
						#region QUANTITY NO PACKAGING TYPE

						//no packaging types, so enter quantity on 'normal' way

						if (BackRequestedAfterCallingBeforeReceptionQuantityHook(sboProviderService, null, currentDockCode, purchaseDocRef, itemInfo))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityWithItemInfoScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_ENTER_QUANTITY",
							"BarcodeController", barcodeController,
							"ItemDescription", itemDescription,
							"ItemCode", itemInfo.ItemCode,
							"ItemBarcode", itemInfo.CodeBars,
							"BarcodesAndUomQuantities", itemInfo.PurchaseBarcodesAndUomQuantities,
							"InitialQuantity", initialQuantity != 0 ? initialQuantity : (itemInfo.DefaultQuantityOnLogisticUnit > 0 ? itemInfo.DefaultQuantityOnLogisticUnit : 1),
							"InnerQuantity", numberOfItemsInPurchaseUnit.Value,
							"MaximumQuantity", maxQuantity < 0 ? maxQuantity : (itemInfo.UomDecimals == 0 ? Math.Floor(maxQuantity / numberOfItemsInPurchaseUnit.Value) : Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(maxQuantity / numberOfItemsInPurchaseUnit.Value, itemInfo.UomDecimals)),
							"NumberOfDigitsAfterDecimal", itemInfo.UomDecimals,
							"InnerUOM", itemInfo.InventoryUom,
							//"UOM", string.IsNullOrEmpty(itemInfo.PurchaseUom) ? itemInfo.InventoryUom : itemInfo.PurchaseUom,
							"UOM", (numberOfItemsInPurchaseUnit.HasValue && numberOfItemsInPurchaseUnit.Value != 1) ? itemInfo.PurchaseUom : itemInfo.InventoryUom,
							"BatchNumber", batchNumber,
							"BestBeforeDate", bestBeforeDate,
							"SSCC", SSCC,
							"DocNum", Convert.ToString(purchaseOrderDocNum),
							"StillToQuantity", (itemInfo.UomDecimals == 0 ? Math.Floor((stillToReceiveInventory / batchReceivingNumber) / numberOfItemsInPurchaseUnit.Value) : Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal((stillToReceiveInventory / batchReceivingNumber) / numberOfItemsInPurchaseUnit.Value, itemInfo.UomDecimals)),
							"MessageKeyStillQuantity", "MSG_LBL_STILL_TO_RECEIVE",
							"GS1WeightAI", itemInfo.BarcodeType == PmxBarcodeTypes.VariableGTIN ? itemInfo.UomGS1MeasureType : 0,
							"AdditionalInfo", customerInfo,
							"SetFocusToQuantityControl", setFocusToQuantityControl
							));

						msg = WaitForMessage();
						goBack = false;
						// reset error
						initialErrorKey = null;

						if (msg.Name.EndsWith(".QuantityEntered"))
						{
							quantity = ExtractParameter<double>(msg.Parameters, "quantity");

							goto Step_CheckQuantity;
						}
						else if (msg.Name.EndsWith(".BackRequested"))
						{
							goBack = true;
							goto Step_GoBackOnEnterQuantity;

						}
					}
				}

				#endregion

			}

			Step_GoBackOnEnterQuantity:

			//go back

			goBack = true;

			if (itemInfo.HasBestBeforeDate)
			{
				goto Step_BestBeforeDate;
			}
			else if (itemInfo.HasSecondBatchNumber)
			{
				goto Step_SecondBatchNumber;
			}
			else if (itemInfo.HasBatchnumber)
			{
				goto Step_SelectBatchNumber;
			}



			if (this.GetNumberOfRows(dsItems) == 1)
			{
				//go back
				if (currentLogUnit != null &&
					currentLogUnit.ItemsOnLogisticUnit.Count > 0)
				{
					//we want to go back, but we have some items in the list
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_PRODUCTS_ON_PALLET_TO_RECEIVE",
							"MessageKey", "MSG_QUESTION_PRODUCTS_ON_PALLET_TO_RECEIVE"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_ScanLabelStart;
					}

					goto Step_EnterQuantity;
				}
			}

			//there where more items in the list
			goto Step_IdentifyProduct;



			#endregion

			Step_CheckQuantity:

			#region CHECK QUANTITY

			//if allowed, check the serial numbers
			if (checkDataInUDFPOLine && isPOManuallySelected && serialNumbersToCheck.Count > 0 &&
				(itemInfo.HasSerialNumber || itemInfo.HasSAPSerialNumber))
			{
				bool failedCheck = false;
				foreach (string serialNumber in listOfSerialNumbers)
				{
					failedCheck = !serialNumbersToCheck.Contains(serialNumber);
					if (failedCheck) break;
				}

				if (failedCheck)
				{
					//ask user what to do
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
								"TitleKey", "MSG_TITLE_ENTERED_DATA_FAILED_CHECK",
							"MessageKey", "MSG_QUESTION_ENTERED_DATA_FAILED_CHECK_SERIALNUMBER"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						if (!(barcode != null &&
							barcode.GS1DataObject != null &&
							!string.IsNullOrEmpty(barcode.GS1DataObject.SerialNumber)))
						{
							goto Step_EnterQuantity;
						}
						else if (isGS1DataEnteredOnScanLogDataScreen)
						{
							goBack = true;
							goto Step_ScanLabelStart;
						}
					}
					goBack = true;
					goto Step_IdentifyProduct;
				}

			}


			double qtyToCheck = 0;
			if ((itemInfo.ListOfPackagingTypes != null &&
				   listOfPackagingTypeInfosForScreen != null &&
			   listOfPackagingTypeInfosForScreen.Count > 0))
			{
				qtyToCheck = quantity;
				//if packaging types are entered, the quantity is in inventory uom,
			}
			else
			{
				qtyToCheck = quantity * numberOfItemsInPurchaseUnit.Value;
			}

			double fullQuantity = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(qtyToCheck * batchReceivingNumber, itemInfo.UomDecimals);

			if (fullQuantity > stillToReceiveInventory &&
				stillToReceiveInventory > 0)
			{
				//ask user what to do
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_RECEIVING_MORE_THAN_ORDERED",
						"MessageKey", "MSG_QUESTION_RECEIVING_MORE_THAN_ORDERED"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					//re-enter the quantity
					goto Step_EnterQuantity;
				}
			}

			goto Step_PrintItemLabel;

			#endregion

			Step_PrintItemLabel:

			#region PRINT ITEM LABEL
			numberOfItemLabels = 0;
			if (itemInfo.PrintItemLabel &&
				//Only print here if weight was not asked.
				!(itemInfo.IsCatchWeightItem && askForWeight))
			{
				if (currentPmxOseCompany.GoodsReceiptAutoPrintItemLabels)
				{
					if (askUserToPrintLabels)
					{
						//ask user if he wants to print item labels
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
								"MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
						msg = WaitForMessage();

						if (msg.Name.EndsWith(".Yes"))
						{
							Convert.ToInt32(Math.Ceiling(quantity));
						}
						else
						{
							goto Step_EnterCurrentDataIntoList;
						}
					}
					numberOfItemLabels = Convert.ToInt32(Math.Ceiling(quantity));
				}
				else
				{
					//ask user if he wants to print item labels
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
							"MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						//numberOfItemLabels = Convert.ToInt32(Math.Ceiling(quantity));
						numberOfItemLabels = null;
					}
					else
					{
						goto Step_EnterCurrentDataIntoList;
					}

				}


				// print the labels

				//Set the data into binders
				this.m_printItemLabelFlow_LocationCode = currentDockCode;
				this.m_printItemLabelFlow_NeededBatchNumber = batchNumber;
				this.m_printItemLabelFlow_NeededBatchNumber2 = secondBatchNumber;
				this.m_printItemLabelFlow_NeededBestBeforeDate = bestBeforeDate;
				this.m_printItemLabelFlow_NeededItemCode = itemInfo.ItemCode;
				this.m_printItemLabelFlow_NeededNrOfLabels = numberOfItemLabels;
				this.m_printItemLabelFlow_EnteredQuantity = qtyToCheck;
				this.m_printItemLabelFlow_EnteredQuantityUom2 = null;
				this.m_printItemLabelFlow_ShowPrintMessage = true;

				m_printItemLabelFlow_NeededPackagingType = null;
				m_printItemLabelFlow_NeededQuantityByPackagingType = null;

				if (numberOfItemLabels.HasValue)
				{
					this.m_printItemLabelFlow_NeededPackagingType = purchaseUom;

					if (numberOfItemsInPurchaseUnit.HasValue)
					{
						this.m_printItemLabelFlow_NeededQuantityByPackagingType = Convert.ToInt32(Math.Ceiling(numberOfItemsInPurchaseUnit.Value));
					}
				}



				//call flow
				base.StartSubFlowAndWait(m_printItemLabelFlow, true);
				goBack = false;
				if (m_printItemLabelFlow_BackRequested)
				{
					goto Step_EnterQuantity;
				}


				goto Step_EnterCurrentDataIntoList;


			}
			else
			{
				goto Step_EnterCurrentDataIntoList;
			}

			#endregion

			Step_EnterCurrentDataIntoList:

			#region ENTER CURRENT DATA INTO LIST

			if (itemInfo.ListOfPackagingTypes != null && listOfPackagingTypeInfosForScreen != null && listOfPackagingTypeInfosForScreen.Count > 0)
			{
				//if packaging types are entered, the quantity is in inventory uom,
				//so we need to adjust the quantity to receive
				quantity /= numberOfItemsInPurchaseUnit.Value;
			}

			//Use the first logistic unit. This because the code at this moment only supports
			//reception of one logistic unit that must be saved as a good receipt.
			LogisticUnitItemGoodReceipt logUnitItem = new LogisticUnitItemGoodReceipt();
			logUnitItem.BatchNumber1 = batchNumber;
			logUnitItem.BatchNumber2 = secondBatchNumber;
			logUnitItem.BestBeforeDate = bestBeforeDate;
			logUnitItem.ItemCode = itemInfo.ItemCode;
			logUnitItem.PurchaseDocRef = purchaseDocRef;
			logUnitItem.IsLogisticCarrierForLogisticUnit = false;
			logUnitItem.IsLogisticCarrier = itemInfo.IsLogisticCarrier;
			logUnitItem.QuantityPerUom = numberOfItemsInPurchaseUnit.Value;// ((itemInfo.HasSAPSerialNumber || itemInfo.HasSerialNumber) && listOfSerialNumbers.Count > 0) ? 1 : numberOfItemsInPurchaseUnit.Value;
			logUnitItem.Quantity = quantity;
			logUnitItem.QuantityUom2 = quantityUom2;
			logUnitItem.PurchaseDocumentLineNum = lineNr;

			logUnitItem.Uom2 = quantityUom2.HasValue ? itemInfo.Uom2 : null;

			logUnitItem.ListOfSerialNumbers = listOfSerialNumbers;

			logUnitItem.ListOfReasonsByZoneType = listOfReasonsCodesAndZoneTypes;

			if (listOfReasonCodeByItem.ContainsKey(logUnitItem.ItemCode))
			{
				logUnitItem.ReasonInfo = listOfReasonCodeByItem[logUnitItem.ItemCode];
			}

			if (listOfPackagingTypeInfosForScreen != null && listOfPackagingTypeInfosForScreen.Count > 0)
			{
				//Screen list in in wrong order, so reversing order
				Collection<PackagingTypeInfo> tempList = new Collection<PackagingTypeInfo>();
				for (int iLoop = listOfPackagingTypeInfosForScreen.Count - 1; iLoop >= 0; iLoop--)
				{
					tempList.Add(listOfPackagingTypeInfosForScreen[iLoop]);
				}

				logUnitItem.FullListOfPackagingTypes = tempList;
			}

			logUnitItem.ListOfPackagingTypes = currentListOfTransactionalPackagingTypeInfos;
			logUnitItem.ListOfBatchAttributes = currentListOfTransactionalBatchAttributeInfos;

			logUnitItem.AutoSelectPO = autoSelectPO;

			if (currentLogisticCarrier != null)
			{
				//add logistic carrier to list when it is set
				LogisticUnitItemGoodReceipt logUnitItemLogCar = new LogisticUnitItemGoodReceipt();

				logUnitItemLogCar.IsLogisticCarrierForLogisticUnit = currentLogisticCarrier.IsLogisticCarrierForLogisticUnit;
				logUnitItemLogCar.IsLogisticCarrier = currentLogisticCarrier.IsLogisticCarrier;
				logUnitItemLogCar.BatchNumber1 = currentLogisticCarrier.BatchNumber1;
				logUnitItemLogCar.BatchNumber2 = currentLogisticCarrier.BatchNumber2;
				logUnitItemLogCar.BestBeforeDate = currentLogisticCarrier.BestBeforeDate;
				logUnitItemLogCar.ItemCode = currentLogisticCarrier.ItemCode;
				logUnitItemLogCar.Quantity = currentLogisticCarrier.Quantity;
				logUnitItemLogCar.PurchaseDocumentLineNum = lineNr ?? -1;
				logUnitItemLogCar.PurchaseDocRef = purchaseDocRef;

				currentLogUnit.ItemsOnLogisticUnit.Add(logUnitItemLogCar);
			}

			//HOOK
			m_afterLogisticUnitItemEnteredHookScript_LogUnitItem = new LogisticUnitItemGoodReceipt();
			m_afterLogisticUnitItemEnteredHookScript_LogUnitItem = logUnitItem;
			m_afterLogisticUnitItemEnteredHookScript.BindParameters(
				 this.DefaultCultureInfo,
				 this.PmxOseCompany,
				  new ReadOnlyBinder<int?>("ContainerDocEntry", containerDocEntry),
				  new ReadWriteBinder<LogisticUnitItemGoodReceipt>("LogUnitItem", this, "m_afterLogisticUnitItemEnteredHookScript_LogUnitItem"),
				  new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")
				  );

			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_afterLogisticUnitItemEnteredHookScript, true);


			if (m_subFlowBackRequested)
			{
				goBack = true;

				goto Step_EnterQuantity;

			}


			logUnitItem = m_afterLogisticUnitItemEnteredHookScript_LogUnitItem;
			currentLogUnit.ItemsOnLogisticUnit.Add(logUnitItem);

			currentLogisticCarrier = null;

			goto Step_AskToAddMore;


			#endregion

			Step_AskToAddMore:

			#region ASK TO ADD MORE

			if (businessPartnerInfo != null)
			{
				if (this.IsLUOnBarcode(barcode) || businessPartnerInfo.HasNoMixedLogUnitsOnReception)
				{
					//no more items allowed
					goto Step_EnterDataIntoListToReceive;
				}
			}

			if (purchaseDocRef != null || containerDocEntry.HasValue)
			{
				//check if there are still items to receive
				query = BuildQuery.GetCodeNameItemsOrderByName(null, null, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, null, sboProviderService.GetDbTool());
				dsCheckItems = sboProviderService.RunView(true, null, null, query);

				RemoveReceivedItems(purchaseDeliveryNoteList, dsCheckItems, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

				if (this.GetNumberOfRows(dsCheckItems) == 0)
				{
					//no more rows, so add the data
					goto Step_EnterDataIntoListToReceive;
				}
			}


			//ask user if he wants to add another item to the list
			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"TitleKey", "MSG_TITLE_ADD_MORE_PRODUCTS",
					"MessageKey", "MSG_QUESTION_ADD_MORE_PRODUCTS"));
			msg = WaitForMessage();

			if (msg.Name.EndsWith(".Yes"))
			{

				goto Step_ClearDataBeforeNextItem;
			}

			goto Step_EnterDataIntoListToReceive;

			#endregion



			Step_EnterDataIntoListToReceive:

			#region ENTER DATA INTO LIST TO RECEIVE

			if (receiveGoodsOnSelectedLocation && !crossDocking)
			{

				Step_IdentifyDestinationLocation:

				#region IDENTIFY DESTINATION LOCATION

				string firstItemCode = null;

				for (int i = 0; i < currentLogUnit.ItemsOnLogisticUnit.Count; i++)
				{
					if (!currentLogUnit.ItemsOnLogisticUnit[i].IsLogisticCarrier)
					{
						firstItemCode = currentLogUnit.ItemsOnLogisticUnit[i].ItemCode;
						break;
					}
				}

				if (firstItemCode == null)
				{
					firstItemCode = currentLogUnit.ItemsOnLogisticUnit[0].ItemCode;
				}

				if (itemInfo == null || itemInfo.ItemCode != firstItemCode)
				{
					itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), firstItemCode);
				}

				string defaultLocation;

				if (!itemInfo.DefaultWarehouseLocationOrZone.TryGetValue(whsInfo.PmxWarehouseCode, out defaultLocation))
				{
					//defaultLocation = itemInfo.DefaultStorageLocationOrZoneCode;
				}

				bool isOnlyItemOnUnit = true;
				string locationUom = null;

				for (int x = 1; x < currentLogUnit.ItemsOnLogisticUnit.Count; x++)
				{
					if (firstItemCode != currentLogUnit.ItemsOnLogisticUnit[x].ItemCode && !currentLogUnit.ItemsOnLogisticUnit[x].IsLogisticCarrier)
					{
						defaultLocation = null;
						isOnlyItemOnUnit = false;
						break;
					}

				}

				string destinationLocation = null;
				string enteredDestinationLocationCode = null;
				string buttonKeyIdentifyDestinationLocationScreen = null;


				//Get whether locations are suggested or not
				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxOseStorageLocationProvider slProv = new PmxOseStorageLocationProvider(conn);
						PmxItemTransactionalInfoProvider itriProv = new PmxItemTransactionalInfoProvider(conn);

						LocationSuggestionRequest request = new LocationSuggestionRequest();

						request.ListOfItemsToMove = new Collection<MoveItemInfo>();

						if (listOfUsedLocations.Count > 0)
						{
							request.ListOfUsedLocationCodes = listOfUsedLocations;
						}

						foreach (LogisticUnitItemGoodReceipt info in currentLogUnit.ItemsOnLogisticUnit)
						{
							MoveItemInfo move = new MoveItemInfo();

							move.ItemCode = info.ItemCode;
							move.Quantity = info.Quantity;
							move.QuantityPerUom = info.QuantityPerUom;

							move.DestinationLogisticUnitIdentKey = currentLogUnit.LogisticUnitId;
							move.ItemTransactionalInfoKey = itriProv.GetItemTransactionalInfoKey(info.ItemCode, info.BatchNumber1, info.BatchNumber2, info.BestBeforeDate);

							request.ListOfItemsToMove.Add(move);
						}



						listOfLocationInfos = slProv.GetSuggestedLocations(LocationRequestType.PutAway, request, whsInfo.PmxWarehouseCode, out useLocationSuggestions, out listOfLocationInfosAsDictionary);

						return null;
					});

				buttons = new List<string>();

				if (useLocationSuggestions)
				{
					buttons.Add("MSG_BUTTON_SELECT_OTHER_LOCATION");
					if (listOfLocationInfos == null ||
						listOfLocationInfos.Count == 0)
					{
						//No locations found.
						//But not going back, becasue this is impossible
						goto Step_AfterLocationSelection;
					}

					defaultLocation = listOfLocationInfos[0].LocationCode;
				}
				else
				{
					buttons.Add("MSG_BUTTON_SELECT_LOCATION");
					buttons.Add("MSG_BUTTON_SELECT_EMPTY_LOCATION");
				}

				//Try to get additional info of the default location
				double? currentStockOnLocation = null;
				double? maximumStockOnLocation = null;

				if (isOnlyItemOnUnit &&
					!string.IsNullOrEmpty(defaultLocation))
				{
					currentStockOnLocation = GetCurrentStock(defaultLocation, firstItemCode, out maximumStockOnLocation, session, sboProviderService);
					//iteminfo should be the one for the item code in 'firstItemCode'
					locationUom = itemInfo.InventoryUom;
				}

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SCAN_DESTINATION_LOCATION",
					"HideDefaultLocation", defaultLocation == null,
					"DefaultLocation", defaultLocation,
					"LocationCurrentStock", currentStockOnLocation,
					"LocationMaximumStock", maximumStockOnLocation,
					"Uom", locationUom,
					"EnabledToolBarButtons", ToolBarButtons.Forward,
					"AllowedButtons", buttons.ToArray()));
				msg = WaitForMessage();
				goBack = false;
				if (msg.Name.EndsWith(".ButtonClicked"))
				{
					buttonKeyIdentifyDestinationLocationScreen = ExtractParameter<string>(msg.Parameters, "buttonKey");
					goto Step_SelectDestinationLocation;
				}

				if (msg.Name.EndsWith(".LocationEntered"))
				{
					enteredDestinationLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

					enteredDestinationLocationCode = sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
						return prov.GetLocationCodeFromScannedValue(enteredDestinationLocationCode);
					});

					goto Step_SelectDestinationLocation;
				}

				//go back is not allowed
				goto Step_IdentifyDestinationLocation;

				#endregion

				Step_SelectDestinationLocation:

				#region SELECT DESTINATION LOCATION

				DataSet dsDestinationLocations = null;

				//get list of items/quantities
				PmxDictionary<string, PmxStorageLocationCanItemBeStoredInfo> canItemBeStoredInfoPerItemCode = new PmxDictionary<string, PmxStorageLocationCanItemBeStoredInfo>();
				foreach (LogisticUnitItemGoodReceipt info in currentLogUnit.ItemsOnLogisticUnit)
				{
					PmxStorageLocationCanItemBeStoredInfo entry;
					if (canItemBeStoredInfoPerItemCode.TryGetValue(info.ItemCode, out entry))
					{
						entry.Quantity += info.Quantity;
					}
					else
					{
						entry = new PmxStorageLocationCanItemBeStoredInfo(info.Quantity, null, null);
						canItemBeStoredInfoPerItemCode.Add(info.ItemCode, entry);
					}

					int? itri = sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(
						delegate (PmxDbConnection conn, object[] parameters)
						{
							PmxItemTransactionalInfoProvider prov = new PmxItemTransactionalInfoProvider(conn);
							return prov.GetItemTransactionalInfoKey(info.ItemCode, info.BatchNumber1, info.BatchNumber2, info.BestBeforeDate);
						}));

					if (!itri.HasValue &&
						(!string.IsNullOrEmpty(info.BatchNumber1) ||
						!string.IsNullOrEmpty(info.BatchNumber2) ||
						info.BestBeforeDate.HasValue))
					{
						//itri not known yet, but item has batch or BBD, so set a default itri
						itri = -1;
					}

					if (itri.HasValue)
					{
						if (entry.ItriKeys == null)
						{
							entry.ItriKeys = new List<int>();
						}
						entry.ItriKeys.Add(itri.Value);
					}

				}

				if (!useLocationSuggestions)
				{

					//get zonetypes
					Collection<string> itemCodes = new Collection<string>();
					foreach (string tempItemCode in canItemBeStoredInfoPerItemCode.Keys)
					{
						itemCodes.Add(tempItemCode);
					}
					query = BuildQuery.GetZoneTypesForItems(itemCodes);
					dsZoneTypes = sboProviderService.RunView(false, null, null, query);

					Collection<string> listOfZoneTypes = new Collection<string>();
					if (this.GetNumberOfRows(dsZoneTypes) > 0)
					{
						foreach (DataRow row in dsZoneTypes.Tables[0].Rows)
						{
							listOfZoneTypes.Add(row[PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME] as string);
						}
					}

					string whsCodeToUse = whsInfo.PmxWarehouseCode;

					query = BuildQuery.GetStorageLocationForZoneTypes(enteredDestinationLocationCode, whsCodeToUse, listOfZoneTypes,
					buttonKeyIdentifyDestinationLocationScreen == "MSG_BUTTON_SELECT_EMPTY_LOCATION");

					dsDestinationLocations = sboProviderService.RunView(false, null, null, query);
				}
				else
				{
					//We are using suggested locations.
					//So now we need to check if the entered value is available in the list.

					if (!string.IsNullOrEmpty(enteredDestinationLocationCode))
					{
						//a location was entered, so check if it is available
						if (listOfLocationInfosAsDictionary == null || !listOfLocationInfosAsDictionary.ContainsKey(enteredDestinationLocationCode))
						{
							//Location not available
							initialErrorKey = "MSG_ERROR_LOCATION_NOT_AVAILABLE";
							goto Step_IdentifyDestinationLocation;
						}

						//Location is available, so fill in the needed variables:
						destinationLocation = listOfLocationInfosAsDictionary[enteredDestinationLocationCode].LocationCode;
						//destinationPmxWhsCode = listOfLocationInfosAsDictionary[enteredDestinationLocationCode].PmxWhsCode;
						//blockMoveWhenNotEmpty = listOfLocationInfosAsDictionary[enteredDestinationLocationCode].BlockMoveWhenNotEmpty;

						//When the location is not an allowed suggested location, a reason needs to be entered
						needsToEnterReasonForNotAllowedLocation = !listOfLocationInfosAsDictionary[enteredDestinationLocationCode].AllowedSuggestedLocation;


						goto Step_CheckLocation;
					}
					else
					{
						//no locations was entered, so build the list
						dsDestinationLocations = PmxOseStorageLocationProvider.ConvertLocationInfoListToLocationDataSet(listOfLocationInfos);
					}
				}

				if (this.GetNumberOfRows(dsDestinationLocations) == 0)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", "MSG_ERROR_NO_LOCATION_FOUND",
							"NoTranslationOfMessageKey", false,
							"ShowButton", true));
					WaitForMessage();

					goto Step_IdentifyDestinationLocation;
				}
				//if ( !String.IsNullOrEmpty( enteredDestinationLocationCode )
				//    && SystemInt32Convertor.ObjectToInt( dsDestinationLocations.Tables[ 0 ].Rows[ 0 ][ LocationDataSet.Columns.FREE_SPACE_FOR_LOG_UNITS.Name ], 1, 1 ) < 1 )
				//{
				//    // Inform the user with the error
				//    session.ShowScreen( typeof( Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen ),
				//        DefaultCultureInfo.Get(), BuildParamCollection(
				//            "MessageKey", "MSG_ERROR_LOCATION_NOT_AVAILABLE",
				//            "NoTranslationOfMessageKey", false,
				//            "ShowButton", true ) );
				//    WaitForMessage();

				//    goto Step_IdentifyDestinationLocation;
				//}

				//locations found
				if (!string.IsNullOrEmpty(enteredDestinationLocationCode) &&
					this.GetNumberOfRows(dsDestinationLocations) == 1)
				{
					destinationLocation = Convert.ToString(dsDestinationLocations.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
					goto Step_CheckLocation;
				}

				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION",
					"LocationDS", dsDestinationLocations,
					"ShowStopButton", false));

				msg = WaitForMessage();
				goBack = false;
				// Reset latest error
				initialErrorKey = null;

				if (msg.Name.EndsWith(".LocationSelected"))
				{
					destinationLocation = ExtractParameter<string>(msg.Parameters, "locationCode");

					goto Step_CheckLocation;
				}
				else
				{
					//back requested
					goto Step_IdentifyDestinationLocation;
				}

				#endregion

				Step_CheckLocation:

				#region CHECK LOCATION

				if (useLocationSuggestions)
				{
					//For suggested locations, we need to check if we need to enter a reason
					//A location was entered, so check if it is available
					if (listOfLocationInfosAsDictionary == null || !listOfLocationInfosAsDictionary.ContainsKey(destinationLocation))
					{
						//Location not available
						initialErrorKey = "MSG_ERROR_LOCATION_NOT_AVAILABLE";
						goto Step_IdentifyDestinationLocation;
					}

					//When the location is not an allowed suggested location, a reason needs to be entered
					needsToEnterReasonForNotAllowedLocation = !listOfLocationInfosAsDictionary[destinationLocation].AllowedSuggestedLocation;

				}

				try
				{
					if (sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteIsLocationLocked), destinationLocation))
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", "MSG_ERROR_DESTINATION_LOCATION_LOCKED",
								"NoTranslationOfMessageKey", false,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();

						goto Step_IdentifyDestinationLocation;

					}


					//check if items can be stored on location
					bool canBeStored = sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
						return prov.CanItemsBeStoredOnStorageLocation(destinationLocation, canItemBeStoredInfoPerItemCode);
					});
					if (!canBeStored)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", "MSG_ERROR_STORAGE_LOCATION_NOT_ALLOWED_FOR_ITEM",
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();


						goto Step_IdentifyDestinationLocation;

					}


					foreach (KeyValuePair<string, PmxStorageLocationCanItemBeStoredInfo> kv in canItemBeStoredInfoPerItemCode)
					{
						if (kv.Value.Status == PmxStorageLocationCanItemBeStored.BlockDifferentItemOrBatch)
						{
							// Inform the user with the error
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"MessageKey", "MSG_BLOCK_ADDING_DIFFERENT_ITEM_OR_BATCH",
									"NoTranslationOfMessageKey", false,
									"IsError", true,
									"ShowButton", true));
							WaitForMessage();

							goto Step_IdentifyDestinationLocation;
						}
					}

					foreach (KeyValuePair<string, PmxStorageLocationCanItemBeStoredInfo> kv in canItemBeStoredInfoPerItemCode)
					{
						if (kv.Value.Status != PmxStorageLocationCanItemBeStored.Yes)
						{
							// Warn user and ask to continue
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"TitleKey", "MSG_TITLE_CONFIRM_MOVE",
									"MessageKey", "MSG_CONFIRM_ADDING_DIFFERENT_ITEM_OR_BATCH"));
							msg = WaitForMessage();

							if (!msg.Name.EndsWith("IDecisionScreenReply.Yes"))
							{
								goto Step_IdentifyDestinationLocation;
							}
							break;
						}
					}

					//Check if move is to warehouse automation location
					if (listOfLocationsForWA.Contains(destinationLocation))
					{
						//This is not allowed for move to WA locations
						initialErrorKey = "MSG_ERROR_MOVE_TO_WA_LOCATION_NOT_ALLOWED";

						goto Step_IdentifyDestinationLocation;
					}


					currentLogUnit.SpecificLocationCode = destinationLocation;

					listOfUsedLocations.Add(destinationLocation);

				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
								"NoTranslationOfMessageKey", true,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();

					}
					else
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"PmxError", ((ProdumexException)ex).Error,
								"ShowButton", true));
						WaitForMessage();
					}
					goto Step_IdentifyDestinationLocation;
				}

				#endregion

			}

			Step_AfterLocationSelection:


			Step_EnterReasonLocations:

			#region ENTER REASON FOR LOCATIONS

			reasonCodeLocations = null;
			reasonFreeTextLocations = null;
			reasonLocationCode = null;
			reasonNameLocations = null;

			if (needsToEnterReasonForNotAllowedLocation)
			{

				if (listOfLocationInfos != null &&
					listOfLocationInfos.Count > 0)
				{
					reasonLocationCode = listOfLocationInfos[0].LocationCode;
				}

				//get all reasons for deviating standard flows
				query = BuildQuery.GetReasonsForDeviatingFromStandardProcess();
				DataSet dsReasonsLocations = sboProviderService.RunView(false, null, null, query);



				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreen),
						  this.DefaultCultureInfo, BuildParamCollection(
						  "InitialErrorKey", initialErrorKey,
						  "TitleKey", "MSG_TITLE_SELECT_REASON",
						  "MessageKey", "MSG_SELECT_REASON_FOR_SELECTING_OTHER_LOCATION",
						  "ReasonDS", dsReasonsLocations));
				msg = WaitForMessage();
				initialErrorKey = null;

				if (msg.Name != "Produmex.Foundation.SlimScreen.Interfaces.ISelectReasonScreenReply.ReasonSelected")
				{
					//go back
					goto Step_EnterDataIntoListToReceive;
				}

				reasonCodeLocations = ExtractParameter<string>(msg.Parameters, "reasonCode");

				bool enterFreeTextReason = false;

				DataRow[] rows = dsReasonsLocations.Tables[0].Select(String.Format("{0} = '{1}'", ReasonDataSet.Columns.REASON_CODE.Name, reasonCode));
				if (rows.Length > 0)
				{
					reasonName = rows[0][ReasonDataSet.Columns.REASON_NAME.Name] as string;
					enterFreeTextReason = rows[0][ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name] as string == SboBooleanColumnValues.Y.VALUE;
				}

				// Show screen with list
				#region ENTER REASON FREE TEXT

				if (enterFreeTextReason)
				{

					// Show screen with list
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"InitialErrorKey", initialErrorKey,
							"TitleKey", "MSG_TITLE_ENTER_REASON",
							"Information", "MSG_ENTER_FREE_REASON_DEFAULT",
							"Parameters", new object[] { reasonName },
							"AllowToGoBack", true,
							"ForceDataEntry", true,
							"AllowMultiLine", true,
							"MinimumNumberOfCharacters", Definitions.MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON
							));

					msg = WaitForMessage();

					initialErrorKey = null;

					if (msg.Name.EndsWith(".StringEntered"))
					{
						reasonFreeTextLocations = ExtractParameter<string>(msg.Parameters, "stringValue");

					}
					else
					{
						//go back
						goto Step_EnterReasonLocations;
					}


				}

				ReasonInfo reasonInfo = new ReasonInfo();
				reasonInfo.ReasonCode = reasonCodeLocations;
				reasonInfo.ReasonFreeText = reasonFreeTextLocations;
				reasonInfo.ReasonLocationCode = reasonLocationCode;

				currentLogUnit.Reason = reasonInfo;

				#endregion
			}

			#endregion

			m_subFlowLUIDs = new List<LogisticUnitGoodsReceipt>();

			//currentLogUnit.SpecificLocationCode = null;

			Collection<PmxDictionary<string, object>> listOfListOfParameters = new Collection<PmxDictionary<string, object>>();

			// Check for possibility of splitting received quantity over multiple LogUnits with DefaultQuantityOnLogisticUnit
			string splitItemCode = null;
			double splitQty = 0.0;
			int splitInNrOfLogUnits = 1;
			double qtyOnLogUnitInPurchaseUom = double.MaxValue;
			if (splitQtyInLogUnitsOfItemDefQtyOnLogUnit && itemInfo.DefaultQuantityOnLogisticUnit > 0)
			{
				if (currentLogUnit.ItemsOnLogisticUnit.Count <= 2)
				{
					foreach (LogisticUnitItemGoodReceipt item in currentLogUnit.ItemsOnLogisticUnit)
					{
						if (!item.IsLogisticCarrier && !item.IsLogisticCarrierForLogisticUnit)
						{
							if (splitItemCode != null ||
								(item.ListOfSerialNumbers != null && item.ListOfSerialNumbers.Count > 0))
							{
								splitItemCode = null;  // Failure, cannot have more than 1 splitItem, serial numbers are also not allowed.
								break;
							}
							splitItemCode = item.ItemCode;
							splitQty += item.Quantity;

							itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), splitItemCode);
							qtyOnLogUnitInPurchaseUom = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(itemInfo.DefaultQuantityOnLogisticUnit / item.QuantityPerUom, 6);
						}
					}
				}
				if (splitItemCode != null)
				{
					splitInNrOfLogUnits = (int)Produmex.Foundation.Utilities.FormattingHelper.CeilingForDecimals(splitQty / qtyOnLogUnitInPurchaseUom, 0);
				}
			}

			LogisticUnitGoodsReceipt logUnitTemplate = currentLogUnit;

			for (int splitLogUnitIdx = 0; splitLogUnitIdx < splitInNrOfLogUnits; splitLogUnitIdx++)
			{
				if (splitInNrOfLogUnits > 1)
				{
					// We change an item (splitItem) in currentLogUnit.ItemsOnLogisticUnit and the last one can be different, so make a copy
					// Also, maybe some other properties of currentLogUnit are modified
					currentLogUnit = RuntimeHelper.DeepCopy<LogisticUnitGoodsReceipt>(logUnitTemplate);

					foreach (LogisticUnitItemGoodReceipt item in currentLogUnit.ItemsOnLogisticUnit)
					{
						if (item.ItemCode == splitItemCode)
						{
							item.Quantity = Math.Min(splitQty, qtyOnLogUnitInPurchaseUom);
							splitQty = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(splitQty - item.Quantity, itemInfo.UomDecimals);
							break;
						}
					}
				}

				for (int iLoop = 0; iLoop < batchReceivingNumber; iLoop++)
				{
					//check if there is locked stock (not when splitting over multiple LogUnits)
					if ((!splitQtyInLogUnitsOfItemDefQtyOnLogUnit || SboQuantity.IsZeroOrLess(itemInfo.DefaultQuantityOnLogisticUnit, itemInfo.UomDecimals))  // Currently only supported for non-splitting
						&& purchaseDocRef != null
						&& string.IsNullOrEmpty(currentLogUnit.SSCC))
					{
						Collection<string> listOfItemCodes = new Collection<string>();
						foreach (LogisticUnitItemGoodReceipt item in currentLogUnit.ItemsOnLogisticUnit)
						{
							if (!listOfItemCodes.Contains(item.ItemCode))
							{
								listOfItemCodes.Add(item.ItemCode);
							}
						}
						query = BuildQuery.GetLockedStockForBaseOrder(listOfItemCodes, purchaseDocRef);
						dsSSCC = sboProviderService.RunView(true, null, null, query);

						if (this.GetNumberOfRows(dsSSCC) > 0)
						{
							//we found stock, so we ask user what to do
							//ask user if he wants to add another item to the list
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifySSCCListScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"TitleKey", "MSG_TITLE_FOUND_STOCK_FOR_SAME_ORDER",
									"BarcodeController", barcodeController,
									"SSCCDS", dsSSCC,
									"ButtonKey", "MSG_BUTTON_MAKE_NEW_SSCC"
									));
							msg = WaitForMessage();

							if (msg.Name.EndsWith(".SSCCEntered"))
							{
								int scannedLUID = ExtractParameter<int>(msg.Parameters, "luid");

								DataRow[] rows = dsSSCC.Tables[0].Select(SSCCDataSet.Columns.LUID.Name + " = " + scannedLUID);
								if (rows.Length > 0)
								{
									currentLogUnit.SpecificLocationCode = Convert.ToString((rows[0][SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name]));
									currentLogUnit.SSCC = ExtractParameter<string>(msg.Parameters, "sscc");
									currentLogUnit.LogisticUnitId = scannedLUID;

									currentLogUnit.IsMoveToLockedStockLocation = true;
								}

								//if logistic carrier was defined, we need to set IsLogisticCarrier to false,
								//otherwise the system will try to add it as the logistic carrier for the logistic unit.
								//But it is possible the logistic unit to move the items to already has a logistic carrier linked to it.
								foreach (LogisticUnitItemGoodReceipt tempItem in currentLogUnit.ItemsOnLogisticUnit)
								{
									if (tempItem.IsLogisticCarrierForLogisticUnit)
									{
										tempItem.IsLogisticCarrierForLogisticUnit = false;
										//there should be only one logistic carrier
										break;
									}
								}

							}

						}

					}

					// check if there are items that do NOT need an SSCC to be created
					foreach (LogisticUnitItemGoodReceipt itemOnLogUnit in currentLogUnit.ItemsOnLogisticUnit)
					{
						itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemOnLogUnit.ItemCode);
						createSsccOnReception = itemInfo.CreateSsccOnReception;

						if (createSsccOnReception)
						{
							break;
						}
					}

					LogisticUnitGoodsReceipt logUnitToAdd = new LogisticUnitGoodsReceipt();

					if (createSsccOnReception || crossDocking)
					{
						logUnitToAdd.SSCC = currentLogUnit.SSCC;
						logUnitToAdd.LogisticUnitId = currentLogUnit.LogisticUnitId;
					}
					else
					{
						logUnitToAdd.AllowReceptionWithoutLUID = true;
					}
					logUnitToAdd.SupplierPalletNumber = currentLogUnit.SupplierPalletNumber;
					logUnitToAdd.SpecificLocationCode = currentLogUnit.SpecificLocationCode;


					logUnitToAdd.ItemsOnLogisticUnit = currentLogUnit.ItemsOnLogisticUnit;

					logUnitToAdd.Reason = currentLogUnit.Reason;

					if (!logUnitToAdd.LogisticUnitId.HasValue)
					{
						//add luid
						try
						{
							if (createSsccOnReception)
							{
								logUnitToAdd.LogisticUnitId = sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, new DelegateWithPmxDbConnection<int>(ExecuteGenerateLuid), logUnitToAdd.SSCC, logUnitToAdd.SupplierPalletNumber);
								//get the sscc
								if (string.IsNullOrEmpty(logUnitToAdd.SSCC) &&
									logUnitToAdd.LogisticUnitId.HasValue)
								{
									logUnitToAdd.SSCC = sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null, new DelegateWithPmxDbConnection<string>(ExecuteGetSSCCByLUID), logUnitToAdd.LogisticUnitId.Value);
								}
							}

						}
						catch (Exception ex)
						{
							if (!(ex is ProdumexException))
							{
								// Inform the user with the error
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
										"NoTranslationOfMessageKey", true,
										"IsError", true,
										"ShowButton", true));
								WaitForMessage();

							}
							else
							{
								// Inform the user with the error
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"PmxError", ((ProdumexException)ex).Error,
										"ShowButton", true));
								WaitForMessage();
							}

						}
					}

					//only print if not moving to locked stock location

					if (!currentLogUnit.IsMoveToLockedStockLocation &&
						logUnitToAdd.LogisticUnitId.HasValue)
					{

						try
						{
							PmxDictionary<string, object> listOfParameters = new PmxDictionary<string, object>();

							bool isHana = IsHana(sboProviderService);


							if (isHana)
							{
								listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.LUID.NAME_HANA, logUnitToAdd.LogisticUnitId ?? -1);
							}
							else
							{
								listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.LUID.NAME, logUnitToAdd.LogisticUnitId);
							}


							listOfListOfParameters.Add(listOfParameters);

						}
						catch (Produmex.Foundation.ProdumexException ex)
						{
							s_log.Error(ex.ToString());
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
									"MessageAddendum", ex.Message,
									"PmxError", ex.Error,
									"ShowButton", true));

							msg = WaitForMessage();
						}
						catch (Exception ex)
						{
							s_log.Error(ex.ToString());
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", initialErrorKey,
									"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
									"MessageAddendum", ex.Message,
									"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
									"ShowButton", true));

							msg = WaitForMessage();
						}

					}

					purchaseDeliveryNoteList.Add(logUnitToAdd);
					//Add to list to pass to hook flow
					m_subFlowLUIDs.Add(logUnitToAdd);
				}
			}


			Step_AddToASN:

			try
			{

				//Store data into ASN
				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
				   delegate (PmxDbConnection conn, object[] parameters)
				   {
					   PmxAdvanceShippingNoticeProvider prov = new PmxAdvanceShippingNoticeProvider(conn);
					   List<int> tempList = prov.AddASNForPurchase(purchaseDeliveryNoteList, cardCode, businessPartnerInfo != null ? businessPartnerInfo.CardName : null, new FolioNumbering(), containerDocEntry, currentDockCode);

					   if (isNoPo)
					   {
						   asnKeys = tempList;
					   }

					   return null;
				   }));

				//Clear this list, because we don't need it anymore, because it is now stored on the ASN table
				purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();

			}
			catch (Exception ex)
			{
				//Ask if 
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));

					WaitForMessage();

				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));

					WaitForMessage();
				}

				//retry?
				//ask user if he wants to retry
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"TitleKey", "MSG_TITLE_RETRY_UPDATE",
										"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
				msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto Step_AddToASN;
				}

			}



			//only print if not moving to locked stock location
			//When no parameters are filled in, stock will be moved to locked stock location
			if (listOfListOfParameters.Count > 0)
			{
				//print the label
				bool proceed = false;
				while (!proceed)
				{
					numberOfReports = 0;

					try
					{
						if (askUserToPrintLabels)
						{
							//ask user if he wants to print logistic label
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"TitleKey", "MSG_TITLE_PRINT_LABEL",
									"MessageKey", "MSG_QUESTION_PRINT_LABEL"));
							msg = WaitForMessage();

							if (!msg.Name.EndsWith(".Yes"))
							{
								break;
							}

						}

						//trigger event
						numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
						 delegate (PmxDbConnection conn, object[] parameters2)
						 {
							 PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
							 return printProv.PrintReportsForEventWithParametersCollection(PmxPrintReportEventType.ReceptionNewLogisticUnitIdentified,
							 currentDockCode, deviceID, listOfListOfParameters, true, out askReprint);
						 });

					}
					catch (Produmex.Foundation.ProdumexException ex)
					{
						s_log.Error(ex.ToString());
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
								"MessageAddendum", ex.Message,
								"PmxError", ex.Error,
								"ShowButton", true));

						msg = WaitForMessage();
					}
					catch (Exception ex)
					{
						s_log.Error(ex.ToString());
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
								"MessageAddendum", ex.Message,
								"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
								"ShowButton", true));

						msg = WaitForMessage();
					}

					if (numberOfReports > 0 && askReprint)
					{
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", "MSG_QUESTION_REPRINT_LABEL",
								"TitleKey", "MSG_TITLE_REPRINT_LABEL"));

						msg = WaitForMessage();

						if (msg.Name.EndsWith(".No"))
						{
							proceed = true;
						}

					}
					else
					{
						proceed = true;
					}

				}
			}

			goto Step_AfterLUIDReceived;

			#endregion


			Step_AfterLUIDReceived:


			#region AFTER LUID RECEIVED

			m_afterLuidReceivedHookFlow.BindParameters(
			this.DefaultCultureInfo,
			this.PmxOseCompany,
			new ReadWriteBinder<List<LogisticUnitGoodsReceipt>>("LogisticUnits", this, "m_subFlowLUIDs")
			);
			this.StartSubFlowAndWait(m_afterLuidReceivedHookFlow, true);


			currentLogUnit = null;

			if (gotoReceiveItem)
			{
				goto Step_ReceiveItems;
			}

			if (receiveAllItemsOnOnePurchaseDelivery)
			{
				if (purchaseDocRef != null || containerDocEntry.HasValue)
				{
					//check if there are still items to receive
					query = BuildQuery.GetCodeNameItemsOrderByName(null, null, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, null, sboProviderService.GetDbTool());
					dsCheckItems = sboProviderService.RunView(true, null, null, query);

					RemoveReceivedItems(purchaseDeliveryNoteList, dsCheckItems, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

					if (this.GetNumberOfRows(dsCheckItems) == 0)
					{
						//no more rows, so receive the data
						goto Step_AskEndQualityValues;
					}
				}

				if (filterButtonKey == "MSG_BUTTON_PO_FILTER_NO_PO")
				{
					goto Step_AskEndQualityValues;
				}

				goto Step_ShowMessageNextLogisticUnit;
			}
			else
			{
				goto Step_AskEndQualityValues;
			}

			#endregion

			Step_AskEndQualityValues:

			#region ASK END QUALITY VALUES

			m_enterQualityValuesFlow.BindParameters(
			 this.DefaultCultureInfo,
			 this.PmxOseCompany,
			 new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.End),
			 new ReadOnlyBinder<string>("BaseType", SboObjectTypes.PurchaseDelivery),
			 new ReadOnlyBinder<int?>("BaseEntry", null),
			 new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
			 new ReadOnlyBinder<bool>("SaveData", false),

			 new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
			 new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

			 );
			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

			if (m_subFlowBackRequested)
			{
				//go back is not allowed
				goto Step_EnterDataIntoListToReceive;
			}

			if (!currentListOfQualityValueInfos.ContainsKey(PmxMomentOfCapture.End))
			{
				currentListOfQualityValueInfos[PmxMomentOfCapture.End] = m_enterQualityValuesFlow_ListOfQualityValues;
			}

			goto Step_ReceiveItems;

			#endregion

			Step_ReceiveItems:

			#region RECEIVE ITEMS

			listOfGeneratedDocEntry = new KeyValuePair<Collection<DocumentRef>, Collection<int>>();
			gotoReceiveItem = false;
			//add current pallet, if it was not added yet
			if (currentLogUnit != null &&
				currentLogUnit.ItemsOnLogisticUnit.Count > 0 &&
				!goBack)
			{
				gotoReceiveItem = true;
				goto Step_EnterDataIntoListToReceive;
			}

			try
			{
				listOfGeneratedDocEntry = sboProviderService.InvokeMethodWithDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(true, false, null, null, new DelegateWithPmxDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);

						PmxDictionary<string, PmxParameterValue> overruleParams = null;
						if (crossDocking && crossDockingPickListType != null)
						{
							overruleParams = new PmxDictionary<string, PmxParameterValue>();
							overruleParams.Add("PickListType", crossDockingPickListType);
						}

						return prov.AddPurchaseDeliveryNoteBasedOnASN(cardCode, new FolioNumbering(), currentDockCode, supplierRef, containerDocEntry, purchaseDocRef, currentListOfQualityValueInfos, asnKeys, autoSelectPO, overruleParams, reserveInvoicesAreAllowed);

					}));

				purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();
				listOfReasonCodeByItem = new PmxDictionary<string, ReasonInfo>();
				asnKeys = null;
			}
			catch (Exception ex)
			{
				s_log.Error("", ex);
				if (ex is ProdumexException)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();
				}

				//ask user if he wants to try again after error
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
						"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_ReceiveItems;
				}
				goto Step_SelectPOFilter;
			}

			goto Step_PrintDeliveryDocument;

			#endregion

			Step_PrintDeliveryDocument:

			#region PRINT DELIVERY DOCUMENT

			numberOfReports = 0;
			try
			{
				foreach (DocumentRef docRef in listOfGeneratedDocEntry.Key)
				{
					//trigger event
					numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
						 delegate (PmxDbConnection conn, object[] parameters2)
						 {
							 PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
							 return printProv.PrintReportsForEvent(PmxPrintReportEventType.ReceptionPurchaseDeliveryNoteCreated,
							 currentDockCode, deviceID, docRef.DocEntry, out askReprint);
						 });
				}

			}
			catch (Produmex.Foundation.ProdumexException ex)
			{
				s_log.Error(ex.ToString());
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", ex.Error,
						"ShowButton", true));

				msg = WaitForMessage();
			}
			catch (Exception ex)
			{
				s_log.Error(ex.ToString());
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
						"ShowButton", true));

				msg = WaitForMessage();
			}

			goto Step_DocumentsPrinted;

			#endregion

			Step_DocumentsPrinted:

			#region DOCUMENTS PRINTED

			if (numberOfReports > 0 && askReprint)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
						"TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"));

				msg = WaitForMessage();

				if (msg.Name.EndsWith(".Yes"))
				{
					goto Step_PrintDeliveryDocument;
				}

			}

			goto Step_GeneratePutAway;


			#endregion

			Step_GeneratePutAway:

			#region GENERATE PUT AWAY

			if (!crossDocking)
			{
				try
				{
					sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(ExecuteGeneratePutAwayOrder), listOfGeneratedDocEntry.Key, currentDockCode);
				}
				catch (Exception ex)
				{
					s_log.Error("", ex);
					if (ex is ProdumexException)
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"PmxError", ((ProdumexException)ex).Error,
								"ShowButton", true));
						WaitForMessage();
					}
					else
					{
						// Inform the user with the error
						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
							DefaultCultureInfo.Get(), BuildParamCollection(
								"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
								"NoTranslationOfMessageKey", true,
								"IsError", true,
								"ShowButton", true));
						WaitForMessage();
					}

					//ask user if he wants to try again after error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
							"MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
					msg = WaitForMessage();

					if (msg.Name.EndsWith(".Yes"))
					{
						goto Step_GeneratePutAway;
					}
				}
			}

			goto Step_MoveForLogisticUnitsWithOnlyLogicCarriers;

			#endregion

			Step_MoveForLogisticUnitsWithOnlyLogicCarriers:

			#region MOVE LOGISTIC CARRIERS

			try
			{
				sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteCreateMoveForLogisticUnitsWithOnlyLogicCarriersIfConfigured), currentDockCode);
			}
			catch (Produmex.Foundation.ProdumexException ex)
			{
				s_log.Error(ex.ToString());
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", ex.Error,
						"ShowButton", true));

				msg = WaitForMessage();
			}
			catch (Exception ex)
			{
				s_log.Error(ex.ToString());
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
						"MessageAddendum", ex.Message,
						"PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
						"ShowButton", true));

				msg = WaitForMessage();
			}

			//// Inform the user the items are received
			//session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
			//    DefaultCultureInfo.Get(), BuildParamCollection(
			//        "MessageKey", "MSG_ITEMS_ARE_RECEIVED",
			//        "ShowButton", true));
			//WaitForMessage();


			if (goBack)
			{
				cardCode = null;
				goBack = false;
				if (filterButtonKey == "MSG_BUTTON_PO_FILTER_NO_PO")
				{
					goto Step_SelectPOFilter;
				}
				else if (filterButtonKey == "MSG_BUTTON_PO_FILTER_EAN")
				{
					goto Step_FilterOnEAN;
				}
				else if (isContainer)
				{
					goto Step_SelectContainer;
				}
				else
				{
					goto Step_SelectOrder;
				}
			}


			if (purchaseDocRef != null || containerDocEntry.HasValue)
			{
				//check if there are still items to receive
				query = BuildQuery.GetCodeNameItemsOrderByName(null, null, purchaseDocRef, whsInfo, cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing, true, false, crossDocking, containerDocEntry, (checkDataInUDFPOLine && isPOManuallySelected), isNoPo, autoSelectPO, null, sboProviderService.GetDbTool());
				dsCheckItems = sboProviderService.RunView(true, null, null, query);

				RemoveReceivedItems(purchaseDeliveryNoteList, dsCheckItems, currentLogUnit, batchReceivingNumber, sboProviderService, purchaseDocRef, containerDocEntry);

				if (this.GetNumberOfRows(dsCheckItems) > 0)
				{
					//same purchase order, so enter next item
					goto Step_ShowMessageNextLogisticUnit;
				}
			}


			goto Step_SelectPOFilter;

			#endregion


		}

		private int? GetLuidFromASN(ISboProviderService sboProviderService, string cardCode, int? containerDocEntry, bool autoSelectPO, DocumentRef purchaseDocRef, string sscc)
		{
			return sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(
								delegate (PmxDbConnection conn, object[] parameters)
								{
									PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

									List<PmxAdvanceShippingNotice> asnBOs = null;
									if (autoSelectPO)
									{
										asnBOs = asnProv.GetOpenAsnListForAutoSelectPO(cardCode, conn.InternalKeySboUser, "", 1);
									}
									else
									{
										asnBOs = asnProv.GetOpenAsnList(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : purchaseDocRef.DocType,
											containerDocEntry.HasValue ? containerDocEntry.Value : purchaseDocRef.DocEntry, "", 1, conn.InternalKeySboUser);
									}
									int? luid = asnBOs.Where(w => w.SSCC == sscc).Select(s => s.LUID).FirstOrDefault();

									if (!luid.HasValue)
									{
										luid = asnBOs.Where(w => w.MasterSSCC == sscc).Select(s => s.MasterLUID).FirstOrDefault();
									}

									return luid;

								}));
		}

		private bool BackRequestedAfterCallingBeforeReceptionQuantityHook(ISboProviderService sboProviderService, WeighData weighData, string dockCode, DocumentRef purchaseDocRef, PmxItemInfo itemInfo)
		{
			m_beforeReceptionQuantityHookScript_ScaleCode = weighData?.ScaleCode;

			m_beforeReceptionQuantityHookScript.BindParameters(
				this.DefaultCultureInfo,
				this.PmxOseCompany,
				new ReadOnlyBinder<PmxItemInfo>("ItemInfo", itemInfo),
				new ReadOnlyBinder<DocumentRef>("DocRef", purchaseDocRef),
				new ReadOnlyBinder<string>("DockCode", dockCode),
				new ReadWriteBinder<string>("ScaleCode", this, "m_beforeReceptionQuantityHookScript_ScaleCode"),
				new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested")
				);

			m_subFlowBackRequested = false;
			this.StartSubFlowAndWait(m_beforeReceptionQuantityHookScript, true);

			if (weighData != null && !m_subFlowBackRequested && weighData.ScaleCode != m_beforeReceptionQuantityHookScript_ScaleCode)
			{
				weighData.ScaleCode = m_beforeReceptionQuantityHookScript_ScaleCode;
				weighData.ScaleData = null;
				initialErrorKey = GetScaleData(sboProviderService, weighData, m_uomCoefficients);
			}

			return m_subFlowBackRequested;
		}

		private string GetScaleData(ISboProviderService sboProviderService, WeighData weightData, PmxDictionary<int, double> uomCoefficients)
		{
			string initialErrorKey = null;

			weightData.ScaleData = sboProviderService.InvokeMethodWithDbConnection<PmxOseScale>(true, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters)
						{
							SboUomProvider prov = new SboUomProvider((SboConnection)conn);
							int uomEntry = prov.GetUomEntryOfUomCode(weightData.ItemUom);

							try
							{
								uomEntry = prov.GetUomEntryOfUomCode(weightData.ItemUom);
							}
							catch (ProdumexException ex)
							{
								s_log.Error(string.Format("There is no UomEntry for {0} ", weightData.ItemUom));
								initialErrorKey = ex.Message;
								return null;
							}

							PmxOseScaleProvider scaleProvider = new PmxOseScaleProvider(conn);
							PmxOseScale scale = null;

							scale = scaleProvider.GetScaleForLocationWithUomAndQuantity(weightData.ScaleCode, weightData.LocationCode, uomCoefficients, uomEntry, weightData.Quantity);

							if (scale != null)
							{
								weightData.ScaleCoefficient = prov.GetCoefficientForUom(scale.UomEntry, uomEntry, uomCoefficients);
								weightData.ScaleUom = prov.GetUomCodeOfUomEntry(scale.UomEntry);
								weightData.ScaleCode = scale.Code;
								if (!weightData.ScaleCoefficient.HasValue)
								{
									initialErrorKey = string.Format("There is no conversion between {0} and {1}", weightData.ItemUom, weightData.ScaleUom);
									s_log.Error(initialErrorKey);
									return null;
								}


								NumberFormatInfo nfi = new NumberFormatInfo();
								nfi.NumberDecimalSeparator = ".";

								string[] accuracyComponents = scale.Accuracy.ToString(nfi).Split('.');

								weightData.ScaleUomDecimals = accuracyComponents.Length == 2 ? accuracyComponents[1].Length : 0;

								foreach (PmxOseScaleCommand scaleCommand in scale.ScaleCommands)
								{
									switch (scaleCommand.CommandType)
									{
										case PmxScaleCommandType.GetWeight:
											weightData.WeighCommand = scaleCommand.CommandFormat;
											break;
										case PmxScaleCommandType.SetTare:
											weightData.TareCommand = scaleCommand.CommandFormat;
											break;
										case PmxScaleCommandType.SetZero:
											weightData.ZeroCommand = scaleCommand.CommandFormat;
											break;
										case PmxScaleCommandType.WeightReturn:
											weightData.WeighResultFormat = scaleCommand.CommandFormat;
											break;
										case PmxScaleCommandType.TareReturn:
											break;
										case PmxScaleCommandType.ZeroReturn:
											break;
										default:
											break;
									}
								}
							}
							return scale;
						});

			return initialErrorKey;
		}

		private double? GetCoefficientForUoms(ISboProviderService sboProviderService, string uomFrom, string uomTo, PmxDictionary<int, double> uomCoefficients)
		{
			double? result;
			result = sboProviderService.InvokeMethodWithDbConnection<double?>(true, false, null, null,
						delegate (PmxDbConnection conn, object[] parameters)
						{
							SboUomProvider prov = new SboUomProvider((SboConnection)conn);
							int uomFromEntry = prov.GetUomEntryOfUomCode(uomFrom);
							int uomToEntry = prov.GetUomEntryOfUomCode(uomTo);

							return prov.GetCoefficientForUom(uomToEntry, uomFromEntry, uomCoefficients);
						});
			return result;
		}


		/// <summary>
		/// Executes the cross docking.
		/// </summary>
		/// <param name="purchaseOrderDocEntry">The purchase order document entry.</param>
		/// <param name="whsInfo">The WHS information.</param>
		/// <param name="dockCode">The dock code.</param>
		/// <param name="crossDockingPickListType">Type of the cross docking pick list.</param>
		/// <param name="session">The session.</param>
		/// <param name="sboProviderService">The sbo provider service.</param>
		/// <returns></returns>
		private bool ExecuteCrossDocking(int? purchaseOrderDocEntry, PmxOseWarehouseInfo whsInfo, string dockCode, PmxParameterValue crossDockingPickListType, Session session, ISboProviderService sboProviderService)
		{
			string initialErrorKey = null;

			Step_RefreshPO:


			DataSet dsPendingLogUnits = sboProviderService.InvokeMethodWithDbConnection<DataSet>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
			{
				DataSet ds = new DataSet();
				DataTable tbl = ds.Tables.Add();

				DbCommand dbCmd = conn.CreateCommand(BuildQuery.GetReceivedButUnfinishedOrders(dockCode, conn.DbCultureInfo));
				using (DbDataReader rdr = dbCmd.ExecuteReader())
				{
					tbl.Load(rdr);
				}

				return ds;
			});

			int? purchaseOrderToCrossDock = null;
			if (purchaseOrderDocEntry.HasValue && dsPendingLogUnits.Tables[0].Select(PurchaseOrderDataSet.Columns.PO_ID.Name + " = " + purchaseOrderDocEntry.Value.ToString()).Length > 0)
			{
				purchaseOrderToCrossDock = purchaseOrderDocEntry;
			}
			else if (dsPendingLogUnits.Tables[0].Rows.Count > 0)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectPOScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
					"InitialErrorKey", initialErrorKey,
					"TitleKey", "MSG_TITLE_SELECT_RECEIVED_PO_FOR_CROSS_DOCKING",
					"PODS", dsPendingLogUnits
					));
				Message msg = WaitForMessage();

				if (msg.Name.EndsWith("ISelectPOScreenReply.BackRequested"))
				{
					return false;
				}

				if (msg.Name.EndsWith("ISelectPOScreenReply.POSelected"))
				{
					purchaseOrderToCrossDock = ExtractParameter<int>(msg.Parameters, "docEntry");
				}

				if (msg.Name.EndsWith("ISelectPOScreenReply.RefreshRequested"))
				{
					goto Step_RefreshPO;
				}
			}
			if (!purchaseOrderToCrossDock.HasValue)
			{
				return true;
			}

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
				"MessageKey", "MSG_PROCESSING_CROSS_DOCKING"));

			DataRow[] selectedPoRows = dsPendingLogUnits.Tables[0].Select(PurchaseOrderDataSet.Columns.PO_ID.Name + " = " + purchaseOrderToCrossDock.Value.ToString());

			int salesOrderDocEntry = Convert.ToInt32(selectedPoRows[0]["ORDR_DocEntry"]);
			int? generatedPickListDocEntry = null;
			int? generatedWaveKey = null;

			Step_PerformCrossDock:


			bool crossDockErrorOccurred = false;
			try
			{

				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate (PmxDbConnection dbConn, object[] parameters)
				{
					PmxPurchaseDeliveryNoteProvider pdnProv = new PmxPurchaseDeliveryNoteProvider();
					PmxPickListProposalProvider plpProv = new PmxPickListProposalProvider(dbConn);
					PmxPickListProvider plProv = new PmxPickListProvider(dbConn);

					Collection<DocumentRef> salesOrderDocEntries = new Collection<DocumentRef>();
					salesOrderDocEntries.Add(new DocumentRef(SboObjectType.SalesOrder, salesOrderDocEntry));
					PmxDictionary<string, PmxParameterValue> overruleParams = null;
					if (crossDockingPickListType != null)
					{
						overruleParams = new PmxDictionary<string, PmxParameterValue>();
						overruleParams.Add("PickListType", crossDockingPickListType);
					}
					Collection<DocumentRef> salesOrdersInError = new Collection<DocumentRef>();

					Collection<int> generatedPickListProposalDocEntries = plpProv.GeneratePickListProposalsForSalesOrders(salesOrderDocEntries, whsInfo.PmxWarehouseCode, dockCode, null, overruleParams, out salesOrdersInError);
					if (generatedPickListProposalDocEntries.Count > 0)
					{
						Collection<int> generatedPickListDocEntries = plProv.GeneratePickListWave(generatedPickListProposalDocEntries);

						if (generatedPickListDocEntries.Count > 0)
						{
							generatedPickListDocEntry = generatedPickListDocEntries[0];

							foreach (int plDocEntry in generatedPickListDocEntries)
							{
								plProv.MakePicklistReadyFromDocEntry(plDocEntry);
							}

							// DbCommand dbCmd = dbConn.CreateCommand( BuildQuery.GetPurchaseOrderReceivedStock( purchaseOrderToCrossDock.Value, dbConn.DbCultureInfo ) );
							DbCommand dbCmd = dbConn.CreateCommand(BuildQuery.GetSalesOrderAndPickListLockedStock(purchaseOrderToCrossDock.Value, -1 /*salesOrderDocEntry*/, generatedPickListDocEntry.Value, dbConn.DbCultureInfo));
							DataTable tblPurchaseOrderReceivedStock = new DataTable();
							using (DbDataReader rdr = dbCmd.ExecuteReader())
							{
								tblPurchaseOrderReceivedStock.Load(rdr);
							}

							// Build key column for row comparison
							tblPurchaseOrderReceivedStock.Columns.Add("key", typeof(string));
							foreach (DataRow row in tblPurchaseOrderReceivedStock.Rows)
							{
								row["key"] = String.Format("{0}/{1}/{2}/{3}",
									row[PmxInventoryLockingDetailTable.Columns.ItemCode.NAME],
									row[PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME],
									row[PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME],
									row[PmxInventoryLockingDetailTable.Columns.QualityStatusCode.NAME]);
							}


							string query = BuildQuery.GetSerialNumbersFromPicklist(generatedPickListDocEntries, sboProviderService.GetDbTool());
							DataSet dsSerialNumbers = SboRecordsetHelper.ConvertRecordsetToDataSet(s_log, SboRecordsetHelper.RunQuery(s_log, query, dbConn));

							List<int> fullPalletPickedLuids = new List<int>();
							foreach (int plDocEntry in generatedPickListDocEntries)
							{
								using (PmxPickList pl = plProv.GetBO(plDocEntry))
								{
									generatedWaveKey = pl.WaveKey;

									foreach (PmxPickListLine line in pl.Lines)
									{
										// if ( line.LogisticUnitIdentKey.HasValue )
										{
											foreach (DataRow row in tblPurchaseOrderReceivedStock.Rows)
											{
												if ((string)row[PmxInventoryLockingDetailTable.Columns.BaseType.NAME] == Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickList
													&& (int)row[PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME] == plDocEntry
													&& (int)row[PmxInventoryLockingDetailTable.Columns.BaseLine.NAME] == line.LineNum)
												{
													double stockQty = Convert.ToDouble(row[PmxInventoryDetailTable.Columns.Quantity.NAME]);
													double? stockQtyUom2 = Convertor.ConvertToDoubleNullable(row[PmxInventoryDetailTable.Columns.QuantityUom2.NAME]);

													Collection<string> listOfPickedSerialNumbers = new Collection<string>();
													foreach (DataRow rowSerialNumber in dsSerialNumbers.Tables[0].Select(PmxPickListLineTable.Columns.DocEntry.NAME + " = " + plDocEntry.ToString() + " and  " + PmxPickListLineTable.Columns.LineNum.NAME + " = " + line.LineNum.ToString()))
													{
														listOfPickedSerialNumbers.Add(rowSerialNumber[PmxSerialNumberTable.Columns.SerialNumber.NAME].ToString());
													}

													//  double stockQuantityPerUom = Convert.ToDouble(row[PmxInventoryDetailTable.Columns.QuantityPerUom.NAME]);

													// Add quantity from sales-order (not picklist) locked rows
													//foreach ( DataRow salesOrderLockedRow in tblPurchaseOrderReceivedStock.Rows )
													//{
													//    if ( (string)salesOrderLockedRow[ PmxInventoryLockingDetailTable.Columns.BaseType.NAME ] == SboObjectTypes.SalesOrder
													//        && (string)salesOrderLockedRow[ "key" ] == (string)row[ "key" ] )
													//    {
													//        double lockQty = Convert.ToDouble( salesOrderLockedRow[ PmxInventoryLockingDetailTable.Columns.Quantity.NAME ] );
													//        if ( lockQty > 0.0 )
													//        {
													//            stockQty += lockQty;
													//            salesOrderLockedRow[ PmxInventoryLockingDetailTable.Columns.Quantity.NAME ] = 0.0;
													//        }
													//    }
													//}
													// if ( stockQty > 0.0 )
													{
														int rxdStockLuid = Convert.ToInt32(row[PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME]);
														string itemCode = Convert.ToString(row[PmxInventoryLockingDetailTable.Columns.ItemCode.NAME]);
														int? itriKey = SystemInt32Convertor.ObjectToIntOrNull(row[PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME]);
														//if ( line.LogisticUnitIdentKey.Value == rxdStockLuid
														//    && line.ItemCode == itemCode
														//    && line.ItemTransactionalInfoKey == itriKey )  // Do more matching?
														{
															PickingItemInfo pickItemInfo = new PickingItemInfo();
															pickItemInfo.PickListDocEntry = line.DocEntry.Value;
															pickItemInfo.PickListLineID = line.LineNum.Value;
															pickItemInfo.ItemCode = line.ItemCode;
															pickItemInfo.ItemDescription = line.ItemDescription;
															pickItemInfo.CustomItemDescription = line.ItemDescription;
															pickItemInfo.ItemTransactionalInfoKey = line.ItemTransactionalInfoKey;
															pickItemInfo.QualityStatusCode = line.QualityStatusCode;
															pickItemInfo.Quantity = Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(Math.Max(line.Quantity * line.QuantityPerUom, stockQty) / line.QuantityPerUom, 6);  // Get everything
															pickItemInfo.QuantityPerUom = line.QuantityPerUom;
															pickItemInfo.Uom = line.Uom;
															pickItemInfo.LocationCode = dockCode;  // Must be on cross-docking dock
															pickItemInfo.LogisticUnitIdentKey = rxdStockLuid;
															pickItemInfo.ListOfPickedSerialNumbers = listOfPickedSerialNumbers;

															if (line.QuantityUom2.HasValue && stockQtyUom2.HasValue)
															{
																pickItemInfo.QuantityUom2 = Math.Max(line.QuantityUom2.Value, stockQtyUom2.Value);
															}

															bool fullPallet = !fullPalletPickedLuids.Contains(rxdStockLuid);
															if (fullPallet) fullPalletPickedLuids.Add(rxdStockLuid);

															plProv.PickItem(pickItemInfo, plDocEntry, pickItemInfo.LogisticUnitIdentKey, dockCode, null, fullPallet, false, listOfPickedSerialNumbers, false, null, false);

															row[PmxInventoryDetailTable.Columns.Quantity.NAME] = 0.0;
														}
													}
												}
											} // foreach ( DataRow row in tblPurchaseOrderReceivedStock.Rows )
										}
									}  // foreach ( PmxPickListLine line in pl.Lines )
								}  // Using ( PmxPickList pl = plProv.GetBO( plDocEntry ) )
							}
						}
					}

					return null;
				});
			}
			catch (Exception ex)
			{
				crossDockErrorOccurred = true;
				//Ask if 
				s_log.Error("", ex);
				if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
							"NoTranslationOfMessageKey", true,
							"IsError", true,
							"ShowButton", true));
					WaitForMessage();

				}
				else
				{
					// Inform the user with the error
					session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
						DefaultCultureInfo.Get(), BuildParamCollection(
							"PmxError", ((ProdumexException)ex).Error,
							"ShowButton", true));
					WaitForMessage();
				}

				//retry?
				//ask user if he wants to retry
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
					DefaultCultureInfo.Get(), BuildParamCollection(
						"TitleKey", "MSG_TITLE_RETRY_UPDATE",
						"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
				Message msg = WaitForMessage();

				if (msg.Name.ToLower().EndsWith(".yes"))
				{
					goto Step_PerformCrossDock;
				}

			}

			if (generatedWaveKey.HasValue && !crossDockErrorOccurred)
			{
				WorkflowInstanceBase pickFlow = this.Create("PickingScript");

				pickFlow.BindParameters(
					this.DefaultCultureInfo,
					this.PmxOseCompany,
					new ReadOnlyBinder<int>("CrossDockingPickListDocEntry", generatedPickListDocEntry.Value),
					new ReadOnlyBinder<int>("CrossDockingWaveKey", generatedWaveKey.Value));

				this.StartSubFlowAndWait(pickFlow, true);
			}

			return true;
		}

		#endregion

		#region HELPERS


		private double? GetCurrentStock(string defaultLocation, string itemCode, out double? maximumStockOnLocation, Session session, ISboProviderService sboProviderService)
		{
			maximumStockOnLocation = null;
			double? currentStock = null;

			double? maximumStockOnLocationTemp = null;

			if (!string.IsNullOrEmpty(defaultLocation) &&
				!string.IsNullOrEmpty(itemCode))
			{
				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
						 delegate (PmxDbConnection conn, object[] parameters2)
						 {
							 PmxInventoryProvider invtProv = new PmxInventoryProvider(conn);
							 PmxOseStorageLocationProvider locProv = new PmxOseStorageLocationProvider(conn);

							 using (PmxOseStorageLocationBase ose = locProv.GetOseStorageLocationByCode(defaultLocation))
							 {
								 if (ose == null)
								 {
									 return null;
								 }
							 }

							 currentStock = invtProv.GetStockQuantity(itemCode, null, false, null, null, defaultLocation, null, null, false, false, false);

							 maximumStockOnLocationTemp = invtProv.GetMaximumQuantityMasterDataForLocation(defaultLocation, itemCode);

							 return null;
						 });
			}

			maximumStockOnLocation = maximumStockOnLocationTemp;

			return currentStock;
		}




		private string GetCustomerInfo(DataSet dsCustomerInfo)
		{
			string info = null;

			if (this.GetNumberOfRows(dsCustomerInfo) > 0)
			{
				string newLine = "";
				info = "";
				foreach (DataRow row in dsCustomerInfo.Tables[0].Rows)
				{
					string numAtCard = row[SboSalesOrderTable.Columns.NumAtCard.NAME] as string;
					if (!string.IsNullOrEmpty(numAtCard))
					{
						numAtCard = "(" + numAtCard + ")";
					}
					info += string.Format("{4}{0}({1}) - {2}{3}",
						row[SboSalesOrderTable.Columns.CardName.NAME],
						row[SboSalesOrderTable.Columns.CardCode.NAME],
						row[SboSalesOrderTable.Columns.DocNum.NAME],
						numAtCard,
						newLine);

					newLine = Environment.NewLine;
				}
			}

			return info;
		}

		/// <summary>
		/// Determines whether this instance is hana.
		/// </summary>
		/// <returns></returns>
		private bool IsHana(ISboProviderService sboProviderService)
		{
			bool isHana = sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(
				delegate (PmxDbConnection conn, object[] parameters)
				{
					return conn.DbTool.DbServerType == DbServerType.HANA;
				}));
			return isHana;
		}

		/// <summary>
		/// Extracts the string values from pipes.
		/// </summary>
		/// <param name="cellValue">The cell value.</param>
		/// <returns></returns>
		private List<string> ExtractStringValuesFromPipes(string cellValue)
		{
			List<string> list = new List<string>();
			string[] split = cellValue.ToString().Split(SeparatorDefinition.SEPARATORVALUE);
			if (!string.IsNullOrEmpty(split[0]))
			{
				foreach (string s in split)
				{
					list.Add(s);
				}
			}
			return list;
		}

		/// <summary>
		/// Extracts the date time values from pipes.
		/// </summary>
		/// <param name="cellValue">The cell value.</param>
		/// <returns></returns>
		private List<DateTime?> ExtractDateTimeValuesFromPipes(string cellValue)
		{
			List<DateTime?> list = new List<DateTime?>();
			string[] split = cellValue.Split(SeparatorDefinition.SEPARATORVALUE);
			if (!string.IsNullOrEmpty(split[0]))
			{
				foreach (string s in split)
				{
					list.Add(DateTime.ParseExact(s, "yyyyMMdd", CultureInfo.CurrentUICulture));
				}
			}
			return list;
		}


		/// <summary>
		/// Executes the is location locked.
		/// </summary>
		/// <param name="conn">The connection.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteIsLocationLocked(PmxDbConnection conn, object[] parameters)
		{
			PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
			return prov.IsLocationLocked((string)parameters[0]);
		}

		/// <summary>
		/// Gets the transactional packaging types text.
		/// </summary>
		/// <param name="quantity">The quantity.</param>
		/// <param name="uom">The uom.</param>
		/// <param name="listOfTransactionalPackagingTypes">The list of transactional packaging types.</param>
		/// <returns></returns>
		private string GetTransactionalPackagingTypesText(double quantity, string uom, Collection<PackagingTypeInfo> listOfTransactionalPackagingTypes)
		{

			if (listOfTransactionalPackagingTypes == null ||
				listOfTransactionalPackagingTypes.Count == 0)
			{
				return string.Format("# {0} {1}", quantity.ToString(), uom);
			}

			//we found packagingTypes
			double currentQty = quantity;
			StringBuilder sb = new StringBuilder();
			string separator = "";
			//loop from end
			for (int iLoop = 0; iLoop < listOfTransactionalPackagingTypes.Count; iLoop++)
			{
				double qty = currentQty / listOfTransactionalPackagingTypes[iLoop].QuantityPerPack;

				qty = Math.Floor(qty);


				sb.AppendFormat("{0}{1} {2}",
					separator, qty.ToString(), listOfTransactionalPackagingTypes[iLoop].PackagingTypeName);

				currentQty -= qty * listOfTransactionalPackagingTypes[iLoop].QuantityPerPack;

				separator = Definitions.SEPARATOR;

			}

			return sb.ToString();


		}


		/// <summary>
		/// Gets the quantity in packaging types text.
		/// </summary>
		/// <param name="listOfTransactionalPackagingTypes">The list of transactional packaging types.</param>
		/// <returns></returns>
		private string GetQuantityInPackagingTypesText(Collection<PackagingTypeInfo> listOfTransactionalPackagingTypes)
		{

			if (listOfTransactionalPackagingTypes == null ||
				listOfTransactionalPackagingTypes.Count == 0)
			{
				return null;
			}

			//we found packagingTypes
			StringBuilder sb = new StringBuilder();
			string separator = "";



			for (int iLoop = 0; iLoop < listOfTransactionalPackagingTypes.Count - 1; iLoop++)
			{

				sb.AppendFormat("{0}1 {1} = {2} {3}", separator, listOfTransactionalPackagingTypes[iLoop].PackagingTypeName, listOfTransactionalPackagingTypes[iLoop].QuantityPerPack / listOfTransactionalPackagingTypes[iLoop + 1].QuantityPerPack, listOfTransactionalPackagingTypes[iLoop + 1].PackagingTypeName);


				separator = Definitions.SEPARATOR;

			}

			return sb.ToString();

		}

		/// <summary>
		/// Gets the name of the pack type.
		/// </summary>
		/// <param name="packagingTypeCode">The packaging type code.</param>
		/// <param name="listOfPackagingTypes">The list of packaging types.</param>
		/// <returns></returns>
		private static string GetPackTypeName(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						return packType.PackagingTypeName;
					}
				}
			}

			return null;
		}

		/// <summary>
		/// Gets the pack barcode.
		/// </summary>
		/// <param name="packagingTypeCode">The packaging type code.</param>
		/// <param name="listOfPackagingTypes">The list of packaging types.</param>
		/// <returns></returns>
		private static Collection<string> GetPackBarcode(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						if (packType.Barcodes.Count > 0)
						{
							return packType.Barcodes;
						}
						else
						{
							return null;
						}
					}
				}
			}

			return null;
		}

		/// <summary>
		/// Gets the hide during entering quantity.
		/// </summary>
		/// <param name="packagingTypeCode">The packaging type code.</param>
		/// <param name="listOfPackagingTypes">The list of packaging types.</param>
		/// <returns></returns>
		private static bool GetHideDuringEnteringQuantity(string packagingTypeCode, Collection<PmxItemPackagingTypeInfo> listOfPackagingTypes)
		{
			if (listOfPackagingTypes != null)
			{
				foreach (PmxItemPackagingTypeInfo packType in listOfPackagingTypes)
				{
					if (packType.PackagingTypeCode == packagingTypeCode)
					{
						return packType.HideDuringEnteringQuantity;
					}
				}
			}

			return true;
		}

		///// <summary>
		///// Executes the print events.
		///// </summary>
		///// <param name="conn">The conn.</param>
		///// <param name="parameters">The parameters.</param>
		///// <returns></returns>
		//private static int ExecutePrintEvents(PmxDbConnection conn, object[] parameters)
		//{
		//    PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

		//    return prov.PrintReportsForEvent((PmxPrintReportEventType)parameters[0], (string)parameters[1], (string)parameters[2], (int)parameters[3]);
		//}


		/// <summary>
		/// Executes the generate luid.
		/// </summary>
		/// <param name="conn">The connection.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static int ExecuteGenerateLuid(PmxDbConnection conn, object[] parameters)
		{
			PmxLogisticUnitIDProvider logUnitProv = new PmxLogisticUnitIDProvider(conn);
			string sscc = (string)parameters[0];
			string supplierPalletNumber = (string)parameters[1];
			int logisticUnitId = -1;
			if (string.IsNullOrEmpty(sscc))
			{
				logisticUnitId = logUnitProv.GenerateNewLogisticUnit(supplierPalletNumber);
			}
			else
			{
				//generate for sscc
				logisticUnitId = logUnitProv.NewLogisticUnitID(sscc, supplierPalletNumber);
			}

			return logisticUnitId;
		}


		/// <summary>
		/// Executes the get create item transactional information key.
		/// </summary>
		/// <param name="conn">The connection.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static int? ExecuteGetCreateItemTransactionalInfoKey(PmxDbConnection conn, object[] parameters)
		{
			PmxItemTransactionalInfoProvider prov = new PmxItemTransactionalInfoProvider(conn);
			string itemCode = (string)parameters[0];
			string batchNumber = (string)parameters[1];
			string batchNumber2 = (string)parameters[2];
			DateTime? bbd = (DateTime?)parameters[3];

			return prov.GetCreateItemTransactionalInfoKey(itemCode, batchNumber, batchNumber2, bbd);
		}

		/// <summary>
		/// Executes the get receive all items on one purchase delivery.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteGetReceiveAllItemsOnOnePurchaseDelivery(PmxDbConnection conn, object[] parameters)
		{
			PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);
			return prov.ReceiveAllItemsOnOnePurchaseDelivery();
		}

		/// <summary>
		/// Executes the get allow to select batches.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteGetAllowToSelectBatches(PmxDbConnection conn, object[] parameters)
		{
			PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);

			return prov.AllowToSelectBatches();
		}


		/// <summary>
		/// Executes the generate put away order.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static object ExecuteGeneratePutAwayOrder(PmxDbConnection conn, object[] parameters)
		{
			PmxMoveOrderProvider prov = new PmxMoveOrderProvider(conn);
			PmxPriorityProvider prioProv = new PmxPriorityProvider(conn);

			foreach (DocumentRef docRef in (Collection<DocumentRef>)parameters[0])
			{
				prov.GeneratePutAwayOrder(docRef.DocEntry, prioProv.GetDefaultPriority().Sequence, null, true, (string)parameters[1]);
			}

			return null;
		}

		/// <summary>
		/// Executes the create move for logistic units with only logic carriers if configured.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteCreateMoveForLogisticUnitsWithOnlyLogicCarriersIfConfigured(PmxDbConnection conn, object[] parameters)
		{
			PmxMoveProvider prov = new PmxMoveProvider(conn);

			return prov.CreateMoveForLogisticUnitsWithOnlyLogicCarriersIfConfigured((string)parameters[0]);
		}


		///// <summary>
		///// Executes the print reports for event collection.
		///// </summary>
		///// <param name="conn">The conn.</param>
		///// <param name="parameters">The parameters.</param>
		///// <returns></returns>
		//private static int ExecutePrintReportsForEventWithParametersCollection(PmxDbConnection conn, object[] parameters)
		//{
		//    PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

		//    return prov.PrintReportsForEventWithParametersCollection((PmxPrintReportEventType)parameters[0], (string)parameters[1], (string)parameters[2], (Collection<PmxDictionary<string, object>>)parameters[3], (bool)parameters[4]);
		//}

		///// <summary>
		///// Executes the print reports for event.
		///// </summary>
		///// <param name="conn">The conn.</param>
		///// <param name="parameters">The parameters.</param>
		///// <returns></returns>
		//private static int ExecutePrintReportsForEventWithParameters(PmxDbConnection conn, object[] parameters)
		//{
		//    PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

		//    return prov.PrintReportsForEventWithParameters((PmxPrintReportEventType)parameters[0], (string)parameters[1], (string)parameters[2], (PmxDictionary<string, object>)parameters[3], (bool)parameters[4]);
		//}

		///// <summary>
		///// Executes the add purchase delivery note list of logistic units.
		///// </summary>
		///// <param name="conn">The conn.</param>
		///// <param name="parameters">The parameters.</param>
		///// <returns></returns>
		//private KeyValuePair<int, Collection<int>> ExecuteAddPurchaseDeliveryNoteListOfLogisticUnits(PmxDbConnection conn, object[] parameters)
		//{
		//    PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider((SboConnection)conn);

		//    return prov.AddPurchaseDeliveryNoteListOfLogisticUnits((string)parameters[0], (string)parameters[1], (Collection<LogisticUnitGoodsReceipt>)parameters[2], (string)parameters[3]);
		//}

		/// <summary>
		/// Sets the item transactional packaging type info list.
		/// </summary>
		/// <param name="itemInfo">The item info.</param>
		/// <returns></returns>
		private Collection<ItemTransactionalPackagingTypeInfo> SetItemTransactionalPackagingTypeInfoList(PmxItemInfo itemInfo)
		{
			//get the packaging types qty from item
			Collection<ItemTransactionalPackagingTypeInfo> listOfTransactionalPackagingTypeInfos = new Collection<ItemTransactionalPackagingTypeInfo>();

			foreach (PmxItemPackagingTypeInfo packInfo in itemInfo.ListOfPackagingTypes)
			{
				ItemTransactionalPackagingTypeInfo newPackInfo = new ItemTransactionalPackagingTypeInfo();

				newPackInfo.PackagingTypeCode = packInfo.PackagingTypeCode;
				newPackInfo.Quantity = packInfo.Quantity;
				newPackInfo.ItemCode = itemInfo.ItemCode;

				listOfTransactionalPackagingTypeInfos.Add(newPackInfo);

			}

			return listOfTransactionalPackagingTypeInfos;
		}


		/// <summary>
		/// Gets the transactional packaging info.
		/// </summary>
		/// <param name="purchaseDeliveryNoteList">The purchase delivery note list.</param>
		/// <param name="batchNumber">The batch number.</param>
		/// <param name="secondBatchNumber">The second batch number.</param>
		/// <param name="bestBeforeDate">The best before date.</param>
		/// <param name="itemCode">The item code.</param>
		/// <returns></returns>
		private static Collection<ItemTransactionalPackagingTypeInfo> GetTransactionalPackagingInfo(Collection<LogisticUnitGoodsReceipt> purchaseDeliveryNoteList, string batchNumber, string secondBatchNumber, DateTime? bestBeforeDate, string itemCode)
		{
			//does it exist in the just received items?
			foreach (LogisticUnitGoodsReceipt receipt in purchaseDeliveryNoteList)
			{
				foreach (LogisticUnitItemGoodReceipt receiptItem in receipt.ItemsOnLogisticUnit)
				{
					if (batchNumber == receiptItem.BatchNumber1 &&
						secondBatchNumber == receiptItem.BatchNumber2 &&
						bestBeforeDate == receiptItem.BestBeforeDate &&
						itemCode == receiptItem.ItemCode)
					{
						//we found the packagingData
						return receiptItem.ListOfPackagingTypes;
					}
				}
			}
			return null;
		}

		/// <summary>
		/// Executes the generate batch number1 for reception.
		/// </summary>
		/// <param name="conn">The connection.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static GeneratedBatchNumberReception ExecuteGenerateBatchNumber1ForReception(PmxDbConnection conn, object[] parameters)
		{
			PmxBatchNumberReceptionProvider prov = new PmxBatchNumberReceptionProvider(conn);
			if ((bool)parameters[4])
			{
				return prov.GenerateBatchNumber1ForReception((DocumentRef)parameters[0], (int?)parameters[1], (string)parameters[2], (string)parameters[3]);
			}
			else
			{
				return prov.GenerateBatchNumber2ForReception((DocumentRef)parameters[0], (int?)parameters[1], (string)parameters[2], (string)parameters[3]);
			}
		}


		/// <summary>
		/// Removes the received items.
		/// </summary>
		/// <param name="purchaseDeliveryNoteList">The purchase delivery note list.</param>
		/// <param name="dsProducts">The ds products.</param>
		/// <param name="currentUnit">The current unit.</param>
		private void RemoveReceivedItems(Collection<LogisticUnitGoodsReceipt> purchaseDeliveryNoteList, DataSet dsProducts, LogisticUnitGoodsReceipt currentUnit, int batchReceivingNumber, ISboProviderService sboProviderService, DocumentRef docRef, int? containerDocEntry)
		{

			//Only if there are base documents we can remove the ASN 
			if (containerDocEntry.HasValue || docRef != null)
			{

				//get asn
				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxAdvanceShippingNoticeProvider asnProv = new PmxAdvanceShippingNoticeProvider(conn);

						List<PmxAdvanceShippingNotice> asnBOs = asnProv.GetOpenAsnList(containerDocEntry.HasValue ? Data.BusinessObjects.Definitions.PmxObjectTypes.PmxContainer : docRef.DocType,
							containerDocEntry.HasValue ? containerDocEntry.Value : docRef.DocEntry, "", 0, conn.InternalKeySboUser);


						foreach (PmxAdvanceShippingNotice asn in asnBOs)
						{
							if (dsProducts != null &&
								dsProducts.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME))
							{
								string filter = ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + asn.ItemCode + "' ";
								filter += " AND " + SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME + "= " + DataSetCommandHelper.FormatAndEscapeColumnValue(asn.QuantityPerUom, CultureInfo.InvariantCulture);

								//DataRow[] rows = dsProducts.Tables[0].Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + logUnitItem.ItemCode + "'");
								DataRow[] rows = dsProducts.Tables[0].Select(filter);
								double alreadyReceived = asn.InventoryQuantity / asn.QuantityPerUom;
								for (int iLoop = 0; iLoop < rows.Length; iLoop++)
								{
									if (!asn.BaseLine.HasValue ||
										Convert.ToInt32(rows[iLoop][SboPurchaseOrderLineTable.Columns.LineNum.NAME]) == asn.BaseLine.Value)
									{

										if (alreadyReceived <= 0)
										{
											break;
										}


										double stillToReceive = Convertor.ConvertToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME]);
										double numOfUnits = Convertor.ConvertToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.NumPerMsr.NAME]);

										double qtyReduced = 0;

										if (stillToReceive <= alreadyReceived)
										{
											//no more quantity to receive, so delete the entry
											rows[iLoop].Delete();
											qtyReduced = stillToReceive;
										}
										else
										{
											//reduce the quantity
											rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME] = stillToReceive - alreadyReceived;
											rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME] = (stillToReceive * numOfUnits) - (alreadyReceived * numOfUnits);
											qtyReduced = alreadyReceived;

										}

										alreadyReceived -= qtyReduced;
									}
								}
								dsProducts.AcceptChanges();
							}

						}


						return null;
					}));
			}
			if (currentUnit != null)
			{
				foreach (LogisticUnitItemGoodReceipt logUnitItem in currentUnit.ItemsOnLogisticUnit)
				{
					if (dsProducts != null &&
						dsProducts.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME))
					{
						string filter = ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + logUnitItem.ItemCode + "' ";
						filter += " AND " + SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME + "= " + DataSetCommandHelper.FormatAndEscapeColumnValue(logUnitItem.QuantityPerUom, CultureInfo.InvariantCulture);

						//DataRow[] rows = dsProducts.Tables[0].Select(ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + logUnitItem.ItemCode + "'");
						DataRow[] rows = dsProducts.Tables[0].Select(filter);
						double alreadyReceived = logUnitItem.Quantity * batchReceivingNumber;
						for (int iLoop = 0; iLoop < rows.Length; iLoop++)
						{
							if (!logUnitItem.PurchaseDocumentLineNum.HasValue ||
							   Convert.ToInt32(rows[iLoop][SboPurchaseOrderLineTable.Columns.LineNum.NAME]) == logUnitItem.PurchaseDocumentLineNum.Value)
							{
								double numOfUnits = Convertor.ConvertToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.NumPerMsr.NAME]);
								double stillToReceive = Convert.ToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME]);

								if (stillToReceive <= alreadyReceived)
								{
									//no more quantity to receive, so delete the entry
									rows[iLoop].Delete();

									alreadyReceived -= stillToReceive;

								}
								else
								{
									//reduce the quantity
									rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME] = stillToReceive - alreadyReceived;
									rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME] = (stillToReceive * numOfUnits) - (alreadyReceived * numOfUnits);
									break;
								}
							}
						}
						dsProducts.AcceptChanges();
					}
				}
			}

		}

		/// <summary>
		/// Gets the still to receive.
		/// </summary>
		/// <param name="dsProducts">The ds products.</param>
		/// <param name="itemCode">The item code.</param>
		/// <param name="quantityPerUom">The quantity per uom.</param>
		/// <returns></returns>
		private double GetStillToReceive(DataSet dsProducts, string itemCode, double? quantityPerUom)
		{
			double stillToReceive = 0;
			if (dsProducts != null &&
					dsProducts.Tables[0].Columns.Contains(SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME))
			{
				string filter = ProductDataSet.Columns.PRODUCT_CODE.Name + "= '" + itemCode + "' ";
				if (quantityPerUom.HasValue)
				{
					filter += " AND " + SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME + "= " + DataSetCommandHelper.FormatAndEscapeColumnValue(quantityPerUom, CultureInfo.InvariantCulture);
				}
				DataRow[] rows = dsProducts.Tables[0].Select(filter);
				stillToReceive = 0;
				for (int iLoop = 0; iLoop < rows.Length; iLoop++)
				{
					//stillToReceive += Convert.ToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME]);
					//stillToReceive += (Convert.ToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME])
					//    * Convert.ToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.NumPerMsr.NAME]));
					stillToReceive += (Convert.ToDouble(rows[iLoop][SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME]));
				}
			}

			return stillToReceive;
		}

		///// <summary>
		///// Gets the still automatic receive with packaging.
		///// </summary>
		///// <param name="dsProducts">The ds products.</param>
		///// <param name="itemInfo">The item information.</param>
		///// <param name="listOfPackagingTypeInfosForScreen">The list of packaging type infos for screen.</param>
		///// <returns></returns>
		//private double GetStillToReceiveWithPackaging(DataSet dsProducts, PmxItemInfo itemInfo, Collection<PackagingTypeInfo> listOfPackagingTypeInfosForScreen)
		//{
		//    double stillToReceive = GetStillToReceive(dsProducts, itemInfo.ItemCode);
		//    double numberOfItemsPerUnit = string.IsNullOrEmpty(itemInfo.PurchaseUom) ? 1 : itemInfo.NumberOfItemsPerPurchaseUnit;
		//    return Produmex.Foundation.Utilities.FormattingHelper.RoundSAPDecimal(stillToReceive / numberOfItemsPerUnit, itemInfo.UomDecimals);
		//}



		/// <summary>
		/// Executes the can item be stored on storage location.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteCanItemBeStoredOnStorageLocation(PmxDbConnection conn, object[] parameters)
		{
			PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
			return prov.CanItemBeStoredOnStorageLocation((string)parameters[0], (string)parameters[1], new PmxStorageLocationCanItemBeStoredInfo());
		}

		/// <summary>
		/// Executes the get LUID by SSCC.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static int ExecuteGetLUIDBySSCC(PmxDbConnection conn, object[] parameters)
		{
			PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);
			return prov.GetLUIDBySSCC((string)parameters[0]);
		}

		/// <summary>
		/// Executes the get SSCC by LUID.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static string ExecuteGetSSCCByLUID(PmxDbConnection conn, object[] parameters)
		{
			PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);
			return prov.GetSSCCByLUID((int)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null &&
				ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}

			return 0;
		}

		/// <summary>
		/// Checks if there are Products the on barcode.
		/// </summary>
		/// <param name="BarcodeContainerObject">The barcode container object.</param>
		/// <returns>
		/// True, if there is a product on the barcode, false if not
		/// </returns>
		public bool IsProductOnBarcode(BarcodeContainer BarcodeContainerObject)
		{
			if (BarcodeContainerObject != null &&
				(!string.IsNullOrEmpty(BarcodeHelper.GetProductIdentification(BarcodeContainerObject)) || !string.IsNullOrEmpty(BarcodeContainerObject.ItemCode)))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Checks if there is a LU the on barcode.
		/// </summary>
		/// <param name="barcodeContainerObject">The barcode container object.</param>
		/// <returns>
		/// True, if there is a LU on the barcode, false if not
		/// </returns>
		public bool IsLUOnBarcode(BarcodeContainer barcodeContainerObject)
		{
			if (barcodeContainerObject != null &&
				barcodeContainerObject.GS1DataObject != null &&
				barcodeContainerObject.GS1DataObject.GTINLU != null)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Checks if there is an SSCC the on barcode.
		/// </summary>
		/// <param name="GS1DataScanned">The G s1 data scanned.</param>
		/// <returns>
		/// True, if there is an sscc on the barcode, false if not
		/// </returns>
		public bool IsSSCCOnBarcode(GS1Data GS1DataScanned)
		{
			if (GS1DataScanned != null &&
				(GS1DataScanned.Sscc != null))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Checks if there is a best before Date the on barcode.
		/// </summary>
		/// <param name="GS1DataScanned">The G s1 data scanned.</param>
		/// <returns>
		/// True, if there is an best before date on the barcode, false if not
		/// </returns>
		public bool IsBestBeforeDateOnBarcode(GS1Data GS1DataScanned)
		{
			if (GS1DataScanned != null &&
				(GS1DataScanned.BestBeforeDate != null))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Checks if there is a quantity the on barcode.
		/// </summary>
		/// <param name="barcode">The barcode.</param>
		/// <returns>
		/// True, if there is a quantity on the barcode, false if not
		/// </returns>
		public bool IsQuantityOnBarcode(BarcodeContainer barcode)
		{
			if (barcode != null &&
			barcode.GS1DataObject != null &&
				barcode.GS1DataObject.Count != null)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Checks if there is a batch number the on barcode.
		/// </summary>
		/// <param name="GS1DataScanned">The G s1 data scanned.</param>
		/// <returns>
		/// True, if there is a batch number on the barcode, false if not
		/// </returns>
		public bool IsBatchNumberOnBarcode(GS1Data GS1DataScanned)
		{
			if (GS1DataScanned != null &&
				(GS1DataScanned.LotNumber != null))
			{
				return true;
			}
			else
			{
				return false;
			}
		}


		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}


		/// <summary>
		/// Executes the get PMX warehouse code.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Executes the need to enter supplier pallet number.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteNeedToEnterSupplierPalletNumber(PmxDbConnection conn, object[] parameters)
		{
			PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
			return prov.NeedToEnterSupplierPalletNumber((string)parameters[0]);
		}
		/// <summary>
		/// Executes the check if labels are allowed to be printed on reception for a vendor
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteCheckIfLabelScanningAllowedOnReception(PmxDbConnection conn, object[] parameters)
		{
			PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
			return prov.CheckIfLabelScanningAllowedOnReception((string)parameters[0]);
		}

		/// <summary>
		/// Executes the check if logistic carriers are allowed on reception for a vendor
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static bool ExecuteCheckIfLogisticCarriersAreAllowedOnReception(PmxDbConnection conn, object[] parameters)
		{
			PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
			return prov.CheckIfLogisticCarriersAreAllowedOnReception((string)parameters[0]);
		}

		/// <summary>
		/// Executes the generate BBD for reception.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static GeneratedBestBeforeDateReception ExecuteGenerateBBDForReception(PmxDbConnection conn, object[] parameters)
		{
			PmxGeneralProvider genProv = new PmxGeneralProvider(conn);
			if (parameters.Length > 1)
			{
				return genProv.GenerateBestBeforeDateForReception((string)parameters[0], (DateTime?)parameters[1]);
			}
			else
			{
				return genProv.GenerateBestBeforeDateForReception((string)parameters[0], null);
			}
		}

		/// <summary>
		/// Checks if allow to calculate based on manufacturing date.
		/// </summary>
		/// <param name="conn">The connection.</param>
		/// <returns></returns>
		private static bool ExecuteCheckIfAllowToCalculateBasedOnManufacturingDate(PmxDbConnection conn, object[] parameters)
		{
			PmxGeneralProvider genProv = new PmxGeneralProvider(conn);
			return genProv.CheckIfAllowToCalculateBasedOnManufacturingDate();

		}



		/// <summary>
		/// Executes the check allow multiple identical log units.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxBusinessPartnerInfo ExecuteGetBusinessPartnerInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
			return prov.GetBusinessPartnerInfo((string)parameters[0]);
		}


		#endregion

		internal class WeighData
		{
			internal string ItemUom;
			internal string LocationCode;
			internal double Quantity;
			internal double? ScaleCoefficient;
			internal PmxOseScale ScaleData;
			internal string ScaleUom;
			internal int ScaleUomDecimals;
			internal string TareCommand;
			internal string ZeroCommand;
			internal string WeighResultFormat;
			internal string WeighCommand;
			internal string ScaleCode;
			internal int NumberOfScales;
		}

		#region BUILD QUERY

		/// <summary>
		/// Static class grouping our native queries
		/// </summary>
		internal static class BuildQuery
		{
			/// <summary>
			/// The m_sbo boolean convertor
			/// </summary>
			private static SboBooleanConvertor m_sboBooleanConvertor = null;
			/// <summary>
			/// Gets the boolean convertor.
			/// </summary>
			/// <value>
			/// The boolean convertor.
			/// </value>
			private static SboBooleanConvertor BooleanConvertor {
				get {
					if (m_sboBooleanConvertor == null)
					{
						m_sboBooleanConvertor = new SboBooleanConvertor();
					}
					return m_sboBooleanConvertor;
				}
			}

			/// <summary>
			/// Gets the reasons.
			/// </summary>
			/// <returns></returns>
			public static string GetReasonsForDeviatingFromStandardProcess()
			{
				StringBuilder strBuilder = new StringBuilder();

				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME, ReasonDataSet.Columns.REASON_CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Name.NAME, ReasonDataSet.Columns.REASON_NAME.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.RequiresFreeText.NAME, ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReasonTable.NAME));

				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.UseWhenDeviatingFromStandardProcess.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.CurrentCulture));

				strBuilder.AppendFormat(
					" ORDER BY CASE WHEN {0} IS NULL THEN 1 ELSE 0 END, {0} ",
					 SqlCommandHelper.EscapeTableColumnName(PmxReasonTable.NAME, PmxReasonTable.Columns.SequenceNumber.NAME));

				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets the name and the code of the  location, ordered by name.
			/// </summary>
			/// <param name="isLoading">Is the dock for loading? When null, no check will be done on this value</param>
			/// <param name="isUnloading">Is the dock for unloading? When null, no check will be done on this value</param>
			/// <param name="useAnd">if set to <c>true</c> the operand between Loading and Unloading is 'AND'.
			/// Otherwise it is 'OR'.</param>
			/// <param name="pmxWarehouseCode">The PMX warehouse code.</param>
			/// <param name="deviceID">The device ID.</param>
			/// <returns>
			/// The sql statement to get the dock
			/// </returns>
			public static string GetCodeNameDockOrderByName(bool? isLoading, bool? isUnloading, bool useAnd, string pmxWarehouseCode, string deviceID)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


				strBuilder.Append("\r\n FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));

				//inner join PmxOseDock - PmxOse
				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

				//inner join PmxOseDock - PmxOseStorageLocation
				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

				strBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				strBuilder.Append("\r\n WHERE ");

				//Dock must be active
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				if (!String.IsNullOrEmpty(pmxWarehouseCode) || !String.IsNullOrEmpty(deviceID))
				{
					strBuilder.Append(" AND ( ");
					if (!String.IsNullOrEmpty(pmxWarehouseCode))
					{
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
					}
					if (!String.IsNullOrEmpty(deviceID))
					{
						if (!String.IsNullOrEmpty(pmxWarehouseCode)) strBuilder.Append(" OR ");

						strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
							SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
							SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
					}
					strBuilder.Append(" )");
				}

				if (isUnloading.HasValue || isLoading.HasValue)
				{
					strBuilder.Append(" AND (");

					if (isUnloading.HasValue)
					{
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsUnloadingDock.NAME);
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isUnloading.Value), CultureInfo.InvariantCulture));
					}

					if (isLoading.HasValue)
					{
						if (isUnloading.HasValue)
						{
							strBuilder.Append(useAnd ? " AND " : " OR ");
						}

						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsLoadingDock.NAME);
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isLoading.Value), CultureInfo.InvariantCulture));
					}

					strBuilder.Append(" )");
				}

				strBuilder.Append("\r\n ORDER BY ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME);

				return strBuilder.ToString();
			}



			/// <summary>
			/// Gets all packaging types.
			/// </summary>
			/// <returns></returns>
			public static string GetAllPackagingTypes()
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME, SboUomTable.Columns.UomCode.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomName.NAME, SboUomTable.Columns.UomName.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets all batches in stock.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <returns></returns>
			public static string GetAllBatchesInStock(string itemCode)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, TextDataSet.Columns.TEXT.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));


				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.ItemCode.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));


				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets all internal batches in stock.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="batchNumber">The batch number.</param>
			/// <returns></returns>
			public static string GetAllInternalBatchesInStock(string itemCode, string batchNumber)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, TextDataSet.Columns.TEXT.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));


				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.ItemCode.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));



				return strBuilder.ToString();
			}



			/// <summary>
			/// Command for getting all PO's order by PO Number
			/// </summary>
			/// <param name="cardCode">The card code.</param>
			/// <param name="gtin">The gtin.</param>
			/// <param name="crossDocking">if set to <c>true</c> [cross docking].</param>
			/// <param name="includeReserveInvoices">if set to <c>true</c> [include reserve invoices].</param>
			/// <param name="sboWarehouseCode">The sbo warehouse code.</param>
			/// <param name="dbTool">The db tool.</param>
			/// <returns>
			/// Command for getting all PO's order by PO Number
			/// </returns>
			public static string GetPOsOrderByPONumber(string cardCode, string gtin, bool crossDocking, bool includeReserveInvoices, string sboWarehouseCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.AppendFormat("SELECT DISTINCT {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.ObjType.NAME, PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocNum.NAME, PurchaseOrderDataSet.Columns.PO_NUMBER.Name));
				qryBuilder.AppendFormat(", CAST( {0} AS NVARCHAR ) AS {1}",
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocNum.NAME),
					SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.PO_NUMBER_ASSTRING.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME, PurchaseOrderDataSet.Columns.PO_ID.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.CardCode.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.CardName.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_NAME.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.NumAtCard.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_REFERENCE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.SupplierLineRef.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_LINE_REF.Name));

				qryBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));

				qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME));

				qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ItemCode.NAME));

				if (crossDocking)
				{
					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.ObjType.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.Confirmed.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				}

				qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocStatus.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Canceled.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Confirmed.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				if (cardCode != null &&
					cardCode.Trim() != "")
				{
					qryBuilder.Append(" AND ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.CardCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));
				}

				if (gtin != null &&
					gtin.Trim() != "")
				{
					qryBuilder.Append(" AND ");
					Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcodePackagingType(qryBuilder, gtin);
				}

				if (!string.IsNullOrEmpty(sboWarehouseCode))
				{
					qryBuilder.Append(" AND ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.WarehouseCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sboWarehouseCode, CultureInfo.InvariantCulture));
				}


				if (crossDocking)
				{
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, CultureInfo.InvariantCulture));
				}
				else  // Cross-docking not yet supported for reserve invoice, remove this "else" when completed
					if (includeReserveInvoices)
				{
					qryBuilder.AppendFormat("\r\n UNION ALL SELECT DISTINCT {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.ObjType.NAME, PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocNum.NAME, PurchaseOrderDataSet.Columns.PO_NUMBER.Name));
					qryBuilder.AppendFormat(", CAST( {0} AS NVARCHAR ) AS {1}",
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocNum.NAME),
						SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.PO_NUMBER_ASSTRING.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME, PurchaseOrderDataSet.Columns.PO_ID.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.CardCode.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.CardName.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_NAME.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.NumAtCard.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_REFERENCE.Name));
					qryBuilder.AppendFormat(", {0}",
						SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.SupplierLineRef.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_LINE_REF.Name));

					qryBuilder.AppendFormat("\r\n FROM {0}",
						SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));

					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME));

					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(qryBuilder.WITH_NOLOCK);
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ItemCode.NAME));

					qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.InventoryStatus.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.Canceled.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.IsReserveInvoice.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineStatus.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.InventoryStatus.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryItem.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

					if (cardCode != null && cardCode.Trim() != "")
					{
						qryBuilder.AppendFormat(" AND {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.CardCode.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));
					}

					if (gtin != null && gtin.Trim() != "")
					{
						qryBuilder.Append(" AND ");
						Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcodePackagingType(qryBuilder, gtin);
					}
				}

				qryBuilder.AppendFormat("\r\n ORDER BY {0}",
					SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.PO_NUMBER.Name));

				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the customer information.
			/// </summary>
			/// <param name="docEntry">The document entry.</param>
			/// <param name="objType">Type of the object.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetCustomerInfo(int docEntry, string objType, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.AppendFormat("SELECT DISTINCT {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME, SboSalesOrderTable.Columns.DocEntry.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocNum.NAME, SboSalesOrderTable.Columns.DocNum.NAME));
				qryBuilder.AppendFormat(", {0}",
				   SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardCode.NAME, SboSalesOrderTable.Columns.CardCode.NAME));
				qryBuilder.AppendFormat(", {0}",
				   SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardName.NAME, SboSalesOrderTable.Columns.CardName.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.NumAtCard.NAME, SboSalesOrderTable.Columns.NumAtCard.NAME));

				if (objType == SboObjectTypes.PurchaseInvoice)
				{
					qryBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));

					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME));



					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseEntry.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseLine.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
					   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
					   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseType.NAME));

					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseOrderTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME));
				}
				else
				{
					qryBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));



					qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME));
				}



				qryBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.ObjType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.Confirmed.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));




				qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
					objType == SboObjectTypes.PurchaseInvoice ? SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME) : SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));


				qryBuilder.AppendFormat("\r\n ORDER BY {0}",
					SqlCommandHelper.EscapeColumnName(PmxSalesOrderTable.Columns.CardCode.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(PmxSalesOrderTable.Columns.DocEntry.NAME));

				return qryBuilder.ToString();
			}



			/// <summary>
			/// Gets the customer information for container.
			/// </summary>
			/// <param name="containerDocEntry">The container document entry.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetCustomerInfoForContainer(int containerDocEntry, string itemCode, DbTool dbTool)
			{
				string poLineInvoiceAlias = "PO_LINE_INVOICE";

				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.AppendFormat("SELECT DISTINCT {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME, SboSalesOrderTable.Columns.DocEntry.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocNum.NAME, SboSalesOrderTable.Columns.DocNum.NAME));
				qryBuilder.AppendFormat(", {0}",
				   SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardCode.NAME, SboSalesOrderTable.Columns.CardCode.NAME));
				qryBuilder.AppendFormat(", {0}",
				   SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardName.NAME, SboSalesOrderTable.Columns.CardName.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.NumAtCard.NAME, SboSalesOrderTable.Columns.NumAtCard.NAME));


				qryBuilder.AppendFormat("\r\n FROM {0}",
				   SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));

				//Join PO line
				qryBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
					   SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseLine.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseType.NAME));
				qryBuilder.AppendFormat("\r\n AND {0} = {1}",
				  SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ItemCode.NAME),
				  SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				//Join Purchase invoice
				qryBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					 SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineNum.NAME),
					 SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseLine.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ObjectType.NAME),
				   SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseType.NAME));
				qryBuilder.AppendFormat("\r\n AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ItemCode.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));


				qryBuilder.AppendFormat("\r\n LEFT JOIN {0} AS {1}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME),
						SqlCommandHelper.EscapeTableName(poLineInvoiceAlias));
				qryBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(poLineInvoiceAlias, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(poLineInvoiceAlias, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseLine.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(poLineInvoiceAlias, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseType.NAME));


				qryBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
				qryBuilder.AppendFormat(" ON {0} = COALESCE ({1},{2})",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.ObjType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
					SqlCommandHelper.EscapeTableColumnName(poLineInvoiceAlias, PmxPurchaseOrderLineTable.Columns.BaseType.NAME));
				qryBuilder.AppendFormat(" AND {0} = COALESCE ({1},{2})",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(poLineInvoiceAlias, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME));
				qryBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.Confirmed.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));




				qryBuilder.AppendFormat("\r\n WHERE {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(containerDocEntry, CultureInfo.InvariantCulture));


				qryBuilder.AppendFormat("\r\n AND {0} IS NOT NULL ",
					SqlCommandHelper.EscapeTableColumnName(SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME));





				qryBuilder.AppendFormat("\r\n ORDER BY {0}",
					SqlCommandHelper.EscapeColumnName(PmxSalesOrderTable.Columns.CardCode.NAME));
				qryBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeColumnName(PmxSalesOrderTable.Columns.DocEntry.NAME));

				return qryBuilder.ToString();
			}



			/// <summary>
			/// Gets the containers.
			/// </summary>
			/// <param name="sboWarehouseCode">The sbo warehouse code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			public static string GetContainers(string sboWarehouseCode, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.DocEntry.NAME, ContainerDataSet.Columns.DOC_ENTRY.Name);

				// AsString because DataSet filtering using a string containing non-numeric chars will give error on numeric columns
				qryBuilder.Append(", CAST(");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerTable.Columns.DocEntry.NAME));
				qryBuilder.Append(" AS NVARCHAR) AS ");
				qryBuilder.Append(qryBuilder.AsIdentifier(ContainerDataSet.Columns.DOC_ENTRY_AS_STRING.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerCode.NAME, ContainerDataSet.Columns.CONTAINER_CODE.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerDescription.NAME, ContainerDataSet.Columns.DESCRIPTION.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.EstimatedDateOfArrival.NAME, ContainerDataSet.Columns.ETA.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.CardCode.NAME, PmxContainerTable.Columns.CardCode.NAME);

				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.DocEntry.NAME);


				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderLineTable.NAME, SboPurchaseOrderLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseLine.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderLineTable.NAME, SboPurchaseOrderLineTable.Columns.LineNum.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseType.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderLineTable.NAME, SboPurchaseOrderLineTable.Columns.ObjectType.NAME);


				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderLineTable.NAME, SboPurchaseOrderLineTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderTable.NAME, SboPurchaseOrderTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseOrderLineTable.NAME, SboPurchaseOrderLineTable.Columns.DocEntry.NAME);


				qryBuilder.Append(" WHERE ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerType.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxContainerTypeColumnValues.Purchase.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerStatus.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxContainerStatusColumnValues.InTransit.VALUE, CultureInfo.InvariantCulture));


				//only for open lines
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

				//only approved orders
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Confirmed.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				//correct whs
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.WarehouseCode.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sboWarehouseCode, CultureInfo.InvariantCulture));

				//
				qryBuilder.Append("\r\n UNION SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.DocEntry.NAME, ContainerDataSet.Columns.DOC_ENTRY.Name);

				// AsString because DataSet filtering using a string containing non-numeric chars will give error on numeric columns
				qryBuilder.Append(", CAST(");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerTable.Columns.DocEntry.NAME));
				qryBuilder.Append(" AS NVARCHAR) AS ");
				qryBuilder.Append(qryBuilder.AsIdentifier(ContainerDataSet.Columns.DOC_ENTRY_AS_STRING.Name));

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerCode.NAME, ContainerDataSet.Columns.CONTAINER_CODE.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerDescription.NAME, ContainerDataSet.Columns.DESCRIPTION.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.EstimatedDateOfArrival.NAME, ContainerDataSet.Columns.ETA.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.CardCode.NAME, PmxContainerTable.Columns.CardCode.NAME);

				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerTable.NAME));

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.DocEntry.NAME);


				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseLine.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.LineNum.NAME);
				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.BaseType.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.ObjectType.NAME);


				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.DocEntry.NAME);


				qryBuilder.Append(" WHERE ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerType.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxContainerTypeColumnValues.Purchase.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxContainerTable.NAME, PmxContainerTable.Columns.ContainerStatus.NAME);
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxContainerStatusColumnValues.InTransit.VALUE, CultureInfo.InvariantCulture));


				//only for open lines
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.LineStatus.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

				//only approved orders
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceTable.Columns.Confirmed.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				//correct whs
				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.WarehouseCode.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sboWarehouseCode, CultureInfo.InvariantCulture));


				qryBuilder.Append(" ORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ContainerDataSet.Columns.ETA.Name));
				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerTable.Columns.ContainerCode.NAME));

				return qryBuilder.ToString();
			}

			/// <summary>
			/// Gets the existing ITRI.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="batchNumber">The batch number.</param>
			/// <param name="batchNumber2">The batch number2.</param>
			/// <param name="bestBeforeDate">The best before date.</param>
			/// <returns></returns>
			public static string GetExistingITRI(string itemCode, string batchNumber, string batchNumber2, DateTime? bestBeforeDate)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.PackagingTypeCode.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.Quantity.NAME));


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));

				strBuilder.Append(" ON ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.ItemTransactionalInfoKey.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.InternalKey.NAME));



				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.ItemCode.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

				if (!string.IsNullOrEmpty(batchNumber))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber, CultureInfo.InvariantCulture));
				}

				if (!string.IsNullOrEmpty(batchNumber2))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(batchNumber2, CultureInfo.InvariantCulture));
				}

				if (bestBeforeDate.HasValue)
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(bestBeforeDate.Value, CultureInfo.InvariantCulture));
				}

				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalPackagingTypeTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTransactionalPackagingTypeTable.Columns.Quantity.NAME));


				return strBuilder.ToString();
			}


			/// <summary>
			/// Command for getting all suppliers order by name
			/// </summary>
			/// <returns>
			/// Command for getting all suppliers order by name
			/// </returns>
			public static string GetSuppliersOrderByName()
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddColumnNameWithAs(strBuilder, SboBusinessPartnerTable.Columns.CardCode.NAME, SupplierDataSet.Columns.SUPPLIER_CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddColumnNameWithAs(strBuilder, SboBusinessPartnerTable.Columns.CardName.NAME, SupplierDataSet.Columns.SUPPLIER_NAME.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardType.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBusinessPartnerTypeColumnValues.Vendor.VALUE, CultureInfo.InvariantCulture));

				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardName.NAME));
				return strBuilder.ToString();
			}

			/// <summary>
			/// Command for getting all products order by name
			/// </summary>
			/// <param name="vendorCode">The vendor code.</param>
			/// <param name="gtin">The gtin.</param>
			/// <param name="purchaseDocRef">The purchase doc ref.</param>
			/// <param name="whsInfo"> The warehouse info.</param>
			/// <param name="cardCode">The card code.</param>
			/// <param name="packagingUnit">The packaging unit.</param>
			/// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
			/// <param name="distinct">if set to <c>true</c> [distinct].</param>
			/// <param name="crossDocking">if set to <c>true</c> [cross docking].</param>
			/// <param name="containerDocEntry">The container doc entry.</param>
			/// <param name="addLineNumberBatchAndSerialNumberColumns">if set to <c>true</c> [add line number batch and serial number columns].</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="dbTool">The db tool.</param>
			/// <returns>
			/// Command for getting all products order by name
			/// </returns>
			/// <exception cref="System.ArgumentException">Only PurchaseOrder or Purchase(Reserve)Invoice supported;purDocEntry.DocType</exception>
			public static string GetCodeNameItemsOrderByName(string vendorCode, string gtin, DocumentRef purchaseDocRef, PmxOseWarehouseInfo whsInfo, string cardCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, bool filterOutFrozenItems, bool distinct, bool crossDocking, int? containerDocEntry, bool addLineNumberBatchAndSerialNumberColumns, bool isNoPo, bool autoSelectPO, string itemCode, DbTool dbTool)
			{
				string purchaseDocTable = SboPurchaseOrderTable.NAME;
				string purchaseLineTable = SboPurchaseOrderLineTable.NAME;
				string purchaseInvoiceDocTable = SboPurchaseInvoiceTable.NAME;
				string purchaseInvoiceLineTable = SboPurchaseInvoiceLineTable.NAME;

				if (purchaseDocRef != null)
				{
					switch (purchaseDocRef.DocType)
					{
						case SboObjectTypes.PurchaseOrder:
							purchaseDocTable = SboPurchaseOrderTable.NAME;
							purchaseLineTable = SboPurchaseOrderLineTable.NAME;
							break;
						case SboObjectTypes.PurchaseInvoice:
							purchaseDocTable = SboPurchaseInvoiceTable.NAME;
							purchaseLineTable = SboPurchaseInvoiceLineTable.NAME;
							break;
						default:
							throw new ArgumentException("Only PurchaseOrder or Purchase(Reserve)Invoice supported", "purDocEntry.DocType");
					}
				}

				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT ");
				if (distinct)
				{
					qryBuilder.Append(" DISTINCT ");
				}
				else if (addLineNumberBatchAndSerialNumberColumns)
				{
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME, ProductDataSet.Columns.LINE_NUMBER.Name);
					qryBuilder.Append(", ");
				}

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

				qryBuilder.Append(", CASE");
				qryBuilder.Flags(QBF.Translate);
				if (purchaseDocRef != null || !String.IsNullOrEmpty(cardCode))
				{
					qryBuilder.AppendFormat(" WHEN {0} <> '' THEN {0} || ' - ' || {1} || ' - ' || {2}  ||  ' - ' || {3}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.SubstituteItemDescr.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.Substitute.NAME),
						SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderLineTable.NAME, "U_SubSup"));

				}
				qryBuilder.AppendFormat(" WHEN {0} <> '' THEN {0} || ' - ' || {1} || ' - ' || {2} ",
					SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.VendorItemDescr.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME),
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderLineTable.NAME, "U_SubSup"));
				qryBuilder.Flags(QBF.TranslateOff);
				qryBuilder.AppendFormat(" ELSE CONCAT(CONCAT({0},' - '),IFNULL({1},'')) END",
					SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME),
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseOrderLineTable.NAME, "U_SubSup"));

				qryBuilder.Append(" AS ", SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name));


				qryBuilder.Append(", ");
				Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(qryBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.HasBestBeforeDate.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.PrintItemLabelReception.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ManBtchNum.NAME));

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticUnit.NAME));
				qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.NumberOfItemsPerPurchaseUnit.NAME));
				qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.PurchaseUom.NAME));
				qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME));
				qryBuilder.Append(", ");
				Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(qryBuilder, packagingUnit, SboItemTable.Columns.CodeBars.NAME);
				//SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.PurchaseBarcode.NAME);

				if ((purchaseDocRef != null || containerDocEntry.HasValue) && !distinct)
				{
					if (containerDocEntry.HasValue)
					{
						qryBuilder.Append(", ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.OpenQuantity.NAME, SboPurchaseInvoiceLineTable.Columns.OpenCreQty.NAME);
					}
					else
					{
						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.OpenCreQty.NAME));
					}





					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineNum.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.DocEntry.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.ObjectType.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.QuantityPerUom.NAME));

					if (containerDocEntry.HasValue)
					{
						qryBuilder.AppendFormat(", {0} * {1} AS {2} ",
						SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.OpenQuantity.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.QuantityPerUom.NAME),
						SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.OpenInvQty.NAME));
					}
					else
					{
						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.OpenInvQty.NAME));
					}

					qryBuilder.AppendFormat(", CASE WHEN {0} = -1 THEN " + qryBuilder.IFNULL + "({2}, {3}) ELSE  " + qryBuilder.IFNULL + "({1}, {2}) END AS {4} ",
						SqlCommandHelper.EscapeTableColumnName(purchaseLineTable, SboPurchaseOrderLineTable.Columns.UomEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(purchaseLineTable, SboPurchaseOrderLineTable.Columns.UomCode.NAME),
						SqlCommandHelper.EscapeTableColumnName(purchaseLineTable, SboPurchaseOrderLineTable.Columns.Uom.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME),
						SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.UomCode.NAME));
				}

				//if (poDocEntry.HasValue && !distinct)
				//{

				//    strBuilder.Append(", ");
				//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.NAME));
				//    strBuilder.Append(".");
				//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.DocEntry.NAME));

				//    strBuilder.Append(", ");
				//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.NAME));
				//    strBuilder.Append(".");
				//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineNum.NAME));

				//}

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

				if (!distinct && addLineNumberBatchAndSerialNumberColumns)
				{
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxBatchNumber.NAME, ProductDataSet.Columns.BATCH1.Name);
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxBatchNumber2.NAME, ProductDataSet.Columns.BATCH2.Name);
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BestBeforeDate.NAME, ProductDataSet.Columns.BEST_BEFORE_DATE.Name);
					qryBuilder.Append(", ");
					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxSerialNumber.NAME, ProductDataSet.Columns.SERIAL_NUMBER_INFO.Name);
				}

				qryBuilder.Append("\r\n FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				if (purchaseDocRef != null || containerDocEntry.HasValue)
				{
					//inner join Product - PO Line
					qryBuilder.Append("\r\n INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
					qryBuilder.Append(" ON ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));

					//inner join PO - PO Line
					qryBuilder.Append("\r\n INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseDocTable));
					qryBuilder.Append(" ON ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseDocTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.Columns.DocEntry.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.Columns.DocEntry.NAME));

					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxAlternateCatNumTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.CardCode.NAME),
						SqlCommandHelper.EscapeTableColumnName(purchaseDocTable, SboPurchaseOrderTable.Columns.CardCode.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.ItemCode.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.Substitute.NAME),
						SqlCommandHelper.EscapeTableColumnName(purchaseLineTable, SboPurchaseOrderLineTable.Columns.BPCatalogNumber.NAME));

					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
					qryBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, purchaseLineTable, SboPurchaseOrderLineTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
					qryBuilder.Append(" <> -1 ");

					if (containerDocEntry.HasValue)
					{
						//inner join Container line - PO Line
						qryBuilder.Append("\r\n INNER JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
						qryBuilder.Append(" ON ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseEntry.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.Columns.DocEntry.NAME));
						qryBuilder.Append(" AND ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseLine.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.Columns.LineNum.NAME));
						qryBuilder.Append(" AND ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseType.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.Columns.ObjectType.NAME));
					}
				}
				else if (!String.IsNullOrEmpty(cardCode))
				{
					qryBuilder.Append("\r\n LEFT JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxAlternateCatNumTable.NAME));
					qryBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.CardCode.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));
					qryBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.ItemCode.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
				}

				string operand = "\r\n WHERE ";
				if (!string.IsNullOrEmpty(vendorCode))
				{
					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.CardCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(vendorCode, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";
				}

				if (!string.IsNullOrEmpty(gtin))
				{
					qryBuilder.Append(operand);
					Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcodePackagingType(qryBuilder, gtin);
					operand = "\r\n AND ";
				}

				if (purchaseDocRef != null)
				{
					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseDocTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.DocEntry.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(purchaseDocRef.DocEntry, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";

					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";

					if (purchaseDocRef.DocType == SboObjectTypes.PurchaseInvoice)  // Is this also valid for SboPurchaseOrderLineTable ??
					{
						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.InventoryStatus.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
					}
				}

				if (whsInfo != null && !isNoPo && !autoSelectPO && purchaseDocRef != null)
				{
					qryBuilder.AppendFormat("{0}{1} = {2}",
						operand,
						SqlCommandHelper.EscapeTableColumnName(purchaseLineTable, SboPurchaseOrderLineTable.Columns.WarehouseCode.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(whsInfo.SboWarehouseCode, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";
				}

				if (containerDocEntry.HasValue)
				{
					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseLineTable));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";

					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.Columns.DocEntry.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(containerDocEntry.Value, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";

					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.Columns.OpenQuantity.NAME));
					qryBuilder.Append(" > 0 ");
				}

				if (!string.IsNullOrEmpty(itemCode))
				{
					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";
				}

				qryBuilder.Append(operand);
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				operand = "\r\n AND ";

				switch (packagingUnit)
				{
					case Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing:
						qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.PurchaseItem.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
						break;
					case Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales:
						qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.SalesItem.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
						break;
					default:
						qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryItem.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
						break;
				}

				if (filterOutFrozenItems)
				{
					Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(qryBuilder, " AND ");
				}

				if (crossDocking && purchaseDocRef.DocType == SboObjectTypes.PurchaseOrder)
				{
					qryBuilder.AppendFormat("\r\n AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, CultureInfo.InvariantCulture));
				}

				if (purchaseDocRef == null)
				{
					qryBuilder.Append("\r\n UNION ");

					qryBuilder.Append("SELECT ");
					if (distinct)
					{
						qryBuilder.Append(" DISTINCT ");
					}
					else if (addLineNumberBatchAndSerialNumberColumns)
					{
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineNum.NAME, ProductDataSet.Columns.LINE_NUMBER.Name);
						qryBuilder.Append(", ");
					}

					SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

					qryBuilder.Append(", CASE");
					qryBuilder.Flags(QBF.Translate);
					if (purchaseDocRef != null || !String.IsNullOrEmpty(cardCode))
					{
						qryBuilder.AppendFormat(" WHEN {0} <> '' THEN {0} || ' - ' || {1} || ' - ' || {2} ",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.SubstituteItemDescr.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.Substitute.NAME));
					}
					qryBuilder.AppendFormat(" WHEN {0} <> '' THEN {0} || ' - ' || {1} ",
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.VendorItemDescr.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME));
					qryBuilder.Flags(QBF.TranslateOff);
					qryBuilder.AppendFormat(" ELSE {0} END",
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME));
					qryBuilder.Append(" AS ", SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name));

					qryBuilder.Append(", ");
					Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(qryBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.HasBestBeforeDate.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.PrintItemLabelReception.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ManBtchNum.NAME));

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticUnit.NAME));
					qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.NumberOfItemsPerPurchaseUnit.NAME));
					qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.PurchaseUom.NAME));
					qryBuilder.Append(", ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryUom.NAME));
					qryBuilder.Append(", ");
					Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(qryBuilder, packagingUnit, SboItemTable.Columns.CodeBars.NAME);
					//SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.PurchaseBarcode.NAME);

					if ((purchaseDocRef != null || containerDocEntry.HasValue) && !distinct)
					{
						if (containerDocEntry.HasValue)
						{
							qryBuilder.Append(", ");
							SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.OpenQuantity.NAME, SboPurchaseInvoiceLineTable.Columns.OpenCreQty.NAME);
						}
						else
						{
							qryBuilder.Append(", ");
							qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.OpenCreQty.NAME));
						}

						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.LineNum.NAME));

						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.DocEntry.NAME));

						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.ObjectType.NAME));

						qryBuilder.Append(", ");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.QuantityPerUom.NAME));

						if (containerDocEntry.HasValue)
						{
							qryBuilder.AppendFormat(", {0} * {1} AS {2} ",
							SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.OpenQuantity.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxContainerLineTable.NAME, PmxContainerLineTable.Columns.QuantityPerUom.NAME),
							SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.OpenInvQty.NAME));
						}
						else
						{
							qryBuilder.Append(", ");
							qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.OpenInvQty.NAME));
						}

						qryBuilder.AppendFormat(", CASE WHEN {0} = -1 THEN " + qryBuilder.IFNULL + "({2}, {3}) ELSE  " + qryBuilder.IFNULL + "({1}, {2}) END AS {4} ",
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.UomEntry.NAME),
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.UomCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.Uom.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, PmxItemTable.Columns.InventoryUom.NAME),
							SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.UomCode.NAME));
					}

					//if (poDocEntry.HasValue && !distinct)
					//{

					//    strBuilder.Append(", ");
					//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.NAME));
					//    strBuilder.Append(".");
					//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.DocEntry.NAME));

					//    strBuilder.Append(", ");
					//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.NAME));
					//    strBuilder.Append(".");
					//    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineNum.NAME));

					//}

					qryBuilder.Append(", ");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

					if (!distinct && addLineNumberBatchAndSerialNumberColumns)
					{
						qryBuilder.Append(", ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxBatchNumber.NAME, ProductDataSet.Columns.BATCH1.Name);
						qryBuilder.Append(", ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxBatchNumber2.NAME, ProductDataSet.Columns.BATCH2.Name);
						qryBuilder.Append(", ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BestBeforeDate.NAME, ProductDataSet.Columns.BEST_BEFORE_DATE.Name);
						qryBuilder.Append(", ");
						SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.PmxSerialNumber.NAME, ProductDataSet.Columns.SERIAL_NUMBER_INFO.Name);
					}

					qryBuilder.Append("\r\n FROM ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(qryBuilder.WITH_NOLOCK);
					if (purchaseDocRef != null || containerDocEntry.HasValue)
					{
						//inner join Product - PO Line
						qryBuilder.Append("\r\n INNER JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
						qryBuilder.Append(" ON ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.Columns.ItemCode.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));

						//inner join PO - PO Line
						qryBuilder.Append("\r\n INNER JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceDocTable));
						qryBuilder.Append(" ON ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceDocTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.Columns.DocEntry.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.Columns.DocEntry.NAME));

						qryBuilder.Append("\r\n LEFT JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxAlternateCatNumTable.NAME));
						qryBuilder.AppendFormat(" ON {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.CardCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceDocTable, SboPurchaseInvoiceTable.Columns.CardCode.NAME));
						qryBuilder.AppendFormat(" AND {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.ItemCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
						qryBuilder.AppendFormat(" AND {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.Substitute.NAME),
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.BPCatalogNumber.NAME));

						qryBuilder.Append("\r\n LEFT JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
						qryBuilder.Append(" ON ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
						qryBuilder.Append(" = ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.UomEntry.NAME);
						qryBuilder.Append(" AND ");
						SqlCommandHelper.AddTableColumnName(qryBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
						qryBuilder.Append(" <> -1 ");

						if (containerDocEntry.HasValue)
						{
							//inner join Container line - PO Line
							qryBuilder.Append("\r\n INNER JOIN ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
							qryBuilder.Append(" ON ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseEntry.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.Columns.DocEntry.NAME));
							qryBuilder.Append(" AND ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseLine.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.Columns.LineNum.NAME));
							qryBuilder.Append(" AND ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.NAME));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxContainerLineTable.Columns.BaseType.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(purchaseInvoiceLineTable));
							qryBuilder.Append(".");
							qryBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.Columns.ObjectType.NAME));
						}
					}
					else if (!String.IsNullOrEmpty(cardCode))
					{
						qryBuilder.Append("\r\n LEFT JOIN ");
						qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxAlternateCatNumTable.NAME));
						qryBuilder.AppendFormat(" ON {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.CardCode.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));
						qryBuilder.AppendFormat(" AND {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxAlternateCatNumTable.NAME, PmxAlternateCatNumTable.Columns.ItemCode.NAME),
							SqlCommandHelper.EscapeTableColumnName(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
					}

					operand = "\r\n WHERE ";
					if (!string.IsNullOrEmpty(vendorCode))
					{
						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.CardCode.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(vendorCode, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
					}

					if (!string.IsNullOrEmpty(gtin))
					{
						qryBuilder.Append(operand);
						Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcodePackagingType(qryBuilder, gtin);
						operand = "\r\n AND ";
					}

					if (whsInfo != null && !isNoPo && !autoSelectPO)
					{
						qryBuilder.AppendFormat("{0}{1} = {2}",
							operand,
							SqlCommandHelper.EscapeTableColumnName(purchaseInvoiceLineTable, SboPurchaseInvoiceLineTable.Columns.WarehouseCode.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(whsInfo.SboWarehouseCode, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
					}

					if (containerDocEntry.HasValue)
					{
						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(purchaseInvoiceLineTable));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.InventoryStatus.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";

						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.Columns.DocEntry.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(containerDocEntry.Value, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";

						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxContainerLineTable.Columns.OpenQuantity.NAME));
						qryBuilder.Append(" > 0 ");
					}

					if (!string.IsNullOrEmpty(itemCode))
					{
						qryBuilder.Append(operand);
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
						qryBuilder.Append(".");
						qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
						qryBuilder.Append(" = ");
						qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
						operand = "\r\n AND ";
					}

					qryBuilder.Append(operand);
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
					qryBuilder.Append(".");
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
					qryBuilder.Append(" = ");
					qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
					operand = "\r\n AND ";

					switch (packagingUnit)
					{
						case Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Purchasing:
							qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.PurchaseItem.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
							operand = "\r\n AND ";
							break;
						case Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales:
							qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.SalesItem.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
							operand = "\r\n AND ";
							break;
						default:
							qryBuilder.Append(operand, qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.InventoryItem.NAME));
							qryBuilder.Append(" = ");
							qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
							operand = "\r\n AND ";
							break;
					}

					if (filterOutFrozenItems)
					{
						Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(qryBuilder, " AND ");
					}

					if (crossDocking && purchaseDocRef.DocType == SboObjectTypes.PurchaseOrder)
					{
						qryBuilder.AppendFormat("\r\n AND {0} = {1}",
							SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
							SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, CultureInfo.InvariantCulture));
					}
				}

				qryBuilder.Append("\r\n ORDER BY ");

				if (addLineNumberBatchAndSerialNumberColumns && !distinct)
				{
					qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.LINE_NUMBER.Name));
					qryBuilder.Append(",");
				}
				//strBuilder.Append( SqlCommandHelper.EscapeColumnName( PmxItemTable.VIEW_NAME ) );
				//strBuilder.Append( "." );
				//strBuilder.Append( SqlCommandHelper.EscapeColumnName( PmxItemTable.Columns.CustomItemDescr.NAME ) );
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name));
				return qryBuilder.ToString();
			}

			/// <summary>
			/// Command for getting all logistic carriers order by name
			/// </summary>
			/// <param name="onlyInStock">if set to <c>true</c> the system will get the logistic carriers that are in stock and have no products on it (LUID IS NULL)</param>
			/// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
			/// <param name="dbTool">The db tool.</param>
			/// <returns>
			/// Command for getting all logistic carriers order by name
			/// </returns>
			public static string GetLogisticCarriersOrderByName(bool onlyInStock, bool filterOutFrozenItems, DbTool dbTool)
			{
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxItemTable.VIEW_NAME, SboItemTable.Columns.CodeBars.NAME, ProductDataSet.Columns.GTIN.Name);

				qryBuilder.Append(", ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));



				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);
				if (onlyInStock)
				{
					qryBuilder.Append(" INNER JOIN ");
					qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					qryBuilder.Append(" ON ", qryBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" = ", qryBuilder.AsIdentifier(PmxItemTable.VIEW_NAME, SboItemTable.Columns.ItemCode.NAME));
					qryBuilder.Append(" AND ", qryBuilder.AsIdentifier(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));
					qryBuilder.Append(" IS NULL ");
				}




				qryBuilder.Append(" WHERE ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.PurchaseItem.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				qryBuilder.Append(" AND ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
				qryBuilder.Append(" = ");
				qryBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				if (filterOutFrozenItems)
				{
					Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(qryBuilder, " AND ");
				}


				qryBuilder.Append(" ORDER BY ");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(".");
				qryBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
				return qryBuilder.ToString();

			}


			/// <summary>
			/// Gets the locked stock for base order.
			/// </summary>
			/// <param name="itemCodes">The item codes.</param>
			/// <param name="purchaseDocRef">The purchase doc ref.</param>
			/// <returns></returns>
			public static string GetLockedStockForBaseOrder(Collection<string> itemCodes, DocumentRef purchaseDocRef)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, SSCCDataSet.Columns.LUID.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name);

				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));

				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));

				if (purchaseDocRef.DocType == SboObjectTypes.PurchaseOrder)
				{
					strBuilder.AppendFormat("\r\n FROM {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
				}
				else if (purchaseDocRef.DocType == SboObjectTypes.PurchaseInvoice)
				{
					strBuilder.AppendFormat("\r\n FROM {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME));

					strBuilder.AppendFormat("\r\n INNER JOIN {0}",
						SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
					strBuilder.AppendFormat(" ON {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseType.NAME));
					strBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseEntry.NAME));
					strBuilder.AppendFormat(" AND {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.BaseLine.NAME));
				}

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME);
				strBuilder.Append(" = CAST(");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME);
				strBuilder.Append(" AS nvarchar(20)) AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME);


				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));


				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));


				if (purchaseDocRef.DocType == SboObjectTypes.PurchaseOrder)
				{
					strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(purchaseDocRef.DocEntry, CultureInfo.InvariantCulture));
					strBuilder.AppendFormat(" AND {0} IN ( {1} )",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ItemCode.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(itemCodes, CultureInfo.InvariantCulture));
				}
				else if (purchaseDocRef.DocType == SboObjectTypes.PurchaseInvoice)
				{
					strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(purchaseDocRef.DocEntry, CultureInfo.InvariantCulture));
					strBuilder.AppendFormat(" AND {0} IN ( {1} )",
						SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ItemCode.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(itemCodes, CultureInfo.InvariantCulture));
				}

				strBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);


				return strBuilder.ToString();

			}




			/// <summary>
			/// Gets the stock with later BBD.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="bbd">The BBD.</param>
			/// <returns></returns>
			public static string GetStockWithLaterBBD(string itemCode, DateTime bbd)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));


				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));


				strBuilder.Append(" AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
				strBuilder.Append(" > ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboHelper.ConvertDateToSboDate(bbd), CultureInfo.InvariantCulture));




				strBuilder.Append(" ORDER BY ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
				strBuilder.Append(" DESC ");


				return strBuilder.ToString();

			}

			/// <summary>
			/// Gets the received but unfinished orders.
			/// </summary>
			/// <param name="currentDockCode">The current dock code.</param>
			/// <param name="dbCulture">The database culture.</param>
			/// <returns></returns>
			internal static string GetReceivedButUnfinishedOrders(string currentDockCode, CultureInfo dbCulture)
			{
				StringBuilder strBuilder = new StringBuilder();

				strBuilder.AppendFormat("SELECT DISTINCT {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocEntry.NAME, PurchaseOrderDataSet.Columns.PO_ID.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocNum.NAME, PurchaseOrderDataSet.Columns.PO_NUMBER.Name));
				strBuilder.AppendFormat(", CAST( {0} AS NVARCHAR ) AS {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocNum.NAME),
					SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.PO_NUMBER_ASSTRING.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.CardCode.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.CardName.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_NAME.Name));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.NumAtCard.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_REFERENCE.Name));

				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME, "ORDR_DocEntry"));

				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.ObjType.NAME, PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name));


				strBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME));
				strBuilder.AppendFormat(" AND ( {0} = {1} OR ( {0} IS NULL AND {1} IS NULL ) )",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemTransactionalInfoKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.StorLocCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseEntry.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME));

				strBuilder.AppendFormat("\r\n INNER JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxSalesOrderTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME));

				strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseEntry.NAME));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseLine.NAME));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineStatus.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue("O", dbCulture));

				strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(currentDockCode, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.TransType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.PurchaseDelivery, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.PurchaseOrder, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue((int)SboObjectType.SalesOrder, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocStatus.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, dbCulture));
				strBuilder.AppendFormat(" AND {0} IS NULL",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the sales order and pick list locked stock.
			/// Locked for:
			/// * SalesOrder -&gt; used for auto-overpicking
			/// * PickList -&gt; used for auto-picking, only returns stock locked on LUID's which are received on the given PurchaseOrder
			/// </summary>
			/// <param name="purchaseOrderDocEntry">The purchase order doc entry.</param>
			/// <param name="salesOrderDocEntry">The sales order doc entry.</param>
			/// <param name="pickListDocEntry">The pick list doc entry.</param>
			/// <param name="dbCulture">The db culture.</param>
			/// <returns></returns>
			internal static string GetSalesOrderAndPickListLockedStock(int purchaseOrderDocEntry, int salesOrderDocEntry, int pickListDocEntry, CultureInfo dbCulture)
			{
				StringBuilder strBuilder = new StringBuilder();

				strBuilder.AppendFormat("SELECT {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseLine.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.Quantity.NAME));
				strBuilder.AppendFormat(", {0}",
					SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QuantityUom2.NAME, PmxInventoryLockingDetailTable.Columns.QuantityUom2.NAME));
				//strBuilder.AppendFormat( ", {0}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryTotalTable.Columns.QuantityPerUom.NAME ) );
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME));
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QualityStatusCode.NAME));
				//strBuilder.AppendFormat( ", {0}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME ) );
				//strBuilder.AppendFormat( ", {0}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME ) );

				strBuilder.AppendFormat("\r\n FROM {0}",
					SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));

				//strBuilder.AppendFormat( "\r\n INNER JOIN {0}",
				//    SqlCommandHelper.EscapeTableName( PmxInventoryTotalTable.NAME ) );
				//strBuilder.AppendFormat( " ON {0} = {1}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME ),
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME ) );
				//strBuilder.AppendFormat( " AND ( {0} = {1} OR ( {0} IS NULL AND {1} IS NULL ) )",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME ),
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME ) );
				//strBuilder.AppendFormat( " AND {0} = {1}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ),
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME ) );
				//strBuilder.AppendFormat( " AND {0} = {1}",
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME ),
				//    SqlCommandHelper.EscapeTableColumnName( PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QualityStatusCode.NAME ) );

				strBuilder.AppendFormat("\r\n WHERE ( {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1} )",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(salesOrderDocEntry, dbCulture));

				strBuilder.AppendFormat(" OR ( {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxPurchaseOrderLineTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickList, dbCulture));
				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, dbCulture));
				strBuilder.AppendFormat(" AND {0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} AND {5} = {6} ) )",
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME),
					SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseType.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.PurchaseOrder, dbCulture),
					SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(purchaseOrderDocEntry, dbCulture));

				return strBuilder.ToString();
			}

			//internal static string GetPurchaseOrderReceivedStock( int purchaseOrderDocEntry, CultureInfo dbCulture )
			//{
			//    StringBuilder strBuilder = new StringBuilder();

			//    strBuilder.AppendFormat( "SELECT {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemCode.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemTransactionalInfoKey.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.QualityStatusCode.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.Quantity.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.QuantityPerUom.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.StorLocCode.NAME ) );
			//    strBuilder.AppendFormat( ", {0}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME ) );

			//    strBuilder.AppendFormat( "\r\n FROM {0}",
			//        SqlCommandHelper.EscapeTableName( PmxInventoryTotalTable.NAME ) );

			//    strBuilder.AppendFormat( "\r\n INNER JOIN {0}",
			//        SqlCommandHelper.EscapeTableName( PmxInventoryDetailTable.NAME ) );
			//    strBuilder.AppendFormat( " ON {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemCode.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME ) );
			//    strBuilder.AppendFormat( " AND ( {0} = {1} OR ( {0} IS NULL AND {1} IS NULL ) )",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.ItemTransactionalInfoKey.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME ) );
			//    strBuilder.AppendFormat( " AND {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.StorLocCode.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME ) );
			//    strBuilder.AppendFormat( " AND {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME ) );

			//    strBuilder.AppendFormat( "\r\n INNER JOIN {0}",
			//        SqlCommandHelper.EscapeTableName( PmxPurchaseOrderLineTable.NAME ) );
			//    strBuilder.AppendFormat( " ON {0} = {1}",
			//        SqlCommandHelper.FormatAndEscapeColumnValue( SboObjectTypes.PurchaseOrder, dbCulture ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseType.NAME ) );
			//    strBuilder.AppendFormat( " AND {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseEntry.NAME ) );
			//    strBuilder.AppendFormat( " AND {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME ),
			//        SqlCommandHelper.EscapeTableColumnName( PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.BaseLine.NAME ) );

			//    strBuilder.AppendFormat( "\r\n WHERE {0} = {1}",
			//        SqlCommandHelper.EscapeTableColumnName( PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME ),
			//        SqlCommandHelper.FormatAndEscapeColumnValue( purchaseOrderDocEntry, dbCulture ) );

			//    return strBuilder.ToString();
			//}


			/// <summary>
			/// Gets the zone types for items.
			/// </summary>
			/// <param name="itemCodes">The item codes.</param>
			/// <returns></returns>
			public static string GetZoneTypesForItems(Collection<string> itemCodes)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT DISTINCT");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemZoneTypeTable.NAME));


				string operand = "";
				strBuilder.Append(" WHERE ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME);
				strBuilder.Append(" IN ( ");

				foreach (string itemCode in itemCodes)
				{
					strBuilder.Append(operand);
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
					operand = ", ";
				}
				strBuilder.Append(") ");


				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the name and the code of the  location, ordered by name.
			/// </summary>
			/// <param name="locationCode">The location code.</param>
			/// <param name="warehouseCode">The warehouse code.</param>
			/// <param name="zoneTypeCodes">The zone type codes.</param>
			/// <param name="emptyLocations">if set to <c>true</c> [empty locations].</param>
			/// <returns>
			/// The sql statement to get the location
			/// </returns>
			public static string GetStorageLocationForZoneTypes(string locationCode, string warehouseCode, Collection<string> zoneTypeCodes, bool emptyLocations)
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);


				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

				strBuilder.Append("\r\n INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

				strBuilder.Append("\r\n AND ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



				if (emptyLocations)
				{
					strBuilder.Append(" LEFT JOIN ");
					strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(" ON ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					strBuilder.Append(" = ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

				}

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


				if (!string.IsNullOrEmpty(locationCode))
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
				}

				if (warehouseCode != null &&
					warehouseCode.Trim() != "")
				{
					strBuilder.Append(" AND ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

				}

				if (zoneTypeCodes != null &&
					zoneTypeCodes.Count > 0)
				{
					foreach (string zoneTypeCode in zoneTypeCodes)
					{
						strBuilder.Append(" AND ");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));
						strBuilder.Append(" IN ( ");

						strBuilder.Append("SELECT ");

						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Code.NAME));


						strBuilder.Append(" FROM ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
						strBuilder.Append(" INNER JOIN ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
						strBuilder.Append(" ON ");
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
						strBuilder.Append(" = ");
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
						strBuilder.Append(" RIGHT OUTER JOIN ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkZoneZoneTypeTable.NAME));
						strBuilder.Append(" ON ");
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxZoneCode.NAME);
						strBuilder.Append(" = ");
						SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkZoneZoneTypeTable.NAME, PmxLinkZoneZoneTypeTable.Columns.ParentCode.NAME);


						strBuilder.Append(" WHERE ");


						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLinkZoneZoneTypeTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLinkZoneZoneTypeTable.Columns.ZoneTypeCode.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(zoneTypeCode, CultureInfo.InvariantCulture));


						strBuilder.Append(") ");

					}
				}

				if (emptyLocations)
				{
					//If there is no link to the inventory, the location is empty
					strBuilder.Append(" AND ");
					SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
					strBuilder.Append(" IS NULL ");

				}

				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets the reasons.
			/// </summary>
			/// <returns></returns>
			public static string GetReasons()
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT  ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME, ReasonDataSet.Columns.REASON_CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Name.NAME, ReasonDataSet.Columns.REASON_NAME.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.RequiresFreeText.NAME, ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReasonTable.NAME));

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReasonTable.Columns.UseForPurchaseReceipt.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				strBuilder.AppendFormat(
			  " ORDER BY CASE WHEN {0} IS NULL THEN 1 ELSE 0 END, {0} ",
			   SqlCommandHelper.EscapeTableColumnName(PmxReasonTable.NAME, PmxReasonTable.Columns.SequenceNumber.NAME));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the reasons for zonetypes
			/// </summary>
			/// <returns></returns>
			public static string GetReasonsForZoneTypes()
			{
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT  ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME, ReasonDataSet.Columns.REASON_CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Name.NAME, ReasonDataSet.Columns.REASON_NAME.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.RequiresFreeText.NAME, ReasonDataSet.Columns.REASON_REQUIRES_FREE_TEXT.Name);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReasonTable.NAME));

				strBuilder.Append(" WHERE ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReasonTable.Columns.UseForZoneType.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				strBuilder.AppendFormat(
			  " ORDER BY CASE WHEN {0} IS NULL THEN 1 ELSE 0 END, {0} ",
			   SqlCommandHelper.EscapeTableColumnName(PmxReasonTable.NAME, PmxReasonTable.Columns.SequenceNumber.NAME));

				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets the zone types on Item
			/// </summary>
			/// <param name="dbCulture">The database culture.</param>
			/// <returns></returns>
			public static string GetZoneTypesOnItem(
				// string itemCode, 
				CultureInfo dbCulture)
			{
				/*
                 * SELECT [PMX_ITZT].[ItemCode], PMX_ITZT.ZoneTypeCode, PMX_ZOTY.CODE, PMX_ZOTY.Name 
FROM PMX_ITZT 
INNER JOIN PMX_ZOTY 
ON 
PMX_ITZT.[ZoneTypeCode] = PMX_ZOTY.[Code]
 WHERE PMX_ZOTY.ShowZoneTYpe = 'N'
                AND
PMX_ITZT = 'itemCode'
                 */
				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Name.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemZoneTypeTable.NAME));

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxZoneTypeTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME);

				strBuilder.AppendFormat(" WHERE {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.ShowZoneType.NAME),
				SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbCulture));
				//strBuilder.AppendFormat(" AND {0} = {1}",
				//SqlCommandHelper.EscapeTableColumnName(PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME),
				//SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbCulture));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the zone types on item code and possible reasons.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="mustHaveReasons">if set to <c>true</c> [must have reasons].</param>
			/// <param name="dbCulture">The database culture.</param>
			/// <returns></returns>
			public static string GetZoneTypesOnItemCodeAndPossibleReasons(string itemCode, bool mustHaveReasons, CultureInfo dbCulture)
			{
				/*                 
SELECT [PMX_ITZT].[ItemCode], [PMX_ITZT].[ZoneTypeCode], [PMX_ZOTY].[Name],
[PMX_QLOG].[QualityLogOperationCode], 
[PMX_REAS].[Code], 
[PMX_REAS].[Name]
FROM [PMX_ITZT] 
                 */

				StringBuilder strBuilder = new StringBuilder();
				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME, ZoneTypeDataSet.Columns.ZONETYPE_CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Name.NAME, ZoneTypeDataSet.Columns.ZONETYPE_NAME.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME, ZoneTypeDataSet.Columns.REASON_CODE.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Name.NAME, ZoneTypeDataSet.Columns.REASON_NAME.Name);
				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.NeedsReason.NAME, ZoneTypeDataSet.Columns.NEEDS_REASON.Name);


				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemZoneTypeTable.NAME));

				/*
                 * 
INNER JOIN [PMX_ZOTY] 
ON [PMX_ITZT].[ZoneTypeCode] = [PMX_ZOTY].[Code] 
INNER JOIN [PMX_QLOG]
ON [PMX_QLOG].[QualityLogOperationType] = [PMX_ZOTY].[Code]
FULL OUTER JOIN[PMX_REAS]
ON [PMX_REAS].[Code] = [PMX_QLOG].[ReasonCode]
                 */

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxZoneTypeTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME);

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxQualityLogTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxQualityLogTable.NAME, PmxQualityLogTable.Columns.QualityLogOperationType.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME);

				strBuilder.Append(" FULL OUTER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReasonTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxReasonTable.NAME, PmxReasonTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxQualityLogTable.NAME, PmxQualityLogTable.Columns.ReasonCode.NAME);



				strBuilder.AppendFormat(" WHERE {0} = {1}",
				SqlCommandHelper.EscapeTableColumnName(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.ShowZoneType.NAME),
				SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbCulture));

				strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbCulture));

				if (mustHaveReasons)
				{
					strBuilder.AppendFormat(" AND {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.NeedsReason.NAME),
					SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbCulture));
				}

				return strBuilder.ToString();
			}


			/// <summary>
			/// Gets the zone types on item code.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="dbCulture">The database culture.</param>
			/// <param name="dbTool">The dbTool</param>
			/// <returns></returns>
			public static string GetZoneTypesOnItemCode(string itemCode, CultureInfo dbCulture, DbTool dbTool)
			{
				/*
                SELECT [PMX_ZOTY].[Code] AS [ZoneTypeCode], [PMX_ZOTY].[Name] AS [ZoneTypeName], 
 '' AS [ReasonCode],  '' AS [ReasonName], [PMX_ZOTY].[NeedsReason] AS [NeedsReason] 
FROM [OITM] 
INNER JOIN [PMX_ITZT] 
ON [PMX_ITZT].[ItemCode] = [OITM].[ItemCode]
INNER JOIN [PMX_ZOTY]
ON [PMX_ZOTY].[Code] = [PMX_ITZT].[ZoneTypeCode]
WHERE [PMX_ZOTY].[ShowZoneType] = N'Y' AND [PMX_ITZT].[ItemCode] = N'RWU001'
                 */
				DbQueryBuilder qryBuilder = dbTool.GetQueryBuilder();
				qryBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME, ZoneTypeDataSet.Columns.ZONETYPE_CODE.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Name.NAME, ZoneTypeDataSet.Columns.ZONETYPE_NAME.Name);
				//strBuilder.Append(", ");
				//SqlCommandHelper.AddEmptyStringColumnWithName(strBuilder, ZoneTypeDataSet.Columns.REASON_CODE.Name);
				//strBuilder.Append(", ");
				//SqlCommandHelper.AddEmptyStringColumnWithName(strBuilder, ZoneTypeDataSet.Columns.REASON_NAME.Name);
				qryBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(qryBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.NeedsReason.NAME, ZoneTypeDataSet.Columns.NEEDS_REASON.Name);

				qryBuilder.Append(" FROM ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.VIEW_NAME));
				qryBuilder.Append(qryBuilder.WITH_NOLOCK);

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemZoneTypeTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemTable.VIEW_NAME, PmxItemTable.Columns.ItemCode.NAME);

				qryBuilder.Append(" INNER JOIN ");
				qryBuilder.Append(SqlCommandHelper.EscapeTableName(PmxZoneTypeTable.NAME));
				qryBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME);
				qryBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(qryBuilder, PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME);

				qryBuilder.AppendFormat(" WHERE {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.ShowZoneType.NAME),
				   SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbCulture));

				qryBuilder.AppendFormat(" AND {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME),
				   SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbCulture));
				return qryBuilder.ToString();
			}


			/// <summary>
			/// Gets the open purchase order lines for card code and item code.
			/// </summary>
			/// <param name="itemCode">The item code.</param>
			/// <param name="cardCode">The card code.</param>
			/// <param name="dbTool">The database tool.</param>
			/// <returns></returns>
			internal static string GetOpenPOLinesForCardCodeAndItemCode(string itemCode, string cardCode, DbTool dbTool)
			{
				StringBuilder strBuilder = new StringBuilder();

				strBuilder.Append("SELECT ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME);

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME));

				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPurchaseOrderTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocEntry.NAME);


				strBuilder.AppendFormat(" WHERE {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineStatus.NAME),
				   SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, dbTool.CultureInfo));

				strBuilder.AppendFormat(" AND {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ItemCode.NAME),
				   SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, dbTool.CultureInfo));

				strBuilder.AppendFormat(" AND {0} = {1}",
				   SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.CardCode.NAME),
				   SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, dbTool.CultureInfo));

				return strBuilder.ToString();
			}

			/// <summary>
			/// Gets the query to get the remarks
			/// </summary>
			/// <param name="docEntry">The doc entry.</param>
			/// <param name="objType">Type of the obj.</param>
			/// <param name="viewName">Name of the view.</param>
			/// <param name="dbTool">The dbtool</param>
			/// <returns></returns>
			public static string GetRemarks(int docEntry, string objType, string viewName, DbTool dbTool)
			{
				return Produmex.Sbo.Logex.Data.SboCommands.GetReceptionRemarks(docEntry, objType, viewName, dbTool);
			}

			/// <summary>
			/// Gets the query to get the purchase line remarks
			/// </summary>
			/// <param name="docEntry">The doc entry.</param>
			/// <param name="objType">Type of the obj.</param>
			/// <param name="lineNr">The line number (can be null).</param>
			/// <param name="itemCode">The itemCode.</param>
			/// <param name="viewName">Name of the view.</param>
			/// <param name="dbTool">The dbtool</param>
			/// <returns>Returns the query string</returns>
			public static string GetPurchaseLineRemarks(int docEntry, string objType, int? lineNr, string itemCode, string viewName, DbTool dbTool)
			{
				return Produmex.Sbo.Logex.Data.SboCommands.GetReceptionPurchaseLineRemarks(docEntry, objType, lineNr, itemCode, viewName, dbTool);
			}

			/// <summary>
			/// Gets the query to get the serial numbers
			/// </summary>
			/// <param name="docEntry">The doc entry.</param>         
			/// <param name="dbTool">The dbtool</param>
			/// <returns>Returns the query string</returns>
			public static string GetSerialNumbersFromPicklist(Collection<int> docEntries, DbTool dbTool)
			{
				List<int> listOfDocEntries = docEntries.Cast<int>().ToList();
				return Produmex.Sbo.Logex.Data.SboCommands.GetSerialNumbersFromPicklist(listOfDocEntries, dbTool);
			}
		}


		#endregion

	}
}